@@ -1,7 +1,9 @@
 #!/usr/bin/env bash
 set -ex
 
-git submodule update --init --recursive
+if [ -d .git ]; then
+    git submodule update --init --recursive
+fi
 
 : ${BUILD_DIR:=build}
 : ${CEPH_GIT_DIR:=..}
@@ -11,21 +13,27 @@ if [ -e $BUILD_DIR ]; then
     exit 1
 fi
 
-PYBUILD="2"
+PYBUILD="3"
+ARGS="-GNinja"
 if [ -r /etc/os-release ]; then
   source /etc/os-release
   case "$ID" in
       fedora)
-          PYBUILD="3.7"
-          if [ "$VERSION_ID" -eq "32" ] ; then
-              PYBUILD="3.8"
+          if [ "$VERSION_ID" -ge "35" ] ; then
+            PYBUILD="3.10"
           elif [ "$VERSION_ID" -ge "33" ] ; then
-              PYBUILD="3.9"
+            PYBUILD="3.9"
+          elif [ "$VERSION_ID" -ge "32" ] ; then
+            PYBUILD="3.8"
+          else
+            PYBUILD="3.7"
           fi
           ;;
-      rhel|centos)
+      almalinux|rocky|rhel|centos)
           MAJOR_VER=$(echo "$VERSION_ID" | sed -e 's/\..*$//')
-          if [ "$MAJOR_VER" -ge "8" ] ; then
+          if [ "$MAJOR_VER" -ge "9" ] ; then
+              PYBUILD="3.9"
+          elif [ "$MAJOR_VER" -ge "8" ] ; then
               PYBUILD="3.6"
           fi
           ;;
@@ -44,9 +52,7 @@ else
   exit 1
 fi
 
-if [[ "$PYBUILD" =~ ^3(\..*)?$ ]] ; then
-    ARGS+=" -DWITH_PYTHON3=${PYBUILD}"
-fi
+ARGS+=" -DWITH_PYTHON3=${PYBUILD}"
 
 if type ccache > /dev/null 2>&1 ; then
     echo "enabling ccache"
@@ -72,7 +78,7 @@ EOF
 
 echo done.
 
-if [[ ! $ARGS =~ "-DCMAKE_BUILD_TYPE" ]]; then
+if [[ ! "$ARGS $@" =~ "-DCMAKE_BUILD_TYPE" ]]; then
   cat <<EOF
 
 ****
