@@ -22,9 +22,9 @@ struct AuthNoneAuthorizeHandler : public AuthAuthorizeHandler {
   bool verify_authorizer(
     CephContext *cct,
     const KeyStore& keys,
-    const bufferlist& authorizer_data,
+    const ceph::buffer::list& authorizer_data,
     size_t connection_secret_required_len,
-    bufferlist *authorizer_reply,
+    ceph::buffer::list *authorizer_reply,
     EntityName *entity_name,
     uint64_t *global_id,
     AuthCapsInfo *caps_info,
