@@ -2,7 +2,7 @@
 // impl/compose.hpp
 // ~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2019 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,11 +16,14 @@
 #endif // defined(_MSC_VER) && (_MSC_VER >= 1200)
 
 #include <boost/asio/detail/config.hpp>
+#include <boost/asio/associated_executor.hpp>
 #include <boost/asio/detail/handler_alloc_helpers.hpp>
 #include <boost/asio/detail/handler_cont_helpers.hpp>
 #include <boost/asio/detail/handler_invoke_helpers.hpp>
 #include <boost/asio/detail/type_traits.hpp>
 #include <boost/asio/detail/variadic_templates.hpp>
+#include <boost/asio/execution/executor.hpp>
+#include <boost/asio/execution/outstanding_work.hpp>
 #include <boost/asio/executor_work_guard.hpp>
 #include <boost/asio/is_executor.hpp>
 #include <boost/asio/system_executor.hpp>
@@ -32,6 +35,50 @@ namespace asio {
 
 namespace detail
 {
+  template <typename Executor, typename = void>
+  class composed_work_guard
+  {
+  public:
+    typedef typename decay<
+        typename prefer_result<Executor,
+          execution::outstanding_work_t::tracked_t
+        >::type
+      >::type executor_type;
+
+    composed_work_guard(const Executor& ex)
+      : executor_(boost::asio::prefer(ex, execution::outstanding_work.tracked))
+    {
+    }
+
+    void reset()
+    {
+    }
+
+    executor_type get_executor() const BOOST_ASIO_NOEXCEPT
+    {
+      return executor_;
+    }
+
+  private:
+    executor_type executor_;
+  };
+
+#if !defined(BOOST_ASIO_NO_TS_EXECUTORS)
+
+  template <typename Executor>
+  struct composed_work_guard<Executor,
+      typename enable_if<
+        !execution::is_executor<Executor>::value
+      >::type> : executor_work_guard<Executor>
+  {
+    composed_work_guard(const Executor& ex)
+      : executor_work_guard<Executor>(ex)
+    {
+    }
+  };
+
+#endif // !defined(BOOST_ASIO_NO_TS_EXECUTORS)
+
   template <typename>
   struct composed_io_executors;
 
@@ -159,7 +206,7 @@ namespace detail
     }
 
     typedef system_executor head_type;
-    executor_work_guard<system_executor> head_;
+    composed_work_guard<system_executor> head_;
   };
 
   template <typename Head>
@@ -178,7 +225,7 @@ namespace detail
     }
 
     typedef Head head_type;
-    executor_work_guard<Head> head_;
+    composed_work_guard<Head> head_;
   };
 
 #if defined(BOOST_ASIO_HAS_VARIADIC_TEMPLATES)
@@ -201,7 +248,7 @@ namespace detail
     }
 
     typedef Head head_type;
-    executor_work_guard<Head> head_;
+    composed_work_guard<Head> head_;
     composed_work<void(Tail...)> tail_;
   };
 
@@ -227,7 +274,7 @@ namespace detail
     } \
   \
     typedef Head head_type; \
-    executor_work_guard<Head> head_; \
+    composed_work_guard<Head> head_; \
     composed_work<void(BOOST_ASIO_VARIADIC_TARGS(n))> tail_; \
   }; \
   /**/
@@ -249,12 +296,13 @@ namespace detail
 #endif // defined(BOOST_ASIO_HAS_VARIADIC_TEMPLATES)
   {
   public:
-    composed_op(BOOST_ASIO_MOVE_ARG(Impl) impl,
-        BOOST_ASIO_MOVE_ARG(Work) work,
-        BOOST_ASIO_MOVE_ARG(Handler) handler)
-      : impl_(BOOST_ASIO_MOVE_CAST(Impl)(impl)),
-        work_(BOOST_ASIO_MOVE_CAST(Work)(work)),
-        handler_(BOOST_ASIO_MOVE_CAST(Handler)(handler)),
+    template <typename I, typename W, typename H>
+    composed_op(BOOST_ASIO_MOVE_ARG(I) impl,
+        BOOST_ASIO_MOVE_ARG(W) work,
+        BOOST_ASIO_MOVE_ARG(H) handler)
+      : impl_(BOOST_ASIO_MOVE_CAST(I)(impl)),
+        work_(BOOST_ASIO_MOVE_CAST(W)(work)),
+        handler_(BOOST_ASIO_MOVE_CAST(H)(handler)),
         invocations_(0)
     {
     }
@@ -270,7 +318,10 @@ namespace detail
 #endif // defined(BOOST_ASIO_HAS_MOVE)
 
     typedef typename associated_executor<Handler,
-        typename Work::head_type>::type executor_type;
+        typename composed_work_guard<
+          typename Work::head_type
+        >::executor_type
+      >::type executor_type;
 
     executor_type get_executor() const BOOST_ASIO_NOEXCEPT
     {
@@ -290,7 +341,7 @@ namespace detail
     template<typename... T>
     void operator()(BOOST_ASIO_MOVE_ARG(T)... t)
     {
-      if (invocations_ < ~unsigned(0))
+      if (invocations_ < ~0u)
         ++invocations_;
       impl_(*this, BOOST_ASIO_MOVE_CAST(T)(t)...);
     }
@@ -305,7 +356,7 @@ namespace detail
 
     void operator()()
     {
-      if (invocations_ < ~unsigned(0))
+      if (invocations_ < ~0u)
         ++invocations_;
       impl_(*this);
     }
@@ -320,7 +371,7 @@ namespace detail
     template<BOOST_ASIO_VARIADIC_TPARAMS(n)> \
     void operator()(BOOST_ASIO_VARIADIC_MOVE_PARAMS(n)) \
     { \
-      if (invocations_ < ~unsigned(0)) \
+      if (invocations_ < ~0u) \
         ++invocations_; \
       impl_(*this, BOOST_ASIO_VARIADIC_MOVE_ARGS(n)); \
     } \
@@ -345,19 +396,29 @@ namespace detail
   };
 
   template <typename Impl, typename Work, typename Handler, typename Signature>
-  inline void* asio_handler_allocate(std::size_t size,
+  inline asio_handler_allocate_is_deprecated
+  asio_handler_allocate(std::size_t size,
       composed_op<Impl, Work, Handler, Signature>* this_handler)
   {
+#if defined(BOOST_ASIO_NO_DEPRECATED)
+    boost_asio_handler_alloc_helpers::allocate(size, this_handler->handler_);
+    return asio_handler_allocate_is_no_longer_used();
+#else // defined(BOOST_ASIO_NO_DEPRECATED)
     return boost_asio_handler_alloc_helpers::allocate(
         size, this_handler->handler_);
+#endif // defined(BOOST_ASIO_NO_DEPRECATED)
   }
 
   template <typename Impl, typename Work, typename Handler, typename Signature>
-  inline void asio_handler_deallocate(void* pointer, std::size_t size,
+  inline asio_handler_deallocate_is_deprecated
+  asio_handler_deallocate(void* pointer, std::size_t size,
       composed_op<Impl, Work, Handler, Signature>* this_handler)
   {
     boost_asio_handler_alloc_helpers::deallocate(
         pointer, size, this_handler->handler_);
+#if defined(BOOST_ASIO_NO_DEPRECATED)
+    return asio_handler_deallocate_is_no_longer_used();
+#endif // defined(BOOST_ASIO_NO_DEPRECATED)
   }
 
   template <typename Impl, typename Work, typename Handler, typename Signature>
@@ -371,20 +432,28 @@ namespace detail
 
   template <typename Function, typename Impl,
       typename Work, typename Handler, typename Signature>
-  inline void asio_handler_invoke(Function& function,
+  inline asio_handler_invoke_is_deprecated
+  asio_handler_invoke(Function& function,
       composed_op<Impl, Work, Handler, Signature>* this_handler)
   {
     boost_asio_handler_invoke_helpers::invoke(
         function, this_handler->handler_);
+#if defined(BOOST_ASIO_NO_DEPRECATED)
+    return asio_handler_invoke_is_no_longer_used();
+#endif // defined(BOOST_ASIO_NO_DEPRECATED)
   }
 
   template <typename Function, typename Impl,
       typename Work, typename Handler, typename Signature>
-  inline void asio_handler_invoke(const Function& function,
+  inline asio_handler_invoke_is_deprecated
+  asio_handler_invoke(const Function& function,
       composed_op<Impl, Work, Handler, Signature>* this_handler)
   {
     boost_asio_handler_invoke_helpers::invoke(
         function, this_handler->handler_);
+#if defined(BOOST_ASIO_NO_DEPRECATED)
+    return asio_handler_invoke_is_no_longer_used();
+#endif // defined(BOOST_ASIO_NO_DEPRECATED)
   }
 
   template <typename Signature, typename Executors>
@@ -394,7 +463,7 @@ namespace detail
     typedef typename composed_io_executors<Executors>::head_type executor_type;
 
     template <typename T>
-    explicit initiate_composed_op(BOOST_ASIO_MOVE_ARG(T) executors)
+    explicit initiate_composed_op(int, BOOST_ASIO_MOVE_ARG(T) executors)
       : executors_(BOOST_ASIO_MOVE_CAST(T)(executors))
     {
     }
@@ -423,20 +492,27 @@ namespace detail
   inline initiate_composed_op<Signature, Executors> make_initiate_composed_op(
       BOOST_ASIO_MOVE_ARG(composed_io_executors<Executors>) executors)
   {
-    return initiate_composed_op<Signature, Executors>(
+    return initiate_composed_op<Signature, Executors>(0,
         BOOST_ASIO_MOVE_CAST(composed_io_executors<Executors>)(executors));
   }
 
   template <typename IoObject>
   inline typename IoObject::executor_type
-  get_composed_io_executor(IoObject& io_object)
+  get_composed_io_executor(IoObject& io_object,
+      typename enable_if<
+        !is_executor<IoObject>::value
+          && !execution::is_executor<IoObject>::value
+      >::type* = 0)
   {
     return io_object.get_executor();
   }
 
   template <typename Executor>
   inline const Executor& get_composed_io_executor(const Executor& ex,
-      typename enable_if<is_executor<Executor>::value>::type* = 0)
+      typename enable_if<
+        is_executor<Executor>::value
+          || execution::is_executor<Executor>::value
+      >::type* = 0)
   {
     return ex;
   }
@@ -497,6 +573,30 @@ async_compose(BOOST_ASIO_MOVE_ARG(Implementation) implementation,
   detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T3)(x3)), \
   detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T4)(x4)), \
   detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T5)(x5))
+# define BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_6 \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T1)(x1)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T2)(x2)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T3)(x3)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T4)(x4)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T5)(x5)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T6)(x6))
+# define BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_7 \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T1)(x1)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T2)(x2)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T3)(x3)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T4)(x4)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T5)(x5)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T6)(x6)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T7)(x7))
+# define BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_8 \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T1)(x1)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T2)(x2)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T3)(x3)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T4)(x4)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T5)(x5)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T6)(x6)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T7)(x7)), \
+  detail::get_composed_io_executor(BOOST_ASIO_MOVE_CAST(T8)(x8))
 
 #define BOOST_ASIO_PRIVATE_ASYNC_COMPOSE_DEF(n) \
   template <typename CompletionToken, typename Signature, \
@@ -522,6 +622,9 @@ async_compose(BOOST_ASIO_MOVE_ARG(Implementation) implementation,
 #undef BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_3
 #undef BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_4
 #undef BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_5
+#undef BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_6
+#undef BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_7
+#undef BOOST_ASIO_PRIVATE_GET_COMPOSED_IO_EXECUTOR_8
 
 #endif // defined(BOOST_ASIO_HAS_VARIADIC_TEMPLATES)
 #endif // !defined(GENERATING_DOCUMENTATION)
