@@ -4,8 +4,8 @@
 //
 // Copyright (c) 2011-2015 Adam Wulkiewicz, Lodz, Poland.
 //
-// This file was modified by Oracle on 2019.
-// Modifications copyright (c) 2019 Oracle and/or its affiliates.
+// This file was modified by Oracle on 2019-2020.
+// Modifications copyright (c) 2019-2020 Oracle and/or its affiliates.
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 //
 // Use, modification and distribution is subject to the Boost Software License,
@@ -15,10 +15,11 @@
 #ifndef BOOST_GEOMETRY_INDEX_DETAIL_PREDICATES_HPP
 #define BOOST_GEOMETRY_INDEX_DETAIL_PREDICATES_HPP
 
+#include <tuple>
+#include <type_traits>
 //#include <utility>
 
-#include <boost/mpl/assert.hpp>
-#include <boost/tuple/tuple.hpp>
+#include <boost/geometry/core/static_assert.hpp>
 
 #include <boost/geometry/index/detail/tags.hpp>
 
@@ -121,10 +122,9 @@ struct path
 template <typename Predicate, typename Tag>
 struct predicate_check
 {
-    BOOST_MPL_ASSERT_MSG(
-        (false),
-        NOT_IMPLEMENTED_FOR_THIS_PREDICATE_OR_TAG,
-        (predicate_check));
+    BOOST_GEOMETRY_STATIC_ASSERT_FALSE(
+        "Not implemented for this Predicate or Tag.",
+        Predicate, Tag);
 };
 
 // ------------------------------------------------------------------ //
@@ -154,7 +154,9 @@ struct predicate_check<predicates::satisfies<Fun, true>, value_tag>
 template <typename Tag>
 struct spatial_predicate_call
 {
-    BOOST_MPL_ASSERT_MSG(false, NOT_IMPLEMENTED_FOR_THIS_TAG, (Tag));
+    BOOST_GEOMETRY_STATIC_ASSERT_FALSE(
+        "Not implemented for this Tag.",
+        Tag);
 };
 
 template <>
@@ -521,22 +523,10 @@ struct predicates_length
     static const unsigned value = 1;
 };
 
-//template <typename F, typename S>
-//struct predicates_length< std::pair<F, S> >
-//{
-//    static const unsigned value = 2;
-//};
-
-//template <typename T0, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8, typename T9>
-//struct predicates_length< boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> >
-//{
-//    static const unsigned value = boost::tuples::length< boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> >::value;
-//};
-
-template <typename Head, typename Tail>
-struct predicates_length< boost::tuples::cons<Head, Tail> >
+template <typename ...Ts>
+struct predicates_length<std::tuple<Ts...>>
 {
-    static const unsigned value = boost::tuples::length< boost::tuples::cons<Head, Tail> >::value;
+    static const unsigned value = std::tuple_size<std::tuple<Ts...>>::value;
 };
 
 // ------------------------------------------------------------------ //
@@ -546,93 +536,27 @@ struct predicates_length< boost::tuples::cons<Head, Tail> >
 template <unsigned I, typename T>
 struct predicates_element
 {
-    BOOST_MPL_ASSERT_MSG((I < 1), INVALID_INDEX, (predicates_element));
+    BOOST_GEOMETRY_STATIC_ASSERT((I < 1),
+        "Invalid I index.",
+        std::integral_constant<unsigned, I>);
+
     typedef T type;
     static type const& get(T const& p) { return p; }
 };
 
-//template <unsigned I, typename F, typename S>
-//struct predicates_element< I, std::pair<F, S> >
-//{
-//    BOOST_MPL_ASSERT_MSG((I < 2), INVALID_INDEX, (predicates_element));
-//
-//    typedef F type;
-//    static type const& get(std::pair<F, S> const& p) { return p.first; }
-//};
-//
-//template <typename F, typename S>
-//struct predicates_element< 1, std::pair<F, S> >
-//{
-//    typedef S type;
-//    static type const& get(std::pair<F, S> const& p) { return p.second; }
-//};
-//
-//template <unsigned I, typename T0, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8, typename T9>
-//struct predicates_element< I, boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> >
-//{
-//    typedef boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> predicate_type;
-//
-//    typedef typename boost::tuples::element<I, predicate_type>::type type;
-//    static type const& get(predicate_type const& p) { return boost::get<I>(p); }
-//};
-
-template <unsigned I, typename Head, typename Tail>
-struct predicates_element< I, boost::tuples::cons<Head, Tail> >
+template <unsigned I, typename ...Ts>
+struct predicates_element<I, std::tuple<Ts...>>
 {
-    typedef boost::tuples::cons<Head, Tail> predicate_type;
+    typedef std::tuple<Ts...> predicate_type;
 
-    typedef typename boost::tuples::element<I, predicate_type>::type type;
-    static type const& get(predicate_type const& p) { return boost::get<I>(p); }
+    typedef typename std::tuple_element<I, predicate_type>::type type;
+    static type const& get(predicate_type const& p) { return std::get<I>(p); }
 };
 
 // ------------------------------------------------------------------ //
 // predicates_check
 // ------------------------------------------------------------------ //
 
-//template <typename PairPredicates, typename Tag, unsigned First, unsigned Last>
-//struct predicates_check_pair {};
-//
-//template <typename PairPredicates, typename Tag, unsigned I>
-//struct predicates_check_pair<PairPredicates, Tag, I, I>
-//{
-//    template <typename Value, typename Indexable>
-//    static inline bool apply(PairPredicates const& , Value const& , Indexable const& )
-//    {
-//        return true;
-//    }
-//};
-//
-//template <typename PairPredicates, typename Tag>
-//struct predicates_check_pair<PairPredicates, Tag, 0, 1>
-//{
-//    template <typename Value, typename Indexable>
-//    static inline bool apply(PairPredicates const& p, Value const& v, Indexable const& i)
-//    {
-//        return predicate_check<typename PairPredicates::first_type, Tag>::apply(p.first, v, i);
-//    }
-//};
-//
-//template <typename PairPredicates, typename Tag>
-//struct predicates_check_pair<PairPredicates, Tag, 1, 2>
-//{
-//    template <typename Value, typename Indexable>
-//    static inline bool apply(PairPredicates const& p, Value const& v, Indexable const& i)
-//    {
-//        return predicate_check<typename PairPredicates::second_type, Tag>::apply(p.second, v, i);
-//    }
-//};
-//
-//template <typename PairPredicates, typename Tag>
-//struct predicates_check_pair<PairPredicates, Tag, 0, 2>
-//{
-//    template <typename Value, typename Indexable>
-//    static inline bool apply(PairPredicates const& p, Value const& v, Indexable const& i)
-//    {
-//        return predicate_check<typename PairPredicates::first_type, Tag>::apply(p.first, v, i)
-//            && predicate_check<typename PairPredicates::second_type, Tag>::apply(p.second, v, i);
-//    }
-//};
-
 template <typename TuplePredicates, typename Tag, unsigned First, unsigned Last>
 struct predicates_check_tuple
 {
@@ -641,9 +565,9 @@ struct predicates_check_tuple
     {
         return predicate_check
                 <
-                    typename boost::tuples::element<First, TuplePredicates>::type,
+                    typename std::tuple_element<First, TuplePredicates>::type,
                     Tag
-                >::apply(boost::get<First>(p), v, i, s)
+                >::apply(std::get<First>(p), v, i, s)
             && predicates_check_tuple<TuplePredicates, Tag, First+1, Last>::apply(p, v, i, s);
     }
 };
@@ -662,7 +586,9 @@ template <typename Predicate, typename Tag, unsigned First, unsigned Last>
 struct predicates_check_impl
 {
     static const bool check = First < 1 && Last <= 1 && First <= Last;
-    BOOST_MPL_ASSERT_MSG((check), INVALID_INDEXES, (predicates_check_impl));
+    BOOST_GEOMETRY_STATIC_ASSERT((check),
+        "Invalid First or Last index.",
+        std::integer_sequence<unsigned, First, Last>);
 
     template <typename Value, typename Indexable, typename Strategy>
     static inline bool apply(Predicate const& p, Value const& v, Indexable const& i, Strategy const& s)
@@ -671,63 +597,25 @@ struct predicates_check_impl
     }
 };
 
-//template <typename Predicate1, typename Predicate2, typename Tag, size_t First, size_t Last>
-//struct predicates_check_impl<std::pair<Predicate1, Predicate2>, Tag, First, Last>
-//{
-//    BOOST_MPL_ASSERT_MSG((First < 2 && Last <= 2 && First <= Last), INVALID_INDEXES, (predicates_check_impl));
-//
-//    template <typename Value, typename Indexable>
-//    static inline bool apply(std::pair<Predicate1, Predicate2> const& p, Value const& v, Indexable const& i)
-//    {
-//        return predicate_check<Predicate1, Tag>::apply(p.first, v, i)
-//            && predicate_check<Predicate2, Tag>::apply(p.second, v, i);
-//    }
-//};
-//
-//template <
-//    typename T0, typename T1, typename T2, typename T3, typename T4,
-//    typename T5, typename T6, typename T7, typename T8, typename T9,
-//    typename Tag, unsigned First, unsigned Last
-//>
-//struct predicates_check_impl<
-//    boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9>,
-//    Tag, First, Last
-//>
-//{
-//    typedef boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> predicates_type;
-//
-//    static const unsigned pred_len = boost::tuples::length<predicates_type>::value;
-//    BOOST_MPL_ASSERT_MSG((First < pred_len && Last <= pred_len && First <= Last), INVALID_INDEXES, (predicates_check_impl));
-//
-//    template <typename Value, typename Indexable>
-//    static inline bool apply(predicates_type const& p, Value const& v, Indexable const& i)
-//    {
-//        return predicates_check_tuple<
-//            predicates_type,
-//            Tag, First, Last
-//        >::apply(p, v, i);
-//    }
-//};
-
-template <typename Head, typename Tail, typename Tag, unsigned First, unsigned Last>
-struct predicates_check_impl<
-    boost::tuples::cons<Head, Tail>,
-    Tag, First, Last
->
+template <typename ...Ts, typename Tag, unsigned First, unsigned Last>
+struct predicates_check_impl<std::tuple<Ts...>, Tag, First, Last>
 {
-    typedef boost::tuples::cons<Head, Tail> predicates_type;
+    typedef std::tuple<Ts...> predicates_type;
 
-    static const unsigned pred_len = boost::tuples::length<predicates_type>::value;
+    static const unsigned pred_len = std::tuple_size<predicates_type>::value;
     static const bool check = First < pred_len && Last <= pred_len && First <= Last;
-    BOOST_MPL_ASSERT_MSG((check), INVALID_INDEXES, (predicates_check_impl));
+    BOOST_GEOMETRY_STATIC_ASSERT((check),
+        "Invalid First or Last index.",
+        std::integer_sequence<unsigned, First, Last>);
 
     template <typename Value, typename Indexable, typename Strategy>
     static inline bool apply(predicates_type const& p, Value const& v, Indexable const& i, Strategy const& s)
     {
-        return predicates_check_tuple<
-            predicates_type,
-            Tag, First, Last
-        >::apply(p, v, i, s);
+        return predicates_check_tuple
+                <
+                    predicates_type,
+                    Tag, First, Last
+                >::apply(p, v, i, s);
     }
 };
 
@@ -770,18 +658,11 @@ struct predicates_count_distance
     static const unsigned value = predicates_is_distance<T>::value;
 };
 
-//template <typename F, typename S>
-//struct predicates_count_distance< std::pair<F, S> >
-//{
-//    static const unsigned value = predicates_is_distance<F>::value
-//                                + predicates_is_distance<S>::value;
-//};
-
 template <typename Tuple, unsigned N>
 struct predicates_count_distance_tuple
 {
     static const unsigned value =
-        predicates_is_distance<typename boost::tuples::element<N-1, Tuple>::type>::value
+        predicates_is_distance<typename std::tuple_element<N-1, Tuple>::type>::value
         + predicates_count_distance_tuple<Tuple, N-1>::value;
 };
 
@@ -789,24 +670,15 @@ template <typename Tuple>
 struct predicates_count_distance_tuple<Tuple, 1>
 {
     static const unsigned value =
-        predicates_is_distance<typename boost::tuples::element<0, Tuple>::type>::value;
+        predicates_is_distance<typename std::tuple_element<0, Tuple>::type>::value;
 };
 
-//template <typename T0, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8, typename T9>
-//struct predicates_count_distance< boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> >
-//{
-//    static const unsigned value = predicates_count_distance_tuple<
-//        boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9>,
-//        boost::tuples::length< boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> >::value
-//    >::value;
-//};
-
-template <typename Head, typename Tail>
-struct predicates_count_distance< boost::tuples::cons<Head, Tail> >
+template <typename ...Ts>
+struct predicates_count_distance<std::tuple<Ts...>>
 {
     static const unsigned value = predicates_count_distance_tuple<
-        boost::tuples::cons<Head, Tail>,
-        boost::tuples::length< boost::tuples::cons<Head, Tail> >::value
+        std::tuple<Ts...>,
+        std::tuple_size<std::tuple<Ts...>>::value
     >::value;
 };
 
@@ -818,47 +690,31 @@ struct predicates_find_distance
     static const unsigned value = predicates_is_distance<T>::value ? 0 : 1;
 };
 
-//template <typename F, typename S>
-//struct predicates_find_distance< std::pair<F, S> >
-//{
-//    static const unsigned value = predicates_is_distance<F>::value ? 0 :
-//                                    (predicates_is_distance<S>::value ? 1 : 2);
-//};
-
 template <typename Tuple, unsigned N>
 struct predicates_find_distance_tuple
 {
     static const bool is_found = predicates_find_distance_tuple<Tuple, N-1>::is_found
-                                || predicates_is_distance<typename boost::tuples::element<N-1, Tuple>::type>::value;
+                                || predicates_is_distance<typename std::tuple_element<N-1, Tuple>::type>::value;
 
     static const unsigned value = predicates_find_distance_tuple<Tuple, N-1>::is_found ?
         predicates_find_distance_tuple<Tuple, N-1>::value :
-        (predicates_is_distance<typename boost::tuples::element<N-1, Tuple>::type>::value ?
-            N-1 : boost::tuples::length<Tuple>::value);
+        (predicates_is_distance<typename std::tuple_element<N-1, Tuple>::type>::value ?
+            N-1 : std::tuple_size<Tuple>::value);
 };
 
 template <typename Tuple>
 struct predicates_find_distance_tuple<Tuple, 1>
 {
-    static const bool is_found = predicates_is_distance<typename boost::tuples::element<0, Tuple>::type>::value;
-    static const unsigned value = is_found ? 0 : boost::tuples::length<Tuple>::value;
+    static const bool is_found = predicates_is_distance<typename std::tuple_element<0, Tuple>::type>::value;
+    static const unsigned value = is_found ? 0 : std::tuple_size<Tuple>::value;
 };
 
-//template <typename T0, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8, typename T9>
-//struct predicates_find_distance< boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> >
-//{
-//    static const unsigned value = predicates_find_distance_tuple<
-//        boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9>,
-//        boost::tuples::length< boost::tuple<T0, T1, T2, T3, T4, T5, T6, T7, T8, T9> >::value
-//    >::value;
-//};
-
-template <typename Head, typename Tail>
-struct predicates_find_distance< boost::tuples::cons<Head, Tail> >
+template <typename ...Ts>
+struct predicates_find_distance<std::tuple<Ts...>>
 {
     static const unsigned value = predicates_find_distance_tuple<
-        boost::tuples::cons<Head, Tail>,
-        boost::tuples::length< boost::tuples::cons<Head, Tail> >::value
+        std::tuple<Ts...>,
+        std::tuple_size<std::tuple<Ts...>>::value
     >::value;
 };
 
