@@ -37,7 +37,7 @@
 // This is the only way we can avoid
 // warning: non-standard suffix on floating constant [-Wpedantic]
 // when building with -Wall -pedantic.  Neither __extension__
-// nor #pragma dianostic ignored work :(
+// nor #pragma diagnostic ignored work :(
 //
 #pragma GCC system_header
 #endif
@@ -49,19 +49,19 @@ namespace boost
       namespace detail
       {
          // owens_t_znorm1(x) = P(-oo<Z<=x)-0.5 with Z being normally distributed.
-         template<typename RealType>
-         inline RealType owens_t_znorm1(const RealType x)
+         template<typename RealType, class Policy>
+         inline RealType owens_t_znorm1(const RealType x, const Policy& pol)
          {
             using namespace boost::math::constants;
-            return erf(x*one_div_root_two<RealType>())*half<RealType>();
+            return boost::math::erf(x*one_div_root_two<RealType>(), pol)*half<RealType>();
          } // RealType owens_t_znorm1(const RealType x)
 
          // owens_t_znorm2(x) = P(x<=Z<oo) with Z being normally distributed.
-         template<typename RealType>
-         inline RealType owens_t_znorm2(const RealType x)
+         template<typename RealType, class Policy>
+         inline RealType owens_t_znorm2(const RealType x, const Policy& pol)
          {
             using namespace boost::math::constants;
-            return erfc(x*one_div_root_two<RealType>())*half<RealType>();
+            return boost::math::erfc(x*one_div_root_two<RealType>(), pol)*half<RealType>();
          } // RealType owens_t_znorm2(const RealType x)
 
          // Auxiliary function, it computes an array key that is used to determine
@@ -117,23 +117,23 @@ namespace boost
                }
             } // for(unsigned short i = 0; i != 7; i++)
 
-            // interprete select array as 8x15 matrix
+            // interpret select array as 8x15 matrix
             return select[iaint*15 + ihint];
 
          } // unsigned short owens_t_compute_code(const RealType h, const RealType a)
 
          template<typename RealType>
-         inline unsigned short owens_t_get_order_imp(const unsigned short icode, RealType, const mpl::int_<53>&)
+         inline unsigned short owens_t_get_order_imp(const unsigned short icode, RealType, const boost::integral_constant<int, 53>&)
          {
             static const unsigned short ord[] = {2, 3, 4, 5, 7, 10, 12, 18, 10, 20, 30, 0, 4, 7, 8, 20, 0, 0}; // 18 entries
 
             BOOST_ASSERT(icode<18);
 
             return ord[icode];
-         } // unsigned short owens_t_get_order(const unsigned short icode, RealType, mpl::int<53> const&)
+         } // unsigned short owens_t_get_order(const unsigned short icode, RealType, boost::integral_constant<int, 53> const&)
 
          template<typename RealType>
-         inline unsigned short owens_t_get_order_imp(const unsigned short icode, RealType, const mpl::int_<64>&)
+         inline unsigned short owens_t_get_order_imp(const unsigned short icode, RealType, const boost::integral_constant<int, 64>&)
         {
            // method ================>>>       {1, 1, 1, 1, 1,  1,  1,  1,  2,  2,  2,  3, 4,  4,  4,  4,  5, 6}
            static const unsigned short ord[] = {3, 4, 5, 6, 8, 11, 13, 19, 10, 20, 30,  0, 7, 10, 11, 23,  0, 0}; // 18 entries
@@ -141,20 +141,16 @@ namespace boost
           BOOST_ASSERT(icode<18);
 
           return ord[icode];
-        } // unsigned short owens_t_get_order(const unsigned short icode, RealType, mpl::int<64> const&)
+        } // unsigned short owens_t_get_order(const unsigned short icode, RealType, boost::integral_constant<int, 64> const&)
 
          template<typename RealType, typename Policy>
          inline unsigned short owens_t_get_order(const unsigned short icode, RealType r, const Policy&)
          {
             typedef typename policies::precision<RealType, Policy>::type precision_type;
-            typedef typename mpl::if_<
-               mpl::or_<
-                  mpl::less_equal<precision_type, mpl::int_<0> >,
-                  mpl::greater<precision_type, mpl::int_<53> >
-               >,
-               mpl::int_<64>,
-               mpl::int_<53>
-            >::type tag_type;
+            typedef boost::integral_constant<int,
+               precision_type::value <= 0 ? 64 :
+               precision_type::value <= 53 ? 53 : 64
+            > tag_type;
 
             return owens_t_get_order_imp(icode, r, tag_type());
          }
@@ -197,7 +193,7 @@ namespace boost
 
          // compute the value of Owen's T function with method T2 from the reference paper
          template<typename RealType, class Policy>
-         inline RealType owens_t_T2(const RealType h, const RealType a, const unsigned short m, const RealType ah, const Policy&, const mpl::false_&)
+         inline RealType owens_t_T2(const RealType h, const RealType a, const unsigned short m, const RealType ah, const Policy& pol, const boost::false_type&)
          {
             BOOST_MATH_STD_USING
             using namespace boost::math::constants;
@@ -210,7 +206,7 @@ namespace boost
             unsigned short ii = 1;
             RealType val = 0;
             RealType vi = a * exp( -ah*ah*half<RealType>() ) * one_div_root_two_pi<RealType>();
-            RealType z = owens_t_znorm1(ah)/h;
+            RealType z = owens_t_znorm1(ah, pol)/h;
 
             while( true )
             {
@@ -229,8 +225,8 @@ namespace boost
          } // RealType owens_t_T2(const RealType h, const RealType a, const unsigned short m, const RealType ah)
 
          // compute the value of Owen's T function with method T3 from the reference paper
-         template<typename RealType>
-         inline RealType owens_t_T3_imp(const RealType h, const RealType a, const RealType ah, const mpl::int_<53>&)
+         template<typename RealType, class Policy>
+         inline RealType owens_t_T3_imp(const RealType h, const RealType a, const RealType ah, const boost::integral_constant<int, 53>&, const Policy& pol)
          {
             BOOST_MATH_STD_USING
             using namespace boost::math::constants;
@@ -259,7 +255,7 @@ namespace boost
             RealType ii = 1;
             unsigned short i = 0;
             RealType vi = a * exp( -ah*ah*half<RealType>() ) * one_div_root_two_pi<RealType>();
-            RealType zi = owens_t_znorm1(ah)/h;
+            RealType zi = owens_t_znorm1(ah, pol)/h;
             RealType val = 0;
 
             while( true )
@@ -281,8 +277,8 @@ namespace boost
          } // RealType owens_t_T3(const RealType h, const RealType a, const RealType ah)
 
         // compute the value of Owen's T function with method T3 from the reference paper
-        template<class RealType>
-        inline RealType owens_t_T3_imp(const RealType h, const RealType a, const RealType ah, const mpl::int_<64>&)
+        template<class RealType, class Policy>
+        inline RealType owens_t_T3_imp(const RealType h, const RealType a, const RealType ah, const boost::integral_constant<int, 64>&, const Policy& pol)
         {
           BOOST_MATH_STD_USING
           using namespace boost::math::constants;
@@ -331,7 +327,7 @@ namespace boost
           RealType ii = 1;
           unsigned short i = 0;
           RealType vi = a * exp( -ah*ah*half<RealType>() ) * one_div_root_two_pi<RealType>();
-          RealType zi = owens_t_znorm1(ah)/h;
+          RealType zi = owens_t_znorm1(ah, pol)/h;
           RealType val = 0;
 
           while( true )
@@ -353,19 +349,15 @@ namespace boost
         } // RealType owens_t_T3(const RealType h, const RealType a, const RealType ah)
 
         template<class RealType, class Policy>
-        inline RealType owens_t_T3(const RealType h, const RealType a, const RealType ah, const Policy&)
+        inline RealType owens_t_T3(const RealType h, const RealType a, const RealType ah, const Policy& pol)
         {
             typedef typename policies::precision<RealType, Policy>::type precision_type;
-            typedef typename mpl::if_<
-               mpl::or_<
-                  mpl::less_equal<precision_type, mpl::int_<0> >,
-                  mpl::greater<precision_type, mpl::int_<53> >
-               >,
-               mpl::int_<64>,
-               mpl::int_<53>
-            >::type tag_type;
+            typedef boost::integral_constant<int,
+               precision_type::value <= 0 ? 64 :
+               precision_type::value <= 53 ? 53 : 64
+            > tag_type;
 
-            return owens_t_T3_imp(h, a, ah, tag_type());
+            return owens_t_T3_imp(h, a, ah, tag_type(), pol);
         }
 
          // compute the value of Owen's T function with method T4 from the reference paper
@@ -399,7 +391,7 @@ namespace boost
 
          // compute the value of Owen's T function with method T5 from the reference paper
          template<typename RealType>
-         inline RealType owens_t_T5_imp(const RealType h, const RealType a, const mpl::int_<53>&)
+         inline RealType owens_t_T5_imp(const RealType h, const RealType a, const boost::integral_constant<int, 53>&)
          {
             BOOST_MATH_STD_USING
             /*
@@ -445,7 +437,7 @@ namespace boost
 
         // compute the value of Owen's T function with method T5 from the reference paper
         template<typename RealType>
-        inline RealType owens_t_T5_imp(const RealType h, const RealType a, const mpl::int_<64>&)
+        inline RealType owens_t_T5_imp(const RealType h, const RealType a, const boost::integral_constant<int, 64>&)
         {
           BOOST_MATH_STD_USING
             /*
@@ -519,27 +511,23 @@ namespace boost
         inline RealType owens_t_T5(const RealType h, const RealType a, const Policy&)
         {
             typedef typename policies::precision<RealType, Policy>::type precision_type;
-            typedef typename mpl::if_<
-               mpl::or_<
-                  mpl::less_equal<precision_type, mpl::int_<0> >,
-                  mpl::greater<precision_type, mpl::int_<53> >
-               >,
-               mpl::int_<64>,
-               mpl::int_<53>
-            >::type tag_type;
+            typedef boost::integral_constant<int,
+               precision_type::value <= 0 ? 64 :
+               precision_type::value <= 53 ? 53 : 64
+            > tag_type;
 
             return owens_t_T5_imp(h, a, tag_type());
         }
 
 
          // compute the value of Owen's T function with method T6 from the reference paper
-         template<typename RealType>
-         inline RealType owens_t_T6(const RealType h, const RealType a)
+         template<typename RealType, class Policy>
+         inline RealType owens_t_T6(const RealType h, const RealType a, const Policy& pol)
          {
             BOOST_MATH_STD_USING
             using namespace boost::math::constants;
 
-            const RealType normh = owens_t_znorm2( h );
+            const RealType normh = owens_t_znorm2(h, pol);
             const RealType y = static_cast<RealType>(1) - a;
             const RealType r = atan2(y, static_cast<RealType>(1 + a) );
 
@@ -581,7 +569,7 @@ namespace boost
             // that each term decreases in size by a factor of 3.  However,
             // that assumption does not apply here, as the underlying T1 series can 
             // go quite strongly divergent in the early terms, before strongly
-            // converging later.  Various "guestimates" have been tried to take account
+            // converging later.  Various "guesstimates" have been tried to take account
             // of this, but they don't always work.... so instead set "n" to the 
             // largest value that won't cause overflow later, and abort iteration
             // when the last accelerated term was small enough...
@@ -627,13 +615,13 @@ namespace boost
                   break;
             }
             abs_err += fabs(c * term);
-            if(sum < 0)  // sum must always be positive, if it's negative something really bad has happend:
+            if(sum < 0)  // sum must always be positive, if it's negative something really bad has happened:
                policies::raise_evaluation_error(function, 0, T(0), pol);
             return std::pair<T, T>((sum / d) / boost::math::constants::two_pi<T>(), abs_err / sum);
          }
 
          template<typename RealType, class Policy>
-         inline RealType owens_t_T2(const RealType h, const RealType a, const unsigned short m, const RealType ah, const Policy&, const mpl::true_&)
+         inline RealType owens_t_T2(const RealType h, const RealType a, const unsigned short m, const RealType ah, const Policy& pol, const boost::true_type&)
          {
             BOOST_MATH_STD_USING
             using namespace boost::math::constants;
@@ -646,7 +634,7 @@ namespace boost
             unsigned short ii = 1;
             RealType val = 0;
             RealType vi = a * exp( -ah*ah*half<RealType>() ) / root_two_pi<RealType>();
-            RealType z = owens_t_znorm1(ah)/h;
+            RealType z = owens_t_znorm1(ah, pol)/h;
             RealType last_z = fabs(z);
             RealType lim = policies::get_epsilon<RealType, Policy>();
 
@@ -672,7 +660,7 @@ namespace boost
          } // RealType owens_t_T2(const RealType h, const RealType a, const unsigned short m, const RealType ah)
 
          template<typename RealType, class Policy>
-         inline std::pair<RealType, RealType> owens_t_T2_accelerated(const RealType h, const RealType a, const RealType ah, const Policy&)
+         inline std::pair<RealType, RealType> owens_t_T2_accelerated(const RealType h, const RealType a, const RealType ah, const Policy& pol)
          {
             //
             // This is the same series as T2, but with acceleration applied.
@@ -690,7 +678,7 @@ namespace boost
             unsigned short ii = 1;
             RealType val = 0;
             RealType vi = a * exp( -ah*ah*half<RealType>() ) / root_two_pi<RealType>();
-            RealType z = boost::math::detail::owens_t_znorm1(ah)/h;
+            RealType z = boost::math::detail::owens_t_znorm1(ah, pol)/h;
             RealType last_z = fabs(z);
 
             //
@@ -699,7 +687,7 @@ namespace boost
             // that each term decreases in size by a factor of 3.  However,
             // that assumption does not apply here, as the underlying T1 series can 
             // go quite strongly divergent in the early terms, before strongly
-            // converging later.  Various "guestimates" have been tried to take account
+            // converging later.  Various "guesstimates" have been tried to take account
             // of this, but they don't always work.... so instead set "n" to the 
             // largest value that won't cause overflow later, and abort iteration
             // when the last accelerated term was small enough...
@@ -788,7 +776,7 @@ namespace boost
          //
          // Note there are different versions for different precisions....
          template<typename RealType, typename Policy>
-         inline RealType owens_t_dispatch(const RealType h, const RealType a, const RealType ah, const Policy& pol, mpl::int_<64> const&)
+         inline RealType owens_t_dispatch(const RealType h, const RealType a, const RealType ah, const Policy& pol, boost::integral_constant<int, 64> const&)
          {
             // Simple main case for 64-bit precision or less, this is as per the Patefield-Tandy paper:
             BOOST_MATH_STD_USING
@@ -806,11 +794,11 @@ namespace boost
             }
             if(a == 1)
             {
-               return owens_t_znorm2(RealType(-h)) * owens_t_znorm2(h) / 2;
+               return owens_t_znorm2(RealType(-h), pol) * owens_t_znorm2(h, pol) / 2;
             }
             if(a >= tools::max_value<RealType>())
             {
-               return owens_t_znorm2(RealType(fabs(h)));
+               return owens_t_znorm2(RealType(fabs(h)), pol);
             }
             RealType val = 0; // avoid compiler warnings, 0 will be overwritten in any case
             const unsigned short icode = owens_t_compute_code(h, a);
@@ -825,7 +813,7 @@ namespace boost
                break;
             case 2: // T2
                typedef typename policies::precision<RealType, Policy>::type precision_type;
-               typedef mpl::bool_<(precision_type::value == 0) || (precision_type::value > 64)> tag_type;
+               typedef boost::integral_constant<bool, (precision_type::value == 0) || (precision_type::value > 64)> tag_type;
                val = owens_t_T2(h, a, m, ah, pol, tag_type());
                break;
             case 3: // T3
@@ -838,7 +826,7 @@ namespace boost
                val = owens_t_T5(h,a, pol);
                break;
             case 6: // T6
-               val = owens_t_T6(h,a);
+               val = owens_t_T6(h,a, pol);
                break;
             default:
                BOOST_THROW_EXCEPTION(std::logic_error("selection routine in Owen's T function failed"));
@@ -847,7 +835,7 @@ namespace boost
          }
 
          template<typename RealType, typename Policy>
-         inline RealType owens_t_dispatch(const RealType h, const RealType a, const RealType ah, const Policy& pol, const mpl::int_<65>&)
+         inline RealType owens_t_dispatch(const RealType h, const RealType a, const RealType ah, const Policy& pol, const boost::integral_constant<int, 65>&)
          {
             // Arbitrary precision version:
             BOOST_MATH_STD_USING
@@ -865,11 +853,11 @@ namespace boost
             }
             if(a == 1)
             {
-               return owens_t_znorm2(RealType(-h)) * owens_t_znorm2(h) / 2;
+               return owens_t_znorm2(RealType(-h), pol) * owens_t_znorm2(h, pol) / 2;
             }
             if(a >= tools::max_value<RealType>())
             {
-               return owens_t_znorm2(RealType(fabs(h)));
+               return owens_t_znorm2(RealType(fabs(h)), pol);
             }
             // Attempt arbitrary precision code, this will throw if it goes wrong:
             typedef typename boost::math::policies::normalise<Policy, boost::math::policies::evaluation_error<> >::type forwarding_policy;
@@ -968,30 +956,26 @@ namespace boost
             //
             // We give up - no arbitrary precision versions succeeded!
             //
-            return owens_t_dispatch(h, a, ah, pol, mpl::int_<64>());
+            return owens_t_dispatch(h, a, ah, pol, boost::integral_constant<int, 64>());
          } // RealType owens_t_dispatch(RealType h, RealType a, RealType ah)
          template<typename RealType, typename Policy>
-         inline RealType owens_t_dispatch(const RealType h, const RealType a, const RealType ah, const Policy& pol, const mpl::int_<0>&)
+         inline RealType owens_t_dispatch(const RealType h, const RealType a, const RealType ah, const Policy& pol, const boost::integral_constant<int, 0>&)
          {
             // We don't know what the precision is until runtime:
             if(tools::digits<RealType>() <= 64)
-               return owens_t_dispatch(h, a, ah, pol, mpl::int_<64>());
-            return owens_t_dispatch(h, a, ah, pol, mpl::int_<65>());
+               return owens_t_dispatch(h, a, ah, pol, boost::integral_constant<int, 64>());
+            return owens_t_dispatch(h, a, ah, pol, boost::integral_constant<int, 65>());
          }
          template<typename RealType, typename Policy>
          inline RealType owens_t_dispatch(const RealType h, const RealType a, const RealType ah, const Policy& pol)
          {
             // Figure out the precision and forward to the correct version:
             typedef typename policies::precision<RealType, Policy>::type precision_type;
-            typedef typename mpl::if_c<
-               precision_type::value == 0,
-               mpl::int_<0>,
-               typename mpl::if_c<
-                  precision_type::value <= 64,
-                  mpl::int_<64>,
-                  mpl::int_<65>
-               >::type
-            >::type tag_type;
+            typedef boost::integral_constant<int,
+               precision_type::value <= 0 ? 0 :
+               precision_type::value <= 64 ? 64 : 65
+            > tag_type;
+
             return owens_t_dispatch(h, a, ah, pol, tag_type());
          }
          // compute Owen's T function, T(h,a), for arbitrary values of h and a
@@ -1019,15 +1003,15 @@ namespace boost
             {
                if( h <= 0.67 )
                {
-                  const RealType normh = owens_t_znorm1(h);
-                  const RealType normah = owens_t_znorm1(fabs_ah);
+                  const RealType normh = owens_t_znorm1(h, pol);
+                  const RealType normah = owens_t_znorm1(fabs_ah, pol);
                   val = static_cast<RealType>(1)/static_cast<RealType>(4) - normh*normah -
                      owens_t_dispatch(fabs_ah, static_cast<RealType>(1 / fabs_a), h, pol);
                } // if( h <= 0.67 )
                else
                {
-                  const RealType normh = detail::owens_t_znorm2(h);
-                  const RealType normah = detail::owens_t_znorm2(fabs_ah);
+                  const RealType normh = detail::owens_t_znorm2(h, pol);
+                  const RealType normah = detail::owens_t_znorm2(fabs_ah, pol);
                   val = constants::half<RealType>()*(normh+normah) - normh*normah -
                      owens_t_dispatch(fabs_ah, static_cast<RealType>(1 / fabs_a), h, pol);
                } // else [if( h <= 0.67 )]
@@ -1052,8 +1036,8 @@ namespace boost
                   do_init(tag());
                }
                template <int N>
-               static void do_init(const mpl::int_<N>&){}
-               static void do_init(const mpl::int_<64>&)
+               static void do_init(const boost::integral_constant<int, N>&){}
+               static void do_init(const boost::integral_constant<int, 64>&)
                {
                   boost::math::owens_t(static_cast<T>(7), static_cast<T>(0.96875), Policy());
                   boost::math::owens_t(static_cast<T>(2), static_cast<T>(0.5), Policy());
@@ -1078,15 +1062,10 @@ namespace boost
          typedef typename tools::promote_args<T1, T2>::type result_type;
          typedef typename policies::evaluation<result_type, Policy>::type value_type;
          typedef typename policies::precision<value_type, Policy>::type precision_type;
-         typedef typename mpl::if_c<
-               precision_type::value == 0,
-               mpl::int_<0>,
-               typename mpl::if_c<
-                  precision_type::value <= 64,
-                  mpl::int_<64>,
-                  mpl::int_<65>
-               >::type
-            >::type tag_type;
+         typedef boost::integral_constant<int,
+            precision_type::value <= 0 ? 0 :
+            precision_type::value <= 64 ? 64 : 65
+         > tag_type;
 
          detail::owens_t_initializer<result_type, Policy, tag_type>::force_instantiate();
             
