@@ -3,8 +3,8 @@
 //Distributed under the Boost Software License, Version 1.0. (See accompanying
 //file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#ifndef UUID_47B1D1217B411E08424FA0ADFD72085
-#define UUID_47B1D1217B411E08424FA0ADFD72085
+#ifndef BOOST_QVM_47B1D1217B411E08424FA0ADFD72085
+#define BOOST_QVM_47B1D1217B411E08424FA0ADFD72085
 
 #include <boost/qvm/mat_traits.hpp>
 #include <boost/qvm/inline.hpp>
