@@ -1,5 +1,5 @@
 //
-// Copyright (w) 2016-2017 Vinnie Falco (vinnie dot falco at gmail dot com)
+// Copyright (c) 2016-2019 Vinnie Falco (vinnie dot falco at gmail dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -10,6 +10,13 @@
 // Test that header file is self-contained.
 #include <boost/beast/websocket/stream.hpp>
 
+#include <boost/asio/io_context.hpp>
+#include <boost/asio/strand.hpp>
+
+#if BOOST_ASIO_HAS_CO_AWAIT
+#include <boost/asio/use_awaitable.hpp>
+#endif
+
 #include "test.hpp"
 
 namespace boost {
@@ -19,12 +26,10 @@ namespace websocket {
 class write_test : public websocket_test_suite
 {
 public:
-    template<class Wrap>
+    template<bool deflateSupported, class Wrap>
     void
     doTestWrite(Wrap const& w)
     {
-        using boost::asio::buffer;
-
         permessage_deflate pmd;
         pmd.client_enable = false;
         pmd.server_enable = false;
@@ -44,29 +49,31 @@ public:
             catch(system_error const& se)
             {
                 BEAST_EXPECTS(
-                    se.code() == boost::asio::error::operation_aborted,
+                    se.code() == net::error::operation_aborted,
                     se.code().message());
             }
         }
 
         // message
-        doTest(pmd, [&](ws_type& ws)
+        doTest<deflateSupported>(pmd,
+        [&](ws_type_t<deflateSupported>& ws)
         {
             ws.auto_fragment(false);
             ws.binary(false);
             std::string const s = "Hello, world!";
-            w.write(ws, buffer(s));
+            w.write(ws, net::buffer(s));
             multi_buffer b;
             w.read(ws, b);
             BEAST_EXPECT(ws.got_text());
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
 
         // empty message
-        doTest(pmd, [&](ws_type& ws)
+        doTest<deflateSupported>(pmd,
+        [&](ws_type_t<deflateSupported>& ws)
         {
             ws.text(true);
-            w.write(ws, boost::asio::null_buffers{});
+            w.write(ws, net::const_buffer{});
             multi_buffer b;
             w.read(ws, b);
             BEAST_EXPECT(ws.got_text());
@@ -74,73 +81,78 @@ public:
         });
 
         // fragmented message
-        doTest(pmd, [&](ws_type& ws)
+        doTest<deflateSupported>(pmd,
+        [&](ws_type_t<deflateSupported>& ws)
         {
             ws.auto_fragment(false);
             ws.binary(false);
             std::string const s = "Hello, world!";
-            w.write_some(ws, false, buffer(s.data(), 5));
-            w.write_some(ws, true, buffer(s.data() + 5, s.size() - 5));
+            w.write_some(ws, false, net::buffer(s.data(), 5));
+            w.write_some(ws, true, net::buffer(s.data() + 5, s.size() - 5));
             multi_buffer b;
             w.read(ws, b);
             BEAST_EXPECT(ws.got_text());
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
 
         // continuation
-        doTest(pmd, [&](ws_type& ws)
+        doTest<deflateSupported>(pmd,
+        [&](ws_type_t<deflateSupported>& ws)
         {
             std::string const s = "Hello";
             std::size_t const chop = 3;
             BOOST_ASSERT(chop < s.size());
             w.write_some(ws, false,
-                buffer(s.data(), chop));
-            w.write_some(ws, true, buffer(
+                net::buffer(s.data(), chop));
+            w.write_some(ws, true, net::buffer(
                 s.data() + chop, s.size() - chop));
             flat_buffer b;
             w.read(ws, b);
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
 
         // mask
-        doTest(pmd, [&](ws_type& ws)
+        doTest<deflateSupported>(pmd,
+        [&](ws_type_t<deflateSupported>& ws)
         {
             ws.auto_fragment(false);
             std::string const s = "Hello";
-            w.write(ws, buffer(s));
+            w.write(ws, net::buffer(s));
             flat_buffer b;
             w.read(ws, b);
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
 
         // mask (large)
-        doTest(pmd, [&](ws_type& ws)
+        doTest<deflateSupported>(pmd,
+        [&](ws_type_t<deflateSupported>& ws)
         {
             ws.auto_fragment(false);
-            ws.write_buffer_size(16);
+            ws.write_buffer_bytes(16);
             std::string const s(32, '*');
-            w.write(ws, buffer(s));
+            w.write(ws, net::buffer(s));
             flat_buffer b;
             w.read(ws, b);
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
 
         // mask, autofrag
-        doTest(pmd, [&](ws_type& ws)
+        doTest<deflateSupported>(pmd,
+        [&](ws_type_t<deflateSupported>& ws)
         {
             ws.auto_fragment(true);
             std::string const s(16384, '*');
-            w.write(ws, buffer(s));
+            w.write(ws, net::buffer(s));
             flat_buffer b;
             w.read(ws, b);
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
 
         // nomask
         doStreamLoop([&](test::stream& ts)
         {
             echo_server es{log, kind::async_client};
-            ws_type ws{ts};
+            ws_type_t<deflateSupported> ws{ts};
             ws.next_layer().connect(es.stream());
             try
             {
@@ -148,10 +160,10 @@ public:
                 w.accept(ws);
                 ws.auto_fragment(false);
                 std::string const s = "Hello";
-                w.write(ws, buffer(s));
+                w.write(ws, net::buffer(s));
                 flat_buffer b;
                 w.read(ws, b);
-                BEAST_EXPECT(to_string(b.data()) == s);
+                BEAST_EXPECT(buffers_to_string(b.data()) == s);
                 w.close(ws, {});
             }
             catch(...)
@@ -166,7 +178,7 @@ public:
         doStreamLoop([&](test::stream& ts)
         {
             echo_server es{log, kind::async_client};
-            ws_type ws{ts};
+            ws_type_t<deflateSupported> ws{ts};
             ws.next_layer().connect(es.stream());
             try
             {
@@ -174,10 +186,10 @@ public:
                 w.accept(ws);
                 ws.auto_fragment(true);
                 std::string const s(16384, '*');
-                w.write(ws, buffer(s));
+                w.write(ws, net::buffer(s));
                 flat_buffer b;
                 w.read(ws, b);
-                BEAST_EXPECT(to_string(b.data()) == s);
+                BEAST_EXPECT(buffers_to_string(b.data()) == s);
                 w.close(ws, {});
             }
             catch(...)
@@ -187,7 +199,13 @@ public:
             }
             ts.close();
         });
+    }
 
+    template<class Wrap>
+    void
+    doTestWriteDeflate(Wrap const& w)
+    {
+        permessage_deflate pmd;
         pmd.client_enable = true;
         pmd.server_enable = true;
         pmd.compLevel = 1;
@@ -197,10 +215,10 @@ public:
         {
             auto const& s = random_string();
             ws.binary(true);
-            w.write(ws, buffer(s));
+            w.write(ws, net::buffer(s));
             flat_buffer b;
             w.read(ws, b);
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
 
         // deflate, continuation
@@ -212,12 +230,12 @@ public:
             // This call should produce no
             // output due to compression latency.
             w.write_some(ws, false,
-                buffer(s.data(), chop));
-            w.write_some(ws, true, buffer(
+                net::buffer(s.data(), chop));
+            w.write_some(ws, true, net::buffer(
                 s.data() + chop, s.size() - chop));
             flat_buffer b;
             w.read(ws, b);
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
 
         // deflate, no context takeover
@@ -226,36 +244,68 @@ public:
         {
             auto const& s = random_string();
             ws.binary(true);
-            w.write(ws, buffer(s));
+            w.write(ws, net::buffer(s));
             flat_buffer b;
             w.read(ws, b);
-            BEAST_EXPECT(to_string(b.data()) == s);
+            BEAST_EXPECT(buffers_to_string(b.data()) == s);
         });
     }
 
     void
     testWrite()
     {
-        using boost::asio::buffer;
-
-        doTestWrite(SyncClient{});
+        doTestWrite<false>(SyncClient{});
+        doTestWrite<true>(SyncClient{});
+        doTestWriteDeflate(SyncClient{});
 
         yield_to([&](yield_context yield)
         {
-            doTestWrite(AsyncClient{yield});
+            doTestWrite<false>(AsyncClient{yield});
+            doTestWrite<true>(AsyncClient{yield});
+            doTestWriteDeflate(AsyncClient{yield});
         });
     }
 
+    void
+    testPausationAbandoning()
+    {
+        struct test_op
+        {
+            std::shared_ptr<stream<test::stream>> s_;
+
+            void operator()(
+                boost::system::error_code = {},
+                std::size_t = 0)
+            {
+                BEAST_FAIL();
+            }
+        };
+
+        std::weak_ptr<stream<test::stream>> weak_ws;
+        {
+            echo_server es{log};
+            net::io_context ioc;
+            auto ws = std::make_shared<stream<test::stream>>(ioc);
+            test_op op{ws};
+            ws->next_layer().connect(es.stream());
+            ws->handshake("localhost", "/");
+            ws->async_ping("", op);
+            BEAST_EXPECT(ws->impl_->wr_block.is_locked());
+            ws->async_write(sbuf("*"), op);
+            weak_ws = ws;
+            ws->next_layer().close();
+        }
+        BEAST_EXPECT(weak_ws.expired());
+    }
+
     void
     testWriteSuspend()
     {
-        using boost::asio::buffer;
-
         // suspend on ping
-        doFailLoop([&](test::fail_counter& fc)
+        doFailLoop([&](test::fail_count& fc)
         {
             echo_server es{log};
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc, fc};
             ws.next_layer().connect(es.stream());
             ws.handshake("localhost", "/");
@@ -268,7 +318,7 @@ public:
                         BOOST_THROW_EXCEPTION(
                             system_error{ec});
                 });
-            BEAST_EXPECT(ws.wr_block_);
+            BEAST_EXPECT(ws.impl_->wr_block.is_locked());
             BEAST_EXPECT(count == 0);
             ws.async_write(sbuf("*"),
                 [&](error_code ec, std::size_t n)
@@ -285,10 +335,10 @@ public:
         });
 
         // suspend on close
-        doFailLoop([&](test::fail_counter& fc)
+        doFailLoop([&](test::fail_count& fc)
         {
             echo_server es{log};
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc, fc};
             ws.next_layer().connect(es.stream());
             ws.handshake("localhost", "/");
@@ -301,13 +351,13 @@ public:
                         BOOST_THROW_EXCEPTION(
                             system_error{ec});
                 });
-            BEAST_EXPECT(ws.wr_block_);
+            BEAST_EXPECT(ws.impl_->wr_block.is_locked());
             BEAST_EXPECT(count == 0);
             ws.async_write(sbuf("*"),
                 [&](error_code ec, std::size_t)
                 {
                     ++count;
-                    if(ec != boost::asio::error::operation_aborted)
+                    if(ec != net::error::operation_aborted)
                         BOOST_THROW_EXCEPTION(
                             system_error{ec});
                 });
@@ -317,10 +367,10 @@ public:
         });
 
         // suspend on read ping + message
-        doFailLoop([&](test::fail_counter& fc)
+        doFailLoop([&](test::fail_count& fc)
         {
             echo_server es{log};
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc, fc};
             ws.next_layer().connect(es.stream());
             ws.handshake("localhost", "/");
@@ -337,7 +387,7 @@ public:
                         BOOST_THROW_EXCEPTION(
                             system_error{ec});
                 });
-            while(! ws.wr_block_)
+            while(! ws.impl_->wr_block.is_locked())
             {
                 ioc.run_one();
                 if(! BEAST_EXPECT(! ioc.stopped()))
@@ -359,10 +409,10 @@ public:
         });
 
         // suspend on ping: nomask, nofrag
-        doFailLoop([&](test::fail_counter& fc)
+        doFailLoop([&](test::fail_count& fc)
         {
             echo_server es{log, kind::async_client};
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc, fc};
             ws.next_layer().connect(es.stream());
             es.async_handshake();
@@ -370,7 +420,7 @@ public:
             std::size_t count = 0;
             std::string const s(16384, '*');
             ws.auto_fragment(false);
-            ws.async_write(buffer(s),
+            ws.async_write(net::buffer(s),
                 [&](error_code ec, std::size_t n)
                 {
                     ++count;
@@ -379,7 +429,7 @@ public:
                             system_error{ec});
                     BEAST_EXPECT(n == 16384);
                 });
-            BEAST_EXPECT(ws.wr_block_);
+            BEAST_EXPECT(ws.impl_->wr_block.is_locked());
             ws.async_ping("",
                 [&](error_code ec)
                 {
@@ -393,10 +443,10 @@ public:
         });
 
         // suspend on ping: nomask, frag
-        doFailLoop([&](test::fail_counter& fc)
+        doFailLoop([&](test::fail_count& fc)
         {
             echo_server es{log, kind::async_client};
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc, fc};
             ws.next_layer().connect(es.stream());
             es.async_handshake();
@@ -404,7 +454,7 @@ public:
             std::size_t count = 0;
             std::string const s(16384, '*');
             ws.auto_fragment(true);
-            ws.async_write(buffer(s),
+            ws.async_write(net::buffer(s),
                 [&](error_code ec, std::size_t n)
                 {
                     ++count;
@@ -413,7 +463,7 @@ public:
                             system_error{ec});
                     BEAST_EXPECT(n == 16384);
                 });
-            BEAST_EXPECT(ws.wr_block_);
+            BEAST_EXPECT(ws.impl_->wr_block.is_locked());
             ws.async_ping("",
                 [&](error_code ec)
                 {
@@ -427,18 +477,17 @@ public:
         });
 
         // suspend on ping: mask, nofrag
-        doFailLoop([&](test::fail_counter& fc)
+        doFailLoop([&](test::fail_count& fc)
         {
             echo_server es{log};
-            error_code ec;
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc, fc};
             ws.next_layer().connect(es.stream());
             ws.handshake("localhost", "/");
             std::size_t count = 0;
             std::string const s(16384, '*');
             ws.auto_fragment(false);
-            ws.async_write(buffer(s),
+            ws.async_write(net::buffer(s),
                 [&](error_code ec, std::size_t n)
                 {
                     ++count;
@@ -447,7 +496,7 @@ public:
                             system_error{ec});
                     BEAST_EXPECT(n == 16384);
                 });
-            BEAST_EXPECT(ws.wr_block_);
+            BEAST_EXPECT(ws.impl_->wr_block.is_locked());
             ws.async_ping("",
                 [&](error_code ec)
                 {
@@ -460,18 +509,17 @@ public:
         });
 
         // suspend on ping: mask, frag
-        doFailLoop([&](test::fail_counter& fc)
+        doFailLoop([&](test::fail_count& fc)
         {
             echo_server es{log};
-            error_code ec;
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc, fc};
             ws.next_layer().connect(es.stream());
             ws.handshake("localhost", "/");
             std::size_t count = 0;
             std::string const s(16384, '*');
             ws.auto_fragment(true);
-            ws.async_write(buffer(s),
+            ws.async_write(net::buffer(s),
                 [&](error_code ec, std::size_t n)
                 {
                     ++count;
@@ -480,7 +528,7 @@ public:
                             system_error{ec});
                     BEAST_EXPECT(n == 16384);
                 });
-            BEAST_EXPECT(ws.wr_block_);
+            BEAST_EXPECT(ws.impl_->wr_block.is_locked());
             ws.async_ping("",
                 [&](error_code ec)
                 {
@@ -493,10 +541,10 @@ public:
         });
 
         // suspend on ping: deflate
-        doFailLoop([&](test::fail_counter& fc)
+        doFailLoop([&](test::fail_count& fc)
         {
             echo_server es{log, kind::async};
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc, fc};
             {
                 permessage_deflate pmd;
@@ -509,7 +557,7 @@ public:
             std::size_t count = 0;
             auto const& s = random_string();
             ws.binary(true);
-            ws.async_write(buffer(s),
+            ws.async_write(net::buffer(s),
                 [&](error_code ec, std::size_t n)
                 {
                     ++count;
@@ -518,7 +566,7 @@ public:
                             system_error{ec});
                     BEAST_EXPECT(n == s.size());
                 });
-            BEAST_EXPECT(ws.wr_block_);
+            BEAST_EXPECT(ws.impl_->wr_block.is_locked());
             ws.async_ping("",
                 [&](error_code ec)
                 {
@@ -540,7 +588,7 @@ public:
             {
                 echo_server es{log, i==1 ?
                     kind::async : kind::sync};
-                boost::asio::io_context ioc;
+                net::io_context ioc;
                 stream<test::stream> ws{ioc};
                 ws.next_layer().connect(es.stream());
 
@@ -549,7 +597,7 @@ public:
                 if(! BEAST_EXPECTS(! ec, ec.message()))
                     break;
                 ws.async_write_some(false,
-                    boost::asio::null_buffers{},
+                    net::const_buffer{},
                     [&](error_code, std::size_t)
                     {
                         fail();
@@ -565,6 +613,44 @@ public:
         }
     }
 
+    void
+    testMoveOnly()
+    {
+        net::io_context ioc;
+        stream<test::stream> ws{ioc};
+        ws.async_write_some(
+            true, net::const_buffer{},
+            move_only_handler{});
+    }
+
+    struct copyable_handler
+    {
+        template<class... Args>
+        void
+        operator()(Args&&...) const
+        {
+        }
+    };
+
+    void
+    testAsioHandlerInvoke()
+    {
+        // make sure things compile, also can set a
+        // breakpoint in asio_handler_invoke to make sure
+        // it is instantiated.
+        {
+            net::io_context ioc;
+            net::strand<
+                net::io_context::executor_type> s(
+                    ioc.get_executor());
+            stream<test::stream> ws{ioc};
+            flat_buffer b;
+            ws.async_write(net::const_buffer{},
+                net::bind_executor(s,
+                    copyable_handler{}));
+        }
+    }
+
     /*
         https://github.com/boostorg/beast/issues/300
 
@@ -577,7 +663,7 @@ public:
         {
             echo_server es{log, i==1 ?
                 kind::async : kind::sync};
-            boost::asio::io_context ioc;
+            net::io_context ioc;
             stream<test::stream> ws{ioc};
             ws.next_layer().connect(es.stream());
 
@@ -593,33 +679,76 @@ public:
         }
     }
 
-    void
-    testContHook()
-    {
-        struct handler
-        {
-            void operator()(error_code) {}
-        };
+    /*
+        https://github.com/boostorg/beast/issues/1666
         
-        char buf[32];
-        stream<test::stream> ws{ioc_};
-        stream<test::stream>::write_some_op<
-            boost::asio::const_buffer,
-                handler> op{handler{}, ws, true,
-                    boost::asio::const_buffer{
-                        buf, sizeof(buf)}};
-        using boost::asio::asio_handler_is_continuation;
-        asio_handler_is_continuation(&op);
+        permessage-deflate not working in version 1.70.
+    */
+    void
+    testIssue1666()
+    {
+        net::io_context ioc;
+        permessage_deflate pmd;
+        pmd.client_enable = true;
+        pmd.server_enable = true;
+        stream<test::stream> ws0{ioc};
+        stream<test::stream> ws1{ioc};
+        ws0.next_layer().connect(ws1.next_layer());
+        ws0.set_option(pmd);
+        ws1.set_option(pmd);
+        ws1.async_accept(
+            [](error_code ec)
+            {
+                BEAST_EXPECTS(! ec, ec.message());
+            });
+        ws0.async_handshake("test", "/",
+            [](error_code ec)
+            {
+                BEAST_EXPECTS(! ec, ec.message());
+            });
+        ioc.run();
+        ioc.restart();
+        std::string s(256, '*');
+        auto const n0 =
+            ws0.next_layer().nwrite_bytes();
+        error_code ec;
+        BEAST_EXPECTS(! ec, ec.message());
+        ws1.write(net::buffer(s), ec);
+        auto const n1 =
+            ws0.next_layer().nwrite_bytes();
+        // Make sure the string was actually compressed
+        BEAST_EXPECT(n1 < n0 + s.size());
     }
 
+#if BOOST_ASIO_HAS_CO_AWAIT
+    void testAwaitableCompiles(
+        stream<test::stream>& s,
+        net::mutable_buffer buf,
+        bool fin)
+    {
+        static_assert(std::is_same_v<
+            net::awaitable<std::size_t>, decltype(
+            s.async_write(buf, net::use_awaitable))>);
+
+        static_assert(std::is_same_v<
+            net::awaitable<std::size_t>, decltype(
+            s.async_write_some(fin, buf, net::use_awaitable))>);
+    }
+#endif
+
     void
     run() override
     {
         testWrite();
+        testPausationAbandoning();
         testWriteSuspend();
         testAsyncWriteFrame();
+        testMoveOnly();
         testIssue300();
-        testContHook();
+        testIssue1666();
+#if BOOST_ASIO_HAS_CO_AWAIT
+        boost::ignore_unused(&write_test::testAwaitableCompiles);
+#endif
     }
 };
 
