@@ -7,8 +7,7 @@
 // See http://www.boost.org/libs/container for documentation.
 //
 //////////////////////////////////////////////////////////////////////////////
-#include <boost/container/detail/config_begin.hpp>
-#include <boost/container/detail/workaround.hpp>
+
 //[doc_emplace
 #include <boost/container/list.hpp>
 #include <cassert>
@@ -41,4 +40,3 @@ int main ()
    return 0;
 }
 //]
-#include <boost/container/detail/config_end.hpp>
