@@ -10,7 +10,6 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 
-#include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/offset_ptr.hpp>
 #include <boost/interprocess/smart_ptr/shared_ptr.hpp>
 #include <boost/interprocess/smart_ptr/weak_ptr.hpp>
@@ -611,5 +610,3 @@ int main()
    test_alias();
 }
 
-#include <boost/interprocess/detail/config_end.hpp>
-
