@@ -83,7 +83,7 @@ bool check_complex(const std::complex<T>& a, const std::complex<T>& b, int max_e
    }
    if((boost::math::isnan)(a.imag()))
    {
-      BOOST_ERROR("Found non-finite value for inaginary part: " << a);
+      BOOST_ERROR("Found non-finite value for imaginary part: " << a);
    }
 
    T rel = boost::math::fabs((b-a)/b) / eps;
@@ -258,7 +258,7 @@ void test_inverse_trig(T)
 
 //
 // check_spots:
-// Various spot values, mostly the C99 special cases (infinites and NAN's).
+// Various spot values, mostly the C99 special cases (infinities and NAN's).
 // TODO: add spot checks for the Wolfram spot values.
 //
 template <class T>
@@ -279,7 +279,7 @@ void check_spots(const T&)
    bool test_infinity = std::numeric_limits<T>::has_infinity;
    T nan = 0;
    bool test_nan = false;
-#if !BOOST_WORKAROUND(__BORLANDC__, BOOST_TESTED_AT(0x564))
+#if !BOOST_WORKAROUND(BOOST_BORLANDC, BOOST_TESTED_AT(0x564))
    // numeric_limits reports that a quiet NaN is present
    // but an attempt to access it will terminate the program!!!!
    if(std::numeric_limits<T>::has_quiet_NaN)
