@@ -11,7 +11,7 @@
 // Basic sanity tests for the beta Distribution.
 
 // http://members.aol.com/iandjmsmith/BETAEX.HTM  beta distribution calculator
-// Appreas to be a 64-bit calculator showing 17 decimal digit (last is noisy).
+// Appears to be a 64-bit calculator showing 17 decimal digit (last is noisy).
 // Similar to mathCAD?
 
 // http://www.nuhertz.com/statmat/distributions.html#Beta
@@ -627,7 +627,7 @@ BOOST_AUTO_TEST_CASE( test_main )
    test_spots(0.0); // Test double.
 #ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
    test_spots(0.0L); // Test long double.
-#if !BOOST_WORKAROUND(__BORLANDC__, BOOST_TESTED_AT(0x582))
+#if !BOOST_WORKAROUND(BOOST_BORLANDC, BOOST_TESTED_AT(0x582))
    test_spots(boost::math::concepts::real_concept(0.)); // Test real concept.
 #endif
 #endif
