@@ -8,12 +8,17 @@
 
 // See library home page at http://www.boost.org/libs/system
 
+#if defined(__GNUC__) && __GNUC__ >= 5 && __cplusplus >= 201103L
+# pragma GCC diagnostic error "-Wsuggest-override"
+#endif
+
 #include <boost/config.hpp>
 
 #if defined( BOOST_GCC ) && BOOST_GCC < 40600
 #pragma GCC diagnostic ignored "-Wnon-virtual-dtor"
 #endif
 
+#include <boost/system/system_error.hpp>
 #include <boost/system/error_code.hpp>
 #include <boost/core/lightweight_test.hpp>
 #include <cerrno>
@@ -38,5 +43,10 @@ int main()
 
     BOOST_TEST( bc == bn );
 
+    boost::system::system_error x( bc, "prefix" );
+
+    BOOST_TEST_EQ( x.code(), bc );
+    BOOST_TEST_EQ( std::string( x.what() ), "prefix: " + bc.message() );
+
     return boost::report_errors();
 }
