@@ -13,28 +13,18 @@
     license reproduced at the end of this file.
 =============================================================================*/
 
-// Tests tokenization of preprocessing numbers.
+// Tests error reporting: Error of rescanning. 
 
-// Note: Wave curently fails the tests 22.1 and 22.2 because of Wave's design
-//       choice to act on C++ tokens and not on the raw inut stream.
-//       This limitation will be removed in a future release.
+// 27.7
+#define SUB1(x, y)      (x - y)
+#define TWO_TOKENS      a,b
+#define SUB(x, y)       SUB1(x, y)
 
-#define STR1(a)     # a
-#define STR(a)      STR1(a)
-#define EXP         1
+// Too many arguments error while rescanning after once replaced to: 
+// SUB(a, b, 1)
 
-// 22.1: 12E+EXP is a preprocessing number, EXP is not expanded.
-//R #line 28 "t_5_025.cpp"
-STR(12E+EXP)        //R "12E+EXP" 
-
-// 22.2: .2e-EXP is also a preprocessing number.
-//R #line 32 "t_5_025.cpp"
-STR(.2e-EXP)        //R ".2e-EXP" 
-
-// 22.3: + or - is allowed only following E or e, 12+EXP is not
-//       a preprocessing number.
-//R #line 37 "t_5_025.cpp"
-STR(12+EXP)         //R "12+1" 
+//E t_6_050.cpp(27): warning: too many macro arguments: SUB1
+SUB(TWO_TOKENS, 1)
 
 /*-
  * Copyright (c) 1998, 2002-2005 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
