@@ -14,7 +14,7 @@ def create_parser(prog, description):
     parser.add_argument(
         '--data',
         required=True,
-    type=arg_validators.ValidDevice(as_string=True),
+        type=arg_validators.ValidRawDevice(as_string=True),
         help='a raw device to use for the OSD',
     )
     parser.add_argument(
@@ -35,16 +35,24 @@ def create_parser(prog, description):
     parser.add_argument(
         '--block.db',
         dest='block_db',
-        help='Path to bluestore block.db block device'
+        help='Path to bluestore block.db block device',
+        type=arg_validators.ValidRawDevice(as_string=True)
     )
     parser.add_argument(
         '--block.wal',
         dest='block_wal',
-        help='Path to bluestore block.wal block device'
+        help='Path to bluestore block.wal block device',
+        type=arg_validators.ValidRawDevice(as_string=True)
     )
     parser.add_argument(
         '--dmcrypt',
-        action='store_true',
+        action=arg_validators.DmcryptAction,
         help='Enable device encryption via dm-crypt',
     )
+    parser.add_argument(
+        '--osd-id',
+        help='Reuse an existing OSD id',
+        default=None,
+        type=arg_validators.valid_osd_id,
+    )
     return parser
