@@ -1,5 +1,6 @@
 import pytest
 from ceph_volume.devices import lvm
+from mock.mock import patch, MagicMock
 
 
 class TestZap(object):
@@ -19,9 +20,18 @@ class TestZap(object):
         '/dev/mapper/foo',
         '/dev/dm-0',
     ])
-    def test_can_not_zap_mapper_device(self, monkeypatch, device_info, capsys, is_root, device_name):
+    @patch('ceph_volume.util.arg_validators.Device')
+    def test_can_not_zap_mapper_device(self, mocked_device, monkeypatch, device_info, capsys, is_root, device_name):
         monkeypatch.setattr('os.path.exists', lambda x: True)
-        device_info()
+        mocked_device.return_value = MagicMock(
+            is_mapper=True,
+            is_mpath=False,
+            used_by_ceph=True,
+            exists=True,
+            has_partitions=False,
+            has_gpt_headers=False,
+            has_fs=False
+        )
         with pytest.raises(SystemExit):
             lvm.zap.Zap(argv=[device_name]).main()
         stdout, stderr = capsys.readouterr()
