@@ -69,8 +69,11 @@ def cephadm_fs(
          mock.patch('platform.processor', return_value='x86_64'), \
          mock.patch('cephadm.extract_uid_gid', return_value=(uid, gid)):
 
-        if not fake_filesystem.is_root():
-            fake_filesystem.set_uid(0)
+        try:
+            if not fake_filesystem.is_root():
+                fake_filesystem.set_uid(0)
+        except AttributeError:
+            pass
 
         fs.create_dir(cd.DATA_DIR)
         fs.create_dir(cd.LOG_DIR)
