@@ -76,8 +76,7 @@ std::string new_oid_prefix(std::string id, std::optional<std::string>& val)
 }
 
 int write_header(cls_method_context_t hctx,
-		 info& header,
-		 bool inc_ver = true)
+		 info& header)
 {
   static constexpr auto HEADER_INSTANCE_SIZE = 16;
   if (header.version.instance.empty()) {
@@ -86,9 +85,6 @@ int write_header(cls_method_context_t hctx,
     cls_gen_rand_base64(buf, sizeof(buf) - 1);
     header.version.instance = buf;
   }
-  if (inc_ver) {
-    ++header.version.ver;
-  }
   ceph::buffer::list bl;
   encode(header, bl);
   return cls_cxx_write_full(hctx, &bl);
@@ -117,7 +113,6 @@ int read_part_header(cls_method_context_t hctx,
   std::ostringstream ss;
   ss << part_header->max_time;
   CLS_LOG(5, "%s:%d read part_header:\n"
-	  "\ttag=%s\n"
 	  "\tmagic=0x%" PRIx64 "\n"
 	  "\tmin_ofs=%" PRId64 "\n"
 	  "\tlast_ofs=%" PRId64 "\n"
@@ -126,7 +121,6 @@ int read_part_header(cls_method_context_t hctx,
 	  "\tmax_index=%" PRId64 "\n"
 	  "\tmax_time=%s\n",
 	  __PRETTY_FUNCTION__, __LINE__,
-	  part_header->tag.c_str(),
 	  part_header->magic,
 	  part_header->min_ofs,
 	  part_header->last_ofs,
@@ -300,7 +294,7 @@ int create_meta(cls_method_context_t hctx,
   header.params.max_entry_size = op.max_entry_size;
   header.params.full_size_threshold = op.max_part_size - op.max_entry_size - part_entry_overhead;
 
-  r = write_header(hctx, header, false);
+  r = write_header(hctx, header);
   if (r < 0) {
     CLS_ERR("%s: failed to write header: r=%d", __PRETTY_FUNCTION__, r);
     return r;
@@ -344,19 +338,16 @@ int update_meta(cls_method_context_t hctx, ceph::buffer::list* in,
     .journal_entries_rm(
       std::move(op.journal_entries_rm));
 
-  auto err = header.apply_update(u);
-  if (err) {
-    std::ostringstream ss;
-    ss << u;
-    CLS_ERR("%s: %s: %s", __PRETTY_FUNCTION__, err->c_str(),
-	    ss.str().c_str());
-    return -EINVAL;
-  }
-
-  r = write_header(hctx, header);
-  if (r < 0) {
-    CLS_ERR("%s: failed to write header: r=%d", __PRETTY_FUNCTION__, r);
-    return r;
+  auto changed = header.apply_update(u);
+  if (changed) {
+    r = write_header(hctx, header);
+    if (r < 0) {
+      CLS_ERR("%s: failed to write header: r=%d", __PRETTY_FUNCTION__, r);
+      return r;
+    }
+  } else {
+    CLS_LOG(10, "%s: No change, nothing to write.",
+	    __PRETTY_FUNCTION__);
   }
 
   return 0;
@@ -406,11 +397,6 @@ int init_part(cls_method_context_t hctx, ceph::buffer::list* in,
 
   std::uint64_t size;
 
-  if (op.tag.empty()) {
-    CLS_ERR("%s: tag required", __PRETTY_FUNCTION__);
-    return -EINVAL;
-  }
-
   int r = cls_cxx_stat2(hctx, &size, nullptr);
   if (r < 0 && r != -ENOENT) {
     CLS_ERR("ERROR: %s: cls_cxx_stat2() on obj returned %d", __PRETTY_FUNCTION__, r);
@@ -424,8 +410,7 @@ int init_part(cls_method_context_t hctx, ceph::buffer::list* in,
       return r;
     }
 
-    if (!(part_header.tag == op.tag &&
-          part_header.params == op.params)) {
+    if (!(part_header.params == op.params)) {
       CLS_ERR("%s: failed to re-create existing part with different "
 	      "params", __PRETTY_FUNCTION__);
       return -EEXIST;
@@ -436,7 +421,6 @@ int init_part(cls_method_context_t hctx, ceph::buffer::list* in,
 
   part_header part_header;
 
-  part_header.tag = op.tag;
   part_header.params = op.params;
 
   part_header.min_ofs = CLS_FIFO_MAX_PART_HEADER_SIZE;
@@ -475,11 +459,6 @@ int push_part(cls_method_context_t hctx, ceph::buffer::list* in,
     return -EINVAL;
   }
 
-  if (op.tag.empty()) {
-    CLS_ERR("%s: tag required", __PRETTY_FUNCTION__);
-    return -EINVAL;
-  }
-
   part_header part_header;
   int r = read_part_header(hctx, &part_header);
   if (r < 0) {
@@ -487,11 +466,6 @@ int push_part(cls_method_context_t hctx, ceph::buffer::list* in,
     return r;
   }
 
-  if (!(part_header.tag == op.tag)) {
-    CLS_ERR("%s: bad tag", __PRETTY_FUNCTION__);
-    return -EINVAL;
-  }
-
   std::uint64_t effective_len = op.total_len + op.data_bufs.size() *
     part_entry_overhead;
 
@@ -782,12 +756,6 @@ int trim_part(cls_method_context_t hctx,
     return r;
   }
 
-  if (op.tag &&
-      !(part_header.tag == *op.tag)) {
-    CLS_ERR("%s: bad tag", __PRETTY_FUNCTION__);
-    return -EINVAL;
-  }
-
   if (op.ofs < part_header.min_ofs) {
     return 0;
   }
@@ -866,12 +834,6 @@ int list_part(cls_method_context_t hctx, ceph::buffer::list* in,
     return r;
   }
 
-  if (op.tag &&
-      !(part_header.tag == *op.tag)) {
-    CLS_ERR("%s: bad tag", __PRETTY_FUNCTION__);
-    return -EINVAL;
-  }
-
   EntryReader reader(hctx, part_header, op.ofs);
 
   if (op.ofs >= part_header.min_ofs &&
@@ -885,8 +847,6 @@ int list_part(cls_method_context_t hctx, ceph::buffer::list* in,
 
   op::list_part_reply reply;
 
-  reply.tag = part_header.tag;
-
   auto max_entries = std::min(op.max_entries, op::MAX_LIST_ENTRIES);
 
   for (int i = 0; i < max_entries && !reader.end(); ++i) {
