@@ -1,3 +1,5 @@
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
 #ifndef CEPH_CLS_LOG_TYPES_H
 #define CEPH_CLS_LOG_TYPES_H
 
@@ -10,15 +12,15 @@ class JSONObj;
 
 
 struct cls_log_entry {
-  string id;
-  string section;
-  string name;
+  std::string id;
+  std::string section;
+  std::string name;
   utime_t timestamp;
-  bufferlist data;
+  ceph::buffer::list data;
 
   cls_log_entry() {}
 
-  void encode(bufferlist& bl) const {
+  void encode(ceph::buffer::list& bl) const {
     ENCODE_START(2, 1, bl);
     encode(section, bl);
     encode(name, bl);
@@ -28,7 +30,7 @@ struct cls_log_entry {
     ENCODE_FINISH(bl);
   }
 
-  void decode(bufferlist::const_iterator& bl) {
+  void decode(ceph::buffer::list::const_iterator& bl) {
     DECODE_START(2, bl);
     decode(section, bl);
     decode(name, bl);
@@ -42,23 +44,30 @@ struct cls_log_entry {
 WRITE_CLASS_ENCODER(cls_log_entry)
 
 struct cls_log_header {
-  string max_marker;
+  std::string max_marker;
   utime_t max_time;
 
-  void encode(bufferlist& bl) const {
+  void encode(ceph::buffer::list& bl) const {
     ENCODE_START(1, 1, bl);
     encode(max_marker, bl);
     encode(max_time, bl);
     ENCODE_FINISH(bl);
   }
 
-  void decode(bufferlist::const_iterator& bl) {
+  void decode(ceph::buffer::list::const_iterator& bl) {
     DECODE_START(1, bl);
     decode(max_marker, bl);
     decode(max_time, bl);
     DECODE_FINISH(bl);
   }
 };
+inline bool operator ==(const cls_log_header& lhs, const cls_log_header& rhs) {
+  return (lhs.max_marker == rhs.max_marker &&
+	  lhs.max_time == rhs.max_time);
+}
+inline bool operator !=(const cls_log_header& lhs, const cls_log_header& rhs) {
+  return !(lhs == rhs);
+}
 WRITE_CLASS_ENCODER(cls_log_header)
 
 
