@@ -608,6 +608,9 @@ int cls_rgw_lc_list(librados::IoCtx& io_ctx, const std::string& oid,
                     std::vector<cls_rgw_lc_entry>& entries);
 #endif
 
+/* multipart */
+void cls_rgw_mp_upload_part_info_update(librados::ObjectWriteOperation& op, const std::string& part_key, const RGWUploadPartInfo& info);
+
 /* resharding */
 void cls_rgw_reshard_add(librados::ObjectWriteOperation& op, const cls_rgw_reshard_entry& entry);
 void cls_rgw_reshard_remove(librados::ObjectWriteOperation& op, const cls_rgw_reshard_entry& entry);
