@@ -1,17 +1,19 @@
-#ifndef CEPH_CLS_RGW_GC_TYPES_H
-#define CEPH_CLS_RGW_GC_TYPES_H
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
+
+#pragma once
 
 #include "include/types.h"
 #include <unordered_map>
 
 struct cls_rgw_gc_urgent_data
 {
-  std::unordered_map<string,ceph::real_time> urgent_data_map;
+  std::unordered_map<std::string, ceph::real_time> urgent_data_map;
   uint32_t num_urgent_data_entries{0}; // requested by user
   uint32_t num_head_urgent_entries{0}; // actual number of entries in queue head
   uint32_t num_xattr_urgent_entries{0}; // actual number of entries in xattr in case of spill over
 
-  void encode(bufferlist& bl) const {
+  void encode(ceph::buffer::list& bl) const {
     ENCODE_START(1, 1, bl);
     encode(urgent_data_map, bl);
     encode(num_urgent_data_entries, bl);
@@ -20,7 +22,7 @@ struct cls_rgw_gc_urgent_data
     ENCODE_FINISH(bl);
   }
 
-  void decode(bufferlist::const_iterator& bl) {
+  void decode(ceph::buffer::list::const_iterator& bl) {
     DECODE_START(1, bl);
     decode(urgent_data_map, bl);
     decode(num_urgent_data_entries, bl);
@@ -30,5 +32,3 @@ struct cls_rgw_gc_urgent_data
   }
 };
 WRITE_CLASS_ENCODER(cls_rgw_gc_urgent_data)
-
-#endif
\ No newline at end of file
