@@ -12,6 +12,7 @@
  * 
  */
 
+#include <utime.h>
 #include <signal.h>
 
 #include "HeartbeatMap.h"
@@ -24,6 +25,10 @@
 #undef dout_prefix
 #define dout_prefix *_dout << "heartbeat_map "
 
+using std::chrono::duration_cast;
+using std::chrono::seconds;
+using std::string;
+
 namespace ceph {
 
 HeartbeatMap::HeartbeatMap(CephContext *cct)
@@ -62,17 +67,17 @@ void HeartbeatMap::remove_worker(const heartbeat_handle_d *h)
 }
 
 bool HeartbeatMap::_check(const heartbeat_handle_d *h, const char *who,
-			  ceph::coarse_mono_clock::rep now)
+			  ceph::coarse_mono_time now)
 {
   bool healthy = true;
-  auto was = h->timeout.load();
-  if (was && was < now) {
+  if (auto was = h->timeout.load(std::memory_order_relaxed);
+      !clock::is_zero(was) && was < now) {
     ldout(m_cct, 1) << who << " '" << h->name << "'"
 		    << " had timed out after " << h->grace << dendl;
     healthy = false;
   }
-  was = h->suicide_timeout;
-  if (was && was < now) {
+  if (auto was = h->suicide_timeout.load(std::memory_order_relaxed);
+      !clock::is_zero(was) && was < now) {
     ldout(m_cct, 1) << who << " '" << h->name << "'"
 		    << " had suicide timed out after " << h->suicide_grace << dendl;
     pthread_kill(h->thread_id, SIGABRT);
@@ -83,33 +88,32 @@ bool HeartbeatMap::_check(const heartbeat_handle_d *h, const char *who,
 }
 
 void HeartbeatMap::reset_timeout(heartbeat_handle_d *h,
-				 ceph::coarse_mono_clock::rep grace,
-				 ceph::coarse_mono_clock::rep suicide_grace)
+				 ceph::timespan grace,
+				 ceph::timespan suicide_grace)
 {
   ldout(m_cct, 20) << "reset_timeout '" << h->name << "' grace " << grace
 		   << " suicide " << suicide_grace << dendl;
-  auto now = chrono::duration_cast<chrono::seconds>(
-	       ceph::coarse_mono_clock::now().time_since_epoch()).count();
+  const auto now = clock::now();
   _check(h, "reset_timeout", now);
 
-  h->timeout = now + grace;
+  h->timeout.store(now + grace, std::memory_order_relaxed);
   h->grace = grace;
 
-  if (suicide_grace)
-    h->suicide_timeout = now + suicide_grace;
-  else
-    h->suicide_timeout = 0;
+  if (suicide_grace > ceph::timespan::zero()) {
+    h->suicide_timeout.store(now + suicide_grace, std::memory_order_relaxed);
+  } else {
+    h->suicide_timeout.store(clock::zero(), std::memory_order_relaxed);
+  }
   h->suicide_grace = suicide_grace;
 }
 
 void HeartbeatMap::clear_timeout(heartbeat_handle_d *h)
 {
   ldout(m_cct, 20) << "clear_timeout '" << h->name << "'" << dendl;
-  auto now = chrono::duration_cast<std::chrono::seconds>(
-	       ceph::coarse_mono_clock::now().time_since_epoch()).count();
+  auto now = clock::now();
   _check(h, "clear_timeout", now);
-  h->timeout = 0;
-  h->suicide_timeout = 0;
+  h->timeout.store(clock::zero(), std::memory_order_relaxed);
+  h->suicide_timeout.store(clock::zero(), std::memory_order_relaxed);
 }
 
 bool HeartbeatMap::is_healthy()
@@ -132,12 +136,11 @@ bool HeartbeatMap::is_healthy()
     healthy = false;
   }
 
-  for (list<heartbeat_handle_d*>::iterator p = m_workers.begin();
+  for (auto p = m_workers.begin();
        p != m_workers.end();
        ++p) {
     heartbeat_handle_d *h = *p;
-    auto epoch = chrono::duration_cast<chrono::seconds>(now.time_since_epoch()).count();
-    if (!_check(h, "is_healthy", epoch)) {
+    if (!_check(h, "is_healthy", now)) {
       healthy = false;
       unhealthy++;
     }
@@ -169,7 +172,7 @@ void HeartbeatMap::check_touch_file()
   if (path.length() && is_healthy()) {
     int fd = ::open(path.c_str(), O_WRONLY|O_CREAT|O_CLOEXEC, 0644);
     if (fd >= 0) {
-      ::utimes(path.c_str(), NULL);
+      ::utime(path.c_str(), NULL);
       ::close(fd);
     } else {
       ldout(m_cct, 0) << "unable to touch " << path << ": "
