@@ -16,6 +16,7 @@
 // -----------------------------------------------------------------------------
 #include "acconfig.h"
 #include "ceph_ver.h"
+#include "common/ceph_context.h"
 #include "CompressionPluginZlib.h"
 
 // -----------------------------------------------------------------------------
@@ -31,7 +32,7 @@ int __ceph_plugin_init(CephContext *cct,
                        const std::string& type,
                        const std::string& name)
 {
-  PluginRegistry *instance = cct->get_plugin_registry();
+  auto instance = cct->get_plugin_registry();
 
   return instance->add(type, name, new CompressionPluginZlib(cct));
 }
