@@ -23,12 +23,12 @@ namespace crimson::os {
 class OSDMeta {
   template<typename T> using Ref = boost::intrusive_ptr<T>;
 
-  crimson::os::FuturizedStore* store;
+  crimson::os::FuturizedStore& store;
   Ref<crimson::os::FuturizedCollection> coll;
 
 public:
   OSDMeta(Ref<crimson::os::FuturizedCollection> coll,
-          crimson::os::FuturizedStore* store)
+          crimson::os::FuturizedStore& store)
     : store{store}, coll{coll}
   {}
 
@@ -46,9 +46,9 @@ public:
   seastar::future<OSDSuperblock> load_superblock();
 
   using ec_profile_t = std::map<std::string, std::string>;
-  seastar::future<pg_pool_t,
-                  std::string,
-                  ec_profile_t> load_final_pool_info(int64_t pool);
+  seastar::future<std::tuple<pg_pool_t,
+			     std::string,
+			     ec_profile_t>> load_final_pool_info(int64_t pool);
 private:
   static ghobject_t osdmap_oid(epoch_t epoch);
   static ghobject_t final_pool_info_oid(int64_t pool);
