@@ -14,10 +14,10 @@ namespace crimson::os {
 /// PG related metadata
 class PGMeta
 {
-  crimson::os::FuturizedStore* store;
+  crimson::os::FuturizedStore& store;
   const spg_t pgid;
 public:
-  PGMeta(crimson::os::FuturizedStore *store, spg_t pgid);
+  PGMeta(crimson::os::FuturizedStore& store, spg_t pgid);
   seastar::future<epoch_t> get_epoch();
   seastar::future<std::tuple<pg_info_t, PastIntervals>> load();
 };
