@@ -1,20 +1,20 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 
-#include "include/compat.h"
+#include <vector>
+
 #include "CrushLocation.h"
 #include "CrushWrapper.h"
+#if defined(WITH_SEASTAR) && !defined(WITH_ALIEN)
+#include "common/SubProcess.h"
+#endif
 #include "common/ceph_context.h"
 #include "common/config.h"
-#include "include/str_list.h"
 #include "common/debug.h"
 #include "common/errno.h"
 #include "include/common_fwd.h"
 #include "include/compat.h"
-
-#include "common/SubProcess.h"
-
-#include <vector>
+#include "include/str_list.h"
 
 namespace TOPNSPC::crush {
 
@@ -48,6 +48,9 @@ int CrushLocation::update_from_hook()
   if (cct->_conf->crush_location_hook.length() == 0)
     return 0;
  
+#if defined(WITH_SEASTAR) && !defined(WITH_ALIEN)
+  ceph_abort_msg("crimson does not support crush_location_hook, it must stay empty");
+#else
   if (0 != access(cct->_conf->crush_location_hook.c_str(), R_OK)) {
     lderr(cct) << "the user define crush location hook: " << cct->_conf->crush_location_hook
                << " may not exist or can not access it" << dendl;
@@ -95,6 +98,7 @@ int CrushLocation::update_from_hook()
   bl.begin().copy(bl.length(), out);
   out.erase(out.find_last_not_of(" \n\r\t")+1);
   return _parse(out);
+#endif // WITH_SEASTAR && !WITH_ALIEN
 }
 
 int CrushLocation::init_on_startup()
