@@ -24,7 +24,6 @@
 
 class ISALCryptoPlugin : public CryptoPlugin {
 
-  CryptoAccelRef cryptoaccel;
 public:
 
   explicit ISALCryptoPlugin(CephContext* cct) : CryptoPlugin(cct)
@@ -32,7 +31,7 @@ public:
   ~ISALCryptoPlugin()
   {}
   virtual int factory(CryptoAccelRef *cs,
-                      ostream *ss)
+                      std::ostream *ss)
   {
     if (cryptoaccel == nullptr)
     {
