@@ -78,7 +78,6 @@ int ErasureCode::create_rule(
   if (ruleid < 0)
     return ruleid;
 
-  crush.set_rule_mask_max_size(ruleid, get_chunk_count());
   return ruleid;
 }
 
@@ -203,12 +202,6 @@ int ErasureCode::encode(const set<int> &want_to_encode,
   return 0;
 }
 
-int ErasureCode::encode_chunks(const set<int> &want_to_encode,
-                               map<int, bufferlist> *encoded)
-{
-  ceph_abort_msg("ErasureCode::encode_chunks not implemented");
-}
- 
 int ErasureCode::_decode(const set<int> &want_to_read,
 			 const map<int, bufferlist> &chunks,
 			 map<int, bufferlist> *decoded)
@@ -254,13 +247,6 @@ int ErasureCode::decode(const set<int> &want_to_read,
   return _decode(want_to_read, chunks, decoded);
 }
 
-int ErasureCode::decode_chunks(const set<int> &want_to_read,
-                               const map<int, bufferlist> &chunks,
-                               map<int, bufferlist> *decoded)
-{
-  ceph_abort_msg("ErasureCode::decode_chunks not implemented");
-}
-
 int ErasureCode::parse(const ErasureCodeProfile &profile,
 		       ostream *ss)
 {
