@@ -29,6 +29,8 @@
 
 #include "include/compat.h"
 
+using std::string;
+
 //
 // derr can be used for functions exclusively called from pidfile_write
 //
@@ -159,12 +161,15 @@ int pidfh::open(std::string_view pid_file)
   pf_dev = st.st_dev;
   pf_ino = st.st_ino;
 
-  struct flock l = {
-    .l_type = F_WRLCK,
-    .l_whence = SEEK_SET,
-    .l_start = 0,
-    .l_len = 0
-  };
+  // Default Windows file share flags prevent other processes from writing
+  // to this file.
+  #ifndef _WIN32
+  struct flock l;
+  l.l_type = F_WRLCK;
+  l.l_whence = SEEK_SET;
+  l.l_start = 0;
+  l.l_len = 0;
+  
   int r = ::fcntl(pf_fd, F_SETLK, &l);
   if (r < 0) {
     if (errno == EAGAIN || errno == EACCES) {
@@ -175,10 +180,12 @@ int pidfh::open(std::string_view pid_file)
       derr << __func__ << ": failed to lock pidfile "
 	   << pf_path << "': " << cpp_strerror(errno) << dendl;
     }
+    const auto lock_errno = errno;
     ::close(pf_fd);
     reset();
-    return -errno;
+    return -lock_errno;
   }
+  #endif
   return 0;
 }
 
