@@ -32,7 +32,8 @@ DeepCopyRequest<I>::DeepCopyRequest(I *src_image_ctx, I *dst_image_ctx,
                                     librados::snap_t dst_snap_id_start,
                                     bool flatten,
                                     const ObjectNumber &object_number,
-                                    ContextWQ *work_queue, SnapSeqs *snap_seqs,
+                                    asio::ContextWQ *work_queue,
+                                    SnapSeqs *snap_seqs,
                                     deep_copy::Handler *handler,
                                     Context *on_finish)
   : RefCountedObject(dst_image_ctx->cct), m_src_image_ctx(src_image_ctx),
