@@ -7,6 +7,7 @@
 #include "common/errno.h"
 #include "common/Cond.h"
 #include "cls/rbd/cls_rbd_client.h"
+#include "librbd/AsioEngine.h"
 #include "librbd/ExclusiveLock.h"
 #include "librbd/ImageCtx.h"
 #include "librbd/ImageState.h"
@@ -24,7 +25,6 @@
 #include <json_spirit/json_spirit.h>
 #include "librbd/journal/DisabledPolicy.h"
 #include "librbd/image/ListWatchersRequest.h"
-#include <experimental/map>
 
 #define dout_subsys ceph_subsys_rbd
 #undef dout_prefix
@@ -34,7 +34,7 @@ namespace librbd {
 namespace api {
 
 template <typename I>
-const typename Trash<I>::TrashImageSources Trash<I>::RESTORE_SOURCE_WHITELIST {
+const typename Trash<I>::TrashImageSources Trash<I>::ALLOWED_RESTORE_SOURCES {
     cls::rbd::TRASH_IMAGE_SOURCE_USER,
     cls::rbd::TRASH_IMAGE_SOURCE_MIRRORING,
     cls::rbd::TRASH_IMAGE_SOURCE_USER_PARENT
@@ -91,13 +91,12 @@ int enable_mirroring(IoCtx &io_ctx, const std::string &image_id) {
 
   ldout(cct, 10) << dendl;
 
-  ThreadPool *thread_pool;
-  ContextWQ *op_work_queue;
-  ImageCtx::get_thread_pool_instance(cct, &thread_pool, &op_work_queue);
+  AsioEngine asio_engine(io_ctx);
+
   C_SaferCond ctx;
   auto req = mirror::EnableRequest<I>::create(
     io_ctx, image_id, cls::rbd::MIRROR_IMAGE_MODE_JOURNAL, "", false,
-    op_work_queue, &ctx);
+    asio_engine.get_work_queue(), &ctx);
   req->send();
   r = ctx.wait();
   if (r < 0) {
@@ -120,7 +119,7 @@ int list_trash_image_specs(
   uint32_t max_read = 1024;
   std::string last_read;
   do {
-    std::map<string, cls::rbd::TrashImageSpec> trash_entries;
+    std::map<std::string, cls::rbd::TrashImageSpec> trash_entries;
     int r = cls_client::trash_list(&io_ctx, last_read, max_read,
                                    &trash_entries);
     if (r < 0 && r != -ENOENT) {
@@ -295,19 +294,20 @@ int Trash<I>::move(librados::IoCtx &io_ctx, rbd_trash_image_source_t source,
     if (r < 0) {
       return r;
     }
-
-    std::experimental::erase_if(
-      trash_image_specs, [image_name](const auto& pair) {
-        const auto& spec = pair.second;
-        return (spec.source != cls::rbd::TRASH_IMAGE_SOURCE_USER ||
-                spec.state != cls::rbd::TRASH_IMAGE_STATE_MOVING ||
-                spec.name != image_name);
-      });
-    if (trash_image_specs.empty()) {
+    if (auto found_image =
+        std::find_if(
+          trash_image_specs.begin(), trash_image_specs.end(),
+          [&](const auto& pair) {
+            const auto& spec = pair.second;
+            return (spec.source == cls::rbd::TRASH_IMAGE_SOURCE_USER &&
+                    spec.state == cls::rbd::TRASH_IMAGE_STATE_MOVING &&
+                    spec.name == image_name);
+          });
+        found_image != trash_image_specs.end()) {
+      image_id = found_image->first;
+    } else {
       return -ENOENT;
     }
-
-    image_id = trash_image_specs.begin()->first;
     ldout(cct, 15) << "derived image id " << image_id << " from existing "
                    << "trash entry" << dendl;
   } else if (r < 0) {
@@ -347,7 +347,7 @@ int Trash<I>::get(IoCtx &io_ctx, const std::string &id,
 }
 
 template <typename I>
-int Trash<I>::list(IoCtx &io_ctx, vector<trash_image_info_t> &entries,
+int Trash<I>::list(IoCtx &io_ctx, std::vector<trash_image_info_t> &entries,
                    bool exclude_user_remove_source) {
   CephContext *cct((CephContext *)io_ctx.cct());
   ldout(cct, 20) << __func__ << " " << &io_ctx << dendl;
@@ -606,13 +606,11 @@ int Trash<I>::remove(IoCtx &io_ctx, const std::string &image_id, bool force,
     return -EBUSY;
   }
 
-  ThreadPool *thread_pool;
-  ContextWQ *op_work_queue;
-  ImageCtx::get_thread_pool_instance(cct, &thread_pool, &op_work_queue);
+  AsioEngine asio_engine(io_ctx);
 
   C_SaferCond cond;
   auto req = librbd::trash::RemoveRequest<I>::create(
-      io_ctx, image_id, op_work_queue, force, prog_ctx, &cond);
+      io_ctx, image_id, asio_engine.get_work_queue(), force, prog_ctx, &cond);
   req->send();
 
   r = cond.wait();
