@@ -19,11 +19,11 @@
 #include <condition_variable>
 #include <mutex>
 
+struct Context;
 
 namespace librbd {
 namespace io {
 
-
 /**
  * AioCompletion is the overall completion for a single
  * rbd I/O request. It may be composed of many AioObjectRequests,
@@ -72,6 +72,8 @@ struct AioCompletion {
   bool was_armed = false;
   bool external_callback = false;
 
+  Context* image_dispatcher_ctx = nullptr;
+
   template <typename T, void (T::*MF)(int)>
   static void callback_adapter(completion_t cb, void *arg) {
     AioCompletion *comp = reinterpret_cast<AioCompletion *>(cb);
