@@ -6,8 +6,20 @@
 #include "common/Formatter.h"
 #include "common/StackStringStream.h"
 
+#include <iostream>
+#include <sstream>
+#include <string>
+
 const mds_gid_t MDS_GID_NONE = mds_gid_t(0);
 
+using std::list;
+using std::make_pair;
+using std::ostream;
+using std::set;
+using std::vector;
+
+using ceph::bufferlist;
+using ceph::Formatter;
 
 /*
  * frag_info_t
@@ -47,6 +59,15 @@ void frag_info_t::dump(Formatter *f) const
   f->dump_unsigned("change_attr", change_attr);
 }
 
+void frag_info_t::decode_json(JSONObj *obj){
+
+  JSONDecoder::decode_json("version", version, obj, true);
+  JSONDecoder::decode_json("mtime", mtime, obj, true);
+  JSONDecoder::decode_json("num_files", nfiles, obj, true);
+  JSONDecoder::decode_json("num_subdirs", nsubdirs, obj, true);
+  JSONDecoder::decode_json("change_attr", change_attr, obj, true);
+}
+
 void frag_info_t::generate_test_instances(std::list<frag_info_t*>& ls)
 {
   ls.push_back(new frag_info_t);
@@ -118,6 +139,16 @@ void nest_info_t::dump(Formatter *f) const
   f->dump_stream("rctime") << rctime;
 }
 
+void nest_info_t::decode_json(JSONObj *obj){
+
+  JSONDecoder::decode_json("version", version, obj, true);
+  JSONDecoder::decode_json("rbytes", rbytes, obj, true);
+  JSONDecoder::decode_json("rfiles", rfiles, obj, true);
+  JSONDecoder::decode_json("rsubdirs", rsubdirs, obj, true);
+  JSONDecoder::decode_json("rsnaps", rsnaps, obj, true);
+  JSONDecoder::decode_json("rctime", rctime, obj, true);
+}
+
 void nest_info_t::generate_test_instances(std::list<nest_info_t*>& ls)
 {
   ls.push_back(new nest_info_t);
@@ -156,6 +187,12 @@ void quota_info_t::dump(Formatter *f) const
   f->dump_int("max_files", max_files);
 }
 
+void  quota_info_t::decode_json(JSONObj *obj){
+
+  JSONDecoder::decode_json("max_bytes", max_bytes, obj, true);
+  JSONDecoder::decode_json("max_files", max_files, obj, true);
+}
+
 void quota_info_t::generate_test_instances(std::list<quota_info_t *>& ls)
 {
   ls.push_back(new quota_info_t);
@@ -204,6 +241,12 @@ void client_writeable_range_t::dump(Formatter *f) const
   f->dump_unsigned("follows", follows);
 }
 
+void client_writeable_range_t::byte_range_t::decode_json(JSONObj *obj){
+
+  JSONDecoder::decode_json("first", first, obj, true);
+  JSONDecoder::decode_json("last", last, obj, true);
+}
+
 void client_writeable_range_t::generate_test_instances(std::list<client_writeable_range_t*>& ls)
 {
   ls.push_back(new client_writeable_range_t);
@@ -236,9 +279,11 @@ void inline_data_t::decode(bufferlist::const_iterator &p)
   decode(version, p);
   uint32_t inline_len;
   decode(inline_len, p);
-  if (inline_len > 0)
-    decode_nohead(inline_len, get_data(), p);
-  else
+  if (inline_len > 0) {
+    ceph::buffer::list bl;
+    decode_nohead(inline_len, bl, p);
+    set_data(bl);
+  } else
     free_data();
 }
 
@@ -305,7 +350,16 @@ void fnode_t::dump(Formatter *f) const
   accounted_rstat.dump(f);
   f->close_section();
 }
-
+void fnode_t::decode_json(JSONObj *obj){
+  JSONDecoder::decode_json("version", version, obj, true);
+  uint64_t tmp;
+  JSONDecoder::decode_json("snap_purged_thru", tmp, obj, true);
+  snap_purged_thru.val = tmp;
+  JSONDecoder::decode_json("fragstat", fragstat, obj, true);
+  JSONDecoder::decode_json("accounted_fragstat", accounted_fragstat, obj, true);
+  JSONDecoder::decode_json("rstat", rstat, obj, true);
+  JSONDecoder::decode_json("accounted_rstat", accounted_rstat, obj, true);
+}
 void fnode_t::generate_test_instances(std::list<fnode_t*>& ls)
 {
   ls.push_back(new fnode_t);
@@ -378,7 +432,7 @@ feature_bitset_t::feature_bitset_t(unsigned long value)
   }
 }
 
-feature_bitset_t::feature_bitset_t(const vector<size_t>& array)
+void feature_bitset_t::init_array(const vector<size_t>& array)
 {
   if (!array.empty()) {
     size_t n = array.back();
@@ -397,6 +451,26 @@ feature_bitset_t::feature_bitset_t(const vector<size_t>& array)
   }
 }
 
+feature_bitset_t::feature_bitset_t(std::string_view str)
+{
+  std::stringstream ss;
+  std::vector<size_t> v;
+  std::string atom;
+
+  ss << str;
+  while (std::getline(ss, atom, ',')) {
+    v.push_back(std::stoul(atom));
+  }
+  std::sort(v.begin(), v.end());
+
+  init_array(v);
+}
+
+feature_bitset_t::feature_bitset_t(const vector<size_t>& array)
+{
+  init_array(array);
+}
+
 feature_bitset_t& feature_bitset_t::operator-=(const feature_bitset_t& other)
 {
   for (size_t i = 0; i < _vec.size(); ++i) {
@@ -441,10 +515,15 @@ void feature_bitset_t::dump(Formatter *f) const {
 void feature_bitset_t::print(ostream& out) const
 {
   std::ios_base::fmtflags f(out.flags());
-  out << "0x";
-  for (int i = _vec.size() - 1; i >= 0; --i)
-    out << std::setfill('0') << std::setw(sizeof(block_type) * 2)
-        << std::hex << _vec[i];
+  int size = _vec.size();
+  if (!size) {
+    out << "0x0";
+  } else {
+    out << "0x";
+    for (int i = size - 1; i >= 0; --i)
+      out << std::setfill('0') << std::setw(sizeof(block_type) * 2)
+          << std::hex << _vec[i];
+  }
   out.flags(f);
 }
 
@@ -515,7 +594,7 @@ void session_info_t::encode(bufferlist& bl, uint64_t features) const
   encode(inst, bl, features);
   encode(completed_requests, bl);
   encode(prealloc_inos, bl);   // hacky, see below.
-  encode(used_inos, bl);
+  encode((__u32)0, bl); // used_inos
   encode(completed_flushes, bl);
   encode(auth_name, bl);
   encode(client_metadata, bl);
@@ -537,9 +616,11 @@ void session_info_t::decode(bufferlist::const_iterator& p)
     decode(completed_requests, p);
   }
   decode(prealloc_inos, p);
-  decode(used_inos, p);
-  prealloc_inos.insert(used_inos);
-  used_inos.clear();
+  {
+    interval_set<inodeno_t> used_inos;
+    decode(used_inos, p);
+    prealloc_inos.insert(used_inos);
+  }
   if (struct_v >= 4 && struct_v < 7) {
     decode(client_metadata.kv_map, p);
   }
@@ -577,15 +658,6 @@ void session_info_t::dump(Formatter *f) const
   }
   f->close_section();
 
-  f->open_array_section("used_inos");
-  for (const auto& [start, len] : used_inos) {
-    f->open_object_section("ino_range");
-    f->dump_stream("start") << start;
-    f->dump_unsigned("length", len);
-    f->close_section();
-  }
-  f->close_section();
-
   f->dump_object("client_metadata", client_metadata);
 }
 
@@ -722,6 +794,10 @@ void mds_table_pending_t::generate_test_instances(std::list<mds_table_pending_t*
   ls.back()->tid = 35434;
 }
 
+void metareqid_t::dump(ceph::Formatter* f) const {
+  f->dump_object("entity", name);
+  f->dump_unsigned("tid", tid);
+}
 
 /*
  * inode_load_vec_t
@@ -849,7 +925,7 @@ void cap_reconnect_t::encode_old(bufferlist& bl) const {
   encode(path, bl);
   capinfo.flock_len = flockbl.length();
   encode(capinfo, bl);
-  encode_nohead(flockbl, bl);
+  ceph::encode_nohead(flockbl, bl);
 }
 
 void cap_reconnect_t::decode(bufferlist::const_iterator& bl) {
@@ -864,7 +940,7 @@ void cap_reconnect_t::decode_old(bufferlist::const_iterator& bl) {
   using ceph::decode;
   decode(path, bl);
   decode(capinfo, bl);
-  decode_nohead(capinfo.flock_len, flockbl, bl);
+  ceph::decode_nohead(capinfo.flock_len, flockbl, bl);
 }
 
 void cap_reconnect_t::dump(Formatter *f) const
