@@ -23,21 +23,21 @@
 #include <map>
 using std::map;
 
-class MMDSLoadTargets : public PaxosServiceMessage {
+class MMDSLoadTargets final : public PaxosServiceMessage {
 public:
   mds_gid_t global_id;
-  set<mds_rank_t> targets;
+  std::set<mds_rank_t> targets;
 
 protected:
   MMDSLoadTargets() : PaxosServiceMessage(MSG_MDS_OFFLOAD_TARGETS, 0) {}
-  MMDSLoadTargets(mds_gid_t g, set<mds_rank_t>& mds_targets) :
+  MMDSLoadTargets(mds_gid_t g, std::set<mds_rank_t>& mds_targets) :
     PaxosServiceMessage(MSG_MDS_OFFLOAD_TARGETS, 0),
     global_id(g), targets(mds_targets) {}
-  ~MMDSLoadTargets() override {}
+  ~MMDSLoadTargets() final {}
 
 public:
   std::string_view get_type_name() const override { return "mds_load_targets"; }
-  void print(ostream& o) const override {
+  void print(std::ostream& o) const override {
     o << "mds_load_targets(" << global_id << " " << targets << ")";
   }
 
@@ -58,6 +58,8 @@ public:
 private:
   template<class T, typename... Args>
   friend boost::intrusive_ptr<T> ceph::make_message(Args&&... args);
+  template<class T, typename... Args>
+  friend MURef<T> crimson::make_message(Args&&... args);
 };
 
 #endif
