@@ -19,14 +19,14 @@
 
 #include "messages/PaxosServiceMessage.h"
 
-class MOSDAlive : public PaxosServiceMessage {
+class MOSDAlive final : public PaxosServiceMessage {
 public:
   epoch_t want = 0;
 
   MOSDAlive(epoch_t h, epoch_t w) : PaxosServiceMessage{MSG_OSD_ALIVE, h}, want(w) {}
   MOSDAlive() : MOSDAlive{0, 0} {}
 private:
-  ~MOSDAlive() override {}
+  ~MOSDAlive() final {}
 
 public:
   void encode_payload(uint64_t features) override {
@@ -37,11 +37,12 @@ public:
   void decode_payload() override {
     auto p = payload.cbegin();
     paxos_decode(p);
+    using ceph::decode;
     decode(want, p);
   }
 
   std::string_view get_type_name() const override { return "osd_alive"; }
-  void print(ostream &out) const override {
+  void print(std::ostream &out) const override {
     out << "osd_alive(want up_thru " << want << " have " << version << ")";
   }
 private:
