@@ -23,14 +23,14 @@
  * PGCreate - instruct an OSD to create a pg, if it doesn't already exist
  */
 
-class MOSDPGCreate : public Message {
+class MOSDPGCreate final : public Message {
 public:
   static constexpr int HEAD_VERSION = 3;
   static constexpr int COMPAT_VERSION = 3;
 
   version_t          epoch = 0;
-  map<pg_t,pg_create_t> mkpg;
-  map<pg_t,utime_t> ctimes;
+  std::map<pg_t,pg_create_t> mkpg;
+  std::map<pg_t,utime_t> ctimes;
 
   MOSDPGCreate()
     : MOSDPGCreate{0}
@@ -40,9 +40,9 @@ public:
       epoch(e)
   {}
 private:
-  ~MOSDPGCreate() override {}
+  ~MOSDPGCreate() final {}
 
-public:  
+public:
   std::string_view get_type_name() const override { return "pg_create"; }
 
   void encode_payload(uint64_t features) override {
@@ -52,16 +52,15 @@ public:
     encode(ctimes, payload);
   }
   void decode_payload() override {
+    using ceph::decode;
     auto p = payload.cbegin();
     decode(epoch, p);
     decode(mkpg, p);
     decode(ctimes, p);
   }
-  void print(ostream& out) const override {
+  void print(std::ostream& out) const override {
     out << "osd_pg_create(e" << epoch;
-    for (map<pg_t,pg_create_t>::const_iterator i = mkpg.begin();
-         i != mkpg.end();
-         ++i) {
+    for (auto i = mkpg.begin(); i != mkpg.end(); ++i) {
       out << " " << i->first << ":" << i->second.created;
     }
     out << ")";
