@@ -24,9 +24,7 @@
 #include "mon/PaxosService.h"
 #include "mon/MonitorDBStore.h"
 
-class MMonCommand;
-struct MAuth;
-struct MMonGlobalID;
+class MAuth;
 class KeyRing;
 class Monitor;
 
@@ -42,25 +40,12 @@ public:
     IncType inc_type;
     uint64_t max_global_id;
     uint32_t auth_type;
-    bufferlist auth_data;
+    ceph::buffer::list auth_data;
 
     Incremental() : inc_type(GLOBAL_ID), max_global_id(0), auth_type(0) {}
 
-    void encode(bufferlist& bl, uint64_t features=-1) const {
+    void encode(ceph::buffer::list& bl, uint64_t features=-1) const {
       using ceph::encode;
-      if ((features & CEPH_FEATURE_MONENC) == 0) {
-	__u8 v = 1;
-	encode(v, bl);
-	__u32 _type = (__u32)inc_type;
-	encode(_type, bl);
-	if (_type == GLOBAL_ID) {
-	  encode(max_global_id, bl);
-	} else {
-	  encode(auth_type, bl);
-	  encode(auth_data, bl);
-	}
-	return;
-      } 
       ENCODE_START(2, 2, bl);
       __u32 _type = (__u32)inc_type;
       encode(_type, bl);
@@ -72,12 +57,12 @@ public:
       }
       ENCODE_FINISH(bl);
     }
-    void decode(bufferlist::iterator& bl) {
+    void decode(ceph::buffer::list::const_iterator& bl) {
       DECODE_START_LEGACY_COMPAT_LEN(2, 2, 2, bl);
       __u32 _type;
       decode(_type, bl);
       inc_type = (IncType)_type;
-      assert(inc_type >= GLOBAL_ID && inc_type <= AUTH_DATA);
+      ceph_assert(inc_type >= GLOBAL_ID && inc_type <= AUTH_DATA);
       if (_type == GLOBAL_ID) {
 	decode(max_global_id, bl);
       } else {
@@ -86,13 +71,13 @@ public:
       }
       DECODE_FINISH(bl);
     }
-    void dump(Formatter *f) const {
+    void dump(ceph::Formatter *f) const {
       f->dump_int("type", inc_type);
       f->dump_int("max_global_id", max_global_id);
       f->dump_int("auth_type", auth_type);
       f->dump_int("auth_data_len", auth_data.length());
     }
-    static void generate_test_instances(list<Incremental*>& ls) {
+    static void generate_test_instances(std::list<Incremental*>& ls) {
       ls.push_back(new Incremental);
       ls.push_back(new Incremental);
       ls.back()->inc_type = GLOBAL_ID;
@@ -111,11 +96,13 @@ public:
 
 
 private:
-  vector<Incremental> pending_auth;
-  version_t last_rotating_ver;
+  std::vector<Incremental> pending_auth;
   uint64_t max_global_id;
   uint64_t last_allocated_id;
 
+  // these are protected by mon->auth_lock
+  int mon_num = 0, mon_rank = 0;
+
   bool _upgrade_format_to_dumpling();
   bool _upgrade_format_to_luminous();
   bool _upgrade_format_to_mimic();
@@ -133,8 +120,20 @@ private:
   }
 
   /* validate mon/osd/mds caps; fail on unrecognized service/type */
-  bool valid_caps(const string& type, const string& caps, ostream *out);
-  bool valid_caps(const vector<string>& caps, ostream *out);
+  bool valid_caps(const std::string& type, const std::string& caps, std::ostream *out);
+  bool valid_caps(const std::string& type, const ceph::buffer::list& bl, std::ostream *out) {
+    auto p = bl.begin();
+    std::string v;
+    try {
+      using ceph::decode;
+      decode(v, p);
+    } catch (ceph::buffer::error& e) {
+      *out << "corrupt capability encoding";
+      return false;
+    }
+    return valid_caps(type, v, out);
+  }
+  bool valid_caps(const std::vector<std::string>& caps, std::ostream *out);
 
   void on_active() override;
   bool should_propose(double& delay) override;
@@ -144,8 +143,16 @@ private:
   void update_from_paxos(bool *need_bootstrap) override;
   void create_pending() override;  // prepare a new pending
   bool prepare_global_id(MonOpRequestRef op);
+  bool _should_increase_max_global_id(); ///< called under mon->auth_lock
   void increase_max_global_id();
-  uint64_t assign_global_id(MonOpRequestRef op, bool should_increase_max);
+  uint64_t assign_global_id(bool should_increase_max);
+public:
+  uint64_t _assign_global_id(); ///< called under mon->auth_lock
+  void _set_mon_num_rank(int num, int rank); ///< called under mon->auth_lock
+
+private:
+  bool prepare_used_pending_keys(MonOpRequestRef op);
+
   // propose pending update to peers
   void encode_pending(MonitorDBStore::TransactionRef t) override;
   void encode_full(MonitorDBStore::TransactionRef t) override;
@@ -160,27 +167,27 @@ private:
   bool prepare_command(MonOpRequestRef op);
 
   bool check_rotate();
+  void process_used_pending_keys(const std::map<EntityName,CryptoKey>& keys);
 
   bool entity_is_pending(EntityName& entity);
   int exists_and_matches_entity(
       const auth_entity_t& entity,
       bool has_secret,
-      stringstream& ss);
+      std::stringstream& ss);
   int exists_and_matches_entity(
       const EntityName& name,
       const EntityAuth& auth,
-      const map<string,bufferlist>& caps,
+      const std::map<std::string,ceph::buffer::list>& caps,
       bool has_secret,
-      stringstream& ss);
+      std::stringstream& ss);
   int remove_entity(const EntityName &entity);
   int add_entity(
       const EntityName& name,
       const EntityAuth& auth);
 
  public:
-  AuthMonitor(Monitor *mn, Paxos *p, const string& service_name)
+  AuthMonitor(Monitor &mn, Paxos &p, const std::string& service_name)
     : PaxosService(mn, p, service_name),
-      last_rotating_ver(0),
       max_global_id(0),
       last_allocated_id(0)
   {}
@@ -194,7 +201,7 @@ private:
       const uuid_d& uuid,
       EntityName& cephx_entity,
       EntityName& lockbox_entity,
-      stringstream& ss);
+      std::stringstream& ss);
   int do_osd_destroy(
       const EntityName& cephx_entity,
       const EntityName& lockbox_entity);
@@ -206,15 +213,15 @@ private:
   int validate_osd_new(
       int32_t id,
       const uuid_d& uuid,
-      const string& cephx_secret,
-      const string& lockbox_secret,
+      const std::string& cephx_secret,
+      const std::string& lockbox_secret,
       auth_entity_t& cephx_entity,
       auth_entity_t& lockbox_entity,
-      stringstream& ss);
+      std::stringstream& ss);
 
-  void dump_info(Formatter *f);
+  void dump_info(ceph::Formatter *f);
 
-  bool is_valid_cephx_key(const string& k) {
+  bool is_valid_cephx_key(const std::string& k) {
     if (k.empty())
       return false;
 
@@ -222,7 +229,7 @@ private:
     try {
       ea.key.decode_base64(k);
       return true;
-    } catch (buffer::error& e) { /* fallthrough */ }
+    } catch (ceph::buffer::error& e) { /* fallthrough */ }
     return false;
   }
 };
