@@ -9,6 +9,7 @@
 #include "include/common_fwd.h"
 #include "include/types.h"
 #include "common/entity_name.h"
+#include "mds/mdstypes.h"
 
 static const __u8 MON_CAP_R     = (1 << 1);      // read
 static const __u8 MON_CAP_W     = (1 << 2);      // write
@@ -53,7 +54,7 @@ std::ostream& operator<<(std::ostream& out, const StringConstraint& c);
 
 struct MonCapGrant {
   /*
-   * A grant can come in one of four forms:
+   * A grant can come in one of five forms:
    *
    *  - a blanket allow ('allow rw', 'allow *')
    *    - this will match against any service and the read/write/exec flags
@@ -72,11 +73,16 @@ struct MonCapGrant {
    *      of key/value pairs that constrain use of that command.  if no pairs
    *      are specified, any arguments are allowed; if a pair is specified, that
    *      argument must be present and equal or match a prefix.
+   *
+   *  - an fs name ('allow fsname foo')
+   *    - this will restrict access to MDSMaps in the FSMap to the provided
+   *      fs name.
    */
   std::string service;
   std::string profile;
   std::string command;
   std::map<std::string, StringConstraint> command_args;
+  std::string fs_name;
 
   // restrict by network
   std::string network;
@@ -105,6 +111,7 @@ struct MonCapGrant {
   MonCapGrant(std::string c, std::string a, StringConstraint co) : command(std::move(c)) {
     command_args[a] = co;
   }
+  MonCapGrant(mon_rwxa_t a, std::string fsname) : fs_name(fsname), allow(a) {}
 
   /**
    * check if given request parameters match our constraints
@@ -127,7 +134,8 @@ struct MonCapGrant {
       allow == MON_CAP_ANY &&
       service.length() == 0 &&
       profile.length() == 0 &&
-      command.length() == 0;
+      command.length() == 0 &&
+      fs_name.empty();
   }
 };
 
@@ -174,6 +182,43 @@ struct MonCap {
   void decode(ceph::buffer::list::const_iterator& bl);
   void dump(ceph::Formatter *f) const;
   static void generate_test_instances(std::list<MonCap*>& ls);
+
+  std::vector<std::string> allowed_fs_names() const {
+    std::vector<std::string> ret;
+    for (auto& g : grants) {
+      if (not g.fs_name.empty()) {
+	ret.push_back(g.fs_name);
+      } else {
+	return {};
+      }
+    }
+    return ret;
+  }
+
+  bool fs_name_capable(const EntityName& ename, std::string_view fs_name,
+		       __u8 mask) {
+    for (auto& g : grants) {
+      if (g.is_allow_all()) {
+	return true;
+      }
+
+      if ((g.fs_name.empty() || g.fs_name == fs_name) && (mask & g.allow)) {
+	  return true;
+      }
+
+      g.expand_profile(ename);
+      for (auto& pg : g.profile_grants) {
+	if ((pg.service == "fs" || pg.service == "mds") &&
+	    (pg.fs_name.empty() || pg.fs_name == fs_name) &&
+	    (pg.allow & mask)) {
+	  return true;
+	}
+      }
+    }
+
+    return false;
+  }
+
 };
 WRITE_CLASS_ENCODER(MonCap)
 
