@@ -24,36 +24,23 @@
 #include "common/bit_str.h"
 #include "common/ceph_releases.h"
 
-#define PAXOS_MDSMAP     0
-#define PAXOS_OSDMAP     1
-#define PAXOS_LOG        2
-#define PAXOS_MONMAP     3
-#define PAXOS_AUTH       4
-#define PAXOS_MGR        5
-#define PAXOS_MGRSTAT    6
-#define PAXOS_HEALTH     7
-#define PAXOS_CONFIG     8
-#define PAXOS_NUM        9
-
-inline const char *get_paxos_name(int p) {
-  switch (p) {
-  case PAXOS_MDSMAP: return "mdsmap";
-  case PAXOS_MONMAP: return "monmap";
-  case PAXOS_OSDMAP: return "osdmap";
-  case PAXOS_LOG: return "logm";
-  case PAXOS_AUTH: return "auth";
-  case PAXOS_MGR: return "mgr";
-  case PAXOS_MGRSTAT: return "mgrstat";
-  case PAXOS_HEALTH: return "health";
-  case PAXOS_CONFIG: return "config";
-  default: ceph_abort(); return 0;
-  }
-}
+// use as paxos_service index
+enum {
+  PAXOS_MDSMAP,
+  PAXOS_OSDMAP,
+  PAXOS_LOG,
+  PAXOS_MONMAP,
+  PAXOS_AUTH,
+  PAXOS_MGR,
+  PAXOS_MGRSTAT,
+  PAXOS_HEALTH,
+  PAXOS_CONFIG,
+  PAXOS_KV,
+  PAXOS_NUM
+};
 
 #define CEPH_MON_ONDISK_MAGIC "ceph mon volume v012"
 
-extern const std::string CONFIG_PREFIX;
-
 // map of entity_type -> features -> count
 struct FeatureMap {
   std::map<uint32_t,std::map<uint64_t,uint64_t>> m;
@@ -497,6 +484,10 @@ namespace ceph {
       constexpr mon_feature_t FEATURE_OSDMAP_PRUNE (1ULL << 3);
       constexpr mon_feature_t FEATURE_NAUTILUS(    (1ULL << 4));
       constexpr mon_feature_t FEATURE_OCTOPUS(    (1ULL << 5));
+      constexpr mon_feature_t FEATURE_PACIFIC(    (1ULL << 6));
+      // elector pinging and CONNECTIVITY mode:
+      constexpr mon_feature_t FEATURE_PINGING(    (1ULL << 7));
+      constexpr mon_feature_t FEATURE_QUINCY(    (1ULL << 8));
 
       constexpr mon_feature_t FEATURE_RESERVED(   (1ULL << 63));
       constexpr mon_feature_t FEATURE_NONE(       (0ULL));
@@ -514,6 +505,9 @@ namespace ceph {
           FEATURE_OSDMAP_PRUNE |
 	  FEATURE_NAUTILUS |
 	  FEATURE_OCTOPUS |
+	  FEATURE_PACIFIC |
+	  FEATURE_PINGING |
+	  FEATURE_QUINCY |
 	  FEATURE_NONE
 	  );
       }
@@ -535,6 +529,9 @@ namespace ceph {
 	  FEATURE_NAUTILUS |
 	  FEATURE_OSDMAP_PRUNE |
 	  FEATURE_OCTOPUS |
+	  FEATURE_PACIFIC |
+	  FEATURE_PINGING |
+	  FEATURE_QUINCY |
 	  FEATURE_NONE
 	  );
       }
@@ -553,6 +550,12 @@ namespace ceph {
 
 static inline ceph_release_t infer_ceph_release_from_mon_features(mon_feature_t f)
 {
+  if (f.contains_all(ceph::features::mon::FEATURE_QUINCY)) {
+    return ceph_release_t::quincy;
+  }
+  if (f.contains_all(ceph::features::mon::FEATURE_PACIFIC)) {
+    return ceph_release_t::pacific;
+  }
   if (f.contains_all(ceph::features::mon::FEATURE_OCTOPUS)) {
     return ceph_release_t::octopus;
   }
@@ -584,8 +587,14 @@ static inline const char *ceph::features::mon::get_feature_name(uint64_t b) {
     return "osdmap-prune";
   } else if (f == FEATURE_NAUTILUS) {
     return "nautilus";
+  } else if (f == FEATURE_PINGING) {
+    return "elector-pinging";
   } else if (f == FEATURE_OCTOPUS) {
     return "octopus";
+  } else if (f == FEATURE_PACIFIC) {
+    return "pacific";
+  } else if (f == FEATURE_QUINCY) {
+    return "quincy";
   } else if (f == FEATURE_RESERVED) {
     return "reserved";
   }
@@ -604,8 +613,14 @@ inline mon_feature_t ceph::features::mon::get_feature_by_name(const std::string
     return FEATURE_OSDMAP_PRUNE;
   } else if (n == "nautilus") {
     return FEATURE_NAUTILUS;
+  } else if (n == "feature-pinging") {
+    return FEATURE_PINGING;
   } else if (n == "octopus") {
     return FEATURE_OCTOPUS;
+  } else if (n == "pacific") {
+    return FEATURE_PACIFIC;
+  } else if (n == "quincy") {
+    return FEATURE_QUINCY;
   } else if (n == "reserved") {
     return FEATURE_RESERVED;
   }
@@ -623,22 +638,31 @@ inline std::ostream& operator<<(std::ostream& out, const mon_feature_t& f) {
 struct ProgressEvent {
   std::string message;                  ///< event description
   float progress;                  ///< [0..1]
-
+  bool add_to_ceph_s;
   void encode(ceph::buffer::list& bl) const {
-    ENCODE_START(1, 1, bl);
+    ENCODE_START(2, 1, bl);
     encode(message, bl);
     encode(progress, bl);
+    encode(add_to_ceph_s, bl);
     ENCODE_FINISH(bl);
   }
   void decode(ceph::buffer::list::const_iterator& p) {
-    DECODE_START(1, p);
+    DECODE_START(2, p);
     decode(message, p);
     decode(progress, p);
+    if (struct_v >= 2){
+	decode(add_to_ceph_s, p);
+    } else {
+      if (!message.empty()) {
+	add_to_ceph_s = true;
+      }
+    }
     DECODE_FINISH(p);
   }
   void dump(ceph::Formatter *f) const {
     f->dump_string("message", message);
     f->dump_float("progress", progress);
+    f->dump_bool("add_to_ceph_s", add_to_ceph_s);
   }
 };
 WRITE_CLASS_ENCODER(ProgressEvent)
