@@ -32,7 +32,7 @@ template <typename InetTraits>
 class tcp;
 
 int tcpv4_listen(tcp<ipv4_traits>& tcpv4, uint16_t port, const SocketOptions &opts,
-                 ServerSocket *sa);
+                 int type, unsigned addr_slot, ServerSocket *sa);
 
 int tcpv4_connect(tcp<ipv4_traits>& tcpv4, const entity_addr_t &addr,
                   ConnectedSocket *sa);
