@@ -19,7 +19,7 @@
 #include "include/encoding.h"
 #include "LFNIndex.h"
 
-extern string reverse_hexdigit_bits_string(string l);
+extern std::string reverse_hexdigit_bits_string(std::string l);
 
 /**
  * Implements collection prehashing.
@@ -51,11 +51,11 @@ extern string reverse_hexdigit_bits_string(string l);
 class HashIndex : public LFNIndex {
 private:
   /// Attribute name for storing subdir info @see subdir_info_s
-  static const string SUBDIR_ATTR;
+  static const std::string SUBDIR_ATTR;
   /// Attribute name for storing index-wide settings
-  static const string SETTINGS_ATTR;
+  static const std::string SETTINGS_ATTR;
   /// Attribute name for storing in progress op tag
-  static const string IN_PROGRESS_OP_TAG;
+  static const std::string IN_PROGRESS_OP_TAG;
   /// Size (bits) in object hash
   static const int PATH_HASH_LEN = 32;
   /// Max length of hashed path
@@ -82,7 +82,7 @@ private:
 
     subdir_info_s() : objs(0), subdirs(0), hash_level(0) {}
 
-    void encode(bufferlist &bl) const
+    void encode(ceph::buffer::list &bl) const
     {
       using ceph::encode;
       __u8 v = 1;
@@ -92,7 +92,7 @@ private:
       encode(hash_level, bl);
     }
 
-    void decode(bufferlist::iterator &bl)
+    void decode(ceph::buffer::list::const_iterator &bl)
     {
       using ceph::decode;
       __u8 v;
@@ -107,14 +107,14 @@ private:
   struct settings_s {
     uint32_t split_rand_factor; ///< random factor added to split threshold (only on root of collection)
     settings_s() : split_rand_factor(0) {}
-    void encode(bufferlist &bl) const
+    void encode(ceph::buffer::list &bl) const
     {
       using ceph::encode;
       __u8 v = 1;
       encode(v, bl);
       encode(split_rand_factor, bl);
     }
-    void decode(bufferlist::iterator &bl)
+    void decode(ceph::buffer::list::const_iterator &bl)
     {
       using ceph::decode;
       __u8 v;
@@ -129,12 +129,12 @@ private:
     static const int MERGE = 1;
     static const int COL_SPLIT = 2;
     int op;
-    vector<string> path;
+    std::vector<std::string> path;
 
-    InProgressOp(int op, const vector<string> &path)
+    InProgressOp(int op, const std::vector<std::string> &path)
       : op(op), path(path) {}
 
-    explicit InProgressOp(bufferlist::iterator &bl) {
+    explicit InProgressOp(ceph::buffer::list::const_iterator &bl) {
       decode(bl);
     }
 
@@ -142,7 +142,7 @@ private:
     bool is_col_split() const { return op == COL_SPLIT; }
     bool is_merge() const { return op == MERGE; }
 
-    void encode(bufferlist &bl) const {
+    void encode(ceph::buffer::list &bl) const {
       using ceph::encode;
       __u8 v = 1;
       encode(v, bl);
@@ -150,7 +150,7 @@ private:
       encode(path, bl);
     }
 
-    void decode(bufferlist::iterator &bl) {
+    void decode(ceph::buffer::list::const_iterator &bl) {
       using ceph::decode;
       __u8 v;
       decode(v, bl);
@@ -167,8 +167,8 @@ public:
     CephContext* cct,
     coll_t collection,     ///< [in] Collection
     const char *base_path, ///< [in] Path to the index root.
-    int merge_at,          ///< [in] Merge threshhold.
-    int split_multiple,	   ///< [in] Split threshhold.
+    int merge_at,          ///< [in] Merge threshold.
+    int split_multiple,	   ///< [in] Split threshold.
     uint32_t index_version,///< [in] Index version
     double retry_probability=0) ///< [in] retry probability
     : LFNIndex(cct, collection, base_path, index_version, retry_probability),
@@ -194,6 +194,17 @@ public:
     CollectionIndex* dest
     ) override;
 
+  /// @see CollectionIndex
+  int _merge(
+    uint32_t bits,
+    CollectionIndex* dest
+    ) override;
+
+  int _merge_dirs(
+    HashIndex& from,
+    HashIndex& to,
+    const std::vector<std::string>& path);
+
   /// @see CollectionIndex
   int apply_layout_settings(int target_level) override;
 
@@ -201,19 +212,19 @@ protected:
   int _init() override;
 
   int _created(
-    const vector<string> &path,
+    const std::vector<std::string> &path,
     const ghobject_t &oid,
-    const string &mangled_name
+    const std::string &mangled_name
     ) override;
   int _remove(
-    const vector<string> &path,
+    const std::vector<std::string> &path,
     const ghobject_t &oid,
-    const string &mangled_name
+    const std::string &mangled_name
     ) override;
   int _lookup(
     const ghobject_t &oid,
-    vector<string> *path,
-    string *mangled_name,
+    std::vector<std::string> *path,
+    std::string *mangled_name,
     int *hardlink
     ) override;
 
@@ -230,44 +241,44 @@ protected:
     const ghobject_t &start,
     const ghobject_t &end,
     int max_count,
-    vector<ghobject_t> *ls,
+    std::vector<ghobject_t> *ls,
     ghobject_t *next
     ) override;
 private:
   /// Internal recursively remove path and its subdirs
   int _recursive_remove(
-    const vector<string> &path, ///< [in] path to remove
+    const std::vector<std::string> &path, ///< [in] path to remove
     bool top			///< [in] internal tracking of first caller
     ); /// @return Error Code, 0 on success
   /// Recursively remove path and its subdirs
   int recursive_remove(
-    const vector<string> &path ///< [in] path to remove
+    const std::vector<std::string> &path ///< [in] path to remove
     ); /// @return Error Code, 0 on success
   /// Tag root directory at beginning of col_split
   int start_col_split(
-    const vector<string> &path ///< [in] path to split
+    const std::vector<std::string> &path ///< [in] path to split
     ); ///< @return Error Code, 0 on success
   /// Tag root directory at beginning of split
   int start_split(
-    const vector<string> &path ///< [in] path to split
+    const std::vector<std::string> &path ///< [in] path to split
     ); ///< @return Error Code, 0 on success
   /// Tag root directory at beginning of split
   int start_merge(
-    const vector<string> &path ///< [in] path to merge
+    const std::vector<std::string> &path ///< [in] path to merge
     ); ///< @return Error Code, 0 on success
   /// Remove tag at end of split or merge
   int end_split_or_merge(
-    const vector<string> &path ///< [in] path to split or merged
+    const std::vector<std::string> &path ///< [in] path to split or merged
     ); ///< @return Error Code, 0 on success
   /// Gets info from the xattr on the subdir represented by path
   int get_info(
-    const vector<string> &path, ///< [in] Path from which to read attribute.
+    const std::vector<std::string> &path, ///< [in] Path from which to read attribute.
     subdir_info_s *info		///< [out] Attribute value
     ); /// @return Error Code, 0 on success
 
   /// Sets info to the xattr on the subdir represented by path
   int set_info(
-    const vector<string> &path, ///< [in] Path on which to set attribute.
+    const std::vector<std::string> &path, ///< [in] Path on which to set attribute.
     const subdir_info_s &info  	///< [in] Value to set
     ); /// @return Error Code, 0 on success
 
@@ -284,37 +295,37 @@ private:
 
   /// Initiates merge
   int initiate_merge(
-    const vector<string> &path, ///< [in] Subdir to merge
+    const std::vector<std::string> &path, ///< [in] Subdir to merge
     subdir_info_s info		///< [in] Info attached to path
     ); /// @return Error Code, 0 on success
 
   /// Completes merge
   int complete_merge(
-    const vector<string> &path, ///< [in] Subdir to merge
+    const std::vector<std::string> &path, ///< [in] Subdir to merge
     subdir_info_s info		///< [in] Info attached to path
     ); /// @return Error Code, 0 on success
 
   /// Resets attr to match actual subdir contents
   int reset_attr(
-    const vector<string> &path ///< [in] path to cleanup
+    const std::vector<std::string> &path ///< [in] path to cleanup
     );
 
   /// Initiate Split
   int initiate_split(
-    const vector<string> &path, ///< [in] Subdir to split
+    const std::vector<std::string> &path, ///< [in] Subdir to split
     subdir_info_s info		///< [in] Info attached to path
     ); /// @return Error Code, 0 on success
 
   /// Completes Split
   int complete_split(
-    const vector<string> &path, ///< [in] Subdir to split
+    const std::vector<std::string> &path, ///< [in] Subdir to split
     subdir_info_s info	       ///< [in] Info attached to path
     ); /// @return Error Code, 0 on success
 
   /// Determine path components from hoid hash
   void get_path_components(
     const ghobject_t &oid, ///< [in] Object for which to get path components
-    vector<string> *path   ///< [out] Path components for hoid.
+    std::vector<std::string> *path   ///< [out] Path components for hoid.
     );
 
   /// Pre-hash and split folders to avoid runtime splitting
@@ -323,13 +334,13 @@ private:
 
   /// Initialize the folder (dir info) with the given hash
   /// level and number of its subdirs.
-  int init_split_folder(vector<string> &path, uint32_t hash_level);
+  int init_split_folder(std::vector<std::string> &path, uint32_t hash_level);
 
   /// do collection split for path
   static int col_split_level(
     HashIndex &from,            ///< [in] from index
     HashIndex &dest,            ///< [in] to index
-    const vector<string> &path, ///< [in] path to split
+    const std::vector<std::string> &path, ///< [in] path to split
     uint32_t bits,              ///< [in] num bits to match
     uint32_t match,             ///< [in] bits to match
     unsigned *mkdirred          ///< [in,out] path[:mkdirred] has been mkdirred
@@ -337,34 +348,32 @@ private:
 
 
   /**
-   * Get string representation of ghobject_t/hash
+   * Get std::string representation of ghobject_t/hash
    *
    * e.g: 0x01234567 -> "76543210"
    */
-  static string get_path_str(
-    const ghobject_t &oid ///< [in] Object to get hash string for
-    ); ///< @return Hash string for hoid.
+  static std::string get_path_str(
+    const ghobject_t &oid ///< [in] Object to get hash std::string for
+    ); ///< @return Hash std::string for hoid.
 
-  /// Get string from hash, @see get_path_str
-  static string get_hash_str(
+  /// Get std::string from hash, @see get_path_str
+  static std::string get_hash_str(
     uint32_t hash ///< [in] Hash to convert to a string.
-    ); ///< @return String representation of hash
+    ); ///< @return std::string representation of hash
 
-  /// Get hash from hash prefix string e.g. "FFFFAB" -> 0xFFFFAB00
+  /// Get hash from hash prefix std::string e.g. "FFFFAB" -> 0xFFFFAB00
   static uint32_t hash_prefix_to_hash(
-    string prefix ///< [in] string to convert
+    std::string prefix ///< [in] std::string to convert
     ); ///< @return Hash
 
   /// Get hash mod from path
   static void path_to_hobject_hash_prefix(
-    const vector<string> &path,///< [in] path to convert
+    const std::vector<std::string> &path,///< [in] path to convert
     uint32_t *bits,            ///< [out] bits
     uint32_t *hash             ///< [out] hash
     ) {
-    string hash_str;
-    for (vector<string>::const_iterator i = path.begin();
-	 i != path.end();
-	 ++i) {
+    std::string hash_str;
+    for (auto i = path.begin(); i != path.end(); ++i) {
       hash_str.push_back(*i->begin());
     }
     uint32_t rev_hash = hash_prefix_to_hash(hash_str);
@@ -384,18 +393,18 @@ private:
     return ret;
   }
 
-  /// Convert a number to hex string (upper case).
-  static string to_hex(int n) {
+  /// Convert a number to hex std::string (upper case).
+  static std::string to_hex(int n) {
     ceph_assert(n >= 0 && n < 16);
     char c = (n <= 9 ? ('0' + n) : ('A' + n - 10));
-    string str;
+    std::string str;
     str.append(1, c);
     return str;
   }
 
   struct CmpPairBitwise {
-    bool operator()(const pair<string, ghobject_t>& l,
-		    const pair<string, ghobject_t>& r) const
+    bool operator()(const std::pair<std::string, ghobject_t>& l,
+		    const std::pair<std::string, ghobject_t>& r) const
     {
       if (l.first < r.first)
 	return true;
@@ -408,42 +417,42 @@ private:
   };
 
   struct CmpHexdigitStringBitwise {
-    bool operator()(const string& l, const string& r) const {
+    bool operator()(const std::string& l, const std::string& r) const {
       return reverse_hexdigit_bits_string(l) < reverse_hexdigit_bits_string(r);
     }
   };
 
   /// Get path contents by hash
   int get_path_contents_by_hash_bitwise(
-    const vector<string> &path,             /// [in] Path to list
+    const std::vector<std::string> &path,             /// [in] Path to list
     const ghobject_t *next_object,          /// [in] list > *next_object
-    set<string, CmpHexdigitStringBitwise> *hash_prefixes, /// [out] prefixes in dir
-    set<pair<string, ghobject_t>, CmpPairBitwise> *objects /// [out] objects
+    std::set<std::string, CmpHexdigitStringBitwise> *hash_prefixes, /// [out] prefixes in dir
+    std::set<std::pair<std::string, ghobject_t>, CmpPairBitwise> *objects /// [out] objects
     );
 
   /// List objects in collection in ghobject_t order
   int list_by_hash(
-    const vector<string> &path, /// [in] Path to list
+    const std::vector<std::string> &path, /// [in] Path to list
     const ghobject_t &end,      /// [in] List only objects < end
     int max_count,              /// [in] List at most max_count
     ghobject_t *next,            /// [in,out] List objects >= *next
-    vector<ghobject_t> *out      /// [out] Listed objects
+    std::vector<ghobject_t> *out      /// [out] Listed objects
     ); ///< @return Error Code, 0 on success
   /// List objects in collection in ghobject_t order
   int list_by_hash_bitwise(
-    const vector<string> &path, /// [in] Path to list
+    const std::vector<std::string> &path, /// [in] Path to list
     const ghobject_t &end,      /// [in] List only objects < end
     int max_count,              /// [in] List at most max_count
     ghobject_t *next,            /// [in,out] List objects >= *next
-    vector<ghobject_t> *out      /// [out] Listed objects
+    std::vector<ghobject_t> *out      /// [out] Listed objects
     ); ///< @return Error Code, 0 on success
 
   /// Create the given levels of sub directories from the given root.
   /// The contents of *path* is not changed after calling this function.
-  int recursive_create_path(vector<string>& path, int level);
+  int recursive_create_path(std::vector<std::string>& path, int level);
 
   /// split each dir below the given path
-  int split_dirs(const vector<string> &path, int target_level = 0);
+  int split_dirs(const std::vector<std::string> &path, int target_level = 0);
 
   int write_settings();
 };
