@@ -33,6 +33,9 @@
 #undef dout_prefix
 #define dout_prefix *_dout << "memstore(" << path << ") "
 
+using ceph::decode;
+using ceph::encode;
+
 // for comparing collections for lock ordering
 bool operator>(const MemStore::CollectionRef& l,
 	       const MemStore::CollectionRef& r)
@@ -61,23 +64,21 @@ int MemStore::_save()
 {
   dout(10) << __func__ << dendl;
   dump_all();
-  set<coll_t> collections;
-  for (ceph::unordered_map<coll_t,CollectionRef>::iterator p = coll_map.begin();
-       p != coll_map.end();
-       ++p) {
+  std::set<coll_t> collections;
+  for (auto p = coll_map.begin(); p != coll_map.end(); ++p) {
     dout(20) << __func__ << " coll " << p->first << " " << p->second << dendl;
     collections.insert(p->first);
-    bufferlist bl;
+    ceph::buffer::list bl;
     ceph_assert(p->second);
     p->second->encode(bl);
-    string fn = path + "/" + stringify(p->first);
+    std::string fn = path + "/" + stringify(p->first);
     int r = bl.write_file(fn.c_str());
     if (r < 0)
       return r;
   }
 
-  string fn = path + "/collections";
-  bufferlist bl;
+  std::string fn = path + "/collections";
+  ceph::buffer::list bl;
   encode(collections, bl);
   int r = bl.write_file(fn.c_str());
   if (r < 0)
@@ -88,7 +89,7 @@ int MemStore::_save()
 
 void MemStore::dump_all()
 {
-  Formatter *f = Formatter::create("json-pretty");
+  auto f = ceph::Formatter::create("json-pretty");
   f->open_object_section("store");
   dump(f);
   f->close_section();
@@ -98,17 +99,15 @@ void MemStore::dump_all()
   delete f;
 }
 
-void MemStore::dump(Formatter *f)
+void MemStore::dump(ceph::Formatter *f)
 {
   f->open_array_section("collections");
-  for (ceph::unordered_map<coll_t,CollectionRef>::iterator p = coll_map.begin();
-       p != coll_map.end();
-       ++p) {
+  for (auto p = coll_map.begin(); p != coll_map.end(); ++p) {
     f->open_object_section("collection");
     f->dump_string("name", stringify(p->first));
 
     f->open_array_section("xattrs");
-    for (map<string,bufferptr>::iterator q = p->second->xattr.begin();
+    for (auto q = p->second->xattr.begin();
 	 q != p->second->xattr.end();
 	 ++q) {
       f->open_object_section("xattr");
@@ -119,7 +118,7 @@ void MemStore::dump(Formatter *f)
     f->close_section();
 
     f->open_array_section("objects");
-    for (map<ghobject_t,ObjectRef>::iterator q = p->second->object_map.begin();
+    for (auto q = p->second->object_map.begin();
 	 q != p->second->object_map.end();
 	 ++q) {
       f->open_object_section("object");
@@ -138,22 +137,22 @@ void MemStore::dump(Formatter *f)
 int MemStore::_load()
 {
   dout(10) << __func__ << dendl;
-  bufferlist bl;
-  string fn = path + "/collections";
-  string err;
+  ceph::buffer::list bl;
+  std::string fn = path + "/collections";
+  std::string err;
   int r = bl.read_file(fn.c_str(), &err);
   if (r < 0)
     return r;
 
-  set<coll_t> collections;
+  std::set<coll_t> collections;
   auto p = bl.cbegin();
   decode(collections, p);
 
-  for (set<coll_t>::iterator q = collections.begin();
+  for (auto q = collections.begin();
        q != collections.end();
        ++q) {
-    string fn = path + "/" + stringify(*q);
-    bufferlist cbl;
+    std::string fn = path + "/" + stringify(*q);
+    ceph::buffer::list cbl;
     int r = cbl.read_file(fn.c_str(), &err);
     if (r < 0)
       return r;
@@ -177,7 +176,7 @@ void MemStore::set_fsid(uuid_d u)
 
 uuid_d MemStore::get_fsid()
 {
-  string fsid_str;
+  std::string fsid_str;
   int r = read_meta("fsid", &fsid_str);
   ceph_assert(r >= 0);
   uuid_d uuid;
@@ -188,7 +187,7 @@ uuid_d MemStore::get_fsid()
 
 int MemStore::mkfs()
 {
-  string fsid_str;
+  std::string fsid_str;
   int r = read_meta("fsid", &fsid_str);
   if (r == -ENOENT) {
     uuid_d fsid;
@@ -204,10 +203,10 @@ int MemStore::mkfs()
     dout(1) << __func__ << " had fsid " << fsid_str << dendl;
   }
 
-  string fn = path + "/collections";
+  std::string fn = path + "/collections";
   derr << path << dendl;
-  bufferlist bl;
-  set<coll_t> collections;
+  ceph::buffer::list bl;
+  std::set<coll_t> collections;
   encode(collections, bl);
   r = bl.write_file(fn.c_str());
   if (r < 0)
@@ -311,7 +310,7 @@ int MemStore::read(
   const ghobject_t& oid,
   uint64_t offset,
   size_t len,
-  bufferlist& bl,
+  ceph::buffer::list& bl,
   uint32_t op_flags)
 {
   Collection *c = static_cast<Collection*>(c_.get());
@@ -334,9 +333,9 @@ int MemStore::read(
 }
 
 int MemStore::fiemap(CollectionHandle& ch, const ghobject_t& oid,
-		     uint64_t offset, size_t len, bufferlist& bl)
+		     uint64_t offset, size_t len, ceph::buffer::list& bl)
 {
-  map<uint64_t, uint64_t> destmap;
+  std::map<uint64_t, uint64_t> destmap;
   int r = fiemap(ch, oid, offset, len, destmap);
   if (r >= 0)
     encode(destmap, bl);
@@ -344,7 +343,7 @@ int MemStore::fiemap(CollectionHandle& ch, const ghobject_t& oid,
 }
 
 int MemStore::fiemap(CollectionHandle& ch, const ghobject_t& oid,
-		     uint64_t offset, size_t len, map<uint64_t, uint64_t>& destmap)
+		     uint64_t offset, size_t len, std::map<uint64_t, uint64_t>& destmap)
 {
   dout(10) << __func__ << " " << ch->cid << " " << oid << " " << offset << "~"
 	   << len << dendl;
@@ -366,7 +365,7 @@ int MemStore::fiemap(CollectionHandle& ch, const ghobject_t& oid,
 }
 
 int MemStore::getattr(CollectionHandle &c_, const ghobject_t& oid,
-		      const char *name, bufferptr& value)
+		      const char *name, ceph::buffer::ptr& value)
 {
   Collection *c = static_cast<Collection*>(c_.get());
   dout(10) << __func__ << " " << c->cid << " " << oid << " " << name << dendl;
@@ -375,7 +374,7 @@ int MemStore::getattr(CollectionHandle &c_, const ghobject_t& oid,
   ObjectRef o = c->get_object(oid);
   if (!o)
     return -ENOENT;
-  string k(name);
+  std::string k(name);
   std::lock_guard lock{o->xattr_mutex};
   if (!o->xattr.count(k)) {
     return -ENODATA;
@@ -385,7 +384,7 @@ int MemStore::getattr(CollectionHandle &c_, const ghobject_t& oid,
 }
 
 int MemStore::getattrs(CollectionHandle &c_, const ghobject_t& oid,
-		       map<string,bufferptr>& aset)
+		       std::map<std::string,ceph::buffer::ptr,std::less<>>& aset)
 {
   Collection *c = static_cast<Collection*>(c_.get());
   dout(10) << __func__ << " " << c->cid << " " << oid << dendl;
@@ -400,7 +399,7 @@ int MemStore::getattrs(CollectionHandle &c_, const ghobject_t& oid,
   return 0;
 }
 
-int MemStore::list_collections(vector<coll_t>& ls)
+int MemStore::list_collections(std::vector<coll_t>& ls)
 {
   dout(10) << __func__ << dendl;
   std::shared_lock l{coll_lock};
@@ -440,14 +439,14 @@ int MemStore::collection_list(CollectionHandle& ch,
 			      const ghobject_t& start,
 			      const ghobject_t& end,
 			      int max,
-			      vector<ghobject_t> *ls, ghobject_t *next)
+			      std::vector<ghobject_t> *ls, ghobject_t *next)
 {
   Collection *c = static_cast<Collection*>(ch.get());
   std::shared_lock l{c->lock};
 
   dout(10) << __func__ << " cid " << ch->cid << " start " << start
 	   << " end " << end << dendl;
-  map<ghobject_t,ObjectRef>::iterator p = c->object_map.lower_bound(start);
+  auto p = c->object_map.lower_bound(start);
   while (p != c->object_map.end() &&
 	 ls->size() < (unsigned)max &&
 	 p->first < end) {
@@ -467,8 +466,8 @@ int MemStore::collection_list(CollectionHandle& ch,
 int MemStore::omap_get(
   CollectionHandle& ch,                ///< [in] Collection containing oid
   const ghobject_t &oid,   ///< [in] Object containing omap
-  bufferlist *header,      ///< [out] omap header
-  map<string, bufferlist> *out /// < [out] Key to value map
+  ceph::buffer::list *header,      ///< [out] omap header
+  std::map<std::string, ceph::buffer::list> *out /// < [out] Key to value map
   )
 {
   dout(10) << __func__ << " " << ch->cid << " " << oid << dendl;
@@ -486,7 +485,7 @@ int MemStore::omap_get(
 int MemStore::omap_get_header(
   CollectionHandle& ch,                ///< [in] Collection containing oid
   const ghobject_t &oid,   ///< [in] Object containing omap
-  bufferlist *header,      ///< [out] omap header
+  ceph::buffer::list *header,      ///< [out] omap header
   bool allow_eio ///< [in] don't assert on eio
   )
 {
@@ -503,7 +502,7 @@ int MemStore::omap_get_header(
 int MemStore::omap_get_keys(
   CollectionHandle& ch,              ///< [in] Collection containing oid
   const ghobject_t &oid, ///< [in] Object containing omap
-  set<string> *keys      ///< [out] Keys defined on oid
+  std::set<std::string> *keys      ///< [out] Keys defined on oid
   )
 {
   dout(10) << __func__ << " " << ch->cid << " " << oid << dendl;
@@ -512,9 +511,7 @@ int MemStore::omap_get_keys(
   if (!o)
     return -ENOENT;
   std::lock_guard lock{o->omap_mutex};
-  for (map<string,bufferlist>::iterator p = o->omap.begin();
-       p != o->omap.end();
-       ++p)
+  for (auto p = o->omap.begin(); p != o->omap.end(); ++p)
     keys->insert(p->first);
   return 0;
 }
@@ -522,8 +519,8 @@ int MemStore::omap_get_keys(
 int MemStore::omap_get_values(
   CollectionHandle& ch,                    ///< [in] Collection containing oid
   const ghobject_t &oid,       ///< [in] Object containing omap
-  const set<string> &keys,     ///< [in] Keys to get
-  map<string, bufferlist> *out ///< [out] Returned keys and values
+  const std::set<std::string> &keys,     ///< [in] Keys to get
+  std::map<std::string, ceph::buffer::list> *out ///< [out] Returned keys and values
   )
 {
   dout(10) << __func__ << " " << ch->cid << " " << oid << dendl;
@@ -532,21 +529,43 @@ int MemStore::omap_get_values(
   if (!o)
     return -ENOENT;
   std::lock_guard lock{o->omap_mutex};
-  for (set<string>::const_iterator p = keys.begin();
-       p != keys.end();
-       ++p) {
-    map<string,bufferlist>::iterator q = o->omap.find(*p);
+  for (auto p = keys.begin(); p != keys.end(); ++p) {
+    auto q = o->omap.find(*p);
     if (q != o->omap.end())
       out->insert(*q);
   }
   return 0;
 }
 
+#ifdef WITH_SEASTAR
+int MemStore::omap_get_values(
+  CollectionHandle& ch,                    ///< [in] Collection containing oid
+  const ghobject_t &oid,       ///< [in] Object containing omap
+  const std::optional<std::string> &start_after,     ///< [in] Keys to get
+  std::map<std::string, ceph::buffer::list> *out ///< [out] Returned keys and values
+  )
+{
+  dout(10) << __func__ << " " << ch->cid << " " << oid << dendl;
+  Collection *c = static_cast<Collection*>(ch.get());
+  ObjectRef o = c->get_object(oid);
+  if (!o)
+    return -ENOENT;
+  assert(start_after);
+  std::lock_guard lock{o->omap_mutex};
+  for (auto it = o->omap.upper_bound(*start_after);
+       it != std::end(o->omap);
+       ++it) {
+    out->insert(*it);
+  }
+  return 0;
+}
+#endif
+
 int MemStore::omap_check_keys(
   CollectionHandle& ch,                ///< [in] Collection containing oid
   const ghobject_t &oid,   ///< [in] Object containing omap
-  const set<string> &keys, ///< [in] Keys to check
-  set<string> *out         ///< [out] Subset of keys defined on oid
+  const std::set<std::string> &keys, ///< [in] Keys to check
+  std::set<std::string> *out         ///< [out] Subset of keys defined on oid
   )
 {
   dout(10) << __func__ << " " << ch->cid << " " << oid << dendl;
@@ -555,10 +574,8 @@ int MemStore::omap_check_keys(
   if (!o)
     return -ENOENT;
   std::lock_guard lock{o->omap_mutex};
-  for (set<string>::const_iterator p = keys.begin();
-       p != keys.end();
-       ++p) {
-    map<string,bufferlist>::iterator q = o->omap.find(*p);
+  for (auto p = keys.begin(); p != keys.end(); ++p) {
+    auto q = o->omap.find(*p);
     if (q != o->omap.end())
       out->insert(*p);
   }
@@ -568,7 +585,7 @@ int MemStore::omap_check_keys(
 class MemStore::OmapIteratorImpl : public ObjectMap::ObjectMapIteratorImpl {
   CollectionRef c;
   ObjectRef o;
-  map<string,bufferlist>::iterator it;
+  std::map<std::string,ceph::buffer::list>::iterator it;
 public:
   OmapIteratorImpl(CollectionRef c, ObjectRef o)
     : c(c), o(o), it(o->omap.begin()) {}
@@ -578,12 +595,12 @@ public:
     it = o->omap.begin();
     return 0;
   }
-  int upper_bound(const string &after) override {
+  int upper_bound(const std::string &after) override {
     std::lock_guard lock{o->omap_mutex};
     it = o->omap.upper_bound(after);
     return 0;
   }
-  int lower_bound(const string &to) override {
+  int lower_bound(const std::string &to) override {
     std::lock_guard lock{o->omap_mutex};
     it = o->omap.lower_bound(to);
     return 0;
@@ -597,11 +614,11 @@ public:
     ++it;
     return 0;
   }
-  string key() override {
+  std::string key() override {
     std::lock_guard lock{o->omap_mutex};
     return it->first;
   }
-  bufferlist value() override {
+  ceph::buffer::list value() override {
     std::lock_guard lock{o->omap_mutex};
     return it->second;
   }
@@ -628,7 +645,7 @@ ObjectMap::ObjectMapIterator MemStore::get_omap_iterator(
 
 int MemStore::queue_transactions(
   CollectionHandle& ch,
-  vector<Transaction>& tls,
+  std::vector<Transaction>& tls,
   TrackedOpRef op,
   ThreadPool::TPHandle *handle)
 {
@@ -638,7 +655,7 @@ int MemStore::queue_transactions(
   Collection *c = static_cast<Collection*>(ch.get());
   std::unique_lock lock{c->sequencer_mutex};
 
-  for (vector<Transaction>::iterator p = tls.begin(); p != tls.end(); ++p) {
+  for (auto p = tls.begin(); p != tls.end(); ++p) {
     // poke the TPHandle heartbeat just to exercise that code path
     if (handle)
       handle->reset_tp_timeout();
@@ -686,7 +703,7 @@ void MemStore::_do_transaction(Transaction& t)
         uint64_t off = op->off;
         uint64_t len = op->len;
 	uint32_t fadvise_flags = i.get_fadvise_flags();
-        bufferlist bl;
+        ceph::buffer::list bl;
         i.decode_bl(bl);
 	r = _write(cid, oid, off, len, bl, fadvise_flags);
       }
@@ -729,11 +746,11 @@ void MemStore::_do_transaction(Transaction& t)
       {
         coll_t cid = i.get_cid(op->cid);
         ghobject_t oid = i.get_oid(op->oid);
-        string name = i.decode_string();
-        bufferlist bl;
+        std::string name = i.decode_string();
+        ceph::buffer::list bl;
         i.decode_bl(bl);
-	map<string, bufferptr> to_set;
-	to_set[name] = bufferptr(bl.c_str(), bl.length());
+	std::map<std::string, ceph::buffer::ptr> to_set;
+	to_set[name] = ceph::buffer::ptr(bl.c_str(), bl.length());
 	r = _setattrs(cid, oid, to_set);
       }
       break;
@@ -742,7 +759,7 @@ void MemStore::_do_transaction(Transaction& t)
       {
         coll_t cid = i.get_cid(op->cid);
         ghobject_t oid = i.get_oid(op->oid);
-        map<string, bufferptr> aset;
+	std::map<std::string, ceph::buffer::ptr> aset;
         i.decode_attrset(aset);
 	r = _setattrs(cid, oid, aset);
       }
@@ -752,7 +769,7 @@ void MemStore::_do_transaction(Transaction& t)
       {
         coll_t cid = i.get_cid(op->cid);
         ghobject_t oid = i.get_oid(op->oid);
-        string name = i.decode_string();
+        std::string name = i.decode_string();
 	r = _rmattr(cid, oid, name.c_str());
       }
       break;
@@ -807,8 +824,8 @@ void MemStore::_do_transaction(Transaction& t)
     case Transaction::OP_COLL_HINT:
       {
         coll_t cid = i.get_cid(op->cid);
-        uint32_t type = op->hint_type;
-        bufferlist hint;
+        uint32_t type = op->hint;
+        ceph::buffer::list hint;
         i.decode_bl(hint);
         auto hiter = hint.cbegin();
         if (type == Transaction::COLL_HINT_EXPECTED_NUM_OBJECTS) {
@@ -904,7 +921,7 @@ void MemStore::_do_transaction(Transaction& t)
       {
         coll_t cid = i.get_cid(op->cid);
         ghobject_t oid = i.get_oid(op->oid);
-        bufferlist aset_bl;
+        ceph::buffer::list aset_bl;
         i.decode_attrset_bl(&aset_bl);
 	r = _omap_setkeys(cid, oid, aset_bl);
       }
@@ -913,7 +930,7 @@ void MemStore::_do_transaction(Transaction& t)
       {
         coll_t cid = i.get_cid(op->cid);
         ghobject_t oid = i.get_oid(op->oid);
-        bufferlist keys_bl;
+        ceph::buffer::list keys_bl;
         i.decode_keyset_bl(&keys_bl);
 	r = _omap_rmkeys(cid, oid, keys_bl);
       }
@@ -922,7 +939,7 @@ void MemStore::_do_transaction(Transaction& t)
       {
         coll_t cid = i.get_cid(op->cid);
         ghobject_t oid = i.get_oid(op->oid);
-        string first, last;
+        std::string first, last;
         first = i.decode_string();
         last = i.decode_string();
 	r = _omap_rmkeyrange(cid, oid, first, last);
@@ -932,7 +949,7 @@ void MemStore::_do_transaction(Transaction& t)
       {
         coll_t cid = i.get_cid(op->cid);
         ghobject_t oid = i.get_oid(op->oid);
-        bufferlist bl;
+        ceph::buffer::list bl;
         i.decode_bl(bl);
 	r = _omap_setheader(cid, oid, bl);
       }
@@ -1009,13 +1026,17 @@ void MemStore::_do_transaction(Transaction& t)
 		<< " (op " << pos << ", counting from 0)" << dendl;
 	dout(0) << msg << dendl;
 	dout(0) << " transaction dump:\n";
-	JSONFormatter f(true);
+	ceph::JSONFormatter f(true);
 	f.open_object_section("transaction");
 	t.dump(&f);
 	f.close_section();
 	f.flush(*_dout);
 	*_dout << dendl;
-	ceph_abort_msg("unexpected error");
+        if (!g_conf().get_val<bool>("objectstore_debug_throw_on_failed_txc")) {
+	  ceph_abort_msg("unexpected error");
+        } else {
+	  throw r;
+        }
       }
     }
 
@@ -1035,7 +1056,7 @@ int MemStore::_touch(const coll_t& cid, const ghobject_t& oid)
 }
 
 int MemStore::_write(const coll_t& cid, const ghobject_t& oid,
-		     uint64_t offset, size_t len, const bufferlist& bl,
+		     uint64_t offset, size_t len, const ceph::buffer::list& bl,
 		     uint32_t fadvise_flags)
 {
   dout(10) << __func__ << " " << cid << " " << oid << " "
@@ -1061,7 +1082,7 @@ int MemStore::_zero(const coll_t& cid, const ghobject_t& oid,
 {
   dout(10) << __func__ << " " << cid << " " << oid << " " << offset << "~"
 	   << len << dendl;
-  bufferlist bl;
+  ceph::buffer::list bl;
   bl.append_zero(len);
   return _write(cid, oid, offset, len, bl);
 }
@@ -1103,7 +1124,7 @@ int MemStore::_remove(const coll_t& cid, const ghobject_t& oid)
 }
 
 int MemStore::_setattrs(const coll_t& cid, const ghobject_t& oid,
-			map<string,bufferptr>& aset)
+			std::map<std::string,ceph::buffer::ptr>& aset)
 {
   dout(10) << __func__ << " " << cid << " " << oid << dendl;
   CollectionRef c = get_collection(cid);
@@ -1114,7 +1135,7 @@ int MemStore::_setattrs(const coll_t& cid, const ghobject_t& oid,
   if (!o)
     return -ENOENT;
   std::lock_guard lock{o->xattr_mutex};
-  for (map<string,bufferptr>::const_iterator p = aset.begin(); p != aset.end(); ++p)
+  for (auto p = aset.begin(); p != aset.end(); ++p)
     o->xattr[p->first] = p->second;
   return 0;
 }
@@ -1225,7 +1246,7 @@ int MemStore::_omap_clear(const coll_t& cid, const ghobject_t &oid)
 }
 
 int MemStore::_omap_setkeys(const coll_t& cid, const ghobject_t &oid,
-			    bufferlist& aset_bl)
+			    ceph::buffer::list& aset_bl)
 {
   dout(10) << __func__ << " " << cid << " " << oid << dendl;
   CollectionRef c = get_collection(cid);
@@ -1240,7 +1261,7 @@ int MemStore::_omap_setkeys(const coll_t& cid, const ghobject_t &oid,
   __u32 num;
   decode(num, p);
   while (num--) {
-    string key;
+    std::string key;
     decode(key, p);
     decode(o->omap[key], p);
   }
@@ -1248,7 +1269,7 @@ int MemStore::_omap_setkeys(const coll_t& cid, const ghobject_t &oid,
 }
 
 int MemStore::_omap_rmkeys(const coll_t& cid, const ghobject_t &oid,
-			   bufferlist& keys_bl)
+			   ceph::buffer::list& keys_bl)
 {
   dout(10) << __func__ << " " << cid << " " << oid << dendl;
   CollectionRef c = get_collection(cid);
@@ -1263,7 +1284,7 @@ int MemStore::_omap_rmkeys(const coll_t& cid, const ghobject_t &oid,
   __u32 num;
   decode(num, p);
   while (num--) {
-    string key;
+    std::string key;
     decode(key, p);
     o->omap.erase(key);
   }
@@ -1271,7 +1292,7 @@ int MemStore::_omap_rmkeys(const coll_t& cid, const ghobject_t &oid,
 }
 
 int MemStore::_omap_rmkeyrange(const coll_t& cid, const ghobject_t &oid,
-			       const string& first, const string& last)
+			       const std::string& first, const std::string& last)
 {
   dout(10) << __func__ << " " << cid << " " << oid << " " << first
 	   << " " << last << dendl;
@@ -1283,14 +1304,14 @@ int MemStore::_omap_rmkeyrange(const coll_t& cid, const ghobject_t &oid,
   if (!o)
     return -ENOENT;
   std::lock_guard lock{o->omap_mutex};
-  map<string,bufferlist>::iterator p = o->omap.lower_bound(first);
-  map<string,bufferlist>::iterator e = o->omap.lower_bound(last);
+  auto p = o->omap.lower_bound(first);
+  auto e = o->omap.lower_bound(last);
   o->omap.erase(p, e);
   return 0;
 }
 
 int MemStore::_omap_setheader(const coll_t& cid, const ghobject_t &oid,
-			      const bufferlist &bl)
+			      const ceph::buffer::list &bl)
 {
   dout(10) << __func__ << " " << cid << " " << oid << dendl;
   CollectionRef c = get_collection(cid);
@@ -1406,12 +1427,12 @@ int MemStore::_split_collection(const coll_t& cid, uint32_t bits, uint32_t match
   std::scoped_lock l{std::min(&(*sc), &(*dc))->lock,
                      std::max(&(*sc), &(*dc))->lock};
 
-  map<ghobject_t,ObjectRef>::iterator p = sc->object_map.begin();
+  auto p = sc->object_map.begin();
   while (p != sc->object_map.end()) {
     if (p->first.match(bits, match)) {
       dout(20) << " moving " << p->first << dendl;
-      dc->object_map.insert(make_pair(p->first, p->second));
-      dc->object_hash.insert(make_pair(p->first, p->second));
+      dc->object_map.insert(std::make_pair(p->first, p->second));
+      dc->object_hash.insert(std::make_pair(p->first, p->second));
       sc->object_hash.erase(p->first);
       sc->object_map.erase(p++);
     } else {
@@ -1439,11 +1460,11 @@ int MemStore::_merge_collection(const coll_t& cid, uint32_t bits, coll_t dest)
     std::scoped_lock l{std::min(&(*sc), &(*dc))->lock,
                        std::max(&(*sc), &(*dc))->lock};
 
-    map<ghobject_t,ObjectRef>::iterator p = sc->object_map.begin();
+    auto p = sc->object_map.begin();
     while (p != sc->object_map.end()) {
       dout(20) << " moving " << p->first << dendl;
-      dc->object_map.insert(make_pair(p->first, p->second));
-      dc->object_hash.insert(make_pair(p->first, p->second));
+      dc->object_map.insert(std::make_pair(p->first, p->second));
+      dc->object_hash.insert(std::make_pair(p->first, p->second));
       sc->object_hash.erase(p->first);
       sc->object_map.erase(p++);
     }
@@ -1465,23 +1486,23 @@ int MemStore::_merge_collection(const coll_t& cid, uint32_t bits, coll_t dest)
 namespace {
 struct BufferlistObject : public MemStore::Object {
   ceph::spinlock mutex;
-  bufferlist data;
+  ceph::buffer::list data;
 
   size_t get_size() const override { return data.length(); }
 
-  int read(uint64_t offset, uint64_t len, bufferlist &bl) override;
-  int write(uint64_t offset, const bufferlist &bl) override;
+  int read(uint64_t offset, uint64_t len, ceph::buffer::list &bl) override;
+  int write(uint64_t offset, const ceph::buffer::list &bl) override;
   int clone(Object *src, uint64_t srcoff, uint64_t len,
             uint64_t dstoff) override;
   int truncate(uint64_t offset) override;
 
-  void encode(bufferlist& bl) const override {
+  void encode(ceph::buffer::list& bl) const override {
     ENCODE_START(1, 1, bl);
     encode(data, bl);
     encode_base(bl);
     ENCODE_FINISH(bl);
   }
-  void decode(bufferlist::const_iterator& p) override {
+  void decode(ceph::buffer::list::const_iterator& p) override {
     DECODE_START(1, p);
     decode(data, p);
     decode_base(p);
@@ -1491,21 +1512,21 @@ struct BufferlistObject : public MemStore::Object {
 }
 // BufferlistObject
 int BufferlistObject::read(uint64_t offset, uint64_t len,
-                                     bufferlist &bl)
+                                     ceph::buffer::list &bl)
 {
   std::lock_guard<decltype(mutex)> lock(mutex);
   bl.substr_of(data, offset, len);
   return bl.length();
 }
 
-int BufferlistObject::write(uint64_t offset, const bufferlist &src)
+int BufferlistObject::write(uint64_t offset, const ceph::buffer::list &src)
 {
   unsigned len = src.length();
 
   std::lock_guard<decltype(mutex)> lock(mutex);
 
   // before
-  bufferlist newdata;
+  ceph::buffer::list newdata;
   if (get_size() >= offset) {
     newdata.substr_of(data, 0, offset);
   } else {
@@ -1519,12 +1540,12 @@ int BufferlistObject::write(uint64_t offset, const bufferlist &src)
 
   // after
   if (get_size() > offset + len) {
-    bufferlist tail;
+    ceph::buffer::list tail;
     tail.substr_of(data, offset + len, get_size() - (offset + len));
     newdata.append(tail);
   }
 
-  data.claim(newdata);
+  data = std::move(newdata);
   return 0;
 }
 
@@ -1535,7 +1556,7 @@ int BufferlistObject::clone(Object *src, uint64_t srcoff,
   if (srcbl == nullptr)
     return -ENOTSUP;
 
-  bufferlist bl;
+  ceph::buffer::list bl;
   {
     std::lock_guard<decltype(srcbl->mutex)> lock(srcbl->mutex);
     if (srcoff == dstoff && len == src->get_size()) {
@@ -1551,9 +1572,9 @@ int BufferlistObject::truncate(uint64_t size)
 {
   std::lock_guard<decltype(mutex)> lock(mutex);
   if (get_size() > size) {
-    bufferlist bl;
+    ceph::buffer::list bl;
     bl.substr_of(data, 0, size);
-    data.claim(bl);
+    data = std::move(bl);
   } else if (get_size() == size) {
     // do nothing
   } else {
@@ -1575,20 +1596,20 @@ struct MemStore::PageSetObject : public Object {
 
   size_t get_size() const override { return data_len; }
 
-  int read(uint64_t offset, uint64_t len, bufferlist &bl) override;
-  int write(uint64_t offset, const bufferlist &bl) override;
+  int read(uint64_t offset, uint64_t len, ceph::buffer::list &bl) override;
+  int write(uint64_t offset, const ceph::buffer::list &bl) override;
   int clone(Object *src, uint64_t srcoff, uint64_t len,
             uint64_t dstoff) override;
   int truncate(uint64_t offset) override;
 
-  void encode(bufferlist& bl) const override {
+  void encode(ceph::buffer::list& bl) const override {
     ENCODE_START(1, 1, bl);
     encode(data_len, bl);
     data.encode(bl);
     encode_base(bl);
     ENCODE_FINISH(bl);
   }
-  void decode(bufferlist::const_iterator& p) override {
+  void decode(ceph::buffer::list::const_iterator& p) override {
     DECODE_START(1, p);
     decode(data_len, p);
     data.decode(p);
@@ -1610,7 +1631,7 @@ thread_local PageSet::page_vector MemStore::PageSetObject::tls_pages;
 #define DEFINE_PAGE_VECTOR(name) PageSet::page_vector name;
 #endif
 
-int MemStore::PageSetObject::read(uint64_t offset, uint64_t len, bufferlist& bl)
+int MemStore::PageSetObject::read(uint64_t offset, uint64_t len, ceph::buffer::list& bl)
 {
   const auto start = offset;
   const auto end = offset + len;
@@ -1620,7 +1641,7 @@ int MemStore::PageSetObject::read(uint64_t offset, uint64_t len, bufferlist& bl)
   data.get_range(offset, len, tls_pages);
 
   // allocate a buffer for the data
-  buffer::ptr buf(len);
+  ceph::buffer::ptr buf(len);
 
   auto p = tls_pages.begin();
   while (remaining) {
@@ -1643,7 +1664,7 @@ int MemStore::PageSetObject::read(uint64_t offset, uint64_t len, bufferlist& bl)
 
     // read from page
     const auto page_offset = offset - page->offset;
-    const auto count = min(remaining, data.get_page_size() - page_offset);
+    const auto count = std::min(remaining, data.get_page_size() - page_offset);
 
     buf.copy_in(offset - start, count, page->data + page_offset);
 
@@ -1659,7 +1680,7 @@ int MemStore::PageSetObject::read(uint64_t offset, uint64_t len, bufferlist& bl)
   return len;
 }
 
-int MemStore::PageSetObject::write(uint64_t offset, const bufferlist &src)
+int MemStore::PageSetObject::write(uint64_t offset, const ceph::buffer::list &src)
 {
   unsigned len = src.length();
 
@@ -1673,7 +1694,7 @@ int MemStore::PageSetObject::write(uint64_t offset, const bufferlist &src)
   while (len > 0) {
     unsigned page_offset = offset - (*page)->offset;
     unsigned pageoff = data.get_page_size() - page_offset;
-    unsigned count = min(len, pageoff);
+    unsigned count = std::min(len, pageoff);
     p.copy(count, (*page)->data + page_offset);
     offset += count;
     len -= count;
@@ -1803,5 +1824,5 @@ int MemStore::PageSetObject::truncate(uint64_t size)
 MemStore::ObjectRef MemStore::Collection::create_object() const {
   if (use_page_set)
     return ceph::make_ref<PageSetObject>(cct->_conf->memstore_page_size);
-  return new BufferlistObject();
+  return make_ref<BufferlistObject>();
 }
