@@ -1,19 +1,42 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import
 
-from . import ApiController, RESTController, \
-    allow_empty_body
 from .. import mgr
 from ..security import Scope
 from ..services.ceph_service import CephService
 from ..services.exception import handle_send_command_error
 from ..tools import find_object_in_list, str_to_bool
+from . import APIDoc, APIRouter, EndpointDoc, RESTController, allow_empty_body
+
+MGR_MODULE_SCHEMA = ([{
+    "name": (str, "Module Name"),
+    "enabled": (bool, "Is Module Enabled"),
+    "always_on": (bool, "Is it an always on module?"),
+    "options": ({
+        "Option_name": ({
+            "name": (str, "Name of the option"),
+            "type": (str, "Type of the option"),
+            "level": (str, "Option level"),
+            "flags": (int, "List of flags associated"),
+            "default_value": (int, "Default value for the option"),
+            "min": (str, "Minimum value"),
+            "max": (str, "Maximum value"),
+            "enum_allowed": ([str], ""),
+            "desc": (str, "Description of the option"),
+            "long_desc": (str, "Elaborated description"),
+            "tags": ([str], "Tags associated with the option"),
+            "see_also": ([str], "Related options")
+        }, "Options")
+    }, "Module Options")
+}])
 
 
-@ApiController('/mgr/module', Scope.CONFIG_OPT)
+@APIRouter('/mgr/module', Scope.CONFIG_OPT)
+@APIDoc("Get details of MGR Module", "MgrModule")
 class MgrModules(RESTController):
     ignore_modules = ['selftest']
 
+    @EndpointDoc("List Mgr modules",
+                 responses={200: MGR_MODULE_SCHEMA})
     def list(self):
         """
         Get the list of managed modules.
