@@ -11,10 +11,13 @@ import { PipesModule } from '~/app/shared/pipes/pipes.module';
 import { AuthStorageService } from '~/app/shared/services/auth-storage.service';
 import { configureTestBed } from '~/testing/unit-test-helper';
 import { RbdSnapshotFormModalComponent } from './rbd-snapshot-form-modal.component';
+import { RbdMirroringService } from '~/app/shared/api/rbd-mirroring.service';
+import { of } from 'rxjs';
 
 describe('RbdSnapshotFormModalComponent', () => {
   let component: RbdSnapshotFormModalComponent;
   let fixture: ComponentFixture<RbdSnapshotFormModalComponent>;
+  let rbdMirrorService: RbdMirroringService;
 
   configureTestBed({
     imports: [
@@ -32,6 +35,7 @@ describe('RbdSnapshotFormModalComponent', () => {
   beforeEach(() => {
     fixture = TestBed.createComponent(RbdSnapshotFormModalComponent);
     component = fixture.componentInstance;
+    rbdMirrorService = TestBed.inject(RbdMirroringService);
   });
 
   it('should create', () => {
@@ -59,4 +63,22 @@ describe('RbdSnapshotFormModalComponent', () => {
     const button = fixture.debugElement.nativeElement.querySelector('cd-submit-button');
     expect(button.textContent).toBe('Rename RBD Snapshot');
   });
+
+  it('should enable the mirror image snapshot creation when peer is configured', () => {
+    spyOn(rbdMirrorService, 'getPeerForPool').and.returnValue(of(['test_peer']));
+    component.mirroring = 'snapshot';
+    component.ngOnInit();
+    fixture.detectChanges();
+    const radio = fixture.debugElement.nativeElement.querySelector('#mirrorImageSnapshot');
+    expect(radio.disabled).toBe(false);
+  });
+
+  it('should disable the mirror image snapshot creation when peer is not configured', () => {
+    spyOn(rbdMirrorService, 'getPeerForPool').and.returnValue(of([]));
+    component.mirroring = 'snapshot';
+    component.ngOnInit();
+    fixture.detectChanges();
+    const radio = fixture.debugElement.nativeElement.querySelector('#mirrorImageSnapshot');
+    expect(radio.disabled).toBe(true);
+  });
 });
