@@ -1,7 +1,6 @@
 import { HttpClientTestingModule } from '@angular/common/http/testing';
 import { ComponentFixture, TestBed } from '@angular/core/testing';
 import { ReactiveFormsModule } from '@angular/forms';
-import { ActivatedRoute } from '@angular/router';
 import { RouterTestingModule } from '@angular/router/testing';
 
 import { ToastrModule } from 'ngx-toastr';
@@ -23,12 +22,7 @@ describe('ConfigurationFormComponent', () => {
       ToastrModule.forRoot(),
       SharedModule
     ],
-    declarations: [ConfigurationFormComponent],
-    providers: [
-      {
-        provide: ActivatedRoute
-      }
-    ]
+    declarations: [ConfigurationFormComponent]
   });
 
   beforeEach(() => {
