@@ -1,27 +1,27 @@
 import { Component, OnDestroy, OnInit, TemplateRef, ViewChild } from '@angular/core';
 
-import { I18n } from '@ngx-translate/i18n-polyfill';
-import * as _ from 'lodash';
-import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
+import { NgbModalRef } from '@ng-bootstrap/ng-bootstrap';
+import _ from 'lodash';
 import { Subscription } from 'rxjs';
 
-import { NfsService } from '../../../shared/api/nfs.service';
-import { ListWithDetails } from '../../../shared/classes/list-with-details.class';
-import { CriticalConfirmationModalComponent } from '../../../shared/components/critical-confirmation-modal/critical-confirmation-modal.component';
-import { ActionLabelsI18n } from '../../../shared/constants/app.constants';
-import { TableComponent } from '../../../shared/datatable/table/table.component';
-import { CellTemplate } from '../../../shared/enum/cell-template.enum';
-import { Icons } from '../../../shared/enum/icons.enum';
-import { ViewCacheStatus } from '../../../shared/enum/view-cache-status.enum';
-import { CdTableAction } from '../../../shared/models/cd-table-action';
-import { CdTableColumn } from '../../../shared/models/cd-table-column';
-import { CdTableSelection } from '../../../shared/models/cd-table-selection';
-import { FinishedTask } from '../../../shared/models/finished-task';
-import { Permission } from '../../../shared/models/permissions';
-import { Task } from '../../../shared/models/task';
-import { AuthStorageService } from '../../../shared/services/auth-storage.service';
-import { TaskListService } from '../../../shared/services/task-list.service';
-import { TaskWrapperService } from '../../../shared/services/task-wrapper.service';
+import { NfsService } from '~/app/shared/api/nfs.service';
+import { ListWithDetails } from '~/app/shared/classes/list-with-details.class';
+import { CriticalConfirmationModalComponent } from '~/app/shared/components/critical-confirmation-modal/critical-confirmation-modal.component';
+import { ActionLabelsI18n } from '~/app/shared/constants/app.constants';
+import { TableComponent } from '~/app/shared/datatable/table/table.component';
+import { CellTemplate } from '~/app/shared/enum/cell-template.enum';
+import { Icons } from '~/app/shared/enum/icons.enum';
+import { ViewCacheStatus } from '~/app/shared/enum/view-cache-status.enum';
+import { CdTableAction } from '~/app/shared/models/cd-table-action';
+import { CdTableColumn } from '~/app/shared/models/cd-table-column';
+import { CdTableSelection } from '~/app/shared/models/cd-table-selection';
+import { FinishedTask } from '~/app/shared/models/finished-task';
+import { Permission } from '~/app/shared/models/permissions';
+import { Task } from '~/app/shared/models/task';
+import { AuthStorageService } from '~/app/shared/services/auth-storage.service';
+import { ModalService } from '~/app/shared/services/modal.service';
+import { TaskListService } from '~/app/shared/services/task-list.service';
+import { TaskWrapperService } from '~/app/shared/services/task-wrapper.service';
 
 @Component({
   selector: 'cd-nfs-list',
@@ -30,7 +30,7 @@ import { TaskWrapperService } from '../../../shared/services/task-wrapper.servic
   providers: [TaskListService]
 })
 export class NfsListComponent extends ListWithDetails implements OnInit, OnDestroy {
-  @ViewChild('nfsState', { static: false })
+  @ViewChild('nfsState')
   nfsState: TemplateRef<any>;
   @ViewChild('nfsFsal', { static: true })
   nfsFsal: TemplateRef<any>;
@@ -47,7 +47,7 @@ export class NfsListComponent extends ListWithDetails implements OnInit, OnDestr
   tableActions: CdTableAction[];
   isDefaultCluster = false;
 
-  modalRef: BsModalRef;
+  modalRef: NgbModalRef;
 
   builders = {
     'nfs/create': (metadata: any) => {
@@ -61,8 +61,7 @@ export class NfsListComponent extends ListWithDetails implements OnInit, OnDestr
 
   constructor(
     private authStorageService: AuthStorageService,
-    private i18n: I18n,
-    private modalService: BsModalService,
+    private modalService: ModalService,
     private nfsService: NfsService,
     private taskListService: TaskListService,
     private taskWrapper: TaskWrapperService,
@@ -104,65 +103,42 @@ export class NfsListComponent extends ListWithDetails implements OnInit, OnDestr
   ngOnInit() {
     this.columns = [
       {
-        name: this.i18n('Path'),
+        name: $localize`Path`,
         prop: 'path',
         flexGrow: 2,
         cellTransformation: CellTemplate.executing
       },
       {
-        name: this.i18n('Pseudo'),
+        name: $localize`Pseudo`,
         prop: 'pseudo',
         flexGrow: 2
       },
       {
-        name: this.i18n('Cluster'),
+        name: $localize`Cluster`,
         prop: 'cluster_id',
         flexGrow: 2
       },
       {
-        name: this.i18n('Daemons'),
-        prop: 'daemons',
-        flexGrow: 2
-      },
-      {
-        name: this.i18n('Storage Backend'),
+        name: $localize`Storage Backend`,
         prop: 'fsal',
         flexGrow: 2,
         cellTemplate: this.nfsFsal
       },
       {
-        name: this.i18n('Access Type'),
+        name: $localize`Access Type`,
         prop: 'access_type',
         flexGrow: 2
       }
     ];
 
-    this.nfsService.daemon().subscribe(
-      (daemons: any) => {
-        const clusters = _(daemons)
-          .map((daemon) => daemon.cluster_id)
-          .uniq()
-          .value();
-
-        this.isDefaultCluster = clusters.length === 1 && clusters[0] === '_default_';
-        this.columns[2].isHidden = this.isDefaultCluster;
-        if (this.table) {
-          this.table.updateColumns();
-        }
-
-        this.taskListService.init(
-          () => this.nfsService.list(),
-          (resp) => this.prepareResponse(resp),
-          (exports) => (this.exports = exports),
-          () => this.onFetchError(),
-          this.taskFilter,
-          this.itemFilter,
-          this.builders
-        );
-      },
-      () => {
-        this.onFetchError();
-      }
+    this.taskListService.init(
+      () => this.nfsService.list(),
+      (resp) => this.prepareResponse(resp),
+      (exports) => (this.exports = exports),
+      () => this.onFetchError(),
+      this.taskFilter,
+      this.itemFilter,
+      this.builders
     );
   }
 
@@ -208,18 +184,16 @@ export class NfsListComponent extends ListWithDetails implements OnInit, OnDestr
     const export_id = this.selection.first().export_id;
 
     this.modalRef = this.modalService.show(CriticalConfirmationModalComponent, {
-      initialState: {
-        itemDescription: this.i18n('NFS export'),
-        itemNames: [`${cluster_id}:${export_id}`],
-        submitActionObservable: () =>
-          this.taskWrapper.wrapTaskAroundCall({
-            task: new FinishedTask('nfs/delete', {
-              cluster_id: cluster_id,
-              export_id: export_id
-            }),
-            call: this.nfsService.delete(cluster_id, export_id)
-          })
-      }
+      itemDescription: $localize`NFS export`,
+      itemNames: [`${cluster_id}:${export_id}`],
+      submitActionObservable: () =>
+        this.taskWrapper.wrapTaskAroundCall({
+          task: new FinishedTask('nfs/delete', {
+            cluster_id: cluster_id,
+            export_id: export_id
+          }),
+          call: this.nfsService.delete(cluster_id, export_id)
+        })
     });
   }
 }
