@@ -1,27 +1,27 @@
 import { Component, EventEmitter, OnInit, Output } from '@angular/core';
 import { Validators } from '@angular/forms';
 
-import { I18n } from '@ngx-translate/i18n-polyfill';
-import { BsModalRef } from 'ngx-bootstrap/modal';
+import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
 
-import { ErasureCodeProfileService } from '../../../shared/api/erasure-code-profile.service';
-import { CrushNodeSelectionClass } from '../../../shared/classes/crush.node.selection.class';
-import { ActionLabelsI18n } from '../../../shared/constants/app.constants';
-import { Icons } from '../../../shared/enum/icons.enum';
-import { CdFormBuilder } from '../../../shared/forms/cd-form-builder';
-import { CdFormGroup } from '../../../shared/forms/cd-form-group';
-import { CdValidators } from '../../../shared/forms/cd-validators';
-import { CrushNode } from '../../../shared/models/crush-node';
-import { ErasureCodeProfile } from '../../../shared/models/erasure-code-profile';
-import { FinishedTask } from '../../../shared/models/finished-task';
-import { TaskWrapperService } from '../../../shared/services/task-wrapper.service';
+import { ErasureCodeProfileService } from '~/app/shared/api/erasure-code-profile.service';
+import { CrushNodeSelectionClass } from '~/app/shared/classes/crush.node.selection.class';
+import { ActionLabelsI18n } from '~/app/shared/constants/app.constants';
+import { Icons } from '~/app/shared/enum/icons.enum';
+import { CdFormBuilder } from '~/app/shared/forms/cd-form-builder';
+import { CdFormGroup } from '~/app/shared/forms/cd-form-group';
+import { CdValidators } from '~/app/shared/forms/cd-validators';
+import { CrushNode } from '~/app/shared/models/crush-node';
+import { ErasureCodeProfile } from '~/app/shared/models/erasure-code-profile';
+import { FinishedTask } from '~/app/shared/models/finished-task';
+import { TaskWrapperService } from '~/app/shared/services/task-wrapper.service';
 
 @Component({
   selector: 'cd-erasure-code-profile-form-modal',
   templateUrl: './erasure-code-profile-form-modal.component.html',
   styleUrls: ['./erasure-code-profile-form-modal.component.scss']
 })
-export class ErasureCodeProfileFormModalComponent extends CrushNodeSelectionClass
+export class ErasureCodeProfileFormModalComponent
+  extends CrushNodeSelectionClass
   implements OnInit {
   @Output()
   submitAction = new EventEmitter();
@@ -49,15 +49,14 @@ export class ErasureCodeProfileFormModalComponent extends CrushNodeSelectionClas
 
   constructor(
     private formBuilder: CdFormBuilder,
-    public bsModalRef: BsModalRef,
+    public activeModal: NgbActiveModal,
     private taskWrapper: TaskWrapperService,
     private ecpService: ErasureCodeProfileService,
-    private i18n: I18n,
     public actionLabels: ActionLabelsI18n
   ) {
     super();
     this.action = this.actionLabels.CREATE;
-    this.resource = this.i18n('EC Profile');
+    this.resource = $localize`EC Profile`;
     this.createForm();
     this.setJerasureDefaults();
   }
@@ -80,7 +79,6 @@ export class ErasureCodeProfileFormModalComponent extends CrushNodeSelectionClas
         4, // Will be overwritten with plugin defaults
         [
           Validators.required,
-          Validators.min(2),
           CdValidators.custom('max', () => this.baseValueValidation(true)),
           CdValidators.custom('unequal', (v: number) => this.lrcDataValidation(v)),
           CdValidators.custom('kLowerM', (v: number) => this.shecDataValidation(v))
@@ -88,11 +86,7 @@ export class ErasureCodeProfileFormModalComponent extends CrushNodeSelectionClas
       ],
       m: [
         2, // Will be overwritten with plugin defaults
-        [
-          Validators.required,
-          Validators.min(1),
-          CdValidators.custom('max', () => this.baseValueValidation())
-        ]
+        [Validators.required, CdValidators.custom('max', () => this.baseValueValidation())]
       ],
       crushFailureDomain: '', // Will be preselected
       crushRoot: null, // Will be preselected
@@ -101,13 +95,12 @@ export class ErasureCodeProfileFormModalComponent extends CrushNodeSelectionClas
       // Only for 'jerasure', 'clay' and 'isa' use
       technique: 'reed_sol_van',
       // Only for 'jerasure' use
-      packetSize: [2048, [Validators.min(1)]],
+      packetSize: [2048],
       // Only for 'lrc' use
       l: [
         3, // Will be overwritten with plugin defaults
         [
           Validators.required,
-          Validators.min(1),
           CdValidators.custom('unequal', (v: number) => this.lrcLocalityValidation(v))
         ]
       ],
@@ -117,7 +110,6 @@ export class ErasureCodeProfileFormModalComponent extends CrushNodeSelectionClas
         2, // Will be overwritten with plugin defaults
         [
           Validators.required,
-          Validators.min(1),
           CdValidators.custom('cGreaterM', (v: number) => this.shecDurabilityValidation(v))
         ]
       ],
@@ -417,16 +409,15 @@ export class ErasureCodeProfileFormModalComponent extends CrushNodeSelectionClas
         task: new FinishedTask('ecp/create', { name: profile.name }),
         call: this.ecpService.create(profile)
       })
-      .subscribe(
-        undefined,
-        () => {
+      .subscribe({
+        error: () => {
           this.form.setErrors({ cdSubmitButton: true });
         },
-        () => {
-          this.bsModalRef.hide();
+        complete: () => {
+          this.activeModal.close();
           this.submitAction.emit(profile);
         }
-      );
+      });
   }
 
   private createJson() {
