@@ -2,37 +2,42 @@ import { HttpClientTestingModule } from '@angular/common/http/testing';
 import { ComponentFixture, TestBed } from '@angular/core/testing';
 import { AbstractControl } from '@angular/forms';
 import { By } from '@angular/platform-browser';
+import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
 import { ActivatedRoute, Router, Routes } from '@angular/router';
 import { RouterTestingModule } from '@angular/router/testing';
 
-import * as _ from 'lodash';
-import { NgBootstrapFormValidationModule } from 'ng-bootstrap-form-validation';
-import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
-import { TabsetComponent, TabsModule } from 'ngx-bootstrap/tabs';
+import {
+  NgbActiveModal,
+  NgbModalModule,
+  NgbModalRef,
+  NgbNavModule
+} from '@ng-bootstrap/ng-bootstrap';
+import _ from 'lodash';
 import { ToastrModule } from 'ngx-toastr';
 import { of } from 'rxjs';
 
-import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
+import { DashboardNotFoundError } from '~/app/core/error/error';
+import { ErrorComponent } from '~/app/core/error/error.component';
+import { CrushRuleService } from '~/app/shared/api/crush-rule.service';
+import { ErasureCodeProfileService } from '~/app/shared/api/erasure-code-profile.service';
+import { PoolService } from '~/app/shared/api/pool.service';
+import { CriticalConfirmationModalComponent } from '~/app/shared/components/critical-confirmation-modal/critical-confirmation-modal.component';
+import { SelectBadgesComponent } from '~/app/shared/components/select-badges/select-badges.component';
+import { CdFormGroup } from '~/app/shared/forms/cd-form-group';
+import { ErasureCodeProfile } from '~/app/shared/models/erasure-code-profile';
+import { Permission } from '~/app/shared/models/permissions';
+import { PoolFormInfo } from '~/app/shared/models/pool-form-info';
+import { AuthStorageService } from '~/app/shared/services/auth-storage.service';
+import { ModalService } from '~/app/shared/services/modal.service';
+import { TaskWrapperService } from '~/app/shared/services/task-wrapper.service';
+import { SharedModule } from '~/app/shared/shared.module';
 import {
   configureTestBed,
   FixtureHelper,
   FormHelper,
-  i18nProviders,
   Mocks,
   modalServiceShow
-} from '../../../../testing/unit-test-helper';
-import { NotFoundComponent } from '../../../core/not-found/not-found.component';
-import { CrushRuleService } from '../../../shared/api/crush-rule.service';
-import { ErasureCodeProfileService } from '../../../shared/api/erasure-code-profile.service';
-import { PoolService } from '../../../shared/api/pool.service';
-import { CriticalConfirmationModalComponent } from '../../../shared/components/critical-confirmation-modal/critical-confirmation-modal.component';
-import { SelectBadgesComponent } from '../../../shared/components/select-badges/select-badges.component';
-import { CdFormGroup } from '../../../shared/forms/cd-form-group';
-import { ErasureCodeProfile } from '../../../shared/models/erasure-code-profile';
-import { Permission } from '../../../shared/models/permissions';
-import { PoolFormInfo } from '../../../shared/models/pool-form-info';
-import { AuthStorageService } from '../../../shared/services/auth-storage.service';
-import { TaskWrapperService } from '../../../shared/services/task-wrapper.service';
+} from '~/testing/unit-test-helper';
 import { Pool } from '../pool';
 import { PoolModule } from '../pool.module';
 import { PoolFormComponent } from './pool-form.component';
@@ -48,6 +53,8 @@ describe('PoolFormComponent', () => {
   let router: Router;
   let ecpService: ErasureCodeProfileService;
   let crushRuleService: CrushRuleService;
+  let poolPermissions: Permission;
+  let authStorageService: AuthStorageService;
 
   const setPgNum = (pgs: number): AbstractControl => {
     const control = formHelper.setValue('pgNum', pgs);
@@ -71,7 +78,7 @@ describe('PoolFormComponent', () => {
     poolServiceMethod: 'create' | 'update' = 'create'
   ) => {
     spyOn(poolService, poolServiceMethod).and.stub();
-    const taskWrapper = TestBed.get(TaskWrapperService);
+    const taskWrapper = TestBed.inject(TaskWrapperService);
     spyOn(taskWrapper, 'wrapTaskAroundCall').and.callThrough();
     component.submit();
     expect(poolService[poolServiceMethod]).toHaveBeenCalledWith(pool);
@@ -98,13 +105,11 @@ describe('PoolFormComponent', () => {
       compression_algorithms: ['snappy'],
       compression_modes: ['none', 'passive'],
       crush_rules_replicated: [
-        Mocks.getCrushRule({ id: 0, min: 2, max: 4, name: 'rep1', type: 'replicated' }),
-        Mocks.getCrushRule({ id: 1, min: 3, max: 18, name: 'rep2', type: 'replicated' }),
-        Mocks.getCrushRule({ id: 2, min: 1, max: 9, name: 'used_rule', type: 'replicated' })
-      ],
-      crush_rules_erasure: [
-        Mocks.getCrushRule({ id: 3, min: 1, max: 1, name: 'ecp1', type: 'erasure' })
+        Mocks.getCrushRule({ id: 0, name: 'rep1', type: 'replicated' }),
+        Mocks.getCrushRule({ id: 1, name: 'rep2', type: 'replicated' }),
+        Mocks.getCrushRule({ id: 2, name: 'used_rule', type: 'replicated' })
       ],
+      crush_rules_erasure: [Mocks.getCrushRule({ id: 3, name: 'ecp1', type: 'erasure' })],
       erasure_code_profiles: [ecp1],
       pg_autoscale_default_mode: 'off',
       pg_autoscale_modes: ['off', 'warn', 'on'],
@@ -128,99 +133,98 @@ describe('PoolFormComponent', () => {
     formHelper = new FormHelper(form);
   };
 
-  const routes: Routes = [{ path: '404', component: NotFoundComponent }];
+  const routes: Routes = [{ path: '404', component: ErrorComponent }];
 
-  configureTestBed({
-    declarations: [NotFoundComponent],
-    imports: [
-      BrowserAnimationsModule,
-      HttpClientTestingModule,
-      RouterTestingModule.withRoutes(routes),
-      ToastrModule.forRoot(),
-      TabsModule.forRoot(),
-      PoolModule,
-      NgBootstrapFormValidationModule.forRoot()
-    ],
-    providers: [
-      ErasureCodeProfileService,
-      BsModalRef,
-      SelectBadgesComponent,
-      { provide: ActivatedRoute, useValue: { params: of({ name: 'somePoolName' }) } },
-      i18nProviders
-    ]
-  });
+  configureTestBed(
+    {
+      declarations: [ErrorComponent],
+      imports: [
+        BrowserAnimationsModule,
+        HttpClientTestingModule,
+        RouterTestingModule.withRoutes(routes),
+        ToastrModule.forRoot(),
+        NgbNavModule,
+        PoolModule,
+        SharedModule,
+        NgbModalModule
+      ],
+      providers: [
+        ErasureCodeProfileService,
+        NgbActiveModal,
+        SelectBadgesComponent,
+        { provide: ActivatedRoute, useValue: { params: of({ name: 'somePoolName' }) } }
+      ]
+    },
+    [CriticalConfirmationModalComponent]
+  );
 
   let navigationSpy: jasmine.Spy;
 
   beforeEach(() => {
-    poolService = TestBed.get(PoolService);
+    poolService = TestBed.inject(PoolService);
     setInfo();
     spyOn(poolService, 'getInfo').and.callFake(() => of(infoReturn));
 
-    ecpService = TestBed.get(ErasureCodeProfileService);
-    crushRuleService = TestBed.get(CrushRuleService);
+    ecpService = TestBed.inject(ErasureCodeProfileService);
+    crushRuleService = TestBed.inject(CrushRuleService);
 
-    router = TestBed.get(Router);
+    router = TestBed.inject(Router);
     navigationSpy = spyOn(router, 'navigate').and.stub();
-
+    authStorageService = TestBed.inject(AuthStorageService);
+    spyOn(authStorageService, 'getPermissions').and.callFake(() => ({
+      pool: poolPermissions
+    }));
+    poolPermissions = new Permission(['update', 'delete', 'read', 'create']);
     setUpPoolComponent();
+
+    component.loadingReady();
   });
 
   it('should create', () => {
     expect(component).toBeTruthy();
   });
 
-  describe('redirect not allowed users', () => {
-    let poolPermissions: Permission;
-    let authStorageService: AuthStorageService;
-
-    const expectRedirect = (redirected = true) => {
+  describe('throws error for not allowed users', () => {
+    const expectError = (redirected: boolean) => {
       navigationSpy.calls.reset();
-      component.authenticate();
-      expect(navigationSpy).toHaveBeenCalledTimes(redirected ? 1 : 0);
+      if (redirected) {
+        expect(() => component.authenticate()).toThrowError(DashboardNotFoundError);
+      } else {
+        expect(() => component.authenticate()).not.toThrowError();
+      }
     };
 
     beforeEach(() => {
-      poolPermissions = {
-        create: false,
-        update: false,
-        read: false,
-        delete: false
-      };
-      authStorageService = TestBed.get(AuthStorageService);
-      spyOn(authStorageService, 'getPermissions').and.callFake(() => ({
-        pool: poolPermissions
-      }));
+      poolPermissions = new Permission(['delete']);
     });
 
-    it('navigates to 404 if not allowed', () => {
-      component.authenticate();
-      expect(router.navigate).toHaveBeenCalledWith(['/404']);
+    it('navigates to Dashboard if not allowed', () => {
+      expect(() => component.authenticate()).toThrowError(DashboardNotFoundError);
     });
 
-    it('navigates if user is not allowed', () => {
-      expectRedirect();
+    it('throws error if user is not allowed', () => {
+      expectError(true);
       poolPermissions.read = true;
-      expectRedirect();
+      expectError(true);
       poolPermissions.delete = true;
-      expectRedirect();
+      expectError(true);
       poolPermissions.update = true;
-      expectRedirect();
+      expectError(true);
       component.editing = true;
       poolPermissions.update = false;
       poolPermissions.create = true;
-      expectRedirect();
+      expectError(true);
     });
 
-    it('does not navigate users with right permissions', () => {
+    it('does not throw error for users with right permissions', () => {
       poolPermissions.read = true;
       poolPermissions.create = true;
-      expectRedirect(false);
+      expectError(false);
       component.editing = true;
       poolPermissions.update = true;
-      expectRedirect(false);
+      expectError(false);
       poolPermissions.create = false;
-      expectRedirect(false);
+      expectError(false);
     });
   });
 
@@ -283,7 +287,6 @@ describe('PoolFormComponent', () => {
       formHelper.expectValidChange('poolType', 'replicated');
       form.get('crushRule').updateValueAndValidity();
       formHelper.expectError('crushRule', 'required'); // As multiple rules exist
-      formHelper.expectErrorChange('crushRule', { min_size: 20 }, 'tooFewOsds');
     });
 
     it('validates crushRule with no crush rules', () => {
@@ -297,17 +300,6 @@ describe('PoolFormComponent', () => {
       component.info.nodes = Mocks.getCrushMap();
       formHelper.setValue('poolType', 'replicated');
       formHelper.expectValid('size');
-      formHelper.setValue('crushRule', Mocks.getCrushRule({ min: 2, max: 6 })); // 3 OSDs usable
-      formHelper.expectErrorChange('size', 1, 'min');
-      formHelper.expectErrorChange('size', 4, 'max'); // More than usable
-      formHelper.expectValidChange('size', 3);
-
-      formHelper.setValue(
-        'crushRule',
-        Mocks.getCrushRule({ min: 1, max: 2, failureDomain: 'osd-rack' }) // 4 OSDs usable
-      );
-      formHelper.expectErrorChange('size', 4, 'max'); // More than rule allows
-      formHelper.expectValidChange('size', 2);
     });
 
     it('validates if warning is displayed when size is 1', () => {
@@ -454,13 +446,13 @@ describe('PoolFormComponent', () => {
 
       it('should set size to maximum if size exceeds maximum', () => {
         formHelper.setValue('crushRule', component.info.crush_rules_replicated[0]);
-        expect(form.getValue('size')).toBe(4);
+        expect(form.getValue('size')).toBe(10);
       });
 
       it('should set size to minimum if size is lower than minimum', () => {
         formHelper.setValue('size', -1);
         formHelper.setValue('crushRule', component.info.crush_rules_replicated[0]);
-        expect(form.getValue('size')).toBe(2);
+        expect(form.getValue('size')).toBe(1);
       });
     });
 
@@ -490,7 +482,7 @@ describe('PoolFormComponent', () => {
 
       it('disables rule field if only one rule exists which is used in the disabled field', () => {
         infoReturn.crush_rules_replicated = [
-          Mocks.getCrushRule({ id: 0, min: 2, max: 4, name: 'rep1', type: 'replicated' })
+          Mocks.getCrushRule({ id: 0, name: 'rep1', type: 'replicated' })
         ];
         setUpPoolComponent();
         formHelper.setValue('poolType', 'replicated');
@@ -519,10 +511,6 @@ describe('PoolFormComponent', () => {
   });
 
   describe('getMaxSize and getMinSize', () => {
-    const setCrushRule = ({ min, max }: { min?: number; max?: number }) => {
-      formHelper.setValue('crushRule', Mocks.getCrushRule({ min, max }));
-    };
-
     it('returns 0 if osd count is 0', () => {
       component.info.osd_count = 0;
       expect(component.getMinSize()).toBe(0);
@@ -535,36 +523,22 @@ describe('PoolFormComponent', () => {
       expect(component.getMaxSize()).toBe(0);
     });
 
-    it('returns minimum and maximum of rule', () => {
-      setCrushRule({ min: 2, max: 6 });
-      expect(component.getMinSize()).toBe(2);
-      expect(component.getMaxSize()).toBe(6);
-    });
-
     it('returns 1 as minimum and 3 as maximum if no crush rule is available', () => {
       expect(component.getMinSize()).toBe(1);
       expect(component.getMaxSize()).toBe(3);
     });
 
-    it('returns the osd count as maximum if the rule maximum exceeds it', () => {
-      setCrushRule({ max: 100 });
-      expect(component.getMaxSize()).toBe(15);
-    });
-
     it('should return the osd count as minimum if its lower the the rule minimum', () => {
-      setCrushRule({ min: 20 });
-      expect(component.getMinSize()).toBe(20);
+      component.info.osd_count = 0;
+      formHelper.setValue('crushRule', component.info.crush_rules_replicated[0]);
       const control = form.get('crushRule');
       expect(control.invalid).toBe(true);
       formHelper.expectError(control, 'tooFewOsds');
     });
 
     it('should get the right maximum if the device type is defined', () => {
-      formHelper.setValue(
-        'crushRule',
-        Mocks.getCrushRule({ min: 1, max: 5, itemName: 'default~ssd' })
-      );
-      expect(form.getValue('crushRule').usable_size).toBe(5);
+      formHelper.setValue('crushRule', Mocks.getCrushRule({ itemName: 'default~ssd' }));
+      expect(form.getValue('crushRule').usable_size).toBe(10);
     });
   });
 
@@ -791,9 +765,9 @@ describe('PoolFormComponent', () => {
     it('should select the newly created rule', () => {
       expect(form.getValue('crushRule').rule_name).toBe('rep1');
       const name = 'awesomeRule';
-      spyOn(TestBed.get(BsModalService), 'show').and.callFake(() => {
+      spyOn(TestBed.inject(ModalService), 'show').and.callFake(() => {
         return {
-          content: {
+          componentInstance: {
             submitAction: of({ name })
           }
         };
@@ -854,11 +828,11 @@ describe('PoolFormComponent', () => {
       };
 
       beforeEach(() => {
-        modalSpy = spyOn(TestBed.get(BsModalService), 'show').and.callFake(
-          (deletionClass: any, config: any) => {
-            deletion = Object.assign(new deletionClass(), config.initialState);
+        modalSpy = spyOn(TestBed.inject(ModalService), 'show').and.callFake(
+          (deletionClass: any, initialState: any) => {
+            deletion = Object.assign(new deletionClass(), initialState);
             return {
-              content: deletion
+              componentInstance: deletion
             };
           }
         );
@@ -868,7 +842,7 @@ describe('PoolFormComponent', () => {
           rules.splice(index, 1);
           return of(undefined);
         });
-        taskWrapper = TestBed.get(TaskWrapperService);
+        taskWrapper = TestBed.inject(TaskWrapperService);
         spyOn(taskWrapper, 'wrapTaskAroundCall').and.callThrough();
       });
 
@@ -882,7 +856,7 @@ describe('PoolFormComponent', () => {
         });
 
         it('should not open the tooltip nor the crush info', () => {
-          expect(component.crushDeletionBtn.isOpen).toBe(false);
+          expect(component.crushDeletionBtn.isOpen()).toBe(false);
           expect(component.data.crushInfo).toBe(false);
         });
 
@@ -897,7 +871,6 @@ describe('PoolFormComponent', () => {
       describe('rule in use', () => {
         beforeEach(() => {
           spyOn(global, 'setTimeout').and.callFake((fn: Function) => fn());
-          component.crushInfoTabs = { tabs: [{}, {}, {}] } as TabsetComponent; // Mock it
           deleteSpy.calls.reset();
           selectRuleByIndex(2);
           component.deleteCrushRule();
@@ -905,34 +878,28 @@ describe('PoolFormComponent', () => {
 
         it('should not have called delete and opened the tooltip', () => {
           expect(crushRuleService.delete).not.toHaveBeenCalled();
-          expect(component.crushDeletionBtn.isOpen).toBe(true);
+          expect(component.crushDeletionBtn.isOpen()).toBe(true);
           expect(component.data.crushInfo).toBe(true);
         });
 
-        it('should open the third crush info tab', () => {
-          expect(component.crushInfoTabs).toEqual({
-            tabs: [{}, {}, { active: true }]
-          } as TabsetComponent);
-        });
-
         it('should hide the tooltip when clicking on delete again', () => {
           component.deleteCrushRule();
-          expect(component.crushDeletionBtn.isOpen).toBe(false);
+          expect(component.crushDeletionBtn.isOpen()).toBe(false);
         });
 
         it('should hide the tooltip when clicking on add', () => {
           modalSpy.and.callFake((): any => ({
-            content: {
+            componentInstance: {
               submitAction: of('someRule')
             }
           }));
           component.addCrushRule();
-          expect(component.crushDeletionBtn.isOpen).toBe(false);
+          expect(component.crushDeletionBtn.isOpen()).toBe(false);
         });
 
         it('should hide the tooltip when changing the crush rule', () => {
           selectRuleByIndex(0);
-          expect(component.crushDeletionBtn.isOpen).toBe(false);
+          expect(component.crushDeletionBtn.isOpen()).toBe(false);
         });
       });
     });
@@ -966,9 +933,9 @@ describe('PoolFormComponent', () => {
       spyOn(ecpService, 'list').and.callFake(() => of(infoReturn.erasure_code_profiles));
       expect(form.getValue('erasureProfile').name).toBe('ecp1');
       const name = 'awesomeProfile';
-      spyOn(TestBed.get(BsModalService), 'show').and.callFake(() => {
+      spyOn(TestBed.inject(ModalService), 'show').and.callFake(() => {
         return {
-          content: {
+          componentInstance: {
             submitAction: of({ name })
           }
         };
@@ -985,11 +952,11 @@ describe('PoolFormComponent', () => {
       let deletion: CriticalConfirmationModalComponent;
       let deleteSpy: jasmine.Spy;
       let modalSpy: jasmine.Spy;
-      let modal: any;
+      let modal: NgbModalRef;
 
       const callEcpDeletion = () => {
         component.deleteErasureCodeProfile();
-        modal.ref.content.callSubmitAction();
+        modal.componentInstance.callSubmitAction();
       };
 
       const expectSuccessfulEcpDeletion = (name: string) => {
@@ -1010,10 +977,10 @@ describe('PoolFormComponent', () => {
 
       beforeEach(() => {
         deletion = undefined;
-        modalSpy = spyOn(TestBed.get(BsModalService), 'show').and.callFake(
+        modalSpy = spyOn(TestBed.inject(ModalService), 'show').and.callFake(
           (comp: any, init: any) => {
             modal = modalServiceShow(comp, init);
-            return modal.ref;
+            return modal;
           }
         );
         deleteSpy = spyOn(ecpService, 'delete').and.callFake((name: string) => {
@@ -1022,7 +989,7 @@ describe('PoolFormComponent', () => {
           profiles.splice(index, 1);
           return of({ status: 202 });
         });
-        taskWrapper = TestBed.get(TaskWrapperService);
+        taskWrapper = TestBed.inject(TaskWrapperService);
         spyOn(taskWrapper, 'wrapTaskAroundCall').and.callThrough();
 
         const ecp2 = new ErasureCodeProfile();
@@ -1045,7 +1012,7 @@ describe('PoolFormComponent', () => {
         });
 
         it('should not open the tooltip nor the crush info', () => {
-          expect(component.ecpDeletionBtn.isOpen).toBe(false);
+          expect(component.ecpDeletionBtn.isOpen()).toBe(false);
           expect(component.data.erasureInfo).toBe(false);
         });
 
@@ -1060,7 +1027,6 @@ describe('PoolFormComponent', () => {
       describe('rule in use', () => {
         beforeEach(() => {
           spyOn(global, 'setTimeout').and.callFake((fn: Function) => fn());
-          component.ecpInfoTabs = { tabs: [{}, {}] } as TabsetComponent; // Mock it
           deleteSpy.calls.reset();
           setSelectedEcp('ecp1');
           component.deleteErasureCodeProfile();
@@ -1072,34 +1038,28 @@ describe('PoolFormComponent', () => {
 
         it('should not have called delete and opened the tooltip', () => {
           expect(ecpService.delete).not.toHaveBeenCalled();
-          expect(component.ecpDeletionBtn.isOpen).toBe(true);
+          expect(component.ecpDeletionBtn.isOpen()).toBe(true);
           expect(component.data.erasureInfo).toBe(true);
         });
 
-        it('should open the third crush info tab', () => {
-          expect(component.ecpInfoTabs).toEqual({
-            tabs: [{}, { active: true }]
-          } as TabsetComponent);
-        });
-
         it('should hide the tooltip when clicking on delete again', () => {
           component.deleteErasureCodeProfile();
-          expect(component.ecpDeletionBtn.isOpen).toBe(false);
+          expect(component.ecpDeletionBtn.isOpen()).toBe(false);
         });
 
         it('should hide the tooltip when clicking on add', () => {
           modalSpy.and.callFake((): any => ({
-            content: {
+            componentInstance: {
               submitAction: of('someProfile')
             }
           }));
           component.addErasureCodeProfile();
-          expect(component.ecpDeletionBtn.isOpen).toBe(false);
+          expect(component.ecpDeletionBtn.isOpen()).toBe(false);
         });
 
         it('should hide the tooltip when changing the crush rule', () => {
           setSelectedEcp('someEcpName');
-          expect(component.ecpDeletionBtn.isOpen).toBe(false);
+          expect(component.ecpDeletionBtn.isOpen()).toBe(false);
         });
       });
     });
