@@ -1,5 +1,5 @@
-import { Component, OnInit } from '@angular/core';
-import { Validators } from '@angular/forms';
+import { AfterViewChecked, ChangeDetectorRef, Component, OnInit } from '@angular/core';
+import { AbstractControl, Validators } from '@angular/forms';
 import { ActivatedRoute, Router } from '@angular/router';
 
 import _ from 'lodash';
@@ -15,19 +15,24 @@ import { CdForm } from '~/app/shared/forms/cd-form';
 import { CdFormBuilder } from '~/app/shared/forms/cd-form-builder';
 import { CdFormGroup } from '~/app/shared/forms/cd-form-group';
 import { CdValidators } from '~/app/shared/forms/cd-validators';
+import { ModalService } from '~/app/shared/services/modal.service';
 import { NotificationService } from '~/app/shared/services/notification.service';
+import { RgwBucketEncryptionModel } from '../models/rgw-bucket-encryption';
 import { RgwBucketMfaDelete } from '../models/rgw-bucket-mfa-delete';
 import { RgwBucketVersioning } from '../models/rgw-bucket-versioning';
+import { RgwConfigModalComponent } from '../rgw-config-modal/rgw-config-modal.component';
 
 @Component({
   selector: 'cd-rgw-bucket-form',
   templateUrl: './rgw-bucket-form.component.html',
-  styleUrls: ['./rgw-bucket-form.component.scss']
+  styleUrls: ['./rgw-bucket-form.component.scss'],
+  providers: [RgwBucketEncryptionModel]
 })
-export class RgwBucketFormComponent extends CdForm implements OnInit {
+export class RgwBucketFormComponent extends CdForm implements OnInit, AfterViewChecked {
   bucketForm: CdFormGroup;
   editing = false;
   owners: string[] = null;
+  kmsProviders: string[] = null;
   action: string;
   resource: string;
   zonegroup: string;
@@ -35,6 +40,8 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
   isVersioningAlreadyEnabled = false;
   isMfaDeleteAlreadyEnabled = false;
   icons = Icons;
+  kmsVaultConfig = false;
+  s3VaultConfig = false;
 
   get isVersioningEnabled(): boolean {
     return this.bucketForm.getValue('versioning');
@@ -49,9 +56,12 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
     private formBuilder: CdFormBuilder,
     private rgwBucketService: RgwBucketService,
     private rgwSiteService: RgwSiteService,
+    private modalService: ModalService,
     private rgwUserService: RgwUserService,
     private notificationService: NotificationService,
-    public actionLabels: ActionLabelsI18n
+    private rgwEncryptionModal: RgwBucketEncryptionModel,
+    public actionLabels: ActionLabelsI18n,
+    private readonly changeDetectorRef: ChangeDetectorRef
   ) {
     super();
     this.editing = this.router.url.startsWith(`/rgw/bucket/${URLVerbs.EDIT}`);
@@ -60,6 +70,10 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
     this.createForm();
   }
 
+  ngAfterViewChecked(): void {
+    this.changeDetectorRef.detectChanges();
+  }
+
   createForm() {
     const self = this;
     const lockDaysValidator = CdValidators.custom('lockDays', () => {
@@ -79,12 +93,31 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
           : [CdValidators.bucketName(), CdValidators.bucketExistence(false, this.rgwBucketService)]
       ],
       owner: [null, [Validators.required]],
+      kms_provider: ['vault'],
       'placement-target': [null, this.editing ? [] : [Validators.required]],
       versioning: [null],
       'mfa-delete': [null],
       'mfa-token-serial': [''],
       'mfa-token-pin': [''],
       lock_enabled: [{ value: false, disabled: this.editing }],
+      encryption_enabled: [null],
+      encryption_type: [
+        null,
+        [
+          CdValidators.requiredIf({
+            encryption_enabled: true
+          })
+        ]
+      ],
+      keyId: [
+        null,
+        [
+          CdValidators.requiredIf({
+            encryption_type: 'aws:kms',
+            encryption_enabled: true
+          })
+        ]
+      ],
       lock_mode: ['COMPLIANCE'],
       lock_retention_period_days: [0, [CdValidators.number(false), lockDaysValidator]]
     });
@@ -95,6 +128,21 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
       owners: this.rgwUserService.enumerate()
     };
 
+    this.kmsProviders = this.rgwEncryptionModal.kmsProviders;
+    this.rgwBucketService.getEncryptionConfig().subscribe((data) => {
+      this.kmsVaultConfig = data[0];
+      this.s3VaultConfig = data[1];
+      if (this.kmsVaultConfig && this.s3VaultConfig) {
+        this.bucketForm.get('encryption_type').setValue('');
+      } else if (this.kmsVaultConfig) {
+        this.bucketForm.get('encryption_type').setValue('aws:kms');
+      } else if (this.s3VaultConfig) {
+        this.bucketForm.get('encryption_type').setValue('AES256');
+      } else {
+        this.bucketForm.get('encryption_type').setValue('');
+      }
+    });
+
     if (!this.editing) {
       promises['getPlacementTargets'] = this.rgwSiteService.get('placement-targets');
     }
@@ -137,14 +185,14 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
           // the Angular react framework will throw an error if there is no
           // field for a given key.
           let value: object = _.pick(bidResp, _.keys(defaults));
+
           value['lock_retention_period_days'] = this.rgwBucketService.getLockDays(bidResp);
           value['placement-target'] = bidResp['placement_rule'];
           value['versioning'] = bidResp['versioning'] === RgwBucketVersioning.ENABLED;
           value['mfa-delete'] = bidResp['mfa_delete'] === RgwBucketMfaDelete.ENABLED;
-
+          value['encryption_enabled'] = bidResp['encryption'] === 'Enabled';
           // Append default values.
           value = _.merge(defaults, value);
-
           // Update the form.
           this.bucketForm.setValue(value);
           if (this.editing) {
@@ -156,7 +204,6 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
             }
           }
         }
-
         this.loadingReady();
       });
     });
@@ -168,6 +215,10 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
 
   submit() {
     // Exit immediately if the form isn't dirty.
+    if (this.bucketForm.getValue('encryption_enabled') == null) {
+      this.bucketForm.get('encryption_enabled').setValue(false);
+      this.bucketForm.get('encryption_type').setValue(null);
+    }
     if (this.bucketForm.pristine) {
       this.goToListView();
       return;
@@ -183,6 +234,9 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
           values['id'],
           values['owner'],
           versioning,
+          values['encryption_enabled'],
+          values['encryption_type'],
+          values['keyId'],
           mfaDelete,
           values['mfa-token-serial'],
           values['mfa-token-pin'],
@@ -212,7 +266,10 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
           values['placement-target'],
           values['lock_enabled'],
           values['lock_mode'],
-          values['lock_retention_period_days']
+          values['lock_retention_period_days'],
+          values['encryption_enabled'],
+          values['encryption_type'],
+          values['keyId']
         )
         .subscribe(
           () => {
@@ -261,4 +318,23 @@ export class RgwBucketFormComponent extends CdForm implements OnInit {
   getMfaDeleteStatus() {
     return this.isMfaDeleteEnabled ? RgwBucketMfaDelete.ENABLED : RgwBucketMfaDelete.DISABLED;
   }
+
+  fileUpload(files: FileList, controlName: string) {
+    const file: File = files[0];
+    const reader = new FileReader();
+    reader.addEventListener('load', () => {
+      const control: AbstractControl = this.bucketForm.get(controlName);
+      control.setValue(file);
+      control.markAsDirty();
+      control.markAsTouched();
+      control.updateValueAndValidity();
+    });
+  }
+
+  openConfigModal() {
+    const modalRef = this.modalService.show(RgwConfigModalComponent, null, { size: 'lg' });
+    modalRef.componentInstance.configForm
+      .get('encryptionType')
+      .setValue(this.bucketForm.getValue('encryption_type') || 'AES256');
+  }
 }
