@@ -4,15 +4,20 @@ import { ComponentFixture, TestBed } from '@angular/core/testing';
 import { By } from '@angular/platform-browser';
 import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
 
-import * as _ from 'lodash';
-import { TabsetComponent, TabsetConfig, TabsModule } from 'ngx-bootstrap/tabs';
+import { NgbNavModule } from '@ng-bootstrap/ng-bootstrap';
+import _ from 'lodash';
 import { NgxPipeFunctionModule } from 'ngx-pipe-function';
 import { of } from 'rxjs';
 
-import { configureTestBed, i18nProviders } from '../../../../testing/unit-test-helper';
-import { OsdService } from '../../../shared/api/osd.service';
-import { HddSmartDataV1, NvmeSmartDataV1, SmartDataResult } from '../../../shared/models/smart';
-import { SharedModule } from '../../../shared/shared.module';
+import { OsdService } from '~/app/shared/api/osd.service';
+import {
+  AtaSmartDataV1,
+  IscsiSmartDataV1,
+  NvmeSmartDataV1,
+  SmartDataResult
+} from '~/app/shared/models/smart';
+import { SharedModule } from '~/app/shared/shared.module';
+import { configureTestBed } from '~/testing/unit-test-helper';
 import { SmartListComponent } from './smart-list.component';
 
 describe('OsdSmartListComponent', () => {
@@ -20,8 +25,9 @@ describe('OsdSmartListComponent', () => {
   let fixture: ComponentFixture<SmartListComponent>;
   let osdService: OsdService;
 
-  const SMART_DATA_HDD_VERSION_1_0: HddSmartDataV1 = require('./fixtures/smart_data_version_1_0_hdd_response.json');
+  const SMART_DATA_ATA_VERSION_1_0: AtaSmartDataV1 = require('./fixtures/smart_data_version_1_0_ata_response.json');
   const SMART_DATA_NVME_VERSION_1_0: NvmeSmartDataV1 = require('./fixtures/smart_data_version_1_0_nvme_response.json');
+  const SMART_DATA_SCSI_VERSION_1_0: IscsiSmartDataV1 = require('./fixtures/smart_data_version_1_0_scsi_response.json');
 
   /**
    * Sets attributes for _all_ returned devices according to the given path. The syntax is the same
@@ -38,7 +44,7 @@ describe('OsdSmartListComponent', () => {
    */
   const patchData = (path: string, newValue: any): any => {
     return _.reduce(
-      _.cloneDeep(SMART_DATA_HDD_VERSION_1_0),
+      _.cloneDeep(SMART_DATA_ATA_VERSION_1_0),
       (result: object, dataObj, deviceId) => {
         result[deviceId] = _.set<any>(dataObj, path, newValue);
         return result;
@@ -52,18 +58,21 @@ describe('OsdSmartListComponent', () => {
    * of `OsdService`. Determines which data is returned.
    */
   const initializeComponentWithData = (
-    dataType: 'hdd_v1' | 'nvme_v1',
+    dataType: 'hdd_v1' | 'nvme_v1' | 'hdd_v1_scsi',
     patch: { [path: string]: any } = null,
     simpleChanges?: SimpleChanges
   ) => {
-    let data: HddSmartDataV1 | NvmeSmartDataV1;
+    let data: AtaSmartDataV1 | NvmeSmartDataV1 | IscsiSmartDataV1;
     switch (dataType) {
       case 'hdd_v1':
-        data = SMART_DATA_HDD_VERSION_1_0;
+        data = SMART_DATA_ATA_VERSION_1_0;
         break;
       case 'nvme_v1':
         data = SMART_DATA_NVME_VERSION_1_0;
         break;
+      case 'hdd_v1_scsi':
+        data = SMART_DATA_SCSI_VERSION_1_0;
+        break;
     }
 
     if (_.isObject(patch)) {
@@ -112,12 +121,11 @@ describe('OsdSmartListComponent', () => {
     declarations: [SmartListComponent],
     imports: [
       BrowserAnimationsModule,
-      TabsModule,
       SharedModule,
       HttpClientTestingModule,
+      NgbNavModule,
       NgxPipeFunctionModule
-    ],
-    providers: [i18nProviders, TabsetComponent, TabsetConfig]
+    ]
   });
 
   beforeEach(() => {
@@ -125,14 +133,14 @@ describe('OsdSmartListComponent', () => {
     component = fixture.componentInstance;
     fixture.detectChanges();
 
-    osdService = TestBed.get(OsdService);
+    osdService = TestBed.inject(OsdService);
   });
 
   it('should create', () => {
     expect(component).toBeTruthy();
   });
 
-  describe('tests HDD version 1.x', () => {
+  describe('tests ATA version 1.x', () => {
     beforeEach(() => initializeComponentWithData('hdd_v1'));
 
     it('should return with proper keys', () => {
@@ -170,6 +178,23 @@ describe('OsdSmartListComponent', () => {
     });
   });
 
+  describe('tests SCSI version 1.x', () => {
+    beforeEach(() => initializeComponentWithData('hdd_v1_scsi'));
+
+    it('should return with proper keys', () => {
+      _.each(component.data, (smartData, _deviceId) => {
+        expect(_.keys(smartData)).toEqual(['info', 'smart', 'device', 'identifier']);
+      });
+    });
+
+    it('should not contain excluded keys in `info`', () => {
+      const excludes = ['scsi_error_counter_log', 'scsi_grown_defect_list'];
+      _.each(component.data, (smartData: SmartDataResult, _deviceId) => {
+        _.each(excludes, (exclude) => expect(smartData.info[exclude]).toBeUndefined());
+      });
+    });
+  });
+
   it('should not work for version 2.x', () => {
     initializeComponentWithData('nvme_v1', { json_format_version: [2, 0] });
     expect(component.data).toEqual({});
@@ -214,10 +239,10 @@ describe('OsdSmartListComponent', () => {
     const deviceId: string = _.keys(component.data)[0];
     component.data[deviceId]['info'] = {};
     fixture.detectChanges();
-    component.innerTabset.tabs[0].active = true;
+    component.nav.select(1);
     fixture.detectChanges();
     verifyAlertPanel(
-      'tab.active cd-alert-panel#alert-device-info-unavailable',
+      'cd-alert-panel#alert-device-info-unavailable',
       'No device information available for this device.',
       'info'
     );
@@ -228,10 +253,10 @@ describe('OsdSmartListComponent', () => {
     const deviceId: string = _.keys(component.data)[0];
     component.data[deviceId]['smart'] = {};
     fixture.detectChanges();
-    component.innerTabset.tabs[1].active = true;
+    component.nav.select(2);
     fixture.detectChanges();
     verifyAlertPanel(
-      'tab.active cd-alert-panel#alert-device-smart-data-unavailable',
+      'cd-alert-panel#alert-device-smart-data-unavailable',
       'No SMART data available for this device.',
       'info'
     );
