@@ -2,25 +2,33 @@ import { CommonModule } from '@angular/common';
 import { NgModule } from '@angular/core';
 import { RouterModule } from '@angular/router';
 
+import { NgbDropdownModule } from '@ng-bootstrap/ng-bootstrap';
 import { BlockUIModule } from 'ng-block-ui';
 
-import { SharedModule } from '../shared/shared.module';
-import { ForbiddenComponent } from './forbidden/forbidden.component';
+import { ContextComponent } from '~/app/core/context/context.component';
+import { SharedModule } from '~/app/shared/shared.module';
+import { ErrorComponent } from './error/error.component';
 import { BlankLayoutComponent } from './layouts/blank-layout/blank-layout.component';
 import { LoginLayoutComponent } from './layouts/login-layout/login-layout.component';
 import { WorkbenchLayoutComponent } from './layouts/workbench-layout/workbench-layout.component';
 import { NavigationModule } from './navigation/navigation.module';
-import { NotFoundComponent } from './not-found/not-found.component';
 
 @NgModule({
-  imports: [BlockUIModule.forRoot(), CommonModule, NavigationModule, RouterModule, SharedModule],
+  imports: [
+    BlockUIModule.forRoot(),
+    CommonModule,
+    NavigationModule,
+    NgbDropdownModule,
+    RouterModule,
+    SharedModule
+  ],
   exports: [NavigationModule],
   declarations: [
-    NotFoundComponent,
-    ForbiddenComponent,
+    ContextComponent,
     WorkbenchLayoutComponent,
     BlankLayoutComponent,
-    LoginLayoutComponent
+    LoginLayoutComponent,
+    ErrorComponent
   ]
 })
 export class CoreModule {}
