@@ -1,45 +1,35 @@
 import { Component, OnDestroy, OnInit } from '@angular/core';
 import { Router } from '@angular/router';
 
-import { TooltipConfig } from 'ngx-bootstrap/tooltip';
 import { Subscription } from 'rxjs';
 
-import { SummaryService } from '../../../shared/services/summary.service';
-import { TaskManagerService } from '../../../shared/services/task-manager.service';
+import { FaviconService } from '~/app/shared/services/favicon.service';
+import { SummaryService } from '~/app/shared/services/summary.service';
+import { TaskManagerService } from '~/app/shared/services/task-manager.service';
 
 @Component({
   selector: 'cd-workbench-layout',
   templateUrl: './workbench-layout.component.html',
   styleUrls: ['./workbench-layout.component.scss'],
-  providers: [
-    {
-      provide: TooltipConfig,
-      useFactory: (): TooltipConfig =>
-        Object.assign(new TooltipConfig(), {
-          container: 'body'
-        })
-    }
-  ]
+  providers: [FaviconService]
 })
 export class WorkbenchLayoutComponent implements OnInit, OnDestroy {
   private subs = new Subscription();
 
   constructor(
-    private router: Router,
+    public router: Router,
     private summaryService: SummaryService,
-    private taskManagerService: TaskManagerService
+    private taskManagerService: TaskManagerService,
+    private faviconService: FaviconService
   ) {}
 
   ngOnInit() {
     this.subs.add(this.summaryService.startPolling());
     this.subs.add(this.taskManagerService.init(this.summaryService));
+    this.faviconService.init();
   }
 
   ngOnDestroy() {
     this.subs.unsubscribe();
   }
-
-  isDashboardPage() {
-    return this.router.url === '/dashboard';
-  }
 }
