@@ -1,15 +1,15 @@
-import { Component, OnInit } from '@angular/core';
+import { Component } from '@angular/core';
 
-import { Icons } from '../../../shared/enum/icons.enum';
-import { Permission } from '../../../shared/models/permissions';
-import { AuthStorageService } from '../../../shared/services/auth-storage.service';
+import { Icons } from '~/app/shared/enum/icons.enum';
+import { Permission } from '~/app/shared/models/permissions';
+import { AuthStorageService } from '~/app/shared/services/auth-storage.service';
 
 @Component({
   selector: 'cd-administration',
   templateUrl: './administration.component.html',
   styleUrls: ['./administration.component.scss']
 })
-export class AdministrationComponent implements OnInit {
+export class AdministrationComponent {
   userPermission: Permission;
   configOptPermission: Permission;
   icons = Icons;
@@ -19,6 +19,4 @@ export class AdministrationComponent implements OnInit {
     this.userPermission = permissions.user;
     this.configOptPermission = permissions.configOpt;
   }
-
-  ngOnInit() {}
 }
