@@ -4,11 +4,10 @@ import { Injectable } from '@angular/core';
 import { Observable, of as observableOf } from 'rxjs';
 import { mergeMap } from 'rxjs/operators';
 
-import { RoleFormModel } from '../../core/auth/role-form/role-form.model';
-import { ApiModule } from './api.module';
+import { RoleFormModel } from '~/app/core/auth/role-form/role-form.model';
 
 @Injectable({
-  providedIn: ApiModule
+  providedIn: 'root'
 })
 export class RoleService {
   constructor(private http: HttpClient) {}
