@@ -1,8 +1,8 @@
 import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
 import { TestBed } from '@angular/core/testing';
 
-import { configureTestBed } from '../../../testing/unit-test-helper';
-import { UserFormModel } from '../../core/auth/user-form/user-form.model';
+import { UserFormModel } from '~/app/core/auth/user-form/user-form.model';
+import { configureTestBed } from '~/testing/unit-test-helper';
 import { UserService } from './user.service';
 
 describe('UserService', () => {
@@ -15,8 +15,8 @@ describe('UserService', () => {
   });
 
   beforeEach(() => {
-    service = TestBed.get(UserService);
-    httpTesting = TestBed.get(HttpTestingController);
+    service = TestBed.inject(UserService);
+    httpTesting = TestBed.inject(HttpTestingController);
   });
 
   afterEach(() => {
