@@ -28,7 +28,8 @@ class Features(Enum):
     DASHBOARD = 'dashboard'
 
 
-PREDISABLED_FEATURES = set()  # type: Set[str]
+PREDISABLED_FEATURES = set(
+    [Features.DASHBOARD, Features.DASHBOARD.value])  # type: Set[Features | str]
 
 Feature2Controller = {
     Features.RBD: [Rbd, RbdSnapshot, RbdTrash],
