@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import
 
-from . import CLICommandTestMixin, ControllerTestCase  # pylint: disable=no-name-in-module
+from ..tests import CLICommandTestMixin, ControllerTestCase
 
 
 class TestPluginDebug(ControllerTestCase, CLICommandTestMixin):
