@@ -251,7 +251,8 @@ EXPORT {
                 mock.patch('nfs.export.check_fs', return_value=True), \
                 mock.patch('nfs.ganesha_conf.check_fs', return_value=True), \
                 mock.patch('nfs.export.ExportMgr._create_user_key',
-                           return_value='thekeyforclientabc'):
+                           return_value='thekeyforclientabc'), \
+                mock.patch('nfs.export.cephfs_path_is_dir'):
 
             rados.open_ioctx.return_value.__enter__.return_value = self.io_mock
             rados.open_ioctx.return_value.__exit__ = mock.Mock(return_value=None)
