@@ -22,13 +22,15 @@
 
 #include <include/types.h>
 #include <common/Formatter.h>
+#include <common/dout.h>
 #include "rgw_xml.h"
 #include "rgw_cors.h"
 
 class RGWCORSRule_S3 : public RGWCORSRule, public XMLObj
 {
+  const DoutPrefixProvider *dpp;
   public:
-    RGWCORSRule_S3() {}
+    RGWCORSRule_S3(const DoutPrefixProvider *dpp) : dpp(dpp) {}
     ~RGWCORSRule_S3() override {}
     
     bool xml_end(const char *el) override;
@@ -37,20 +39,22 @@ class RGWCORSRule_S3 : public RGWCORSRule, public XMLObj
 
 class RGWCORSConfiguration_S3 : public RGWCORSConfiguration, public XMLObj
 {
+  const DoutPrefixProvider *dpp;
   public:
-    RGWCORSConfiguration_S3() {}
+    RGWCORSConfiguration_S3(const DoutPrefixProvider *dpp) : dpp(dpp) {}
     ~RGWCORSConfiguration_S3() override {}
 
     bool xml_end(const char *el) override;
-    void to_xml(ostream& out);
+    void to_xml(std::ostream& out);
 };
 
 class RGWCORSXMLParser_S3 : public RGWXMLParser
 {
+  const DoutPrefixProvider *dpp;
   CephContext *cct;
 
   XMLObj *alloc_obj(const char *el) override;
 public:
-  explicit RGWCORSXMLParser_S3(CephContext *_cct) : cct(_cct) {}
+  explicit RGWCORSXMLParser_S3(const DoutPrefixProvider *_dpp, CephContext *_cct) : dpp(_dpp), cct(_cct) {}
 };
 #endif /*CEPH_RGW_CORS_S3_H*/
