@@ -55,7 +55,7 @@ enum RGWOpType {
   RGW_OP_CREATE_ROLE,
   RGW_OP_DELETE_ROLE,
   RGW_OP_GET_ROLE,
-  RGW_OP_MODIFY_ROLE,
+  RGW_OP_MODIFY_ROLE_TRUST_POLICY,
   RGW_OP_LIST_ROLES,
   RGW_OP_PUT_ROLE_POLICY,
   RGW_OP_GET_ROLE_POLICY,
