@@ -170,6 +170,55 @@ public:
   }
 };
 
+namespace {
+// this allows waiting untill "finish()" is called from a different thread
+// waiting could be blocking the waiting thread or yielding, depending
+// with compilation flag support and whether the optional_yield is set
+class Waiter {
+  using Signature = void(boost::system::error_code);
+  using Completion = ceph::async::Completion<Signature>;
+  using CompletionInit = boost::asio::async_completion<yield_context, Signature>;
+  std::unique_ptr<Completion> completion = nullptr;
+  int ret;
+
+  bool done = false;
+  mutable std::mutex lock;
+  mutable std::condition_variable cond;
+
+public:
+  int wait(optional_yield y) {
+    std::unique_lock l{lock};
+    if (done) {
+      return ret;
+    }
+    if (y) {
+      boost::system::error_code ec;
+      auto&& token = y.get_yield_context()[ec];
+      CompletionInit init(token);
+      completion = Completion::create(y.get_io_context().get_executor(),
+          std::move(init.completion_handler));
+      l.unlock();
+      init.result.get();
+      return -ec.value();
+    }
+    cond.wait(l, [this]{return (done==true);});
+    return ret;
+  }
+
+  void finish(int r) {
+    std::unique_lock l{lock};
+    ret = r;
+    done = true;
+    if (completion) {
+      boost::system::error_code ec(-ret, boost::system::system_category());
+      Completion::post(std::move(completion), ec);
+    } else {
+      cond.notify_all();
+    }
+  }
+};
+} // namespace
+
 #ifdef WITH_RADOSGW_AMQP_ENDPOINT
 class RGWPubSubAMQPEndpoint : public RGWPubSubEndpoint {
 private:
@@ -410,12 +459,12 @@ public:
       return amqp::publish(conn, topic, json_format_pubsub_event(event));
     } else {
       // TODO: currently broker and routable are the same - this will require different flags but the same mechanism
-      // note: dynamic allocation of Waiter is needed when this is invoked from a beast coroutine
-      auto w = std::unique_ptr<Waiter>(new Waiter);
+      auto w = std::make_unique<Waiter>();
       const auto rc = amqp::publish_with_confirm(conn, 
         topic,
         json_format_pubsub_event(event),
-        std::bind(&Waiter::finish, w.get(), std::placeholders::_1));
+        [wp = w.get()](int r) { wp->finish(r);}
+      );
       if (rc < 0) {
         // failed to publish, does not wait for reply
         return rc;
@@ -448,8 +497,8 @@ private:
   };
   CephContext* const cct;
   const std::string topic;
-  kafka::connection_ptr_t conn;
   const ack_level_t ack_level;
+  std::string conn_name;
 
 
   ack_level_t get_ack_level(const RGWHTTPArgs& args) {
@@ -470,21 +519,21 @@ private:
   class NoAckPublishCR : public RGWCoroutine {
   private:
     const std::string topic;
-    kafka::connection_ptr_t conn;
+    std::string conn_name;
     const std::string message;
 
   public:
     NoAckPublishCR(CephContext* cct,
               const std::string& _topic,
-              kafka::connection_ptr_t& _conn,
+              const std::string& _conn_name,
               const std::string& _message) :
       RGWCoroutine(cct),
-      topic(_topic), conn(_conn), message(_message) {}
+      topic(_topic), conn_name(_conn_name), message(_message) {}
 
     // send message to endpoint, without waiting for reply
     int operate(const DoutPrefixProvider *dpp) override {
       reenter(this) {
-        const auto rc = kafka::publish(conn, topic, message);
+        const auto rc = kafka::publish(conn_name, topic, message);
         if (rc < 0) {
           return set_cr_error(rc);
         }
@@ -500,23 +549,23 @@ private:
   class AckPublishCR : public RGWCoroutine, public RGWIOProvider {
   private:
     const std::string topic;
-    kafka::connection_ptr_t conn;
+    std::string conn_name;
     const std::string message;
 
   public:
     AckPublishCR(CephContext* cct,
               const std::string& _topic,
-              kafka::connection_ptr_t& _conn,
+              const std::string& _conn_name,
               const std::string& _message) :
       RGWCoroutine(cct),
-      topic(_topic), conn(_conn), message(_message) {}
+      topic(_topic), conn_name(_conn_name), message(_message) {}
 
     // send message to endpoint, waiting for reply
     int operate(const DoutPrefixProvider *dpp) override {
       reenter(this) {
         yield {
           init_new_io(this);
-          const auto rc = kafka::publish_with_confirm(conn, 
+          const auto rc = kafka::publish_with_confirm(conn_name, 
               topic,
               message,
               std::bind(&AckPublishCR::request_complete, this, std::placeholders::_1));
@@ -561,28 +610,26 @@ public:
       CephContext* _cct) : 
         cct(_cct),
         topic(_topic),
-        conn(kafka::connect(_endpoint, get_bool(args, "use-ssl", false), get_bool(args, "verify-ssl", true), args.get_optional("ca-location"))) ,
         ack_level(get_ack_level(args)) {
-    if (!conn) { 
+    if (!kafka::connect(conn_name, _endpoint, get_bool(args, "use-ssl", false), get_bool(args, "verify-ssl", true), 
+          args.get_optional("ca-location"))) {
       throw configuration_error("Kafka: failed to create connection to: " + _endpoint);
     }
   }
 
   RGWCoroutine* send_to_completion_async(const rgw_pubsub_event& event, RGWDataSyncEnv* env) override {
-    ceph_assert(conn);
     if (ack_level == ack_level_t::None) {
-      return new NoAckPublishCR(cct, topic, conn, json_format_pubsub_event(event));
+      return new NoAckPublishCR(cct, topic, conn_name, json_format_pubsub_event(event));
     } else {
-      return new AckPublishCR(cct, topic, conn, json_format_pubsub_event(event));
+      return new AckPublishCR(cct, topic, conn_name, json_format_pubsub_event(event));
     }
   }
   
   RGWCoroutine* send_to_completion_async(const rgw_pubsub_s3_event& event, RGWDataSyncEnv* env) override {
-    ceph_assert(conn);
     if (ack_level == ack_level_t::None) {
-      return new NoAckPublishCR(cct, topic, conn, json_format_pubsub_event(event));
+      return new NoAckPublishCR(cct, topic, conn_name, json_format_pubsub_event(event));
     } else {
-      return new AckPublishCR(cct, topic, conn, json_format_pubsub_event(event));
+      return new AckPublishCR(cct, topic, conn_name, json_format_pubsub_event(event));
     }
   }
 
@@ -641,16 +688,15 @@ public:
   };
 
   int send_to_completion_async(CephContext* cct, const rgw_pubsub_s3_event& event, optional_yield y) override {
-    ceph_assert(conn);
     if (ack_level == ack_level_t::None) {
-      return kafka::publish(conn, topic, json_format_pubsub_event(event));
+      return kafka::publish(conn_name, topic, json_format_pubsub_event(event));
     } else {
-      // note: dynamic allocation of Waiter is needed when this is invoked from a beast coroutine
-      auto w = std::unique_ptr<Waiter>(new Waiter);
-      const auto rc = kafka::publish_with_confirm(conn, 
+      auto w = std::make_unique<Waiter>();
+      const auto rc = kafka::publish_with_confirm(conn_name, 
         topic,
         json_format_pubsub_event(event),
-        std::bind(&Waiter::finish, w.get(), std::placeholders::_1));
+        [wp = w.get()](int r) { wp->finish(r); }
+      );
       if (rc < 0) {
         // failed to publish, does not wait for reply
         return rc;
@@ -661,7 +707,7 @@ public:
 
   std::string to_str() const override {
     std::string str("Kafka Endpoint");
-    str += kafka::to_string(conn);
+    str += "\nBroker: " + conn_name;
     str += "\nTopic: " + topic;
     return str;
   }
