@@ -2295,8 +2295,6 @@ int RGWREST::preprocess(struct req_state *s, rgw::io::BasicClient* cio)
   }
   s->op = op_from_method(info.method);
 
-  info.init_meta_info(s, &s->has_bad_meta);
-
   return 0;
 }
 
@@ -2336,5 +2334,7 @@ RGWHandler_REST* RGWREST::get_handler(
     return nullptr;
   }
 
+  s->info.init_meta_info(s, &s->has_bad_meta, s->prot_flags);
+
   return handler;
 } /* get stream handler */
