@@ -15,6 +15,9 @@
 
 #pragma once
 
+#include "rgw/rgw_rest.h"
+#include "rgw/rgw_auth_s3.h"
+
 class RGWOp_Metadata_List : public RGWRESTOp {
 public:
   RGWOp_Metadata_List() {}
@@ -23,7 +26,7 @@ public:
   int check_caps(const RGWUserCaps& caps) override {
     return caps.check_cap("metadata", RGW_CAP_READ);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
   const char* name() const override { return "list_metadata"; }
 };
 
@@ -35,7 +38,7 @@ public:
   int check_caps(const RGWUserCaps& caps) override {
     return caps.check_cap("metadata", RGW_CAP_READ);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
   const char* name() const override { return "get_metadata"; }
 };
 
@@ -44,12 +47,12 @@ public:
   RGWOp_Metadata_Get_Myself() {}
   ~RGWOp_Metadata_Get_Myself() override {}
 
-  void execute() override;
+  void execute(optional_yield y) override;
 };
 
 class RGWOp_Metadata_Put : public RGWRESTOp {
   int get_data(bufferlist& bl);
-  string update_status;
+  std::string update_status;
   obj_version ondisk_version;
 public:
   RGWOp_Metadata_Put() {}
@@ -58,7 +61,7 @@ public:
   int check_caps(const RGWUserCaps& caps) override {
     return caps.check_cap("metadata", RGW_CAP_WRITE);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
   void send_response() override;
   const char* name() const override { return "set_metadata"; }
   RGWOpType get_type() override { return RGW_OP_ADMIN_SET_METADATA; }
@@ -72,7 +75,7 @@ public:
   int check_caps(const RGWUserCaps& caps) override {
     return caps.check_cap("metadata", RGW_CAP_WRITE);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
   const char* name() const override { return "remove_metadata"; }
 };
 
@@ -82,7 +85,7 @@ protected:
   RGWOp *op_put() override;
   RGWOp *op_delete() override;
 
-  int read_permissions(RGWOp*) override {
+  int read_permissions(RGWOp*, optional_yield y) override {
     return 0;
   }
 public:
@@ -95,7 +98,8 @@ public:
   RGWRESTMgr_Metadata() = default;
   ~RGWRESTMgr_Metadata() override = default;
 
-  RGWHandler_REST* get_handler(struct req_state* const s,
+  RGWHandler_REST* get_handler(rgw::sal::Store* store,
+			       struct req_state* const s,
                                const rgw::auth::StrategyRegistry& auth_registry,
                                const std::string& frontend_prefix) override {
     return new RGWHandler_Metadata(auth_registry);
