@@ -23,13 +23,13 @@ protected:
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield) override;
 
   const char* name() const override { return "pubsub_topic_create"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_TOPIC_CREATE; }
@@ -43,13 +43,13 @@ protected:
   rgw_pubsub_topics result;
 
 public:
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield) override;
 
   const char* name() const override { return "pubsub_topics_list"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_TOPICS_LIST; }
@@ -66,13 +66,13 @@ protected:
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override {
+  int verify_permission(optional_yield y) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
 
   const char* name() const override { return "pubsub_topic_get"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_TOPIC_GET; }
@@ -82,19 +82,19 @@ public:
 // delete a topic
 class RGWPSDeleteTopicOp : public RGWDefaultResponseOp {
 protected:
-  string topic_name;
+  std::string topic_name;
   std::optional<RGWPubSub> ps;
   
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
 
   const char* name() const override { return "pubsub_topic_delete"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_TOPIC_DELETE; }
@@ -112,13 +112,13 @@ protected:
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
 
   const char* name() const override { return "pubsub_subscription_create"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_SUB_CREATE; }
@@ -135,13 +135,13 @@ protected:
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
 
   const char* name() const override { return "pubsub_subscription_get"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_SUB_GET; }
@@ -158,13 +158,13 @@ protected:
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
 
   const char* name() const override { return "pubsub_subscription_delete"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_SUB_DELETE; }
@@ -183,13 +183,13 @@ protected:
 public:
   RGWPSAckSubEventOp() {}
 
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
 
   const char* name() const override { return "pubsub_subscription_ack"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_SUB_ACK; }
@@ -212,13 +212,13 @@ protected:
 public:
   RGWPSPullSubEventsOp() {}
 
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return 0;
   }
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
   }
-  void execute() override;
+  void execute(optional_yield y) override;
 
   const char* name() const override { return "pubsub_subscription_pull"; }
   RGWOpType get_type() override { return RGW_OP_PUBSUB_SUB_PULL; }
@@ -229,13 +229,13 @@ public:
 class RGWPSCreateNotifOp : public RGWDefaultResponseOp {
 protected:
   std::optional<RGWPubSub> ps;
-  string bucket_name;
+  std::string bucket_name;
   RGWBucketInfo bucket_info;
 
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override;
+  int verify_permission(optional_yield y) override;
 
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
@@ -255,7 +255,7 @@ protected:
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override;
+  int verify_permission(optional_yield y) override;
 
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
@@ -275,7 +275,7 @@ protected:
   virtual int get_params() = 0;
 
 public:
-  int verify_permission() override;
+  int verify_permission(optional_yield y) override;
 
   void pre_exec() override {
     rgw_bucket_object_pre_exec(s);
@@ -284,4 +284,3 @@ public:
   RGWOpType get_type() override { return RGW_OP_PUBSUB_NOTIF_LIST; }
   uint32_t op_mask() override { return RGW_OP_TYPE_READ; }
 };
-
