@@ -6,6 +6,7 @@
 #include "rgw_rest_s3.h"
 #include "rgw_rest_config.h"
 #include "rgw_zone.h"
+#include "rgw_sal_rados.h"
 
 #include "services/svc_zone.h"
 #include "services/svc_mdlog.h"
@@ -14,6 +15,8 @@
 
 #define dout_subsys ceph_subsys_rgw
 
+using namespace std;
+
 // reject 'period push' if we would have to fetch too many intermediate periods
 static const uint32_t PERIOD_HISTORY_FETCH_MAX = 64;
 
@@ -23,19 +26,19 @@ class RGWOp_Period_Base : public RGWRESTOp {
   RGWPeriod period;
   std::ostringstream error_stream;
  public:
-  int verify_permission() override { return 0; }
+  int verify_permission(optional_yield) override { return 0; }
   void send_response() override;
 };
 
 // reply with the period object on success
 void RGWOp_Period_Base::send_response()
 {
-  set_req_state_err(s, http_ret, error_stream.str());
+  set_req_state_err(s, op_ret, error_stream.str());
   dump_errno(s);
 
-  if (http_ret < 0) {
+  if (op_ret < 0) {
     if (!s->err.message.empty()) {
-      ldout(s->cct, 4) << "Request failed with " << http_ret
+      ldpp_dout(this, 4) << "Request failed with " << op_ret
           << ": " << s->err.message << dendl;
     }
     end_header(s);
@@ -50,17 +53,17 @@ void RGWOp_Period_Base::send_response()
 // GET /admin/realm/period
 class RGWOp_Period_Get : public RGWOp_Period_Base {
  public:
-  void execute() override;
+  void execute(optional_yield y) override;
   int check_caps(const RGWUserCaps& caps) override {
     return caps.check_cap("zone", RGW_CAP_READ);
   }
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return check_caps(s->user->get_caps());
   }
   const char* name() const override { return "get_period"; }
 };
 
-void RGWOp_Period_Get::execute()
+void RGWOp_Period_Get::execute(optional_yield y)
 {
   string realm_id, realm_name, period_id;
   epoch_t epoch = 0;
@@ -72,45 +75,46 @@ void RGWOp_Period_Get::execute()
   period.set_id(period_id);
   period.set_epoch(epoch);
 
-  http_ret = period.init(store->ctx(), store->svc()->sysobj, realm_id, realm_name);
-  if (http_ret < 0)
-    ldout(store->ctx(), 5) << "failed to read period" << dendl;
+  op_ret = period.init(this, store->ctx(), static_cast<rgw::sal::RadosStore*>(store)->svc()->sysobj, realm_id, y, realm_name);
+  if (op_ret < 0)
+    ldpp_dout(this, 5) << "failed to read period" << dendl;
 }
 
 // POST /admin/realm/period
 class RGWOp_Period_Post : public RGWOp_Period_Base {
  public:
-  void execute() override;
+  void execute(optional_yield y) override;
   int check_caps(const RGWUserCaps& caps) override {
     return caps.check_cap("zone", RGW_CAP_WRITE);
   }
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return check_caps(s->user->get_caps());
   }
   const char* name() const override { return "post_period"; }
+  RGWOpType get_type() override { return RGW_OP_PERIOD_POST; }
 };
 
-void RGWOp_Period_Post::execute()
+void RGWOp_Period_Post::execute(optional_yield y)
 {
   auto cct = store->ctx();
 
   // initialize the period without reading from rados
-  period.init(cct, store->svc()->sysobj, false);
+  period.init(this, cct, static_cast<rgw::sal::RadosStore*>(store)->svc()->sysobj, y, false);
 
   // decode the period from input
   const auto max_size = cct->_conf->rgw_max_put_param_size;
   bool empty;
-  http_ret = rgw_rest_get_json_input(cct, s, period, max_size, &empty);
-  if (http_ret < 0) {
-    lderr(cct) << "failed to decode period" << dendl;
+  op_ret = get_json_input(cct, s, period, max_size, &empty);
+  if (op_ret < 0) {
+    ldpp_dout(this, -1) << "failed to decode period" << dendl;
     return;
   }
 
   // require period.realm_id to match our realm
-  if (period.get_realm() != store->svc()->zone->get_realm().get_id()) {
+  if (period.get_realm() != store->get_zone()->get_realm().get_id()) {
     error_stream << "period with realm id " << period.get_realm()
-        << " doesn't match current realm " << store->svc()->zone->get_realm().get_id() << std::endl;
-    http_ret = -EINVAL;
+        << " doesn't match current realm " << store->get_zone()->get_realm().get_id() << std::endl;
+    op_ret = -EINVAL;
     return;
   }
 
@@ -118,66 +122,66 @@ void RGWOp_Period_Post::execute()
   // period that we haven't restarted with yet. we also don't want to modify
   // the objects in use by RGWRados
   RGWRealm realm(period.get_realm());
-  http_ret = realm.init(cct, store->svc()->sysobj);
-  if (http_ret < 0) {
-    lderr(cct) << "failed to read current realm: "
-        << cpp_strerror(-http_ret) << dendl;
+  op_ret = realm.init(this, cct, static_cast<rgw::sal::RadosStore*>(store)->svc()->sysobj, y);
+  if (op_ret < 0) {
+    ldpp_dout(this, -1) << "failed to read current realm: "
+        << cpp_strerror(-op_ret) << dendl;
     return;
   }
 
   RGWPeriod current_period;
-  http_ret = current_period.init(cct, store->svc()->sysobj, realm.get_id());
-  if (http_ret < 0) {
-    lderr(cct) << "failed to read current period: "
-        << cpp_strerror(-http_ret) << dendl;
+  op_ret = current_period.init(this, cct, static_cast<rgw::sal::RadosStore*>(store)->svc()->sysobj, realm.get_id(), y);
+  if (op_ret < 0) {
+    ldpp_dout(this, -1) << "failed to read current period: "
+        << cpp_strerror(-op_ret) << dendl;
     return;
   }
 
   // if period id is empty, handle as 'period commit'
   if (period.get_id().empty()) {
-    http_ret = period.commit(store, realm, current_period, error_stream);
-    if (http_ret < 0) {
-      lderr(cct) << "master zone failed to commit period" << dendl;
+    op_ret = period.commit(this, store, realm, current_period, error_stream, y);
+    if (op_ret < 0) {
+      ldpp_dout(this, -1) << "master zone failed to commit period" << dendl;
     }
     return;
   }
 
   // if it's not period commit, nobody is allowed to push to the master zone
-  if (period.get_master_zone() == store->svc()->zone->get_zone_params().get_id()) {
-    ldout(cct, 10) << "master zone rejecting period id="
+  if (period.get_master_zone() == store->get_zone()->get_params().get_id()) {
+    ldpp_dout(this, 10) << "master zone rejecting period id="
         << period.get_id() << " epoch=" << period.get_epoch() << dendl;
-    http_ret = -EINVAL; // XXX: error code
+    op_ret = -EINVAL; // XXX: error code
     return;
   }
 
   // write the period to rados
-  http_ret = period.store_info(false);
-  if (http_ret < 0) {
-    lderr(cct) << "failed to store period " << period.get_id() << dendl;
+  op_ret = period.store_info(this, false, y);
+  if (op_ret < 0) {
+    ldpp_dout(this, -1) << "failed to store period " << period.get_id() << dendl;
     return;
   }
   // set as latest epoch
-  http_ret = period.update_latest_epoch(period.get_epoch());
-  if (http_ret == -EEXIST) {
+  op_ret = period.update_latest_epoch(this, period.get_epoch(), y);
+  if (op_ret == -EEXIST) {
     // already have this epoch (or a more recent one)
-    ldout(cct, 4) << "already have epoch >= " << period.get_epoch()
+    ldpp_dout(this, 4) << "already have epoch >= " << period.get_epoch()
         << " for period " << period.get_id() << dendl;
-    http_ret = 0;
+    op_ret = 0;
     return;
   }
-  if (http_ret < 0) {
-    lderr(cct) << "failed to set latest epoch" << dendl;
+  if (op_ret < 0) {
+    ldpp_dout(this, -1) << "failed to set latest epoch" << dendl;
     return;
   }
 
-  auto period_history = store->svc()->mdlog->get_period_history();
+  auto period_history = static_cast<rgw::sal::RadosStore*>(store)->svc()->mdlog->get_period_history();
 
   // decide whether we can set_current_period() or set_latest_epoch()
   if (period.get_id() != current_period.get_id()) {
     auto current_epoch = current_period.get_realm_epoch();
     // discard periods in the past
     if (period.get_realm_epoch() < current_epoch) {
-      ldout(cct, 10) << "discarding period " << period.get_id()
+      ldpp_dout(this, 10) << "discarding period " << period.get_id()
           << " with realm epoch " << period.get_realm_epoch()
           << " older than current epoch " << current_epoch << dendl;
       // return success to ack that we have this period
@@ -185,53 +189,53 @@ void RGWOp_Period_Post::execute()
     }
     // discard periods too far in the future
     if (period.get_realm_epoch() > current_epoch + PERIOD_HISTORY_FETCH_MAX) {
-      lderr(cct) << "discarding period " << period.get_id()
+      ldpp_dout(this, -1) << "discarding period " << period.get_id()
           << " with realm epoch " << period.get_realm_epoch() << " too far in "
           "the future from current epoch " << current_epoch << dendl;
-      http_ret = -ENOENT; // XXX: error code
+      op_ret = -ENOENT; // XXX: error code
       return;
     }
     // attach a copy of the period into the period history
-    auto cursor = period_history->attach(RGWPeriod{period});
+    auto cursor = period_history->attach(this, RGWPeriod{period}, y);
     if (!cursor) {
       // we're missing some history between the new period and current_period
-      http_ret = cursor.get_error();
-      lderr(cct) << "failed to collect the periods between current period "
+      op_ret = cursor.get_error();
+      ldpp_dout(this, -1) << "failed to collect the periods between current period "
           << current_period.get_id() << " (realm epoch " << current_epoch
           << ") and the new period " << period.get_id()
           << " (realm epoch " << period.get_realm_epoch()
-          << "): " << cpp_strerror(-http_ret) << dendl;
+          << "): " << cpp_strerror(-op_ret) << dendl;
       return;
     }
     if (cursor.has_next()) {
       // don't switch if we have a newer period in our history
-      ldout(cct, 4) << "attached period " << period.get_id()
+      ldpp_dout(this, 4) << "attached period " << period.get_id()
           << " to history, but the history contains newer periods" << dendl;
       return;
     }
     // set as current period
-    http_ret = realm.set_current_period(period);
-    if (http_ret < 0) {
-      lderr(cct) << "failed to update realm's current period" << dendl;
+    op_ret = realm.set_current_period(this, period, y);
+    if (op_ret < 0) {
+      ldpp_dout(this, -1) << "failed to update realm's current period" << dendl;
       return;
     }
-    ldout(cct, 4) << "period " << period.get_id()
+    ldpp_dout(this, 4) << "period " << period.get_id()
         << " is newer than current period " << current_period.get_id()
         << ", updating realm's current period and notifying zone" << dendl;
-    realm.notify_new_period(period);
+    realm.notify_new_period(this, period, y);
     return;
   }
   // reflect the period into our local objects
-  http_ret = period.reflect();
-  if (http_ret < 0) {
-    lderr(cct) << "failed to update local objects: "
-        << cpp_strerror(-http_ret) << dendl;
+  op_ret = period.reflect(this, y);
+  if (op_ret < 0) {
+    ldpp_dout(this, -1) << "failed to update local objects: "
+        << cpp_strerror(-op_ret) << dendl;
     return;
   }
-  ldout(cct, 4) << "period epoch " << period.get_epoch()
+  ldpp_dout(this, 4) << "period epoch " << period.get_epoch()
       << " is newer than current epoch " << current_period.get_epoch()
       << ", updating period's latest epoch and notifying zone" << dendl;
-  realm.notify_new_period(period);
+  realm.notify_new_period(this, period, y);
   // update the period history
   period_history->insert(RGWPeriod{period});
 }
@@ -246,7 +250,8 @@ class RGWHandler_Period : public RGWHandler_Auth_S3 {
 
 class RGWRESTMgr_Period : public RGWRESTMgr {
  public:
-  RGWHandler_REST* get_handler(struct req_state*,
+  RGWHandler_REST* get_handler(rgw::sal::Store* store,
+			       struct req_state*,
                                const rgw::auth::StrategyRegistry& auth_registry,
                                const std::string&) override {
     return new RGWHandler_Period(auth_registry);
@@ -261,15 +266,15 @@ public:
   int check_caps(const RGWUserCaps& caps) override {
     return caps.check_cap("zone", RGW_CAP_READ);
   }
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return check_caps(s->user->get_caps());
   }
-  void execute() override;
+  void execute(optional_yield y) override;
   void send_response() override;
   const char* name() const override { return "get_realm"; }
 };
 
-void RGWOp_Realm_Get::execute()
+void RGWOp_Realm_Get::execute(optional_yield y)
 {
   string id;
   RESTArgs::get_string(s, "id", id, &id);
@@ -278,18 +283,18 @@ void RGWOp_Realm_Get::execute()
 
   // read realm
   realm.reset(new RGWRealm(id, name));
-  http_ret = realm->init(g_ceph_context, store->svc()->sysobj);
-  if (http_ret < 0)
-    lderr(store->ctx()) << "failed to read realm id=" << id
+  op_ret = realm->init(this, g_ceph_context, static_cast<rgw::sal::RadosStore*>(store)->svc()->sysobj, y);
+  if (op_ret < 0)
+    ldpp_dout(this, -1) << "failed to read realm id=" << id
         << " name=" << name << dendl;
 }
 
 void RGWOp_Realm_Get::send_response()
 {
-  set_req_state_err(s, http_ret);
+  set_req_state_err(s, op_ret);
   dump_errno(s);
 
-  if (http_ret < 0) {
+  if (op_ret < 0) {
     end_header(s);
     return;
   }
@@ -307,32 +312,32 @@ public:
   int check_caps(const RGWUserCaps& caps) override {
     return caps.check_cap("zone", RGW_CAP_READ);
   }
-  int verify_permission() override {
+  int verify_permission(optional_yield) override {
     return check_caps(s->user->get_caps());
   }
-  void execute() override;
+  void execute(optional_yield y) override;
   void send_response() override;
   const char* name() const override { return "list_realms"; }
 };
 
-void RGWOp_Realm_List::execute()
+void RGWOp_Realm_List::execute(optional_yield y)
 {
   {
     // read default realm
-    RGWRealm realm(store->ctx(), store->svc()->sysobj);
-    [[maybe_unused]] int ret = realm.read_default_id(default_id);
+    RGWRealm realm(store->ctx(), static_cast<rgw::sal::RadosStore*>(store)->svc()->sysobj);
+    [[maybe_unused]] int ret = realm.read_default_id(this, default_id, y);
   }
-  http_ret = store->svc()->zone->list_realms(realms);
-  if (http_ret < 0)
-    lderr(store->ctx()) << "failed to list realms" << dendl;
+  op_ret = static_cast<rgw::sal::RadosStore*>(store)->svc()->zone->list_realms(this, realms);
+  if (op_ret < 0)
+    ldpp_dout(this, -1) << "failed to list realms" << dendl;
 }
 
 void RGWOp_Realm_List::send_response()
 {
-  set_req_state_err(s, http_ret);
+  set_req_state_err(s, op_ret);
   dump_errno(s);
 
-  if (http_ret < 0) {
+  if (op_ret < 0) {
     end_header(s);
     return;
   }
@@ -362,7 +367,8 @@ RGWRESTMgr_Realm::RGWRESTMgr_Realm()
 }
 
 RGWHandler_REST*
-RGWRESTMgr_Realm::get_handler(struct req_state*,
+RGWRESTMgr_Realm::get_handler(rgw::sal::Store* store,
+			      struct req_state*,
                               const rgw::auth::StrategyRegistry& auth_registry,
                               const std::string&)
 {
