@@ -358,6 +358,7 @@ class RadosZone : public Zone {
     virtual bool get_redirect_endpoint(std::string* endpoint) override;
     virtual bool has_zonegroup_api(const std::string& api) const override;
     virtual const std::string& get_current_period_id() override;
+    virtual const std::string_view get_tier_type() override;
 };
 
 class RadosStore : public Store {
@@ -521,6 +522,7 @@ public:
 
   /* For RadosStore code */
   RGWObjManifest& get_manifest() { return info.manifest; }
+  const std::set<std::string>& get_past_prefixes() const { return info.past_prefixes; }
 
   friend class RadosMultipartUpload;
 };
@@ -572,6 +574,11 @@ public:
 			  const rgw_placement_rule *ptail_placement_rule,
 			  uint64_t part_num,
 			  const std::string& part_num_str) override;
+protected:
+  int cleanup_part_history(const DoutPrefixProvider* dpp,
+                           optional_yield y,
+                           RadosMultipartPart* part,
+                           std::list<rgw_obj_index_key>& remove_objs);
 };
 
 class MPRadosSerializer : public MPSerializer {
