@@ -7,7 +7,8 @@
 
 class RGWRESTMgr_PubSub : public RGWRESTMgr {
 public:
-  virtual RGWHandler_REST* get_handler(struct req_state* s,
+  virtual RGWHandler_REST* get_handler(rgw::sal::Store* store,
+			       struct req_state* s,
                                const rgw::auth::StrategyRegistry& auth_registry,
                                const std::string& frontend_prefix) override;
 };
