@@ -7,12 +7,12 @@
 #include <algorithm>
 #include <array>
 #include <cstring>
+#include <string_view>
 #include <tuple>
 #include <utility>
 
 #include <boost/optional.hpp>
 #include <boost/range/adaptor/reversed.hpp>
-#include <boost/utility/string_ref.hpp>
 
 namespace rgw {
 namespace tar {
@@ -107,8 +107,8 @@ public:
     }
   }
 
-  boost::string_ref get_filename() const {
-    return boost::string_ref(header->filename,
+  std::string_view get_filename() const {
+    return std::string_view(header->filename,
                              std::min(sizeof(header->filename),
                                       strlen(header->filename)));
   }
@@ -116,13 +116,13 @@ public:
   size_t get_filesize() const {
     /* The string_ref is pretty suitable here because tar encodes its
      * metadata in ASCII. */
-    const boost::string_ref raw(header->filesize, sizeof(header->filesize));
+    const std::string_view raw(header->filesize, sizeof(header->filesize));
 
     /* We need to find where the padding ends. */
     const auto pad_ends_at = std::min(raw.find_last_not_of('\0'),
                                       raw.find_last_not_of(' '));
     const auto trimmed = raw.substr(0,
-      pad_ends_at == boost::string_ref::npos ? boost::string_ref::npos
+      pad_ends_at == std::string_view::npos ? std::string_view::npos
                                              : pos2len(pad_ends_at));
 
     size_t sum = 0, mul = 1;
