@@ -53,13 +53,10 @@ namespace rgw {
       switch (type) {
       case TOKEN_AD:
 	return "ad";
-	break;
       case TOKEN_LDAP:
 	return "ldap";
-	break;
       case TOKEN_KEYSTONE:
 	return "keystone";
-	break;
       default:
 	return "none";
       };
@@ -84,12 +81,19 @@ namespace rgw {
 	     const std::string& _key)
       : type(_type), id(_id), key(_key) {};
 
-    RGWToken(const string& json) {
+    explicit RGWToken(const string& json) {
       JSONParser p;
       p.parse(json.c_str(), json.length());
       JSONDecoder::decode_json(RGWToken::type_name, *this, &p);
     }
 
+    RGWToken& operator=(const std::string& json) {
+      JSONParser p;
+      p.parse(json.c_str(), json.length());
+      JSONDecoder::decode_json(RGWToken::type_name, *this, &p);
+      return *this;
+    }
+
     void encode(bufferlist& bl) const {
       uint32_t ver = version();
       string typestr{from_type(type)};
@@ -148,13 +152,13 @@ namespace rgw {
       return to_base64(std::move(os.str()));
     }
 
-    friend inline ostream& operator<<(ostream& os, const RGWToken& token);
+    friend inline std::ostream& operator<<(std::ostream& os, const RGWToken& token);
 
     virtual ~RGWToken() {};
   };
   WRITE_CLASS_ENCODER(RGWToken)
 
-  inline ostream& operator<<(ostream& os, const RGWToken& token)
+  inline std::ostream& operator<<(std::ostream& os, const RGWToken& token)
   {
     os << "<<RGWToken"
        << " type=" << RGWToken::from_type(token.type)
