@@ -41,7 +41,7 @@
 #include "port/port.h"
 #include "util/thread_operation.h"
 
-namespace rocksdb {
+namespace ROCKSDB_NAMESPACE {
 
 class ColumnFamilyHandle;
 
@@ -230,4 +230,4 @@ class ThreadStatusUpdater {
 #endif  // ROCKSDB_USING_THREAD_STATUS
 };
 
-}  // namespace rocksdb
+}  // namespace ROCKSDB_NAMESPACE
