@@ -28,7 +28,7 @@
 #include <iostream>
 #include <unordered_map>
 
-#include <seastar/core/future-util.hh>
+#include <seastar/core/seastar.hh>
 #include <seastar/core/scollectd_api.hh>
 #include <seastar/core/metrics_api.hh>
 #include <seastar/core/byteorder.hh>
@@ -86,7 +86,7 @@ registration::registration(type_instance_id&& id)
 
 seastar::metrics::impl::metric_id to_metrics_id(const type_instance_id & id) {
     return seastar::metrics::impl::metric_id(id.plugin(), id.type_instance(),
-            {{seastar::metrics::shard_label.name(), seastar::metrics::impl::shard()}, {seastar::metrics::type_label.name(), id.type()}});
+            {{seastar::metrics::shard_label.name(), seastar::metrics::impl::shard()}});
 }
 
 
@@ -169,7 +169,7 @@ struct cpwriter {
         }
         sstring res = id.name();
         for (auto i : id.labels()) {
-            if (i.first != seastar::metrics::shard_label.name() && i.first != seastar::metrics::type_label.name()) {
+            if (i.first != seastar::metrics::shard_label.name()) {
                 res += "-" + i.second;
             }
         }
@@ -214,10 +214,12 @@ struct cpwriter {
                 write_le(tmpi);
                 break;
             }
-            case data_type::COUNTER:
             case data_type::DERIVE:
+                write(v.i()); // signed int 64, big endian
+                break;
+            case data_type::COUNTER:
             case data_type::ABSOLUTE:
-                write(v.ui()); // big endian
+                write(v.ui()); // unsigned int 64, big endian
                 break;
             default:
                 assert(0);
@@ -275,7 +277,7 @@ struct cpwriter {
         }
         return *this;
     }
-    cpwriter & put(const sstring & host, const seastar::metrics::impl::metric_id & id) {
+    cpwriter & put(const sstring & host, const seastar::metrics::impl::metric_id & id, const type_id& type) {
         const auto ts = std::chrono::system_clock::now().time_since_epoch();
         const auto lrts =
                 std::chrono::duration_cast<std::chrono::seconds>(ts).count();
@@ -289,8 +291,8 @@ struct cpwriter {
         // Optional
         put_cached(part_type::PluginInst,
                 id.instance_id() == per_cpu_plugin_instance ?
-                        to_sstring(engine().cpu_id()) : id.instance_id());
-        put_cached(part_type::Type, id.inherit_type());
+                        to_sstring(this_shard_id()) : id.instance_id());
+        put_cached(part_type::Type, type);
         // Optional
         put_cached(part_type::TypeInst, get_type_instance(id));
         return *this;
@@ -300,7 +302,7 @@ struct cpwriter {
             const type_instance_id & id, const value_list & v) {
         const auto ps = std::chrono::duration_cast<collectd_hres_duration>(
                         period).count();
-            put(host, to_metrics_id(id));
+            put(host, to_metrics_id(id), id.type());
             put(part_type::Values, v);
             if (ps != 0) {
                 put(part_type::IntervalHr, ps);
@@ -310,10 +312,11 @@ struct cpwriter {
 
     cpwriter & put(const sstring & host,
             const duration & period,
+            const type_id& type,
             const seastar::metrics::impl::metric_id & id, const seastar::metrics::impl::metric_value & v) {
         const auto ps = std::chrono::duration_cast<collectd_hres_duration>(
                 period).count();
-        put(host, id);
+        put(host, id, type);
         put(part_type::Values, v);
         if (ps != 0) {
             put(part_type::IntervalHr, ps);
@@ -347,7 +350,7 @@ future<> impl::send_metric(const type_instance_id & id,
 future<> impl::send_notification(const type_instance_id & id,
         const sstring & msg) {
     cpwriter out;
-    out.put(_host, to_metrics_id(id));
+    out.put(_host, to_metrics_id(id), id.type());
     out.put(part_type::Message, msg);
     return _chan.send(_addr, net::packet(out.data(), out.size()));
 }
@@ -357,7 +360,7 @@ void impl::start(const sstring & host, const ipv4_addr & addr, const duration pe
     _period = period;
     _addr = addr;
     _host = host;
-    _chan = engine().net().make_udp_channel();
+    _chan = make_udp_channel();
     _timer.set_callback(std::bind(&impl::run, this));
 
     // dogfood ourselves
@@ -401,7 +404,7 @@ void impl::run() {
     typedef size_t metric_family_id;
     typedef seastar::metrics::impl::value_vector::iterator value_iterator;
     typedef seastar::metrics::impl::metric_metadata_vector::iterator metadata_iterator;
-    typedef std::tuple<metric_family_id, metadata_iterator, value_iterator, cpwriter> context;
+    typedef std::tuple<metric_family_id, metadata_iterator, value_iterator, type_id, cpwriter> context;
 
     auto ctxt = make_lw_shared<context>();
     foreign_ptr<shared_ptr<seastar::metrics::impl::values_copy>> vals =  seastar::metrics::impl::get_values();
@@ -416,6 +419,7 @@ void impl::run() {
     if (values.size() > 0) {
         std::get<value_iterator>(*ctxt) = values[0].begin();
         std::get<metadata_iterator>(*ctxt) = metadata->at(0).metrics.begin();
+        std::get<type_id>(*ctxt) = metadata->at(0).mf.inherit_type;
     }
 
     auto stop_when = [ctxt, metadata]() {
@@ -441,7 +445,7 @@ void impl::run() {
                     continue;
                 }
                 auto m = out.mark();
-                out.put(_host, _period, md_iterator->id, *i);
+                out.put(_host, _period, std::get<type_id>(*ctxt), md_iterator->id, *i);
                 if (!out) {
                     out.reset(m);
                     out_of_space = true;
@@ -457,6 +461,7 @@ void impl::run() {
             if (mf < values.size()) {
                 i = values[mf].begin();
                 md_iterator = metadata->at(mf).metrics.begin();
+                std::get<type_id>(*ctxt) = metadata->at(mf).mf.inherit_type;
             }
         }
         if (out.empty()) {
@@ -496,7 +501,7 @@ std::vector<type_instance_id> impl::get_instance_ids() const {
         // actually active ids
         for (auto i : v.second) {
             if (i.second) {
-                res.emplace_back(i.second->get_id());
+                res.emplace_back(i.second->get_id(), v.second.info().inherit_type);
             }
         }
     }
@@ -522,17 +527,17 @@ future<> send_metric(const type_instance_id & id,
     return get_impl().send_metric(id, values);
 }
 
-void configure(const boost::program_options::variables_map & opts) {
-    bool enable = opts["collectd"].as<bool>();
+void configure(const options& opts) {
+    bool enable = opts.collectd.get_value();
     if (!enable) {
         return;
     }
-    auto addr = ipv4_addr(opts["collectd-address"].as<std::string>());
-    auto period = std::chrono::milliseconds(opts["collectd-poll-period"].as<unsigned>());
+    auto addr = ipv4_addr(opts.collectd_address.get_value());
+    auto period = std::chrono::milliseconds(opts.collectd_poll_period.get_value());
 
-    auto host = (opts["collectd-hostname"].as<std::string>() == "")
+    auto host = (opts.collectd_hostname.get_value() == "")
             ? seastar::metrics::impl::get_local_impl()->get_config().hostname
-            : sstring(opts["collectd-hostname"].as<std::string>());
+            : sstring(opts.collectd_hostname.get_value());
 
     // Now create send loops on each cpu
     for (unsigned c = 0; c < smp::count; c++) {
@@ -543,19 +548,20 @@ void configure(const boost::program_options::variables_map & opts) {
     }
 }
 
-boost::program_options::options_description get_options_description() {
-    namespace bpo = boost::program_options;
-    bpo::options_description opts("COLLECTD options");
-    opts.add_options()("collectd", bpo::value<bool>()->default_value(false),
-            "enable collectd daemon")("collectd-address",
-            bpo::value<std::string>()->default_value("239.192.74.66:25826"),
-            "address to send/broadcast metrics to")("collectd-poll-period",
-            bpo::value<unsigned>()->default_value(1000),
-            "poll period - frequency of sending counter metrics (default: 1000ms, 0 disables)")(
-            "collectd-hostname",
-            bpo::value<std::string>()->default_value(""),
-            "Deprecated option, use metrics-hostname instead");
-    return opts;
+options::options(program_options::option_group* parent_group)
+    : program_options::option_group(parent_group, "COLLECTD options")
+    , collectd(*this, "collectd", false,
+            "enable collectd daemon")
+    , collectd_address(*this, "collectd-address",
+            "239.192.74.66:25826",
+            "address to send/broadcast metrics to")
+    , collectd_poll_period(*this, "collectd-poll-period",
+            1000,
+            "poll period - frequency of sending counter metrics (default: 1000ms, 0 disables)")
+    , collectd_hostname(*this, "collectd-hostname",
+            "",
+            "Deprecated option, use metrics-hostname instead")
+{
 }
 
 static seastar::metrics::impl::register_ref get_register(const scollectd::type_instance_id& i) {
