@@ -30,7 +30,13 @@
 #include <string.h>
 
 #define AVX2
+#define CLEAR_SCRATCH_SIMD_REGS clear_scratch_ymms
+
 #include "intel-ipsec-mb.h"
+#include "include/kasumi_internal.h"
+#include "include/zuc_internal.h"
+#include "include/snow3g.h"
+
 #include "save_xmms.h"
 #include "asm.h"
 #include "des.h"
@@ -53,9 +59,13 @@ JOB_AES_HMAC *submit_job_aes_xcbc_avx(MB_MGR_AES_XCBC_OOO *state,
                                       JOB_AES_HMAC *job);
 JOB_AES_HMAC *flush_job_aes_xcbc_avx(MB_MGR_AES_XCBC_OOO *state);
 
+JOB_AES_HMAC *submit_job_aes_cntr_avx(JOB_AES_HMAC *job);
+
+JOB_AES_HMAC *submit_job_aes_cntr_bit_avx(JOB_AES_HMAC *job);
+
+#define SAVE_XMMS               save_xmms_avx
+#define RESTORE_XMMS            restore_xmms_avx
 
-#define SAVE_XMMS save_xmms_avx
-#define RESTORE_XMMS restore_xmms_avx
 #define SUBMIT_JOB_AES128_ENC submit_job_aes128_enc_avx
 #define SUBMIT_JOB_AES128_DEC submit_job_aes128_dec_avx
 #define FLUSH_JOB_AES128_ENC  flush_job_aes128_enc_avx
@@ -68,10 +78,15 @@ JOB_AES_HMAC *flush_job_aes_xcbc_avx(MB_MGR_AES_XCBC_OOO *state);
 #define SUBMIT_JOB_AES256_DEC submit_job_aes256_dec_avx
 #define FLUSH_JOB_AES256_ENC  flush_job_aes256_enc_avx
 
-#define SUBMIT_JOB_AES128_CNTR submit_job_aes128_cntr_avx
-#define SUBMIT_JOB_AES192_CNTR submit_job_aes192_cntr_avx
-#define SUBMIT_JOB_AES256_CNTR submit_job_aes256_cntr_avx
+#define SUBMIT_JOB_AES_ECB_128_ENC submit_job_aes_ecb_128_enc_avx
+#define SUBMIT_JOB_AES_ECB_128_DEC submit_job_aes_ecb_128_dec_avx
+#define SUBMIT_JOB_AES_ECB_192_ENC submit_job_aes_ecb_192_enc_avx
+#define SUBMIT_JOB_AES_ECB_192_DEC submit_job_aes_ecb_192_dec_avx
+#define SUBMIT_JOB_AES_ECB_256_ENC submit_job_aes_ecb_256_enc_avx
+#define SUBMIT_JOB_AES_ECB_256_DEC submit_job_aes_ecb_256_dec_avx
 
+#define SUBMIT_JOB_AES_CNTR   submit_job_aes_cntr_avx
+#define SUBMIT_JOB_AES_CNTR_BIT   submit_job_aes_cntr_bit_avx
 
 #define AES_CBC_DEC_128       aes_cbc_dec_128_avx
 #define AES_CBC_DEC_192       aes_cbc_dec_192_avx
@@ -81,6 +96,20 @@ JOB_AES_HMAC *flush_job_aes_xcbc_avx(MB_MGR_AES_XCBC_OOO *state);
 #define AES_CNTR_192       aes_cntr_192_avx
 #define AES_CNTR_256       aes_cntr_256_avx
 
+#define AES_CNTR_CCM_128   aes_cntr_ccm_128_avx
+
+#define AES_ECB_ENC_128       aes_ecb_enc_128_avx
+#define AES_ECB_ENC_192       aes_ecb_enc_192_avx
+#define AES_ECB_ENC_256       aes_ecb_enc_256_avx
+#define AES_ECB_DEC_128       aes_ecb_dec_128_avx
+#define AES_ECB_DEC_192       aes_ecb_dec_192_avx
+#define AES_ECB_DEC_256       aes_ecb_dec_256_avx
+
+#define SUBMIT_JOB_PON_ENC        submit_job_pon_enc_avx
+#define SUBMIT_JOB_PON_DEC        submit_job_pon_dec_avx
+#define SUBMIT_JOB_PON_ENC_NO_CTR submit_job_pon_enc_no_ctr_avx
+#define SUBMIT_JOB_PON_DEC_NO_CTR submit_job_pon_dec_no_ctr_avx
+
 #ifndef NO_GCM
 #define AES_GCM_DEC_128   aes_gcm_dec_128_avx_gen4
 #define AES_GCM_ENC_128   aes_gcm_enc_128_avx_gen4
@@ -138,6 +167,10 @@ JOB_AES_HMAC *submit_job_aes_cmac_auth_avx(MB_MGR_CMAC_OOO *state,
 
 JOB_AES_HMAC *flush_job_aes_cmac_auth_avx(MB_MGR_CMAC_OOO *state);
 
+JOB_AES_HMAC *submit_job_aes_ccm_auth_avx(MB_MGR_CCM_OOO *state,
+                                           JOB_AES_HMAC *job);
+
+JOB_AES_HMAC *flush_job_aes_ccm_auth_avx(MB_MGR_CCM_OOO *state);
 
 #define SUBMIT_JOB_HMAC               submit_job_hmac_avx2
 #define FLUSH_JOB_HMAC                flush_job_hmac_avx2
@@ -170,13 +203,12 @@ JOB_AES_HMAC *flush_job_aes_cmac_auth_avx(MB_MGR_CMAC_OOO *state);
 
 #define AES_CFB_128_ONE    aes_cfb_128_one_avx2
 
-void aes128_cbc_mac_x8(AES_ARGS_x8 *args, uint64_t len);
+void aes128_cbc_mac_x8(AES_ARGS *args, uint64_t len);
 
 #define AES128_CBC_MAC     aes128_cbc_mac_x8
 
-#define FLUSH_JOB_AES_CCM_AUTH     flush_job_aes_ccm_auth_arch
-#define SUBMIT_JOB_AES_CCM_AUTH    submit_job_aes_ccm_auth_arch
-#define AES_CCM_MAX_JOBS 8
+#define FLUSH_JOB_AES_CCM_AUTH     flush_job_aes_ccm_auth_avx
+#define SUBMIT_JOB_AES_CCM_AUTH    submit_job_aes_ccm_auth_avx
 
 #define FLUSH_JOB_AES_CMAC_AUTH    flush_job_aes_cmac_auth_avx
 #define SUBMIT_JOB_AES_CMAC_AUTH   submit_job_aes_cmac_auth_avx
@@ -284,6 +316,7 @@ init_mb_mgr_avx2(MB_MGR *state)
 {
         unsigned int j;
         uint8_t *p;
+        size_t size;
 
         state->features = cpu_feature_adjust(state->flags,
                                              cpu_feature_detect());
@@ -294,79 +327,45 @@ init_mb_mgr_avx2(MB_MGR *state)
         }
 
         /* Init AES out-of-order fields */
-        state->aes128_ooo.lens[0] = 0;
-        state->aes128_ooo.lens[1] = 0;
-        state->aes128_ooo.lens[2] = 0;
-        state->aes128_ooo.lens[3] = 0;
-        state->aes128_ooo.lens[4] = 0;
-        state->aes128_ooo.lens[5] = 0;
-        state->aes128_ooo.lens[6] = 0;
-        state->aes128_ooo.lens[7] = 0;
+        memset(state->aes128_ooo.lens, 0xFF,
+               sizeof(state->aes128_ooo.lens));
+        memset(&state->aes128_ooo.lens[0], 0,
+               sizeof(state->aes128_ooo.lens[0]) * 8);
+        memset(state->aes128_ooo.job_in_lane, 0,
+               sizeof(state->aes128_ooo.job_in_lane));
         state->aes128_ooo.unused_lanes = 0xF76543210;
-        state->aes128_ooo.job_in_lane[0] = NULL;
-        state->aes128_ooo.job_in_lane[1] = NULL;
-        state->aes128_ooo.job_in_lane[2] = NULL;
-        state->aes128_ooo.job_in_lane[3] = NULL;
-        state->aes128_ooo.job_in_lane[4] = NULL;
-        state->aes128_ooo.job_in_lane[5] = NULL;
-        state->aes128_ooo.job_in_lane[6] = NULL;
-        state->aes128_ooo.job_in_lane[7] = NULL;
+        state->aes128_ooo.num_lanes_inuse = 0;
 
-        state->aes192_ooo.lens[0] = 0;
-        state->aes192_ooo.lens[1] = 0;
-        state->aes192_ooo.lens[2] = 0;
-        state->aes192_ooo.lens[3] = 0;
-        state->aes192_ooo.lens[4] = 0;
-        state->aes192_ooo.lens[5] = 0;
-        state->aes192_ooo.lens[6] = 0;
-        state->aes192_ooo.lens[7] = 0;
+        memset(state->aes192_ooo.lens, 0xFF,
+               sizeof(state->aes192_ooo.lens));
+        memset(&state->aes192_ooo.lens[0], 0,
+               sizeof(state->aes192_ooo.lens[0]) * 8);
+        memset(state->aes192_ooo.job_in_lane, 0,
+               sizeof(state->aes192_ooo.job_in_lane));
         state->aes192_ooo.unused_lanes = 0xF76543210;
-        state->aes192_ooo.job_in_lane[0] = NULL;
-        state->aes192_ooo.job_in_lane[1] = NULL;
-        state->aes192_ooo.job_in_lane[2] = NULL;
-        state->aes192_ooo.job_in_lane[3] = NULL;
-        state->aes192_ooo.job_in_lane[4] = NULL;
-        state->aes192_ooo.job_in_lane[5] = NULL;
-        state->aes192_ooo.job_in_lane[6] = NULL;
-        state->aes192_ooo.job_in_lane[7] = NULL;
+        state->aes192_ooo.num_lanes_inuse = 0;
 
-
-        state->aes256_ooo.lens[0] = 0;
-        state->aes256_ooo.lens[1] = 0;
-        state->aes256_ooo.lens[2] = 0;
-        state->aes256_ooo.lens[3] = 0;
-        state->aes256_ooo.lens[4] = 0;
-        state->aes256_ooo.lens[5] = 0;
-        state->aes256_ooo.lens[6] = 0;
-        state->aes256_ooo.lens[7] = 0;
+        memset(&state->aes256_ooo.lens, 0xFF,
+               sizeof(state->aes256_ooo.lens));
+        memset(&state->aes256_ooo.lens[0], 0,
+               sizeof(state->aes256_ooo.lens[0]) * 8);
+        memset(state->aes256_ooo.job_in_lane, 0,
+               sizeof(state->aes256_ooo.job_in_lane));
         state->aes256_ooo.unused_lanes = 0xF76543210;
-        state->aes256_ooo.job_in_lane[0] = NULL;
-        state->aes256_ooo.job_in_lane[1] = NULL;
-        state->aes256_ooo.job_in_lane[2] = NULL;
-        state->aes256_ooo.job_in_lane[3] = NULL;
-        state->aes256_ooo.job_in_lane[4] = NULL;
-        state->aes256_ooo.job_in_lane[5] = NULL;
-        state->aes256_ooo.job_in_lane[6] = NULL;
-        state->aes256_ooo.job_in_lane[7] = NULL;
+        state->aes256_ooo.num_lanes_inuse = 0;
 
-        /* DOCSIS SEC BPI uses same settings as AES128 CBC */
-        state->docsis_sec_ooo.lens[0] = 0;
-        state->docsis_sec_ooo.lens[1] = 0;
-        state->docsis_sec_ooo.lens[2] = 0;
-        state->docsis_sec_ooo.lens[3] = 0;
-        state->docsis_sec_ooo.lens[4] = 0;
-        state->docsis_sec_ooo.lens[5] = 0;
-        state->docsis_sec_ooo.lens[6] = 0;
-        state->docsis_sec_ooo.lens[7] = 0;
+        /* DOCSIS SEC BPI (AES CBC + AES CFB for partial block)
+         * uses same settings as AES128 CBC.
+         */
+        memset(state->docsis_sec_ooo.lens, 0xFF,
+               sizeof(state->docsis_sec_ooo.lens));
+        memset(&state->docsis_sec_ooo.lens[0], 0,
+               sizeof(state->docsis_sec_ooo.lens[0]) * 8);
+        memset(state->docsis_sec_ooo.job_in_lane, 0,
+               sizeof(state->docsis_sec_ooo.job_in_lane));
         state->docsis_sec_ooo.unused_lanes = 0xF76543210;
-        state->docsis_sec_ooo.job_in_lane[0] = NULL;
-        state->docsis_sec_ooo.job_in_lane[1] = NULL;
-        state->docsis_sec_ooo.job_in_lane[2] = NULL;
-        state->docsis_sec_ooo.job_in_lane[3] = NULL;
-        state->docsis_sec_ooo.job_in_lane[4] = NULL;
-        state->docsis_sec_ooo.job_in_lane[5] = NULL;
-        state->docsis_sec_ooo.job_in_lane[6] = NULL;
-        state->docsis_sec_ooo.job_in_lane[7] = NULL;
+        state->docsis_sec_ooo.num_lanes_inuse = 0;
+
 
         /* Init HMAC/SHA1 out-of-order fields */
         state->hmac_sha_1_ooo.lens[0] = 0;
@@ -407,14 +406,15 @@ init_mb_mgr_avx2(MB_MGR *state)
          */
         for (j = 0; j < AVX2_NUM_SHA256_LANES; j++) {
                 state->hmac_sha_224_ooo.ldata[j].job_in_lane = NULL;
-                state->hmac_sha_224_ooo.ldata[j].extra_block[64] = 0x80;
-                memset(state->hmac_sha_224_ooo.ldata[j].extra_block + 65,
-                       0x00,
-                       64 + 7);
+
+                p = state->hmac_sha_224_ooo.ldata[j].extra_block;
+                size = sizeof(state->hmac_sha_224_ooo.ldata[j].extra_block);
+                memset (p, 0x00, size);
+                p[64] = 0x80;
+
                 p = state->hmac_sha_224_ooo.ldata[j].outer_block;
-                memset(p + 8*4 + 1,
-                       0x00,
-                       64 - 8*4 - 1 - 2);
+                size = sizeof(state->hmac_sha_224_ooo.ldata[j].outer_block);
+                memset(p, 0x00, size);
                 p[7 * 4] = 0x80;  /* digest 7 words long */
                 p[64 - 2] = 0x02; /* length in little endian = 0x02E0 */
                 p[64 - 1] = 0xE0;
@@ -537,14 +537,15 @@ init_mb_mgr_avx2(MB_MGR *state)
         state->hmac_md5_ooo.num_lanes_inuse = 0;
         for (j = 0; j < AVX2_NUM_MD5_LANES; j++) {
                 state->hmac_md5_ooo.ldata[j].job_in_lane = NULL;
-                state->hmac_md5_ooo.ldata[j].extra_block[64] = 0x80;
-                memset(state->hmac_md5_ooo.ldata[j].extra_block + 65,
-                       0x00,
-                       64 + 7);
+
+                p = state->hmac_md5_ooo.ldata[j].extra_block;
+                size = sizeof(state->hmac_md5_ooo.ldata[j].extra_block);
+                memset (p, 0x00, size);
+                p[64] = 0x80;
+
                 p = state->hmac_md5_ooo.ldata[j].outer_block;
-                memset(p + 5*4 + 1,
-                       0x00,
-                       64 - 5*4 - 1 - 2);
+                size = sizeof(state->hmac_md5_ooo.ldata[j].outer_block);
+                memset(p, 0x00, size);
                 p[4 * 4] = 0x80;
                 p[64 - 7] = 0x02;
                 p[64 - 8] = 0x80;
@@ -611,6 +612,36 @@ init_mb_mgr_avx2(MB_MGR *state)
         state->sha512              = sha512_avx2;
         state->md5_one_block       = md5_one_block_avx2;
         state->aes128_cfb_one      = aes_cfb_128_one_avx2;
+
+        state->eea3_1_buffer       = zuc_eea3_1_buffer_avx;
+        state->eea3_4_buffer       = zuc_eea3_4_buffer_avx;
+        state->eea3_n_buffer       = zuc_eea3_n_buffer_avx;
+        state->eia3_1_buffer       = zuc_eia3_1_buffer_avx;
+
+        state->f8_1_buffer         = kasumi_f8_1_buffer_avx;
+        state->f8_1_buffer_bit     = kasumi_f8_1_buffer_bit_avx;
+        state->f8_2_buffer         = kasumi_f8_2_buffer_avx;
+        state->f8_3_buffer         = kasumi_f8_3_buffer_avx;
+        state->f8_4_buffer         = kasumi_f8_4_buffer_avx;
+        state->f8_n_buffer         = kasumi_f8_n_buffer_avx;
+        state->f9_1_buffer         = kasumi_f9_1_buffer_avx;
+        state->f9_1_buffer_user    = kasumi_f9_1_buffer_user_avx;
+        state->kasumi_init_f8_key_sched = kasumi_init_f8_key_sched_avx;
+        state->kasumi_init_f9_key_sched = kasumi_init_f9_key_sched_avx;
+        state->kasumi_key_sched_size = kasumi_key_sched_size_avx;
+
+        state->snow3g_f8_1_buffer_bit = snow3g_f8_1_buffer_bit_avx2;
+        state->snow3g_f8_1_buffer  = snow3g_f8_1_buffer_avx2;
+        state->snow3g_f8_2_buffer  = snow3g_f8_2_buffer_avx2;
+        state->snow3g_f8_4_buffer  = snow3g_f8_4_buffer_avx2;
+        state->snow3g_f8_8_buffer  = snow3g_f8_8_buffer_avx2;
+        state->snow3g_f8_n_buffer  = snow3g_f8_n_buffer_avx2;
+        state->snow3g_f8_8_buffer_multikey = snow3g_f8_8_buffer_multikey_avx2;
+        state->snow3g_f8_n_buffer_multikey = snow3g_f8_n_buffer_multikey_avx2;
+        state->snow3g_f9_1_buffer = snow3g_f9_1_buffer_avx2;
+        state->snow3g_init_key_sched = snow3g_init_key_sched_avx2;
+        state->snow3g_key_sched_size = snow3g_key_sched_size_avx2;
+
 #ifndef NO_GCM
         state->gcm128_enc          = aes_gcm_enc_128_avx_gen4;
         state->gcm192_enc          = aes_gcm_enc_192_avx_gen4;
