@@ -56,21 +56,21 @@ free_rpc_vhost_scsi_ctrlr(struct rpc_vhost_scsi_ctrlr *req)
 	free(req->cpumask);
 }
 
-static const struct spdk_json_object_decoder rpc_construct_vhost_ctrlr[] = {
+static const struct spdk_json_object_decoder rpc_vhost_create_scsi_ctrlr[] = {
 	{"ctrlr", offsetof(struct rpc_vhost_scsi_ctrlr, ctrlr), spdk_json_decode_string },
 	{"cpumask", offsetof(struct rpc_vhost_scsi_ctrlr, cpumask), spdk_json_decode_string, true},
 };
 
 static void
-spdk_rpc_construct_vhost_scsi_controller(struct spdk_jsonrpc_request *request,
-		const struct spdk_json_val *params)
+rpc_vhost_create_scsi_controller(struct spdk_jsonrpc_request *request,
+				 const struct spdk_json_val *params)
 {
 	struct rpc_vhost_scsi_ctrlr req = {0};
 	struct spdk_json_write_ctx *w;
 	int rc;
 
-	if (spdk_json_decode_object(params, rpc_construct_vhost_ctrlr,
-				    SPDK_COUNTOF(rpc_construct_vhost_ctrlr),
+	if (spdk_json_decode_object(params, rpc_vhost_create_scsi_ctrlr,
+				    SPDK_COUNTOF(rpc_vhost_create_scsi_ctrlr),
 				    &req)) {
 		SPDK_DEBUGLOG(SPDK_LOG_VHOST_RPC, "spdk_json_decode_object failed\n");
 		rc = -EINVAL;
@@ -85,10 +85,6 @@ spdk_rpc_construct_vhost_scsi_controller(struct spdk_jsonrpc_request *request,
 	free_rpc_vhost_scsi_ctrlr(&req);
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w == NULL) {
-		return;
-	}
-
 	spdk_json_write_bool(w, true);
 	spdk_jsonrpc_end_result(request, w);
 	return;
@@ -98,39 +94,40 @@ invalid:
 	spdk_jsonrpc_send_error_response(request, SPDK_JSONRPC_ERROR_INVALID_PARAMS,
 					 spdk_strerror(-rc));
 }
-SPDK_RPC_REGISTER("construct_vhost_scsi_controller", spdk_rpc_construct_vhost_scsi_controller,
+SPDK_RPC_REGISTER("vhost_create_scsi_controller", rpc_vhost_create_scsi_controller,
 		  SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_create_scsi_controller, construct_vhost_scsi_controller)
 
-struct rpc_add_vhost_scsi_ctrlr_lun {
+struct rpc_vhost_scsi_ctrlr_add_target {
 	char *ctrlr;
 	int32_t scsi_target_num;
 	char *bdev_name;
 };
 
 static void
-free_rpc_add_vhost_scsi_ctrlr_lun(struct rpc_add_vhost_scsi_ctrlr_lun *req)
+free_rpc_vhost_scsi_ctrlr_add_target(struct rpc_vhost_scsi_ctrlr_add_target *req)
 {
 	free(req->ctrlr);
 	free(req->bdev_name);
 }
 
-static const struct spdk_json_object_decoder rpc_vhost_add_lun[] = {
-	{"ctrlr", offsetof(struct rpc_add_vhost_scsi_ctrlr_lun, ctrlr), spdk_json_decode_string },
-	{"scsi_target_num", offsetof(struct rpc_add_vhost_scsi_ctrlr_lun, scsi_target_num), spdk_json_decode_int32},
-	{"bdev_name", offsetof(struct rpc_add_vhost_scsi_ctrlr_lun, bdev_name), spdk_json_decode_string },
+static const struct spdk_json_object_decoder rpc_vhost_scsi_ctrlr_add_target[] = {
+	{"ctrlr", offsetof(struct rpc_vhost_scsi_ctrlr_add_target, ctrlr), spdk_json_decode_string },
+	{"scsi_target_num", offsetof(struct rpc_vhost_scsi_ctrlr_add_target, scsi_target_num), spdk_json_decode_int32},
+	{"bdev_name", offsetof(struct rpc_vhost_scsi_ctrlr_add_target, bdev_name), spdk_json_decode_string },
 };
 
 static void
-spdk_rpc_add_vhost_scsi_lun(struct spdk_jsonrpc_request *request,
-			    const struct spdk_json_val *params)
+rpc_vhost_scsi_controller_add_target(struct spdk_jsonrpc_request *request,
+				     const struct spdk_json_val *params)
 {
-	struct rpc_add_vhost_scsi_ctrlr_lun req = {0};
+	struct rpc_vhost_scsi_ctrlr_add_target req = {0};
 	struct spdk_json_write_ctx *w;
 	struct spdk_vhost_dev *vdev;
 	int rc;
 
-	if (spdk_json_decode_object(params, rpc_vhost_add_lun,
-				    SPDK_COUNTOF(rpc_vhost_add_lun),
+	if (spdk_json_decode_object(params, rpc_vhost_scsi_ctrlr_add_target,
+				    SPDK_COUNTOF(rpc_vhost_scsi_ctrlr_add_target),
 				    &req)) {
 		SPDK_DEBUGLOG(SPDK_LOG_VHOST_RPC, "spdk_json_decode_object failed\n");
 		rc = -EINVAL;
@@ -151,23 +148,21 @@ spdk_rpc_add_vhost_scsi_lun(struct spdk_jsonrpc_request *request,
 		goto invalid;
 	}
 
-	free_rpc_add_vhost_scsi_ctrlr_lun(&req);
+	free_rpc_vhost_scsi_ctrlr_add_target(&req);
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w == NULL) {
-		return;
-	}
-
 	spdk_json_write_int32(w, rc);
 	spdk_jsonrpc_end_result(request, w);
 	return;
 
 invalid:
-	free_rpc_add_vhost_scsi_ctrlr_lun(&req);
+	free_rpc_vhost_scsi_ctrlr_add_target(&req);
 	spdk_jsonrpc_send_error_response(request, SPDK_JSONRPC_ERROR_INVALID_PARAMS,
 					 spdk_strerror(-rc));
 }
-SPDK_RPC_REGISTER("add_vhost_scsi_lun", spdk_rpc_add_vhost_scsi_lun, SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER("vhost_scsi_controller_add_target", rpc_vhost_scsi_controller_add_target,
+		  SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_scsi_controller_add_target, add_vhost_scsi_lun)
 
 struct rpc_remove_vhost_scsi_ctrlr_target {
 	char *ctrlr;
@@ -186,24 +181,20 @@ static const struct spdk_json_object_decoder rpc_vhost_remove_target[] = {
 };
 
 static int
-spdk_rpc_remove_vhost_scsi_target_finish_cb(struct spdk_vhost_dev *vdev, void *arg)
+rpc_vhost_scsi_controller_remove_target_finish_cb(struct spdk_vhost_dev *vdev, void *arg)
 {
 	struct spdk_jsonrpc_request *request = arg;
 	struct spdk_json_write_ctx *w;
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w == NULL) {
-		return -1;
-	}
-
 	spdk_json_write_bool(w, true);
 	spdk_jsonrpc_end_result(request, w);
 	return 0;
 }
 
 static void
-spdk_rpc_remove_vhost_scsi_target(struct spdk_jsonrpc_request *request,
-				  const struct spdk_json_val *params)
+rpc_vhost_scsi_controller_remove_target(struct spdk_jsonrpc_request *request,
+					const struct spdk_json_val *params)
 {
 	struct rpc_remove_vhost_scsi_ctrlr_target req = {0};
 	struct spdk_vhost_dev *vdev;
@@ -226,7 +217,7 @@ spdk_rpc_remove_vhost_scsi_target(struct spdk_jsonrpc_request *request,
 	}
 
 	rc = spdk_vhost_scsi_dev_remove_tgt(vdev, req.scsi_target_num,
-					    spdk_rpc_remove_vhost_scsi_target_finish_cb,
+					    rpc_vhost_scsi_controller_remove_target_finish_cb,
 					    request);
 	spdk_vhost_unlock();
 	if (rc < 0) {
@@ -242,13 +233,16 @@ invalid:
 					 spdk_strerror(-rc));
 }
 
-SPDK_RPC_REGISTER("remove_vhost_scsi_target", spdk_rpc_remove_vhost_scsi_target, SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER("vhost_scsi_controller_remove_target",
+		  rpc_vhost_scsi_controller_remove_target, SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_scsi_controller_remove_target, remove_vhost_scsi_target)
 
 struct rpc_vhost_blk_ctrlr {
 	char *ctrlr;
 	char *dev_name;
 	char *cpumask;
 	bool readonly;
+	bool packed_ring;
 };
 
 static const struct spdk_json_object_decoder rpc_construct_vhost_blk_ctrlr[] = {
@@ -256,6 +250,7 @@ static const struct spdk_json_object_decoder rpc_construct_vhost_blk_ctrlr[] = {
 	{"dev_name", offsetof(struct rpc_vhost_blk_ctrlr, dev_name), spdk_json_decode_string },
 	{"cpumask", offsetof(struct rpc_vhost_blk_ctrlr, cpumask), spdk_json_decode_string, true},
 	{"readonly", offsetof(struct rpc_vhost_blk_ctrlr, readonly), spdk_json_decode_bool, true},
+	{"packed_ring", offsetof(struct rpc_vhost_blk_ctrlr, packed_ring), spdk_json_decode_bool, true},
 };
 
 static void
@@ -267,8 +262,8 @@ free_rpc_vhost_blk_ctrlr(struct rpc_vhost_blk_ctrlr *req)
 }
 
 static void
-spdk_rpc_construct_vhost_blk_controller(struct spdk_jsonrpc_request *request,
-					const struct spdk_json_val *params)
+rpc_vhost_create_blk_controller(struct spdk_jsonrpc_request *request,
+				const struct spdk_json_val *params)
 {
 	struct rpc_vhost_blk_ctrlr req = {0};
 	struct spdk_json_write_ctx *w;
@@ -282,7 +277,8 @@ spdk_rpc_construct_vhost_blk_controller(struct spdk_jsonrpc_request *request,
 		goto invalid;
 	}
 
-	rc = spdk_vhost_blk_construct(req.ctrlr, req.cpumask, req.dev_name, req.readonly);
+	rc = spdk_vhost_blk_construct(req.ctrlr, req.cpumask, req.dev_name,
+				      req.readonly, req.packed_ring);
 	if (rc < 0) {
 		goto invalid;
 	}
@@ -290,10 +286,6 @@ spdk_rpc_construct_vhost_blk_controller(struct spdk_jsonrpc_request *request,
 	free_rpc_vhost_blk_ctrlr(&req);
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w == NULL) {
-		return;
-	}
-
 	spdk_json_write_bool(w, true);
 	spdk_jsonrpc_end_result(request, w);
 	return;
@@ -304,34 +296,35 @@ invalid:
 					 spdk_strerror(-rc));
 
 }
-SPDK_RPC_REGISTER("construct_vhost_blk_controller", spdk_rpc_construct_vhost_blk_controller,
+SPDK_RPC_REGISTER("vhost_create_blk_controller", rpc_vhost_create_blk_controller,
 		  SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_create_blk_controller, construct_vhost_blk_controller)
 
-struct rpc_remove_vhost_ctrlr {
+struct rpc_delete_vhost_ctrlr {
 	char *ctrlr;
 };
 
-static const struct spdk_json_object_decoder rpc_remove_vhost_ctrlr[] = {
-	{"ctrlr", offsetof(struct rpc_remove_vhost_ctrlr, ctrlr), spdk_json_decode_string },
+static const struct spdk_json_object_decoder rpc_delete_vhost_ctrlr_decoder[] = {
+	{"ctrlr", offsetof(struct rpc_delete_vhost_ctrlr, ctrlr), spdk_json_decode_string },
 };
 
 static void
-free_rpc_remove_vhost_ctrlr(struct rpc_remove_vhost_ctrlr *req)
+free_rpc_delete_vhost_ctrlr(struct rpc_delete_vhost_ctrlr *req)
 {
 	free(req->ctrlr);
 }
 
 static void
-spdk_rpc_remove_vhost_controller(struct spdk_jsonrpc_request *request,
-				 const struct spdk_json_val *params)
+rpc_vhost_delete_controller(struct spdk_jsonrpc_request *request,
+			    const struct spdk_json_val *params)
 {
-	struct rpc_remove_vhost_ctrlr req = {0};
+	struct rpc_delete_vhost_ctrlr req = {0};
 	struct spdk_json_write_ctx *w;
 	struct spdk_vhost_dev *vdev;
 	int rc;
 
-	if (spdk_json_decode_object(params, rpc_remove_vhost_ctrlr,
-				    SPDK_COUNTOF(rpc_remove_vhost_ctrlr), &req)) {
+	if (spdk_json_decode_object(params, rpc_delete_vhost_ctrlr_decoder,
+				    SPDK_COUNTOF(rpc_delete_vhost_ctrlr_decoder), &req)) {
 		SPDK_DEBUGLOG(SPDK_LOG_VHOST_RPC, "spdk_json_decode_object failed\n");
 		rc = -EINVAL;
 		goto invalid;
@@ -351,32 +344,29 @@ spdk_rpc_remove_vhost_controller(struct spdk_jsonrpc_request *request,
 		goto invalid;
 	}
 
-	free_rpc_remove_vhost_ctrlr(&req);
+	free_rpc_delete_vhost_ctrlr(&req);
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w == NULL) {
-		return;
-	}
-
 	spdk_json_write_bool(w, true);
 	spdk_jsonrpc_end_result(request, w);
 
 	return;
 
 invalid:
-	free_rpc_remove_vhost_ctrlr(&req);
+	free_rpc_delete_vhost_ctrlr(&req);
 	spdk_jsonrpc_send_error_response(request, SPDK_JSONRPC_ERROR_INVALID_PARAMS,
 					 spdk_strerror(-rc));
 
 }
-SPDK_RPC_REGISTER("remove_vhost_controller", spdk_rpc_remove_vhost_controller, SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER("vhost_delete_controller", rpc_vhost_delete_controller, SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_delete_controller, remove_vhost_controller)
 
 struct rpc_get_vhost_ctrlrs {
 	char *name;
 };
 
 static void
-_spdk_rpc_get_vhost_controller(struct spdk_json_write_ctx *w, struct spdk_vhost_dev *vdev)
+_rpc_get_vhost_controller(struct spdk_json_write_ctx *w, struct spdk_vhost_dev *vdev)
 {
 	uint32_t delay_base_us, iops_threshold;
 
@@ -385,13 +375,14 @@ _spdk_rpc_get_vhost_controller(struct spdk_json_write_ctx *w, struct spdk_vhost_
 	spdk_json_write_object_begin(w);
 
 	spdk_json_write_named_string(w, "ctrlr", spdk_vhost_dev_get_name(vdev));
-	spdk_json_write_named_string_fmt(w, "cpumask", "0x%s", spdk_cpuset_fmt(vdev->cpumask));
+	spdk_json_write_named_string_fmt(w, "cpumask", "0x%s",
+					 spdk_cpuset_fmt(spdk_thread_get_cpumask(vdev->thread)));
 	spdk_json_write_named_uint32(w, "delay_base_us", delay_base_us);
 	spdk_json_write_named_uint32(w, "iops_threshold", iops_threshold);
 	spdk_json_write_named_string(w, "socket", vdev->path);
 
 	spdk_json_write_named_object_begin(w, "backend_specific");
-	spdk_vhost_dump_info_json(vdev, w);
+	vhost_dump_info_json(vdev, w);
 	spdk_json_write_object_end(w);
 
 	spdk_json_write_object_end(w);
@@ -408,8 +399,8 @@ free_rpc_get_vhost_ctrlrs(struct rpc_get_vhost_ctrlrs *req)
 }
 
 static void
-spdk_rpc_get_vhost_controllers(struct spdk_jsonrpc_request *request,
-			       const struct spdk_json_val *params)
+rpc_vhost_get_controllers(struct spdk_jsonrpc_request *request,
+			  const struct spdk_json_val *params)
 {
 	struct rpc_get_vhost_ctrlrs req = {0};
 	struct spdk_json_write_ctx *w;
@@ -435,14 +426,9 @@ spdk_rpc_get_vhost_controllers(struct spdk_jsonrpc_request *request,
 		free_rpc_get_vhost_ctrlrs(&req);
 
 		w = spdk_jsonrpc_begin_result(request);
-		if (w == NULL) {
-			spdk_vhost_unlock();
-			return;
-		}
-
 		spdk_json_write_array_begin(w);
 
-		_spdk_rpc_get_vhost_controller(w, vdev);
+		_rpc_get_vhost_controller(w, vdev);
 		spdk_vhost_unlock();
 
 		spdk_json_write_array_end(w);
@@ -453,15 +439,11 @@ spdk_rpc_get_vhost_controllers(struct spdk_jsonrpc_request *request,
 	free_rpc_get_vhost_ctrlrs(&req);
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w == NULL) {
-		spdk_vhost_unlock();
-		return;
-	}
 	spdk_json_write_array_begin(w);
 
 	vdev = spdk_vhost_dev_next(NULL);
 	while (vdev != NULL) {
-		_spdk_rpc_get_vhost_controller(w, vdev);
+		_rpc_get_vhost_controller(w, vdev);
 		vdev = spdk_vhost_dev_next(vdev);
 	}
 	spdk_vhost_unlock();
@@ -475,7 +457,8 @@ invalid:
 	spdk_jsonrpc_send_error_response(request, SPDK_JSONRPC_ERROR_INTERNAL_ERROR,
 					 spdk_strerror(-rc));
 }
-SPDK_RPC_REGISTER("get_vhost_controllers", spdk_rpc_get_vhost_controllers, SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER("vhost_get_controllers", rpc_vhost_get_controllers, SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_get_controllers, get_vhost_controllers)
 
 
 struct rpc_vhost_ctrlr_coalescing {
@@ -497,8 +480,8 @@ free_rpc_set_vhost_controllers_event_coalescing(struct rpc_vhost_ctrlr_coalescin
 }
 
 static void
-spdk_rpc_set_vhost_controller_coalescing(struct spdk_jsonrpc_request *request,
-		const struct spdk_json_val *params)
+rpc_vhost_controller_set_coalescing(struct spdk_jsonrpc_request *request,
+				    const struct spdk_json_val *params)
 {
 	struct rpc_vhost_ctrlr_coalescing req = {0};
 	struct spdk_json_write_ctx *w;
@@ -529,10 +512,8 @@ spdk_rpc_set_vhost_controller_coalescing(struct spdk_jsonrpc_request *request,
 	free_rpc_set_vhost_controllers_event_coalescing(&req);
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w != NULL) {
-		spdk_json_write_bool(w, true);
-		spdk_jsonrpc_end_result(request, w);
-	}
+	spdk_json_write_bool(w, true);
+	spdk_jsonrpc_end_result(request, w);
 
 	return;
 
@@ -541,8 +522,9 @@ invalid:
 	spdk_jsonrpc_send_error_response(request, SPDK_JSONRPC_ERROR_INVALID_PARAMS,
 					 spdk_strerror(-rc));
 }
-SPDK_RPC_REGISTER("set_vhost_controller_coalescing", spdk_rpc_set_vhost_controller_coalescing,
+SPDK_RPC_REGISTER("vhost_controller_set_coalescing", rpc_vhost_controller_set_coalescing,
 		  SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_controller_set_coalescing, set_vhost_controller_coalescing)
 
 #ifdef SPDK_CONFIG_VHOST_INTERNAL_LIB
 
@@ -566,8 +548,8 @@ free_rpc_vhost_nvme_ctrlr(struct rpc_vhost_nvme_ctrlr *req)
 }
 
 static void
-spdk_rpc_construct_vhost_nvme_controller(struct spdk_jsonrpc_request *request,
-		const struct spdk_json_val *params)
+rpc_vhost_create_nvme_controller(struct spdk_jsonrpc_request *request,
+				 const struct spdk_json_val *params)
 {
 	struct rpc_vhost_nvme_ctrlr req = {};
 	struct spdk_json_write_ctx *w;
@@ -580,7 +562,7 @@ spdk_rpc_construct_vhost_nvme_controller(struct spdk_jsonrpc_request *request,
 		goto invalid;
 	}
 
-	rc = spdk_vhost_nvme_dev_construct(req.ctrlr, req.cpumask, req.io_queues);
+	rc = vhost_nvme_dev_construct(req.ctrlr, req.cpumask, req.io_queues);
 	if (rc < 0) {
 		goto invalid;
 	}
@@ -588,10 +570,6 @@ spdk_rpc_construct_vhost_nvme_controller(struct spdk_jsonrpc_request *request,
 	free_rpc_vhost_nvme_ctrlr(&req);
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w == NULL) {
-		return;
-	}
-
 	spdk_json_write_bool(w, true);
 	spdk_jsonrpc_end_result(request, w);
 	return;
@@ -602,31 +580,32 @@ invalid:
 					 spdk_strerror(-rc));
 
 }
-SPDK_RPC_REGISTER("construct_vhost_nvme_controller", spdk_rpc_construct_vhost_nvme_controller,
+SPDK_RPC_REGISTER("vhost_create_nvme_controller", rpc_vhost_create_nvme_controller,
 		  SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_create_nvme_controller, construct_vhost_nvme_controller)
 
-struct rpc_add_vhost_nvme_ctrlr_ns {
+struct rpc_vhost_nvme_ctrlr_add_ns {
 	char *ctrlr;
 	char *bdev_name;
 };
 
 static void
-free_rpc_add_vhost_nvme_ctrlr_ns(struct rpc_add_vhost_nvme_ctrlr_ns *req)
+free_rpc_vhost_nvme_ctrlr_add_ns(struct rpc_vhost_nvme_ctrlr_add_ns *req)
 {
 	free(req->ctrlr);
 	free(req->bdev_name);
 }
 
 static const struct spdk_json_object_decoder rpc_vhost_nvme_add_ns[] = {
-	{"ctrlr", offsetof(struct rpc_add_vhost_nvme_ctrlr_ns, ctrlr), spdk_json_decode_string },
-	{"bdev_name", offsetof(struct rpc_add_vhost_nvme_ctrlr_ns, bdev_name), spdk_json_decode_string },
+	{"ctrlr", offsetof(struct rpc_vhost_nvme_ctrlr_add_ns, ctrlr), spdk_json_decode_string },
+	{"bdev_name", offsetof(struct rpc_vhost_nvme_ctrlr_add_ns, bdev_name), spdk_json_decode_string },
 };
 
 static void
-spdk_rpc_add_vhost_nvme_ns(struct spdk_jsonrpc_request *request,
-			   const struct spdk_json_val *params)
+rpc_vhost_nvme_controller_add_ns(struct spdk_jsonrpc_request *request,
+				 const struct spdk_json_val *params)
 {
-	struct rpc_add_vhost_nvme_ctrlr_ns req = {0};
+	struct rpc_vhost_nvme_ctrlr_add_ns req = {0};
 	struct spdk_json_write_ctx *w;
 	struct spdk_vhost_dev *vdev;
 	int rc;
@@ -647,28 +626,26 @@ spdk_rpc_add_vhost_nvme_ns(struct spdk_jsonrpc_request *request,
 		goto invalid;
 	}
 
-	rc = spdk_vhost_nvme_dev_add_ns(vdev, req.bdev_name);
+	rc = vhost_nvme_dev_add_ns(vdev, req.bdev_name);
 	spdk_vhost_unlock();
 	if (rc < 0) {
 		goto invalid;
 	}
-	free_rpc_add_vhost_nvme_ctrlr_ns(&req);
+	free_rpc_vhost_nvme_ctrlr_add_ns(&req);
 
 	w = spdk_jsonrpc_begin_result(request);
-	if (w == NULL) {
-		return;
-	}
-
 	spdk_json_write_bool(w, true);
 	spdk_jsonrpc_end_result(request, w);
 	return;
 
 invalid:
-	free_rpc_add_vhost_nvme_ctrlr_ns(&req);
+	free_rpc_vhost_nvme_ctrlr_add_ns(&req);
 	spdk_jsonrpc_send_error_response(request, SPDK_JSONRPC_ERROR_INVALID_PARAMS,
 					 spdk_strerror(-rc));
 }
-SPDK_RPC_REGISTER("add_vhost_nvme_ns", spdk_rpc_add_vhost_nvme_ns, SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER("vhost_nvme_controller_add_ns", rpc_vhost_nvme_controller_add_ns,
+		  SPDK_RPC_RUNTIME)
+SPDK_RPC_REGISTER_ALIAS_DEPRECATED(vhost_nvme_controller_add_ns, add_vhost_nvme_ns)
 
 #endif /* SPDK_CONFIG_VHOST_INTERNAL_LIB */
 
