@@ -4,8 +4,8 @@ import os
 import re
 import sys
 
-comment = re.compile('^\s*#')
-assign = re.compile('^\s*([a-zA-Z_]+)\s*(\?)?=\s*([^#]*)')
+comment = re.compile(r'^\s*#')
+assign = re.compile(r'^\s*([a-zA-Z0-9_]+)\s*(\?)?=\s*([^#]*)')
 
 args = os.environ.copy()
 for arg in sys.argv:
