@@ -11,7 +11,7 @@ if [ ! -x $FIO_PATH/fio ]; then
 	error "Invalid path of fio binary"
 fi
 
-function run_spdk_iscsi_fio(){
+function run_spdk_iscsi_fio() {
 	$FIO_PATH/fio $testdir/perf.job "$@" --output-format=json
 }
 
@@ -20,12 +20,12 @@ iscsi_fio_results="$testdir/perf_output/iscsi_fio.json"
 trap "iscsiadm -m node --logout; iscsiadm -m node -o delete; exit 1" ERR SIGTERM SIGABRT
 iscsiadm -m discovery -t sendtargets -p $IP_T:$ISCSI_PORT
 iscsiadm -m node --login -p $IP_T:$ISCSI_PORT
-sleep 1
+waitforiscsidevices 1
 
 disks=($(iscsiadm -m session -P 3 | grep "Attached scsi disk" | awk '{print $4}'))
-for (( i=0; i < ${#disks[@]}; i++ ))
-do
+for ((i = 0; i < ${#disks[@]}; i++)); do
 	filename+=$(printf /dev/%s: "${disks[i]}")
+	waitforfile $filename
 	echo noop > /sys/block/${disks[i]}/queue/scheduler
 	echo "2" > /sys/block/${disks[i]}/queue/nomerges
 	echo "1024" > /sys/block/${disks[i]}/queue/nr_requests
