@@ -76,47 +76,46 @@ def verify(expr, retcode, msg):
 
 def verify_log_flag_rpc_methods(rpc_py, rpc_param):
     rpc = spdk_rpc(rpc_py)
-    output = rpc.get_log_flags()
+    output = rpc.log_get_flags()
     jsonvalue = json.loads(output)
     verify(not jsonvalue[rpc_param['log_flag']], 1,
-           "get_log_flags returned {}, expected false".format(jsonvalue))
-    rpc.set_log_flag(rpc_param['log_flag'])
-    output = rpc.get_log_flags()
+           "log_get_flags returned {}, expected false".format(jsonvalue))
+    rpc.log_set_flag(rpc_param['log_flag'])
+    output = rpc.log_get_flags()
     jsonvalue = json.loads(output)
     verify(jsonvalue[rpc_param['log_flag']], 1,
-           "get_log_flags returned {}, expected true".format(jsonvalue))
-    rpc.clear_log_flag(rpc_param['log_flag'])
-    output = rpc.get_log_flags()
+           "log_get_flags returned {}, expected true".format(jsonvalue))
+    rpc.log_clear_flag(rpc_param['log_flag'])
+    output = rpc.log_get_flags()
     jsonvalue = json.loads(output)
     verify(not jsonvalue[rpc_param['log_flag']], 1,
-           "get_log_flags returned {}, expected false".format(jsonvalue))
+           "log_get_flags returned {}, expected false".format(jsonvalue))
 
     print("verify_log_flag_rpc_methods passed")
 
 
 def verify_iscsi_connection_rpc_methods(rpc_py):
     rpc = spdk_rpc(rpc_py)
-    output = rpc.get_iscsi_connections()
+    output = rpc.iscsi_get_connections()
     jsonvalue = json.loads(output)
     verify(not jsonvalue, 1,
-           "get_iscsi_connections returned {}, expected empty".format(jsonvalue))
+           "iscsi_get_connections returned {}, expected empty".format(jsonvalue))
 
-    rpc.construct_malloc_bdev(rpc_param['malloc_bdev_size'], rpc_param['malloc_block_size'])
-    rpc.add_portal_group(portal_tag, "{}:{}".format(rpc_param['target_ip'], str(rpc_param['port'])))
-    rpc.add_initiator_group(initiator_tag, rpc_param['initiator_name'], rpc_param['netmask'])
+    rpc.bdev_malloc_create(rpc_param['malloc_bdev_size'], rpc_param['malloc_block_size'])
+    rpc.iscsi_create_portal_group(portal_tag, "{}:{}".format(rpc_param['target_ip'], str(rpc_param['port'])))
+    rpc.iscsi_create_initiator_group(initiator_tag, rpc_param['initiator_name'], rpc_param['netmask'])
 
     lun_mapping = "Malloc" + str(rpc_param['lun_total']) + ":0"
     net_mapping = portal_tag + ":" + initiator_tag
-    rpc.construct_target_node(rpc_param['target_name'], rpc_param['alias_name'], lun_mapping, net_mapping, rpc_param['queue_depth'], '-d')
+    rpc.iscsi_create_target_node(rpc_param['target_name'], rpc_param['alias_name'], lun_mapping,
+                                 net_mapping, rpc_param['queue_depth'], '-d')
     check_output('iscsiadm -m discovery -t st -p {}'.format(rpc_param['target_ip']), shell=True)
     check_output('iscsiadm -m node --login', shell=True)
-    name = json.loads(rpc.get_target_nodes())[0]['name']
-    output = rpc.get_iscsi_connections()
+    name = json.loads(rpc.iscsi_get_target_nodes())[0]['name']
+    output = rpc.iscsi_get_connections()
     jsonvalues = json.loads(output)
     verify(jsonvalues[0]['target_node_name'] == rpc_param['target_name'], 1,
            "target node name vaule is {}, expected {}".format(jsonvalues[0]['target_node_name'], rpc_param['target_name']))
-    verify(jsonvalues[0]['id'] == 0, 1,
-           "device id value is {}, expected 0".format(jsonvalues[0]['id']))
     verify(jsonvalues[0]['initiator_addr'] == rpc_param['initiator_ip'], 1,
            "initiator address values is {}, expected {}".format(jsonvalues[0]['initiator_addr'], rpc_param['initiator_ip']))
     verify(jsonvalues[0]['target_addr'] == rpc_param['target_ip'], 1,
@@ -124,38 +123,39 @@ def verify_iscsi_connection_rpc_methods(rpc_py):
 
     check_output('iscsiadm -m node --logout', shell=True)
     check_output('iscsiadm -m node -o delete', shell=True)
-    rpc.delete_initiator_group(initiator_tag)
-    rpc.delete_portal_group(portal_tag)
-    rpc.delete_target_node(name)
-    output = rpc.get_iscsi_connections()
+    rpc.iscsi_delete_initiator_group(initiator_tag)
+    rpc.iscsi_delete_portal_group(portal_tag)
+    rpc.iscsi_delete_target_node(name)
+    output = rpc.iscsi_get_connections()
     jsonvalues = json.loads(output)
     verify(not jsonvalues, 1,
-           "get_iscsi_connections returned {}, expected empty".format(jsonvalues))
+           "iscsi_get_connections returned {}, expected empty".format(jsonvalues))
 
     print("verify_iscsi_connection_rpc_methods passed")
 
 
 def verify_scsi_devices_rpc_methods(rpc_py):
     rpc = spdk_rpc(rpc_py)
-    output = rpc.get_scsi_devices()
+    output = rpc.scsi_get_devices()
     jsonvalue = json.loads(output)
     verify(not jsonvalue, 1,
-           "get_scsi_devices returned {}, expected empty".format(jsonvalue))
+           "scsi_get_devices returned {}, expected empty".format(jsonvalue))
 
-    rpc.construct_malloc_bdev(rpc_param['malloc_bdev_size'], rpc_param['malloc_block_size'])
-    rpc.add_portal_group(portal_tag, "{}:{}".format(rpc_param['target_ip'], str(rpc_param['port'])))
-    rpc.add_initiator_group(initiator_tag, rpc_param['initiator_name'], rpc_param['netmask'])
+    rpc.bdev_malloc_create(rpc_param['malloc_bdev_size'], rpc_param['malloc_block_size'])
+    rpc.iscsi_create_portal_group(portal_tag, "{}:{}".format(rpc_param['target_ip'], str(rpc_param['port'])))
+    rpc.iscsi_create_initiator_group(initiator_tag, rpc_param['initiator_name'], rpc_param['netmask'])
 
     lun_mapping = "Malloc" + str(rpc_param['lun_total']) + ":0"
     net_mapping = portal_tag + ":" + initiator_tag
-    rpc.construct_target_node(rpc_param['target_name'], rpc_param['alias_name'], lun_mapping, net_mapping, rpc_param['queue_depth'], '-d')
+    rpc.iscsi_create_target_node(rpc_param['target_name'], rpc_param['alias_name'], lun_mapping,
+                                 net_mapping, rpc_param['queue_depth'], '-d')
     check_output('iscsiadm -m discovery -t st -p {}'.format(rpc_param['target_ip']), shell=True)
     check_output('iscsiadm -m node --login', shell=True)
-    name = json.loads(rpc.get_target_nodes())[0]['name']
-    output = rpc.get_iscsi_global_params()
+    name = json.loads(rpc.iscsi_get_target_nodes())[0]['name']
+    output = rpc.iscsi_get_options()
     jsonvalues = json.loads(output)
     nodebase = jsonvalues['node_base']
-    output = rpc.get_scsi_devices()
+    output = rpc.scsi_get_devices()
     jsonvalues = json.loads(output)
     verify(jsonvalues[0]['device_name'] == nodebase + ":" + rpc_param['target_name'], 1,
            "device name vaule is {}, expected {}".format(jsonvalues[0]['device_name'], rpc_param['target_name']))
@@ -164,13 +164,13 @@ def verify_scsi_devices_rpc_methods(rpc_py):
 
     check_output('iscsiadm -m node --logout', shell=True)
     check_output('iscsiadm -m node -o delete', shell=True)
-    rpc.delete_initiator_group(initiator_tag)
-    rpc.delete_portal_group(portal_tag)
-    rpc.delete_target_node(name)
-    output = rpc.get_scsi_devices()
+    rpc.iscsi_delete_initiator_group(initiator_tag)
+    rpc.iscsi_delete_portal_group(portal_tag)
+    rpc.iscsi_delete_target_node(name)
+    output = rpc.scsi_get_devices()
     jsonvalues = json.loads(output)
     verify(not jsonvalues, 1,
-           "get_scsi_devices returned {}, expected empty".format(jsonvalues))
+           "scsi_get_devices returned {}, expected empty".format(jsonvalues))
 
     print("verify_scsi_devices_rpc_methods passed")
 
@@ -179,31 +179,31 @@ def create_malloc_bdevs_rpc_methods(rpc_py, rpc_param):
     rpc = spdk_rpc(rpc_py)
 
     for i in range(1, rpc_param['lun_total'] + 1):
-        rpc.construct_malloc_bdev(rpc_param['malloc_bdev_size'], rpc_param['malloc_block_size'])
+        rpc.bdev_malloc_create(rpc_param['malloc_bdev_size'], rpc_param['malloc_block_size'])
 
     print("create_malloc_bdevs_rpc_methods passed")
 
 
 def verify_portal_groups_rpc_methods(rpc_py, rpc_param):
     rpc = spdk_rpc(rpc_py)
-    output = rpc.get_portal_groups()
+    output = rpc.iscsi_get_portal_groups()
     jsonvalues = json.loads(output)
     verify(not jsonvalues, 1,
-           "get_portal_groups returned {} groups, expected empty".format(jsonvalues))
+           "iscsi_get_portal_groups returned {} groups, expected empty".format(jsonvalues))
 
     lo_ip = (target_ip, other_ip)
-    nics = json.loads(rpc.get_interfaces())
+    nics = json.loads(rpc.net_get_interfaces())
     for x in nics:
         if x["ifc_index"] == 'lo':
-            rpc.add_ip_address(x["ifc_index"], lo_ip[1])
+            rpc.net_interface_add_ip_address(x["ifc_index"], lo_ip[1])
     for idx, value in enumerate(lo_ip):
         # The portal group tag must start at 1
         tag = idx + 1
-        rpc.add_portal_group(tag, "{}:{}@{}".format(value, rpc_param['port'], rpc_param['cpumask']))
-        output = rpc.get_portal_groups()
+        rpc.iscsi_create_portal_group(tag, "{}:{}".format(value, rpc_param['port']))
+        output = rpc.iscsi_get_portal_groups()
         jsonvalues = json.loads(output)
         verify(len(jsonvalues) == tag, 1,
-               "get_portal_groups returned {} groups, expected {}".format(len(jsonvalues), tag))
+               "iscsi_get_portal_groups returned {} groups, expected {}".format(len(jsonvalues), tag))
 
     tag_list = []
     for idx, value in enumerate(jsonvalues):
@@ -211,15 +211,13 @@ def verify_portal_groups_rpc_methods(rpc_py, rpc_param):
                "host value is {}, expected {}".format(value['portals'][0]['host'], rpc_param['target_ip']))
         verify(value['portals'][0]['port'] == str(rpc_param['port']), 1,
                "port value is {}, expected {}".format(value['portals'][0]['port'], str(rpc_param['port'])))
-        verify(value['portals'][0]['cpumask'] == format(rpc_param['cpumask'], '#x'), 1,
-               "cpumask value is {}, expected {}".format(value['portals'][0]['cpumask'], format(rpc_param['cpumask'], '#x')))
         tag_list.append(value['tag'])
         verify(value['tag'] == idx + 1, 1,
                "tag value is {}, expected {}".format(value['tag'], idx + 1))
 
     for idx, value in enumerate(tag_list):
-        rpc.delete_portal_group(value)
-        output = rpc.get_portal_groups()
+        rpc.iscsi_delete_portal_group(value)
+        output = rpc.iscsi_get_portal_groups()
         jsonvalues = json.loads(output)
         verify(len(jsonvalues) == (len(tag_list) - (idx + 1)), 1,
                "get_portal_group returned {} groups, expected {}".format(len(jsonvalues), (len(tag_list) - (idx + 1))))
@@ -231,32 +229,30 @@ def verify_portal_groups_rpc_methods(rpc_py, rpc_param):
                    "host value is {}, expected {}".format(jvalue['portals'][0]['host'], lo_ip[idx + jidx + 1]))
             verify(jvalue['portals'][0]['port'] == str(rpc_param['port']), 1,
                    "port value is {}, expected {}".format(jvalue['portals'][0]['port'], str(rpc_param['port'])))
-            verify(jvalue['portals'][0]['cpumask'] == format(rpc_param['cpumask'], '#x'), 1,
-                   "cpumask value is {}, expected {}".format(jvalue['portals'][0]['cpumask'], format(rpc_param['cpumask'], '#x')))
             verify(jvalue['tag'] != value or jvalue['tag'] == tag_list[idx + jidx + 1], 1,
                    "tag value is {}, expected {} and not {}".format(jvalue['tag'], tag_list[idx + jidx + 1], value))
 
     for x in nics:
         if x["ifc_index"] == 'lo':
-            rpc.delete_ip_address(x["ifc_index"], lo_ip[1])
+            rpc.net_interface_delete_ip_address(x["ifc_index"], lo_ip[1])
 
     print("verify_portal_groups_rpc_methods passed")
 
 
 def verify_initiator_groups_rpc_methods(rpc_py, rpc_param):
     rpc = spdk_rpc(rpc_py)
-    output = rpc.get_initiator_groups()
+    output = rpc.iscsi_get_initiator_groups()
     jsonvalues = json.loads(output)
     verify(not jsonvalues, 1,
-           "get_initiator_groups returned {}, expected empty".format(jsonvalues))
+           "iscsi_get_initiator_groups returned {}, expected empty".format(jsonvalues))
     for idx, value in enumerate(rpc_param['netmask']):
         # The initiator group tag must start at 1
         tag = idx + 1
-        rpc.add_initiator_group(tag, rpc_param['initiator_name'], value)
-        output = rpc.get_initiator_groups()
+        rpc.iscsi_create_initiator_group(tag, rpc_param['initiator_name'], value)
+        output = rpc.iscsi_get_initiator_groups()
         jsonvalues = json.loads(output)
         verify(len(jsonvalues) == tag, 1,
-               "get_initiator_groups returned {} groups, expected {}".format(len(jsonvalues), tag))
+               "iscsi_get_initiator_groups returned {} groups, expected {}".format(len(jsonvalues), tag))
 
     tag_list = []
     for idx, value in enumerate(jsonvalues):
@@ -270,12 +266,12 @@ def verify_initiator_groups_rpc_methods(rpc_py, rpc_param):
 
     for idx, value in enumerate(rpc_param['netmask']):
         tag = idx + 1
-        rpc.delete_initiators_from_initiator_group(tag, '-n', rpc_param['initiator_name'], '-m', value)
+        rpc.iscsi_initiator_group_remove_initiators(tag, '-n', rpc_param['initiator_name'], '-m', value)
 
-    output = rpc.get_initiator_groups()
+    output = rpc.iscsi_get_initiator_groups()
     jsonvalues = json.loads(output)
     verify(len(jsonvalues) == tag, 1,
-           "get_initiator_groups returned {} groups, expected {}".format(len(jsonvalues), tag))
+           "iscsi_get_initiator_groups returned {} groups, expected {}".format(len(jsonvalues), tag))
 
     for idx, value in enumerate(jsonvalues):
         verify(value['tag'] == idx + 1, 1,
@@ -289,11 +285,11 @@ def verify_initiator_groups_rpc_methods(rpc_py, rpc_param):
 
     for idx, value in enumerate(rpc_param['netmask']):
         tag = idx + 1
-        rpc.add_initiators_to_initiator_group(tag, '-n', rpc_param['initiator_name'], '-m', value)
-    output = rpc.get_initiator_groups()
+        rpc.iscsi_initiator_group_add_initiators(tag, '-n', rpc_param['initiator_name'], '-m', value)
+    output = rpc.iscsi_get_initiator_groups()
     jsonvalues = json.loads(output)
     verify(len(jsonvalues) == tag, 1,
-           "get_initiator_groups returned {} groups, expected {}".format(len(jsonvalues), tag))
+           "iscsi_get_initiator_groups returned {} groups, expected {}".format(len(jsonvalues), tag))
 
     tag_list = []
     for idx, value in enumerate(jsonvalues):
@@ -306,11 +302,11 @@ def verify_initiator_groups_rpc_methods(rpc_py, rpc_param):
                "netmasks value is {}, expected {}".format(value['netmasks'][0], rpc_param['netmask'][idx]))
 
     for idx, value in enumerate(tag_list):
-        rpc.delete_initiator_group(value)
-        output = rpc.get_initiator_groups()
+        rpc.iscsi_delete_initiator_group(value)
+        output = rpc.iscsi_get_initiator_groups()
         jsonvalues = json.loads(output)
         verify(len(jsonvalues) == (len(tag_list) - (idx + 1)), 1,
-               "get_initiator_groups returned {} groups, expected {}".format(len(jsonvalues), (len(tag_list) - (idx + 1))))
+               "iscsi_get_initiator_groups returned {} groups, expected {}".format(len(jsonvalues), (len(tag_list) - (idx + 1))))
         if not jsonvalues:
             break
         for jidx, jvalue in enumerate(jsonvalues):
@@ -326,25 +322,26 @@ def verify_initiator_groups_rpc_methods(rpc_py, rpc_param):
 
 def verify_target_nodes_rpc_methods(rpc_py, rpc_param):
     rpc = spdk_rpc(rpc_py)
-    output = rpc.get_iscsi_global_params()
+    output = rpc.iscsi_get_options()
     jsonvalues = json.loads(output)
     nodebase = jsonvalues['node_base']
-    output = rpc.get_target_nodes()
+    output = rpc.iscsi_get_target_nodes()
     jsonvalues = json.loads(output)
     verify(not jsonvalues, 1,
-           "get_target_nodes returned {}, expected empty".format(jsonvalues))
+           "iscsi_get_target_nodes returned {}, expected empty".format(jsonvalues))
 
-    rpc.construct_malloc_bdev(rpc_param['malloc_bdev_size'], rpc_param['malloc_block_size'])
-    rpc.add_portal_group(portal_tag, "{}:{}".format(rpc_param['target_ip'], str(rpc_param['port'])))
-    rpc.add_initiator_group(initiator_tag, rpc_param['initiator_name'], rpc_param['netmask'])
+    rpc.bdev_malloc_create(rpc_param['malloc_bdev_size'], rpc_param['malloc_block_size'])
+    rpc.iscsi_create_portal_group(portal_tag, "{}:{}".format(rpc_param['target_ip'], str(rpc_param['port'])))
+    rpc.iscsi_create_initiator_group(initiator_tag, rpc_param['initiator_name'], rpc_param['netmask'])
 
     lun_mapping = "Malloc" + str(rpc_param['lun_total']) + ":0"
     net_mapping = portal_tag + ":" + initiator_tag
-    rpc.construct_target_node(rpc_param['target_name'], rpc_param['alias_name'], lun_mapping, net_mapping, rpc_param['queue_depth'], '-d')
-    output = rpc.get_target_nodes()
+    rpc.iscsi_create_target_node(rpc_param['target_name'], rpc_param['alias_name'], lun_mapping,
+                                 net_mapping, rpc_param['queue_depth'], '-d')
+    output = rpc.iscsi_get_target_nodes()
     jsonvalues = json.loads(output)
     verify(len(jsonvalues) == 1, 1,
-           "get_target_nodes returned {} nodes, expected 1".format(len(jsonvalues)))
+           "iscsi_get_target_nodes returned {} nodes, expected 1".format(len(jsonvalues)))
     bdev_name = jsonvalues[0]['luns'][0]['bdev_name']
     verify(bdev_name == "Malloc" + str(rpc_param['lun_total']), 1,
            "bdev_name value is {}, expected Malloc{}".format(jsonvalues[0]['luns'][0]['bdev_name'], str(rpc_param['lun_total'])))
@@ -374,26 +371,27 @@ def verify_target_nodes_rpc_methods(rpc_py, rpc_param):
     verify(jsonvalues[0]['data_digest'] == rpc_param['data_digest'], 1,
            "data digest value is {}, expected {}".format(jsonvalues[0]['data_digest'], rpc_param['data_digest']))
     lun_id = '1'
-    rpc.target_node_add_lun(name, bdev_name, "-i", lun_id)
-    output = rpc.get_target_nodes()
+    rpc.iscsi_target_node_add_lun(name, bdev_name, "-i", lun_id)
+    output = rpc.iscsi_get_target_nodes()
     jsonvalues = json.loads(output)
     verify(jsonvalues[0]['luns'][1]['bdev_name'] == "Malloc" + str(rpc_param['lun_total']), 1,
            "bdev_name value is {}, expected Malloc{}".format(jsonvalues[0]['luns'][0]['bdev_name'], str(rpc_param['lun_total'])))
     verify(jsonvalues[0]['luns'][1]['lun_id'] == 1, 1,
            "lun id value is {}, expected 1".format(jsonvalues[0]['luns'][1]['lun_id']))
 
-    rpc.delete_target_node(name)
-    output = rpc.get_target_nodes()
+    rpc.iscsi_delete_target_node(name)
+    output = rpc.iscsi_get_target_nodes()
     jsonvalues = json.loads(output)
     verify(not jsonvalues, 1,
-           "get_target_nodes returned {}, expected empty".format(jsonvalues))
+           "iscsi_get_target_nodes returned {}, expected empty".format(jsonvalues))
 
-    rpc.construct_target_node(rpc_param['target_name'], rpc_param['alias_name'], lun_mapping, net_mapping, rpc_param['queue_depth'], '-d')
+    rpc.iscsi_create_target_node(rpc_param['target_name'], rpc_param['alias_name'], lun_mapping,
+                                 net_mapping, rpc_param['queue_depth'], '-d')
 
-    rpc.delete_portal_group(portal_tag)
-    rpc.delete_initiator_group(initiator_tag)
-    rpc.delete_target_node(name)
-    output = rpc.get_target_nodes()
+    rpc.iscsi_delete_portal_group(portal_tag)
+    rpc.iscsi_delete_initiator_group(initiator_tag)
+    rpc.iscsi_delete_target_node(name)
+    output = rpc.iscsi_get_target_nodes()
     jsonvalues = json.loads(output)
     if not jsonvalues:
         print("This issue will be fixed later.")
@@ -401,34 +399,34 @@ def verify_target_nodes_rpc_methods(rpc_py, rpc_param):
     print("verify_target_nodes_rpc_methods passed.")
 
 
-def verify_get_interfaces(rpc_py):
+def verify_net_get_interfaces(rpc_py):
     rpc = spdk_rpc(rpc_py)
-    nics = json.loads(rpc.get_interfaces())
+    nics = json.loads(rpc.net_get_interfaces())
     nics_names = set(x["name"] for x in nics)
-    # parse ip link show to verify the get_interfaces result
+    # parse ip link show to verify the net_get_interfaces result
     ip_show = ns_cmd + " ip link show"
     ifcfg_nics = set(re.findall(r'\S+:\s(\S+?)(?:@\S+){0,1}:\s<.*', check_output(ip_show.split()).decode()))
-    verify(nics_names == ifcfg_nics, 1, "get_interfaces returned {}".format(nics))
-    print("verify_get_interfaces passed.")
+    verify(nics_names == ifcfg_nics, 1, "net_get_interfaces returned {}".format(nics))
+    print("verify_net_get_interfaces passed.")
 
 
 def help_get_interface_ip_list(rpc_py, nic_name):
     rpc = spdk_rpc(rpc_py)
-    nics = json.loads(rpc.get_interfaces())
+    nics = json.loads(rpc.net_get_interfaces())
     nic = list([x for x in nics if x["name"] == nic_name])
     verify(len(nic) != 0, 1,
            "Nic name: {} is not found in {}".format(nic_name, [x["name"] for x in nics]))
     return nic[0]["ip_addr"]
 
 
-def verify_add_delete_ip_address(rpc_py):
+def verify_net_interface_add_delete_ip_address(rpc_py):
     rpc = spdk_rpc(rpc_py)
-    nics = json.loads(rpc.get_interfaces())
+    nics = json.loads(rpc.net_get_interfaces())
     # add ip on up to first 2 nics
     for x in nics[:2]:
         faked_ip = "123.123.{}.{}".format(random.randint(1, 254), random.randint(1, 254))
         ping_cmd = ns_cmd + " ping -c 1 -W 1 " + faked_ip
-        rpc.add_ip_address(x["ifc_index"], faked_ip)
+        rpc.net_interface_add_ip_address(x["ifc_index"], faked_ip)
         verify(faked_ip in help_get_interface_ip_list(rpc_py, x["name"]), 1,
                "add ip {} to nic {} failed.".format(faked_ip, x["name"]))
         try:
@@ -437,7 +435,7 @@ def verify_add_delete_ip_address(rpc_py):
             verify(False, 1,
                    "ping ip {} for {} was failed(adding was successful)".format
                    (faked_ip, x["name"]))
-        rpc.delete_ip_address(x["ifc_index"], faked_ip)
+        rpc.net_interface_delete_ip_address(x["ifc_index"], faked_ip)
         verify(faked_ip not in help_get_interface_ip_list(rpc_py, x["name"]), 1,
                "delete ip {} from nic {} failed.(adding and ping were successful)".format
                (faked_ip, x["name"]))
@@ -454,29 +452,7 @@ def verify_add_delete_ip_address(rpc_py):
             verify(False, 1,
                    "ip {} for {} could be pinged after delete ip(adding/ping/delete were successful)".format
                    (faked_ip, x["name"]))
-    print("verify_add_delete_ip_address passed.")
-
-
-def verify_add_nvme_bdev_rpc_methods(rpc_py):
-    rpc = spdk_rpc(rpc_py)
-    test_pass = 0
-    output = check_output(["lspci", "-mm", "-nn"])
-    addrs = re.findall(r'^([0-9]{2}:[0-9]{2}.[0-9]) "Non-Volatile memory controller \[0108\]".*-p02', output.decode(), re.MULTILINE)
-    for addr in addrs:
-        ctrlr_address = "-b Nvme{} -t pcie -a 0000:{}".format(addrs.index(addr), addr)
-        rpc.construct_nvme_bdev(ctrlr_address)
-        print("add nvme device passed first time")
-        test_pass = 0
-        try:
-            rpc.construct_nvme_bdev(ctrlr_address)
-        except Exception as e:
-            print("add nvme device passed second time")
-            test_pass = 1
-            pass
-        else:
-            pass
-        verify(test_pass == 1, 1, "add nvme device passed second time")
-    print("verify_add_nvme_bdev_rpc_methods passed.")
+    print("verify_net_interface_add_delete_ip_address passed.")
 
 
 if __name__ == "__main__":
@@ -485,15 +461,17 @@ if __name__ == "__main__":
 
     try:
         verify_log_flag_rpc_methods(rpc_py, rpc_param)
-        verify_get_interfaces(rpc_py)
-        verify_add_delete_ip_address(rpc_py)
+        verify_net_get_interfaces(rpc_py)
+        # Add/delete IP will not be supported in VPP.
+        # It has separate vppctl utility for that.
+        if test_type == 'posix':
+            verify_net_interface_add_delete_ip_address(rpc_py)
         create_malloc_bdevs_rpc_methods(rpc_py, rpc_param)
         verify_portal_groups_rpc_methods(rpc_py, rpc_param)
         verify_initiator_groups_rpc_methods(rpc_py, rpc_param)
         verify_target_nodes_rpc_methods(rpc_py, rpc_param)
         verify_scsi_devices_rpc_methods(rpc_py)
         verify_iscsi_connection_rpc_methods(rpc_py)
-        verify_add_nvme_bdev_rpc_methods(rpc_py)
     except RpcException as e:
         print("{}. Exiting with status {}".format(e.message, e.retval))
         raise e
