@@ -35,17 +35,28 @@
 
 #include "CUnit/Basic.h"
 
-#include "common/lib/test_env.c"
+#include "common/lib/ut_multithread.c"
 
 #include "spdk_cunit.h"
 #include "blobfs/blobfs.c"
 #include "blobfs/tree.c"
+#include "blob/blobstore.h"
+
+#include "spdk_internal/thread.h"
 
 #include "unit/lib/blob/bs_dev_common.c"
 
 struct spdk_filesystem *g_fs;
 struct spdk_file *g_file;
 int g_fserrno;
+struct spdk_trace_histories *g_trace_histories;
+DEFINE_STUB_V(spdk_trace_add_register_fn, (struct spdk_trace_register_fn *reg_fn));
+DEFINE_STUB_V(spdk_trace_register_description, (const char *name,
+		uint16_t tpoint_id, uint8_t owner_type,
+		uint8_t object_type, uint8_t new_object,
+		uint8_t arg1_is_ptr, const char *arg1_name));
+DEFINE_STUB_V(_spdk_trace_record, (uint64_t tsc, uint16_t tpoint_id, uint16_t poller_id,
+				   uint32_t size, uint64_t object_id, uint64_t arg1));
 
 /* Return NULL to test hardcoded defaults. */
 struct spdk_conf_section *
@@ -61,12 +72,6 @@ spdk_conf_section_get_intval(struct spdk_conf_section *sp, const char *key)
 	return -1;
 }
 
-static void
-_fs_send_msg(spdk_thread_fn fn, void *ctx, void *thread_ctx)
-{
-	fn(ctx);
-}
-
 static void
 fs_op_complete(void *ctx, int fserrno)
 {
@@ -80,6 +85,13 @@ fs_op_with_handle_complete(void *ctx, struct spdk_filesystem *fs, int fserrno)
 	g_fserrno = fserrno;
 }
 
+static void
+fs_poll_threads(void)
+{
+	poll_threads();
+	while (spdk_thread_poll(g_cache_pool_thread, 0, 0) > 0) {}
+}
+
 static void
 fs_init(void)
 {
@@ -87,18 +99,18 @@ fs_init(void)
 	struct spdk_bs_dev *dev;
 
 	dev = init_dev();
-	spdk_allocate_thread(_fs_send_msg, NULL, NULL, NULL, "thread0");
 
 	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
 	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
 	CU_ASSERT(g_fserrno == 0);
 	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
 
 	g_fserrno = 1;
 	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
-
-	spdk_free_thread();
 }
 
 static void
@@ -131,25 +143,29 @@ fs_open(void)
 
 	dev = init_dev();
 	memset(name, 'a', sizeof(name) - 1);
-	spdk_allocate_thread(_fs_send_msg, NULL, NULL, NULL, "thread0");
 
 	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
 	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
 	CU_ASSERT(g_fserrno == 0);
 	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
 
 	g_fserrno = 0;
 	/* Open should fail, because the file name is too long. */
 	spdk_fs_open_file_async(fs, name, SPDK_BLOBFS_OPEN_CREATE, open_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == -ENAMETOOLONG);
 
 	g_fserrno = 0;
 	spdk_fs_open_file_async(fs, "file1", 0, open_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == -ENOENT);
 
 	g_file = NULL;
 	g_fserrno = 1;
 	spdk_fs_open_file_async(fs, "file1", SPDK_BLOBFS_OPEN_CREATE, open_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	SPDK_CU_ASSERT_FATAL(g_file != NULL);
 	CU_ASSERT(!strcmp("file1", g_file->name));
@@ -166,19 +182,20 @@ fs_open(void)
 	g_fserrno = 0;
 	/* Delete should successful, we will mark the file as deleted. */
 	spdk_fs_delete_file_async(fs, "file1", delete_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(!TAILQ_EMPTY(&fs->files));
 
 	g_fserrno = 1;
 	spdk_file_close_async(g_file, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(TAILQ_EMPTY(&fs->files));
 
 	g_fserrno = 1;
 	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
-
-	spdk_free_thread();
 }
 
 static void
@@ -190,36 +207,40 @@ fs_create(void)
 
 	dev = init_dev();
 	memset(name, 'a', sizeof(name) - 1);
-	spdk_allocate_thread(_fs_send_msg, NULL, NULL, NULL, "thread0");
 
 	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
 	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
 	CU_ASSERT(g_fserrno == 0);
 	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
 
 	g_fserrno = 0;
 	/* Create should fail, because the file name is too long. */
 	spdk_fs_create_file_async(fs, name, create_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == -ENAMETOOLONG);
 
 	g_fserrno = 1;
 	spdk_fs_create_file_async(fs, "file1", create_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 
 	g_fserrno = 1;
 	spdk_fs_create_file_async(fs, "file1", create_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == -EEXIST);
 
 	g_fserrno = 1;
 	spdk_fs_delete_file_async(fs, "file1", delete_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(TAILQ_EMPTY(&fs->files));
 
 	g_fserrno = 1;
 	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
-
-	spdk_free_thread();
 }
 
 static void
@@ -229,73 +250,82 @@ fs_truncate(void)
 	struct spdk_bs_dev *dev;
 
 	dev = init_dev();
-	spdk_allocate_thread(_fs_send_msg, NULL, NULL, NULL, "thread0");
 
 	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
 	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
 	CU_ASSERT(g_fserrno == 0);
 	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
 
 	g_file = NULL;
 	g_fserrno = 1;
 	spdk_fs_open_file_async(fs, "file1", SPDK_BLOBFS_OPEN_CREATE, open_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	SPDK_CU_ASSERT_FATAL(g_file != NULL);
 
 	g_fserrno = 1;
 	spdk_file_truncate_async(g_file, 18 * 1024 * 1024 + 1, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(g_file->length == 18 * 1024 * 1024 + 1);
 
 	g_fserrno = 1;
 	spdk_file_truncate_async(g_file, 1, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(g_file->length == 1);
 
 	g_fserrno = 1;
 	spdk_file_truncate_async(g_file, 18 * 1024 * 1024 + 1, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(g_file->length == 18 * 1024 * 1024 + 1);
 
 	g_fserrno = 1;
 	spdk_file_close_async(g_file, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(g_file->ref_count == 0);
 
 	g_fserrno = 1;
 	spdk_fs_delete_file_async(fs, "file1", delete_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(TAILQ_EMPTY(&fs->files));
 
 	g_fserrno = 1;
 	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
-
-	spdk_free_thread();
 }
 
 static void
 fs_rename(void)
 {
 	struct spdk_filesystem *fs;
-	struct spdk_file *file, *file2;
+	struct spdk_file *file, *file2, *file_iter;
 	struct spdk_bs_dev *dev;
 
 	dev = init_dev();
-	spdk_allocate_thread(_fs_send_msg, NULL, NULL, NULL, "thread0");
 
 	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
 	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
 	CU_ASSERT(g_fserrno == 0);
 	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
 
 	g_fserrno = 1;
 	spdk_fs_create_file_async(fs, "file1", create_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 
 	g_file = NULL;
 	g_fserrno = 1;
 	spdk_fs_open_file_async(fs, "file1", 0, open_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	SPDK_CU_ASSERT_FATAL(g_file != NULL);
 	CU_ASSERT(g_file->ref_count == 1);
@@ -304,12 +334,14 @@ fs_rename(void)
 	g_file = NULL;
 	g_fserrno = 1;
 	spdk_file_close_async(file, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	SPDK_CU_ASSERT_FATAL(file->ref_count == 0);
 
 	g_file = NULL;
 	g_fserrno = 1;
 	spdk_fs_open_file_async(fs, "file2", SPDK_BLOBFS_OPEN_CREATE, open_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	SPDK_CU_ASSERT_FATAL(g_file != NULL);
 	CU_ASSERT(g_file->ref_count == 1);
@@ -318,6 +350,7 @@ fs_rename(void)
 	g_file = NULL;
 	g_fserrno = 1;
 	spdk_file_close_async(file2, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	SPDK_CU_ASSERT_FATAL(file2->ref_count == 0);
 
@@ -327,6 +360,7 @@ fs_rename(void)
 	 */
 	g_fserrno = 1;
 	spdk_fs_rename_file_async(fs, "file1", "file2", fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(file->ref_count == 0);
 	CU_ASSERT(!strcmp(file->name, "file2"));
@@ -335,19 +369,169 @@ fs_rename(void)
 
 	g_fserrno = 0;
 	spdk_fs_delete_file_async(fs, "file1", delete_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == -ENOENT);
 	CU_ASSERT(!TAILQ_EMPTY(&fs->files));
+	TAILQ_FOREACH(file_iter, &fs->files, tailq) {
+		if (file_iter == NULL) {
+			SPDK_CU_ASSERT_FATAL(false);
+		}
+	}
 
 	g_fserrno = 1;
 	spdk_fs_delete_file_async(fs, "file2", delete_cb, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	CU_ASSERT(TAILQ_EMPTY(&fs->files));
 
 	g_fserrno = 1;
 	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+}
+
+static void
+fs_rw_async(void)
+{
+	struct spdk_filesystem *fs;
+	struct spdk_bs_dev *dev;
+	uint8_t w_buf[4096];
+	uint8_t r_buf[4096];
+
+	dev = init_dev();
+
+	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
+	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
+	CU_ASSERT(g_fserrno == 0);
+	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
+
+	g_file = NULL;
+	g_fserrno = 1;
+	spdk_fs_open_file_async(fs, "file1", SPDK_BLOBFS_OPEN_CREATE, open_cb, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+	SPDK_CU_ASSERT_FATAL(g_file != NULL);
+
+	/* Write file */
+	CU_ASSERT(g_file->length == 0);
+	g_fserrno = 1;
+	memset(w_buf, 0x5a, sizeof(w_buf));
+	spdk_file_write_async(g_file, fs->sync_target.sync_io_channel, w_buf, 0, 4096,
+			      fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+	CU_ASSERT(g_file->length == 4096);
+
+	/* Read file */
+	g_fserrno = 1;
+	memset(r_buf, 0x0, sizeof(r_buf));
+	spdk_file_read_async(g_file, fs->sync_target.sync_io_channel, r_buf, 0, 4096,
+			     fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+	CU_ASSERT(memcmp(r_buf, w_buf, sizeof(r_buf)) == 0);
+
+	g_fserrno = 1;
+	spdk_file_close_async(g_file, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 
-	spdk_free_thread();
+	g_fserrno = 1;
+	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+}
+
+static void
+fs_writev_readv_async(void)
+{
+	struct spdk_filesystem *fs;
+	struct spdk_bs_dev *dev;
+	struct iovec w_iov[2];
+	struct iovec r_iov[2];
+	uint8_t w_buf[4096];
+	uint8_t r_buf[4096];
+
+	dev = init_dev();
+
+	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
+	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
+	CU_ASSERT(g_fserrno == 0);
+	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
+
+	g_file = NULL;
+	g_fserrno = 1;
+	spdk_fs_open_file_async(fs, "file1", SPDK_BLOBFS_OPEN_CREATE, open_cb, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+	SPDK_CU_ASSERT_FATAL(g_file != NULL);
+
+	/* Write file */
+	CU_ASSERT(g_file->length == 0);
+	g_fserrno = 1;
+	memset(w_buf, 0x5a, sizeof(w_buf));
+	w_iov[0].iov_base = w_buf;
+	w_iov[0].iov_len = 2048;
+	w_iov[1].iov_base = w_buf + 2048;
+	w_iov[1].iov_len = 2048;
+	spdk_file_writev_async(g_file, fs->sync_target.sync_io_channel,
+			       w_iov, 2, 0, 4096, fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+	CU_ASSERT(g_file->length == 4096);
+
+	/* Read file */
+	g_fserrno = 1;
+	memset(r_buf, 0x0, sizeof(r_buf));
+	r_iov[0].iov_base = r_buf;
+	r_iov[0].iov_len = 2048;
+	r_iov[1].iov_base = r_buf + 2048;
+	r_iov[1].iov_len = 2048;
+	spdk_file_readv_async(g_file, fs->sync_target.sync_io_channel,
+			      r_iov, 2, 0, 4096, fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+	CU_ASSERT(memcmp(r_buf, w_buf, sizeof(r_buf)) == 0);
+
+	/* Overwrite file with block aligned */
+	g_fserrno = 1;
+	memset(w_buf, 0x6a, sizeof(w_buf));
+	w_iov[0].iov_base = w_buf;
+	w_iov[0].iov_len = 2048;
+	w_iov[1].iov_base = w_buf + 2048;
+	w_iov[1].iov_len = 2048;
+	spdk_file_writev_async(g_file, fs->sync_target.sync_io_channel,
+			       w_iov, 2, 0, 4096, fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+	CU_ASSERT(g_file->length == 4096);
+
+	/* Read file to verify the overwritten data */
+	g_fserrno = 1;
+	memset(r_buf, 0x0, sizeof(r_buf));
+	r_iov[0].iov_base = r_buf;
+	r_iov[0].iov_len = 2048;
+	r_iov[1].iov_base = r_buf + 2048;
+	r_iov[1].iov_len = 2048;
+	spdk_file_readv_async(g_file, fs->sync_target.sync_io_channel,
+			      r_iov, 2, 0, 4096, fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+	CU_ASSERT(memcmp(r_buf, w_buf, sizeof(r_buf)) == 0);
+
+	g_fserrno = 1;
+	spdk_file_close_async(g_file, fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
+
+	g_fserrno = 1;
+	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
+	CU_ASSERT(g_fserrno == 0);
 }
 
 static void
@@ -392,38 +576,38 @@ tree_find_buffer_ut(void)
 	level1_0->u.tree[12] = level0_0_12;
 	level1_0->present_mask |= (1ULL << 12);
 
-	buffer = spdk_tree_find_buffer(NULL, 0);
+	buffer = tree_find_buffer(NULL, 0);
 	CU_ASSERT(buffer == NULL);
 
-	buffer = spdk_tree_find_buffer(level0_0_0, 0);
+	buffer = tree_find_buffer(level0_0_0, 0);
 	CU_ASSERT(buffer == NULL);
 
-	buffer = spdk_tree_find_buffer(level0_0_0, CACHE_TREE_LEVEL_SIZE(0) + 1);
+	buffer = tree_find_buffer(level0_0_0, CACHE_TREE_LEVEL_SIZE(0) + 1);
 	CU_ASSERT(buffer == NULL);
 
-	buffer = spdk_tree_find_buffer(level0_0_0, leaf_0_0_4->offset);
+	buffer = tree_find_buffer(level0_0_0, leaf_0_0_4->offset);
 	CU_ASSERT(buffer == leaf_0_0_4);
 
-	buffer = spdk_tree_find_buffer(level1_0, leaf_0_0_4->offset);
+	buffer = tree_find_buffer(level1_0, leaf_0_0_4->offset);
 	CU_ASSERT(buffer == leaf_0_0_4);
 
-	buffer = spdk_tree_find_buffer(level1_0, leaf_0_12_8->offset);
+	buffer = tree_find_buffer(level1_0, leaf_0_12_8->offset);
 	CU_ASSERT(buffer == leaf_0_12_8);
 
-	buffer = spdk_tree_find_buffer(level1_0, leaf_0_12_8->offset + CACHE_BUFFER_SIZE - 1);
+	buffer = tree_find_buffer(level1_0, leaf_0_12_8->offset + CACHE_BUFFER_SIZE - 1);
 	CU_ASSERT(buffer == leaf_0_12_8);
 
-	buffer = spdk_tree_find_buffer(level1_0, leaf_0_12_8->offset - 1);
+	buffer = tree_find_buffer(level1_0, leaf_0_12_8->offset - 1);
 	CU_ASSERT(buffer == NULL);
 
 	leaf_9_23_15->offset = CACHE_TREE_LEVEL_SIZE(2) * 9 +
 			       CACHE_TREE_LEVEL_SIZE(1) * 23 +
 			       CACHE_BUFFER_SIZE * 15;
-	root = spdk_tree_insert_buffer(level1_0, leaf_9_23_15);
+	root = tree_insert_buffer(level1_0, leaf_9_23_15);
 	CU_ASSERT(root != level1_0);
-	buffer = spdk_tree_find_buffer(root, leaf_9_23_15->offset);
+	buffer = tree_find_buffer(root, leaf_9_23_15->offset);
 	CU_ASSERT(buffer == leaf_9_23_15);
-	spdk_tree_free_buffers(root);
+	tree_free_buffers(root);
 	free(root);
 }
 
@@ -435,12 +619,13 @@ channel_ops(void)
 	struct spdk_io_channel *channel;
 
 	dev = init_dev();
-	spdk_allocate_thread(_fs_send_msg, NULL, NULL, NULL, "thread0");
 
 	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
 	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
 	CU_ASSERT(g_fserrno == 0);
 	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
 
 	channel =  spdk_fs_alloc_io_channel(fs);
 	CU_ASSERT(channel != NULL);
@@ -449,10 +634,9 @@ channel_ops(void)
 
 	g_fserrno = 1;
 	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	g_fs = NULL;
-
-	spdk_free_thread();
 }
 
 static void
@@ -460,27 +644,27 @@ channel_ops_sync(void)
 {
 	struct spdk_filesystem *fs;
 	struct spdk_bs_dev *dev;
-	struct spdk_io_channel *channel;
+	struct spdk_fs_thread_ctx *channel;
 
 	dev = init_dev();
-	spdk_allocate_thread(_fs_send_msg, NULL, NULL, NULL, "thread0");
 
 	spdk_fs_init(dev, NULL, NULL, fs_op_with_handle_complete, NULL);
+	fs_poll_threads();
 	SPDK_CU_ASSERT_FATAL(g_fs != NULL);
 	CU_ASSERT(g_fserrno == 0);
 	fs = g_fs;
+	SPDK_CU_ASSERT_FATAL(fs->bs->dev == dev);
 
-	channel =  spdk_fs_alloc_io_channel_sync(fs);
+	channel =  spdk_fs_alloc_thread_ctx(fs);
 	CU_ASSERT(channel != NULL);
 
-	spdk_fs_free_io_channel(channel);
+	spdk_fs_free_thread_ctx(channel);
 
 	g_fserrno = 1;
 	spdk_fs_unload(fs, fs_op_complete, NULL);
+	fs_poll_threads();
 	CU_ASSERT(g_fserrno == 0);
 	g_fs = NULL;
-
-	spdk_free_thread();
 }
 
 int main(int argc, char **argv)
@@ -488,29 +672,24 @@ int main(int argc, char **argv)
 	CU_pSuite	suite = NULL;
 	unsigned int	num_failures;
 
-	if (CU_initialize_registry() != CUE_SUCCESS) {
-		return CU_get_error();
-	}
+	CU_set_error_action(CUEA_ABORT);
+	CU_initialize_registry();
 
 	suite = CU_add_suite("blobfs_async_ut", NULL, NULL);
-	if (suite == NULL) {
-		CU_cleanup_registry();
-		return CU_get_error();
-	}
 
-	if (
-		CU_add_test(suite, "fs_init", fs_init) == NULL ||
-		CU_add_test(suite, "fs_open", fs_open) == NULL ||
-		CU_add_test(suite, "fs_create", fs_create) == NULL ||
-		CU_add_test(suite, "fs_truncate", fs_truncate) == NULL ||
-		CU_add_test(suite, "fs_rename", fs_rename) == NULL ||
-		CU_add_test(suite, "tree_find_buffer", tree_find_buffer_ut) == NULL ||
-		CU_add_test(suite, "channel_ops", channel_ops) == NULL ||
-		CU_add_test(suite, "channel_ops_sync", channel_ops_sync) == NULL
-	) {
-		CU_cleanup_registry();
-		return CU_get_error();
-	}
+	CU_ADD_TEST(suite, fs_init);
+	CU_ADD_TEST(suite, fs_open);
+	CU_ADD_TEST(suite, fs_create);
+	CU_ADD_TEST(suite, fs_truncate);
+	CU_ADD_TEST(suite, fs_rename);
+	CU_ADD_TEST(suite, fs_rw_async);
+	CU_ADD_TEST(suite, fs_writev_readv_async);
+	CU_ADD_TEST(suite, tree_find_buffer_ut);
+	CU_ADD_TEST(suite, channel_ops);
+	CU_ADD_TEST(suite, channel_ops_sync);
+
+	allocate_threads(1);
+	set_thread(0);
 
 	g_dev_buffer = calloc(1, DEV_BUFFER_SIZE);
 	CU_basic_set_mode(CU_BRM_VERBOSE);
@@ -518,5 +697,8 @@ int main(int argc, char **argv)
 	num_failures = CU_get_number_of_failures();
 	CU_cleanup_registry();
 	free(g_dev_buffer);
+
+	free_threads();
+
 	return num_failures;
 }
