@@ -106,19 +106,20 @@ DEFINE_STUB_V(spdk_scsi_dev_queue_mgmt_task,
 DEFINE_STUB_V(spdk_scsi_dev_delete_lun,
 	      (struct spdk_scsi_dev *dev, struct spdk_scsi_lun *lun));
 
-DEFINE_STUB(spdk_scsi_pr_check, int, (struct spdk_scsi_task *task), 0);
+DEFINE_STUB(scsi_pr_check, int, (struct spdk_scsi_task *task), 0);
+DEFINE_STUB(scsi2_reserve_check, int, (struct spdk_scsi_task *task), 0);
 
 void
-spdk_bdev_scsi_reset(struct spdk_scsi_task *task)
+bdev_scsi_reset(struct spdk_scsi_task *task)
 {
 	task->status = SPDK_SCSI_STATUS_GOOD;
 	task->response = SPDK_SCSI_TASK_MGMT_RESP_SUCCESS;
 
-	spdk_scsi_lun_complete_reset_task(task->lun, task);
+	scsi_lun_complete_reset_task(task->lun, task);
 }
 
 int
-spdk_bdev_scsi_execute(struct spdk_scsi_task *task)
+bdev_scsi_execute(struct spdk_scsi_task *task)
 {
 	if (g_lun_execute_fail) {
 		return -EINVAL;
@@ -138,13 +139,12 @@ spdk_bdev_scsi_execute(struct spdk_scsi_task *task)
 DEFINE_STUB(spdk_bdev_get_io_channel, struct spdk_io_channel *,
 	    (struct spdk_bdev_desc *desc), NULL);
 
-static _spdk_scsi_lun *
-lun_construct(void)
+static struct spdk_scsi_lun *lun_construct(void)
 {
 	struct spdk_scsi_lun		*lun;
 	struct spdk_bdev		bdev;
 
-	lun = spdk_scsi_lun_construct(&bdev, NULL, NULL);
+	lun = scsi_lun_construct(&bdev, NULL, NULL);
 
 	SPDK_CU_ASSERT_FATAL(lun != NULL);
 	return lun;
@@ -156,7 +156,7 @@ lun_destruct(struct spdk_scsi_lun *lun)
 	/* LUN will defer its removal if there are any unfinished tasks */
 	SPDK_CU_ASSERT_FATAL(TAILQ_EMPTY(&lun->tasks));
 
-	spdk_scsi_lun_destruct(lun);
+	scsi_lun_destruct(lun);
 }
 
 static void
@@ -182,14 +182,12 @@ lun_task_mgmt_execute_abort_task_not_supported(void)
 	task.lun = lun;
 	task.cdb = cdb;
 
-	spdk_scsi_lun_append_task(lun, &task);
-	spdk_scsi_lun_execute_tasks(lun);
+	scsi_lun_execute_task(lun, &task);
 
 	/* task should now be on the tasks list */
 	CU_ASSERT(!TAILQ_EMPTY(&lun->tasks));
 
-	spdk_scsi_lun_append_mgmt_task(lun, &mgmt_task);
-	spdk_scsi_lun_execute_mgmt_task(lun);
+	scsi_lun_execute_mgmt_task(lun, &mgmt_task);
 
 	/* task abort is not supported */
 	CU_ASSERT(mgmt_task.response == SPDK_SCSI_TASK_MGMT_RESP_REJECT_FUNC_NOT_SUPPORTED);
@@ -197,7 +195,7 @@ lun_task_mgmt_execute_abort_task_not_supported(void)
 	/* task is still on the tasks list */
 	CU_ASSERT_EQUAL(g_task_count, 1);
 
-	spdk_scsi_lun_complete_task(lun, &task);
+	scsi_lun_complete_task(lun, &task);
 	CU_ASSERT_EQUAL(g_task_count, 0);
 
 	lun_destruct(lun);
@@ -227,14 +225,12 @@ lun_task_mgmt_execute_abort_task_all_not_supported(void)
 	task.lun = lun;
 	task.cdb = cdb;
 
-	spdk_scsi_lun_append_task(lun, &task);
-	spdk_scsi_lun_execute_tasks(lun);
+	scsi_lun_execute_task(lun, &task);
 
 	/* task should now be on the tasks list */
 	CU_ASSERT(!TAILQ_EMPTY(&lun->tasks));
 
-	spdk_scsi_lun_append_mgmt_task(lun, &mgmt_task);
-	spdk_scsi_lun_execute_mgmt_task(lun);
+	scsi_lun_execute_mgmt_task(lun, &mgmt_task);
 
 	/* task abort is not supported */
 	CU_ASSERT(mgmt_task.response == SPDK_SCSI_TASK_MGMT_RESP_REJECT_FUNC_NOT_SUPPORTED);
@@ -242,7 +238,7 @@ lun_task_mgmt_execute_abort_task_all_not_supported(void)
 	/* task is still on the tasks list */
 	CU_ASSERT_EQUAL(g_task_count, 1);
 
-	spdk_scsi_lun_complete_task(lun, &task);
+	scsi_lun_complete_task(lun, &task);
 
 	CU_ASSERT_EQUAL(g_task_count, 0);
 
@@ -263,8 +259,7 @@ lun_task_mgmt_execute_lun_reset(void)
 	mgmt_task.lun = lun;
 	mgmt_task.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
 
-	spdk_scsi_lun_append_mgmt_task(lun, &mgmt_task);
-	spdk_scsi_lun_execute_mgmt_task(lun);
+	scsi_lun_execute_mgmt_task(lun, &mgmt_task);
 
 	/* Returns success */
 	CU_ASSERT_EQUAL(mgmt_task.status, SPDK_SCSI_STATUS_GOOD);
@@ -289,8 +284,7 @@ lun_task_mgmt_execute_invalid_case(void)
 	mgmt_task.function = 5;
 
 	/* Pass an invalid value to the switch statement */
-	spdk_scsi_lun_append_mgmt_task(lun, &mgmt_task);
-	spdk_scsi_lun_execute_mgmt_task(lun);
+	scsi_lun_execute_mgmt_task(lun, &mgmt_task);
 
 	/* function code is invalid */
 	CU_ASSERT_EQUAL(mgmt_task.response, SPDK_SCSI_TASK_MGMT_RESP_REJECT_FUNC_NOT_SUPPORTED);
@@ -392,8 +386,7 @@ lun_execute_scsi_task_pending(void)
 	 */
 	CU_ASSERT(TAILQ_EMPTY(&lun->tasks));
 
-	spdk_scsi_lun_append_task(lun, &task);
-	spdk_scsi_lun_execute_tasks(lun);
+	scsi_lun_execute_task(lun, &task);
 
 	/* Assert the task has been successfully added to the tasks queue */
 	CU_ASSERT(!TAILQ_EMPTY(&lun->tasks));
@@ -402,7 +395,7 @@ lun_execute_scsi_task_pending(void)
 	CU_ASSERT_EQUAL(g_task_count, 1);
 
 	/* Need to complete task so LUN might be removed right now */
-	spdk_scsi_lun_complete_task(lun, &task);
+	scsi_lun_complete_task(lun, &task);
 
 	CU_ASSERT_EQUAL(g_task_count, 0);
 
@@ -430,8 +423,7 @@ lun_execute_scsi_task_complete(void)
 	 */
 	CU_ASSERT(TAILQ_EMPTY(&lun->tasks));
 
-	spdk_scsi_lun_append_task(lun, &task);
-	spdk_scsi_lun_execute_tasks(lun);
+	scsi_lun_execute_task(lun, &task);
 
 	/* Assert the task has not been added to the tasks queue */
 	CU_ASSERT(TAILQ_EMPTY(&lun->tasks));
@@ -448,7 +440,7 @@ lun_destruct_success(void)
 
 	lun = lun_construct();
 
-	spdk_scsi_lun_destruct(lun);
+	scsi_lun_destruct(lun);
 
 	CU_ASSERT_EQUAL(g_task_count, 0);
 }
@@ -458,7 +450,7 @@ lun_construct_null_ctx(void)
 {
 	struct spdk_scsi_lun		*lun;
 
-	lun = spdk_scsi_lun_construct(NULL, NULL, NULL);
+	lun = scsi_lun_construct(NULL, NULL, NULL);
 
 	/* lun should be NULL since we passed NULL for the ctx pointer. */
 	CU_ASSERT(lun == NULL);
@@ -496,26 +488,16 @@ lun_reset_task_wait_scsi_task_complete(void)
 	mgmt_task.lun = lun;
 	mgmt_task.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
 
-	/* Append a task to the pending task list. */
-	spdk_scsi_lun_append_task(lun, &task);
-
-	CU_ASSERT(!TAILQ_EMPTY(&lun->pending_tasks));
-
 	/* Execute the task but it is still in the task list. */
-	spdk_scsi_lun_execute_tasks(lun);
+	scsi_lun_execute_task(lun, &task);
 
 	CU_ASSERT(TAILQ_EMPTY(&lun->pending_tasks));
 	CU_ASSERT(!TAILQ_EMPTY(&lun->tasks));
 
-	/* Append a reset task to the pending mgmt task list. */
-	spdk_scsi_lun_append_mgmt_task(lun, &mgmt_task);
-
-	CU_ASSERT(!TAILQ_EMPTY(&lun->pending_mgmt_tasks));
-
 	/* Execute the reset task */
-	spdk_scsi_lun_execute_mgmt_task(lun);
+	scsi_lun_execute_mgmt_task(lun, &mgmt_task);
 
-	/* The reset task should be still on the submitted mgmt task list and
+	/* The reset task should be on the submitted mgmt task list and
 	 * a poller is created because the task prior to the reset task is pending.
 	 */
 	CU_ASSERT(!TAILQ_EMPTY(&lun->mgmt_tasks));
@@ -528,7 +510,7 @@ lun_reset_task_wait_scsi_task_complete(void)
 	CU_ASSERT(lun->reset_poller != NULL);
 
 	/* Complete the task. */
-	spdk_scsi_lun_complete_task(lun, &task);
+	scsi_lun_complete_task(lun, &task);
 
 	CU_ASSERT(TAILQ_EMPTY(&lun->tasks));
 
@@ -567,22 +549,17 @@ lun_reset_task_suspend_scsi_task(void)
 	mgmt_task.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
 
 	/* Append a reset task to the pending mgmt task list. */
-	spdk_scsi_lun_append_mgmt_task(lun, &mgmt_task);
+	scsi_lun_append_mgmt_task(lun, &mgmt_task);
 
 	CU_ASSERT(!TAILQ_EMPTY(&lun->pending_mgmt_tasks));
 
-	/* Append a task to the pending task list. */
-	spdk_scsi_lun_append_task(lun, &task);
-
-	CU_ASSERT(!TAILQ_EMPTY(&lun->pending_tasks));
-
-	/* Execute the task but it is still on the pending task list. */
-	spdk_scsi_lun_execute_tasks(lun);
+	/* Execute the task but it is on the pending task list. */
+	scsi_lun_execute_task(lun, &task);
 
 	CU_ASSERT(!TAILQ_EMPTY(&lun->pending_tasks));
 
 	/* Execute the reset task. The task will be executed then. */
-	spdk_scsi_lun_execute_mgmt_task(lun);
+	_scsi_lun_execute_mgmt_task(lun);
 
 	CU_ASSERT(TAILQ_EMPTY(&lun->mgmt_tasks));
 	CU_ASSERT(lun->reset_poller == NULL);
@@ -597,52 +574,170 @@ lun_reset_task_suspend_scsi_task(void)
 	CU_ASSERT_EQUAL(g_task_count, 0);
 }
 
+static void
+lun_check_pending_tasks_only_for_specific_initiator(void)
+{
+	struct spdk_bdev bdev = {};
+	struct spdk_scsi_lun *lun;
+	struct spdk_scsi_task task1 = {};
+	struct spdk_scsi_task task2 = {};
+	struct spdk_scsi_port initiator_port1 = {};
+	struct spdk_scsi_port initiator_port2 = {};
+	struct spdk_scsi_port initiator_port3 = {};
+
+	lun = scsi_lun_construct(&bdev, NULL, NULL);
+
+	task1.initiator_port = &initiator_port1;
+	task2.initiator_port = &initiator_port2;
+
+	TAILQ_INSERT_TAIL(&lun->tasks, &task1, scsi_link);
+	TAILQ_INSERT_TAIL(&lun->tasks, &task2, scsi_link);
+	CU_ASSERT(scsi_lun_has_outstanding_tasks(lun) == true);
+	CU_ASSERT(_scsi_lun_has_pending_tasks(lun) == false);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, NULL) == true);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, &initiator_port1) == true);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, &initiator_port2) == true);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, &initiator_port3) == false);
+	TAILQ_REMOVE(&lun->tasks, &task1, scsi_link);
+	TAILQ_REMOVE(&lun->tasks, &task2, scsi_link);
+	CU_ASSERT(_scsi_lun_has_pending_tasks(lun) == false);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, NULL) == false);
+
+	TAILQ_INSERT_TAIL(&lun->pending_tasks, &task1, scsi_link);
+	TAILQ_INSERT_TAIL(&lun->pending_tasks, &task2, scsi_link);
+	CU_ASSERT(scsi_lun_has_outstanding_tasks(lun) == false);
+	CU_ASSERT(_scsi_lun_has_pending_tasks(lun) == true);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, NULL) == true);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, &initiator_port1) == true);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, &initiator_port2) == true);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, &initiator_port3) == false);
+	TAILQ_REMOVE(&lun->pending_tasks, &task1, scsi_link);
+	TAILQ_REMOVE(&lun->pending_tasks, &task2, scsi_link);
+	CU_ASSERT(_scsi_lun_has_pending_tasks(lun) == false);
+	CU_ASSERT(scsi_lun_has_pending_tasks(lun, NULL) == false);
+
+	TAILQ_INSERT_TAIL(&lun->mgmt_tasks, &task1, scsi_link);
+	TAILQ_INSERT_TAIL(&lun->mgmt_tasks, &task2, scsi_link);
+	CU_ASSERT(scsi_lun_has_outstanding_mgmt_tasks(lun) == true);
+	CU_ASSERT(_scsi_lun_has_pending_mgmt_tasks(lun) == false);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, NULL) == true);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, &initiator_port1) == true);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, &initiator_port2) == true);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, &initiator_port3) == false);
+	TAILQ_REMOVE(&lun->mgmt_tasks, &task1, scsi_link);
+	TAILQ_REMOVE(&lun->mgmt_tasks, &task2, scsi_link);
+	CU_ASSERT(_scsi_lun_has_pending_mgmt_tasks(lun) == false);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, NULL) == false);
+
+	TAILQ_INSERT_TAIL(&lun->pending_mgmt_tasks, &task1, scsi_link);
+	TAILQ_INSERT_TAIL(&lun->pending_mgmt_tasks, &task2, scsi_link);
+	CU_ASSERT(_scsi_lun_has_pending_mgmt_tasks(lun) == true);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, NULL) == true);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, &initiator_port1) == true);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, &initiator_port2) == true);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, &initiator_port3) == false);
+	TAILQ_REMOVE(&lun->pending_mgmt_tasks, &task1, scsi_link);
+	TAILQ_REMOVE(&lun->pending_mgmt_tasks, &task2, scsi_link);
+	CU_ASSERT(_scsi_lun_has_pending_mgmt_tasks(lun) == false);
+	CU_ASSERT(scsi_lun_has_pending_mgmt_tasks(lun, NULL) == false);
+
+	scsi_lun_remove(lun);
+}
+
+static void
+abort_pending_mgmt_tasks_when_lun_is_removed(void)
+{
+	struct spdk_bdev bdev = {};
+	struct spdk_scsi_lun *lun;
+	struct spdk_scsi_task task1, task2, task3;
+
+	lun = scsi_lun_construct(&bdev, NULL, NULL);
+
+	/* Normal case */
+	ut_init_task(&task1);
+	ut_init_task(&task2);
+	ut_init_task(&task3);
+	task1.lun = lun;
+	task2.lun = lun;
+	task3.lun = lun;
+	task1.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
+	task2.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
+	task3.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
+
+	CU_ASSERT(g_task_count == 3);
+
+	scsi_lun_append_mgmt_task(lun, &task1);
+	scsi_lun_append_mgmt_task(lun, &task2);
+	scsi_lun_append_mgmt_task(lun, &task3);
+
+	CU_ASSERT(!TAILQ_EMPTY(&lun->pending_mgmt_tasks));
+
+	_scsi_lun_execute_mgmt_task(lun);
+
+	CU_ASSERT(TAILQ_EMPTY(&lun->pending_mgmt_tasks));
+	CU_ASSERT(TAILQ_EMPTY(&lun->mgmt_tasks));
+	CU_ASSERT(g_task_count == 0);
+	CU_ASSERT(task1.response == SPDK_SCSI_TASK_MGMT_RESP_SUCCESS);
+	CU_ASSERT(task2.response == SPDK_SCSI_TASK_MGMT_RESP_SUCCESS);
+	CU_ASSERT(task3.response == SPDK_SCSI_TASK_MGMT_RESP_SUCCESS);
+
+	/* LUN hotplug case */
+	ut_init_task(&task1);
+	ut_init_task(&task2);
+	ut_init_task(&task3);
+	task1.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
+	task2.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
+	task3.function = SPDK_SCSI_TASK_FUNC_LUN_RESET;
+
+	CU_ASSERT(g_task_count == 3);
+
+	scsi_lun_append_mgmt_task(lun, &task1);
+	scsi_lun_append_mgmt_task(lun, &task2);
+	scsi_lun_append_mgmt_task(lun, &task3);
+
+	CU_ASSERT(!TAILQ_EMPTY(&lun->pending_mgmt_tasks));
+
+	lun->removed = true;
+
+	_scsi_lun_execute_mgmt_task(lun);
+
+	CU_ASSERT(TAILQ_EMPTY(&lun->pending_mgmt_tasks));
+	CU_ASSERT(TAILQ_EMPTY(&lun->mgmt_tasks));
+	CU_ASSERT(g_task_count == 0);
+	CU_ASSERT(task1.response == SPDK_SCSI_TASK_MGMT_RESP_INVALID_LUN);
+	CU_ASSERT(task2.response == SPDK_SCSI_TASK_MGMT_RESP_INVALID_LUN);
+	CU_ASSERT(task3.response == SPDK_SCSI_TASK_MGMT_RESP_INVALID_LUN);
+
+	scsi_lun_remove(lun);
+}
+
 int
 main(int argc, char **argv)
 {
 	CU_pSuite	suite = NULL;
 	unsigned int	num_failures;
 
-	if (CU_initialize_registry() != CUE_SUCCESS) {
-		return CU_get_error();
-	}
+	CU_set_error_action(CUEA_ABORT);
+	CU_initialize_registry();
 
 	suite = CU_add_suite("lun_suite", NULL, NULL);
-	if (suite == NULL) {
-		CU_cleanup_registry();
-		return CU_get_error();
-	}
 
-	if (
-		CU_add_test(suite, "task management abort task - not supported",
-			    lun_task_mgmt_execute_abort_task_not_supported) == NULL
-		|| CU_add_test(suite, "task management abort task set - success",
-			       lun_task_mgmt_execute_abort_task_all_not_supported) == NULL
-		|| CU_add_test(suite, "task management - lun reset success",
-			       lun_task_mgmt_execute_lun_reset) == NULL
-		|| CU_add_test(suite, "task management - invalid option",
-			       lun_task_mgmt_execute_invalid_case) == NULL
-		|| CU_add_test(suite, "append task - null lun SPDK_SPC_INQUIRY",
-			       lun_append_task_null_lun_task_cdb_spc_inquiry) == NULL
-		|| CU_add_test(suite, "append task - allocated length less than 4096",
-			       lun_append_task_null_lun_alloc_len_lt_4096) == NULL
-		|| CU_add_test(suite, "append task - unsupported lun",
-			       lun_append_task_null_lun_not_supported) == NULL
-		|| CU_add_test(suite, "execute task - scsi task pending",
-			       lun_execute_scsi_task_pending) == NULL
-		|| CU_add_test(suite, "execute task - scsi task complete",
-			       lun_execute_scsi_task_complete) == NULL
-		|| CU_add_test(suite, "destruct task - success", lun_destruct_success) == NULL
-		|| CU_add_test(suite, "construct - null ctx", lun_construct_null_ctx) == NULL
-		|| CU_add_test(suite, "construct - success", lun_construct_success) == NULL
-		|| CU_add_test(suite, "reset task wait for prior task completion",
-			       lun_reset_task_wait_scsi_task_complete) == NULL
-		|| CU_add_test(suite, "reset task suspend subsequent scsi task",
-			       lun_reset_task_suspend_scsi_task) == NULL
-	) {
-		CU_cleanup_registry();
-		return CU_get_error();
-	}
+	CU_ADD_TEST(suite, lun_task_mgmt_execute_abort_task_not_supported);
+	CU_ADD_TEST(suite, lun_task_mgmt_execute_abort_task_all_not_supported);
+	CU_ADD_TEST(suite, lun_task_mgmt_execute_lun_reset);
+	CU_ADD_TEST(suite, lun_task_mgmt_execute_invalid_case);
+	CU_ADD_TEST(suite, lun_append_task_null_lun_task_cdb_spc_inquiry);
+	CU_ADD_TEST(suite, lun_append_task_null_lun_alloc_len_lt_4096);
+	CU_ADD_TEST(suite, lun_append_task_null_lun_not_supported);
+	CU_ADD_TEST(suite, lun_execute_scsi_task_pending);
+	CU_ADD_TEST(suite, lun_execute_scsi_task_complete);
+	CU_ADD_TEST(suite, lun_destruct_success);
+	CU_ADD_TEST(suite, lun_construct_null_ctx);
+	CU_ADD_TEST(suite, lun_construct_success);
+	CU_ADD_TEST(suite, lun_reset_task_wait_scsi_task_complete);
+	CU_ADD_TEST(suite, lun_reset_task_suspend_scsi_task);
+	CU_ADD_TEST(suite, lun_check_pending_tasks_only_for_specific_initiator);
+	CU_ADD_TEST(suite, abort_pending_mgmt_tasks_when_lun_is_removed);
 
 	CU_basic_set_mode(CU_BRM_VERBOSE);
 	allocate_threads(1);
