@@ -1,4 +1,6 @@
 #!/usr/bin/env bash
+# -*- mode:sh; tab-width:4; sh-basic-offset:4; indent-tabs-mode:nil -*-
+# vim: softtabstop=4 shiftwidth=4 expandtab
 #
 # Copyright (C) 2013 Inktank <info@inktank.com>
 # Copyright (C) 2013 Cloudwatt <libre.licensing@cloudwatt.com>
@@ -28,6 +30,7 @@ else
   CEPH_CONF_PATH="$PWD"
 fi
 conf_fn="$CEPH_CONF_PATH/ceph.conf"
+CEPHADM_DIR_PATH="$CEPH_CONF_PATH/../src/cephadm"
 
 MYUID=$(id -u)
 MYNAME=$(id -nu)
@@ -42,6 +45,39 @@ do_killall() {
     $SUDO killall -u $MYNAME $1
 }
 
+maybe_kill() {
+    local p=$1
+    shift
+    local step=$1
+    shift
+    case $step in
+        0)
+            # killing processes
+            pkill -SIGTERM -u $MYUID $p
+            return 1
+            ;;
+        [1-5])
+            # wait for processes to stop
+            if pkill -0 -u $MYUID $p; then
+                # $p is still alive
+                return 1
+            fi
+            ;;
+        8)
+            # kill and print if some left
+            if pkill -0 -u $MYUID $p; then
+                echo "WARNING: $p did not orderly shutdown, killing it hard!" >&2
+                pkill -SIGKILL -u $MYUID $p
+            fi
+            ;;
+    esac
+}
+
+do_killcephadm() {
+    FSID=$($CEPH_BIN/ceph -c $conf_fn fsid)
+    sudo $CEPHADM_DIR_PATH/cephadm rm-cluster --fsid $FSID --force
+}
+
 do_umountall() {
     #VSTART_IP_PORTS is of the format as below
     #"[v[num]:IP:PORT/0,v[num]:IP:PORT/0][v[num]:IP:PORT/0,v[num]:IP:PORT/0]..."
@@ -72,7 +108,7 @@ do_umountall() {
     [ -n "$CEPH_FUSE_MNTS" ] && sudo umount -f $CEPH_FUSE_MNTS
 }
 
-usage="usage: $0 [all] [mon] [mds] [osd] [rgw] [ganesha] [--crimson]\n"
+usage="usage: $0 [all] [mon] [mds] [osd] [rgw] [nfs] [--crimson] [--cephadm]\n"
 
 stop_all=1
 stop_mon=0
@@ -82,6 +118,7 @@ stop_mgr=0
 stop_rgw=0
 stop_ganesha=0
 ceph_osd=ceph-osd
+stop_cephadm=0
 
 while [ $# -ge 1 ]; do
     case $1 in
@@ -108,13 +145,17 @@ while [ $# -ge 1 ]; do
             stop_rgw=1
             stop_all=0
             ;;
-        ganesha | ganesha.nfsd )
+        nfs | ganesha.nfsd )
             stop_ganesha=1
             stop_all=0
             ;;
         --crimson)
             ceph_osd=crimson-osd
             ;;
+        --cephadm)
+            stop_cephadm=1
+            stop_all=0
+            ;;
         * )
             printf "$usage"
             exit
@@ -145,13 +186,30 @@ if [ $stop_all -eq 1 ]; then
         fi
     fi
 
-    for p in ceph-mon ceph-mds $ceph_osd ceph-mgr radosgw lt-radosgw apache2 ganesha.nfsd ; do
-        for try in 0 1 1 1 1 ; do
-            if ! pkill -u $MYUID $p ; then
-                break
+    daemons="$($CEPHADM_DIR_PATH/cephadm ls 2> /dev/null)"
+    if [ $? -eq 0 -a "$daemons" != "[]" ]; then
+        do_killcephadm
+    fi
+
+    # killing processes
+    to_kill="$ceph_osd ceph-mon ceph-mds ceph-mgr radosgw lt-radosgw apache2 ganesha.nfsd"
+    since_kill=0
+    for step in 0 1 1 2 3 5 8; do
+        sleep $step
+        since_kill=$((since_kill + step))
+        survivors=''
+        for p in $to_kill; do
+            if ! maybe_kill "$p" $step; then
+                survivors+=" $p"
             fi
-            sleep $try
         done
+        if [ -z "$survivors" ]; then
+            break
+        fi
+        to_kill=$survivors
+        if [ $since_kill -gt 0 ]; then
+            echo "WARNING: $to_kill still alive after $since_kill seconds" >&2
+        fi
     done
 
     pkill -u $MYUID -f valgrind.bin.\*ceph-mon
@@ -166,4 +224,5 @@ else
     [ $stop_mgr -eq 1 ] && do_killall ceph-mgr
     [ $stop_ganesha -eq 1 ] && do_killall ganesha.nfsd
     [ $stop_rgw -eq 1 ] && do_killall radosgw lt-radosgw apache2
+    [ $stop_cephadm -eq 1 ] && do_killcephadm
 fi
