@@ -14,7 +14,6 @@
 
 #include <iostream>
 #include <regex>                 // For regex, regex_search
-#include <experimental/filesystem> // For extension
 
 #include "common/admin_socket_client.h"     // For AdminSocketClient
 #include "common/ceph_json.h"               // For JSONParser, JSONObjIter
@@ -22,6 +21,8 @@
 
 #include "admin_socket_output.h"
 
+using namespace std;
+
 void AdminSocketOutput::add_target(const std::string& target) {
   if (target == "all") {
     add_target("osd");
