@@ -2,18 +2,21 @@
 
 #include "auth/Auth.h"
 #include "messages/MPing.h"
+#include "common/ceph_argparse.h"
 #include "crimson/auth/DummyAuth.h"
+#include "crimson/common/throttle.h"
 #include "crimson/net/Connection.h"
 #include "crimson/net/Dispatcher.h"
 #include "crimson/net/Messenger.h"
-#include "crimson/net/Config.h"
-#include "crimson/thread/Throttle.h"
 
 #include <seastar/core/alien.hh>
 #include <seastar/core/app-template.hh>
 #include <seastar/core/future-util.hh>
+#include <seastar/core/internal/pollable_fd.hh>
+#include <seastar/core/posix.hh>
 #include <seastar/core/reactor.hh>
 
+using crimson::common::local_conf;
 
 enum class echo_role {
   as_server,
@@ -35,31 +38,26 @@ struct DummyAuthAuthorizer : public AuthAuthorizer {
 };
 
 struct Server {
-  crimson::thread::Throttle byte_throttler;
+  crimson::common::Throttle byte_throttler;
   crimson::net::MessengerRef msgr;
   crimson::auth::DummyAuthClientServer dummy_auth;
-  struct ServerDispatcher : crimson::net::Dispatcher {
+  struct ServerDispatcher final : crimson::net::Dispatcher {
     unsigned count = 0;
     seastar::condition_variable on_reply;
-    seastar::future<> ms_dispatch(crimson::net::Connection* c,
-                                  MessageRef m) override {
+    std::optional<seastar::future<>> ms_dispatch(crimson::net::ConnectionRef c,
+                                                 MessageRef m) final
+    {
       std::cout << "server got ping " << *m << std::endl;
       // reply with a pong
-      return c->send(make_message<MPing>()).then([this] {
+      return c->send(crimson::make_message<MPing>()).then([this] {
         ++count;
         on_reply.signal();
+        return seastar::now();
       });
     }
-    seastar::future<crimson::net::msgr_tag_t, bufferlist>
-    ms_verify_authorizer(entity_type_t peer_type,
-                         auth_proto_t protocol,
-                         bufferlist& auth) override {
-      return seastar::make_ready_future<crimson::net::msgr_tag_t, bufferlist>(
-          0, bufferlist{});
-    }
   } dispatcher;
   Server(crimson::net::MessengerRef msgr)
-    : byte_throttler(crimson::net::conf.osd_client_message_size_cap),
+    : byte_throttler(local_conf()->osd_client_message_size_cap),
       msgr{msgr}
   {
     msgr->set_crc_header();
@@ -68,14 +66,15 @@ struct Server {
 };
 
 struct Client {
-  crimson::thread::Throttle byte_throttler;
+  crimson::common::Throttle byte_throttler;
   crimson::net::MessengerRef msgr;
   crimson::auth::DummyAuthClientServer dummy_auth;
-  struct ClientDispatcher : crimson::net::Dispatcher {
+  struct ClientDispatcher final : crimson::net::Dispatcher {
     unsigned count = 0;
     seastar::condition_variable on_reply;
-    seastar::future<> ms_dispatch(crimson::net::Connection* c,
-                                  MessageRef m) override {
+    std::optional<seastar::future<>> ms_dispatch(crimson::net::ConnectionRef c,
+                                                 MessageRef m) final
+    {
       std::cout << "client got pong " << *m << std::endl;
       ++count;
       on_reply.signal();
@@ -83,7 +82,7 @@ struct Client {
     }
   } dispatcher;
   Client(crimson::net::MessengerRef msgr)
-    : byte_throttler(crimson::net::conf.osd_client_message_size_cap),
+    : byte_throttler(local_conf()->osd_client_message_size_cap),
       msgr{msgr}
   {
     msgr->set_crc_header();
@@ -93,17 +92,19 @@ struct Client {
 } // namespace seastar_pingpong
 
 class SeastarContext {
-  seastar::file_desc begin_fd;
-  seastar::readable_eventfd on_end;
+  int begin_fd;
+  seastar::file_desc on_end;
 
 public:
   SeastarContext()
-    : begin_fd{seastar::file_desc::eventfd(0, 0)}
+    : begin_fd{eventfd(0, 0)},
+      on_end{seastar::file_desc::eventfd(0, 0)}
   {}
 
   template<class Func>
   std::thread with_seastar(Func&& func) {
-    return std::thread{[this, func = std::forward<Func>(func)] {
+    return std::thread{[this, on_end = on_end.get(),
+	                func = std::forward<Func>(func)] {
         // alien: are you ready?
         wait_for_seastar();
         // alien: could you help me apply(func)?
@@ -111,17 +112,36 @@ public:
         // alien: i've sent my request. have you replied it?
         // wait_for_seastar();
         // alien: you are free to go!
-        on_end.write_side().signal(1);
+        ::eventfd_write(on_end, 1);
       }};
   }
 
   void run(seastar::app_template& app, int argc, char** argv) {
     app.run(argc, argv, [this] {
-      return seastar::now().then([this] {
-        return set_seastar_ready();
+      std::vector<const char*> args;
+      std::string cluster;
+      std::string conf_file_list;
+      auto init_params = ceph_argparse_early_args(args,
+                                                CEPH_ENTITY_TYPE_CLIENT,
+                                                &cluster,
+                                                &conf_file_list);
+      return crimson::common::sharded_conf().start(init_params.name, cluster)
+      .then([conf_file_list] {
+        return local_conf().parse_config_files(conf_file_list);
       }).then([this] {
+        return set_seastar_ready();
+      }).then([on_end = std::move(on_end)] () mutable {
         // seastar: let me know once i am free to leave.
-        return on_end.wait().then([](size_t){});
+        return seastar::do_with(seastar::pollable_fd(std::move(on_end)), [] 
+			        (seastar::pollable_fd& on_end_fds) {
+          return on_end_fds.readable().then([&on_end_fds] {
+            eventfd_t result = 0;
+            on_end_fds.get_file_desc().read(&result, sizeof(result));
+            return seastar::make_ready_future<>();
+          });
+        });
+      }).then([]() {
+        return crimson::common::sharded_conf().stop();
       }).handle_exception([](auto ep) {
         std::cerr << "Error: " << ep << std::endl;
       }).finally([] {
@@ -132,14 +152,14 @@ public:
 
   seastar::future<> set_seastar_ready() {
     // seastar: i am ready to serve!
-    ::eventfd_write(begin_fd.get(), 1);
+    ::eventfd_write(begin_fd, 1);
     return seastar::now();
   }
 
 private:
   void wait_for_seastar() {
     eventfd_t result = 0;
-    if (int r = ::eventfd_read(begin_fd.get(), &result); r < 0) {
+    if (int r = ::eventfd_read(begin_fd, &result); r < 0) {
       std::cerr << "unable to eventfd_read():" << errno << std::endl;
     }
   }
@@ -163,14 +183,17 @@ seastar_echo(const entity_addr_t addr, echo_role role, unsigned count)
       server.msgr->set_auth_client(&server.dummy_auth);
       server.msgr->set_auth_server(&server.dummy_auth);
       return server.msgr->bind(entity_addrvec_t{addr}
-      ).then([&server] {
-        return server.msgr->start(&server.dispatcher);
-      }).then([&dispatcher=server.dispatcher, count] {
+      ).safe_then([&server] {
+        return server.msgr->start({&server.dispatcher});
+      }, crimson::net::Messenger::bind_ertr::all_same_way([](auto& e) {
+        ceph_abort_msg("bind failed");
+      })).then([&dispatcher=server.dispatcher, count] {
         return dispatcher.on_reply.wait([&dispatcher, count] {
           return dispatcher.count >= count;
         });
       }).finally([&server] {
         std::cout << "server shutting down" << std::endl;
+        server.msgr->stop();
         return server.msgr->shutdown();
       });
     });
@@ -186,19 +209,20 @@ seastar_echo(const entity_addr_t addr, echo_role role, unsigned count)
       client.msgr->set_require_authorizer(false);
       client.msgr->set_auth_client(&client.dummy_auth);
       client.msgr->set_auth_server(&client.dummy_auth);
-      return client.msgr->start(&client.dispatcher).then(
+      return client.msgr->start({&client.dispatcher}).then(
           [addr, &client, &disp=client.dispatcher, count] {
         auto conn = client.msgr->connect(addr, entity_name_t::TYPE_OSD);
         return seastar::do_until(
           [&disp,count] { return disp.count >= count; },
           [&disp,conn] {
-            return conn->send(make_message<MPing>()).then([&] {
+            return conn->send(crimson::make_message<MPing>()).then([&] {
               return disp.on_reply.wait();
             });
           }
         );
       }).finally([&client] {
         std::cout << "client shutting down" << std::endl;
+        client.msgr->stop();
         return client.msgr->shutdown();
       });
     });
@@ -218,9 +242,7 @@ int main(int argc, char** argv)
     ("nonce", po::value<uint32_t>()->default_value(42),
      "a unique number to identify the pong server")
     ("count", po::value<unsigned>()->default_value(10),
-     "stop after sending/echoing <count> MPing messages")
-    ("v2", po::value<bool>()->default_value(false),
-     "using msgr v2 protocol");
+     "stop after sending/echoing <count> MPing messages");
   po::variables_map vm;
   std::vector<std::string> unrecognized_options;
   try {
@@ -241,11 +263,7 @@ int main(int argc, char** argv)
   }
 
   entity_addr_t addr;
-  if (vm["v2"].as<bool>()) {
-    addr.set_type(entity_addr_t::TYPE_MSGR2);
-  } else {
-    addr.set_type(entity_addr_t::TYPE_LEGACY);
-  }
+  addr.set_type(entity_addr_t::TYPE_MSGR2);
   addr.set_family(AF_INET);
   addr.set_port(vm["port"].as<std::uint16_t>());
   addr.set_nonce(vm["nonce"].as<std::uint32_t>());
@@ -259,7 +277,7 @@ int main(int argc, char** argv)
   seastar::app_template app;
   SeastarContext sc;
   auto job = sc.with_seastar([&] {
-    auto fut = seastar::alien::submit_to(0, [addr, role, count] {
+    auto fut = seastar::alien::submit_to(app.alien(), 0, [addr, role, count] {
       return seastar_echo(addr, role, count);
     });
     fut.wait();
