@@ -23,8 +23,7 @@ int main(int argc, char **argv)
 
   int r = rados.conf_set("lockdep", "true");
   if (r < 0) {
-    std::cerr << "failed to enable lockdep" << std::endl;
-    return -r;
+    std::cerr << "warning: failed to enable lockdep" << std::endl;
   }
   return RUN_ALL_TESTS();
 }
