@@ -17,120 +17,120 @@ using std::pair;
 class AioTestData
 {
 public:
-  AioTestData() : m_complete(false), m_safe(false) {
-    m_sem = sem_open("test_libradosstriper_aio_sem", O_CREAT, 0644, 0);
+  AioTestData() : m_complete(false) {
+    sem_init(&m_sem, 0, 0);
   }
 
   ~AioTestData() {
-    sem_unlink("test_libradosstriper_aio_sem");
-    sem_close(m_sem);
+    sem_destroy(&m_sem);
+  }
+
+  void notify() {
+    sem_post(&m_sem);
+  }
+
+  void wait() {
+    sem_wait(&m_sem);
   }
 
-  sem_t *m_sem;
   bool m_complete;
-  bool m_safe;
+
+private:
+  sem_t m_sem;
 };
 
 void set_completion_complete(rados_completion_t cb, void *arg)
 {
   AioTestData *test = static_cast<AioTestData*>(arg);
   test->m_complete = true;
-  sem_post(test->m_sem);
-}
-
-void set_completion_safe(rados_completion_t cb, void *arg)
-{
-  AioTestData *test = static_cast<AioTestData*>(arg);
-  test->m_safe = true;
-  sem_post(test->m_sem);
+  test->notify();
 }
 
 TEST_F(StriperTest, SimpleWrite) {
   AioTestData test_data;
   rados_completion_t my_completion;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_write(striper, "StriperTest", my_completion, buf, sizeof(buf), 0));
   TestAlarm alarm;
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   rados_aio_release(my_completion);
 }
 
 TEST_F(StriperTestPP, SimpleWritePP) {
   AioTestData test_data;
   AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+    ((void*)&test_data, set_completion_complete);
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   bufferlist bl1;
   bl1.append(buf, sizeof(buf));
   ASSERT_EQ(0, striper.aio_write("SimpleWritePP", my_completion, bl1, sizeof(buf), 0));
   TestAlarm alarm;
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   my_completion->release();
 }
 
 TEST_F(StriperTest, WaitForSafe) {
   AioTestData test_data;
   rados_completion_t my_completion;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_write(striper, "WaitForSafe", my_completion, buf, sizeof(buf), 0));
   TestAlarm alarm;
-  rados_aio_wait_for_safe(my_completion);
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  rados_aio_wait_for_complete(my_completion);
+  test_data.wait();
   rados_aio_release(my_completion);
 }
 
 TEST_F(StriperTestPP, WaitForSafePP) {
   AioTestData test_data;
-  AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion =
+    librados::Rados::aio_create_completion(&test_data,
+                                           set_completion_complete);
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   bufferlist bl1;
   bl1.append(buf, sizeof(buf));
   ASSERT_EQ(0, striper.aio_write("WaitForSafePP", my_completion, bl1, sizeof(buf), 0));
   TestAlarm alarm;
-  my_completion->wait_for_safe();
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  my_completion->wait_for_complete();
+  test_data.wait();
   my_completion->release();
 }
 
 TEST_F(StriperTest, RoundTrip) {
   AioTestData test_data;
   rados_completion_t my_completion;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_write(striper, "RoundTrip", my_completion, buf, sizeof(buf), 0));
   {
     TestAlarm alarm;
-    sem_wait(test_data.m_sem);
-    sem_wait(test_data.m_sem);
+    test_data.wait();
   }
   char buf2[128];
   memset(buf2, 0, sizeof(buf2));
   rados_completion_t my_completion2;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion2));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion2));
   ASSERT_EQ(0, rados_striper_aio_read(striper, "RoundTrip", my_completion2, buf2, sizeof(buf2), 0));
   {
     TestAlarm alarm;
     rados_aio_wait_for_complete(my_completion2);
   }
   ASSERT_EQ(0, memcmp(buf, buf2, sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   rados_aio_release(my_completion);
   rados_aio_release(my_completion2);
 }
@@ -138,37 +138,37 @@ TEST_F(StriperTest, RoundTrip) {
 TEST_F(StriperTest, RoundTrip2) {
   AioTestData test_data;
   rados_completion_t my_completion;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_write(striper, "RoundTrip2", my_completion, buf, sizeof(buf), 0));
   {
     TestAlarm alarm;
-    sem_wait(test_data.m_sem);
-    sem_wait(test_data.m_sem);
+    test_data.wait();
   }
   char buf2[128];
   memset(buf2, 0, sizeof(buf2));
   rados_completion_t my_completion2;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion2));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion2));
   ASSERT_EQ(0, rados_striper_aio_read(striper, "RoundTrip2", my_completion2, buf2, sizeof(buf2), 0));
   {
     TestAlarm alarm;
-    rados_aio_wait_for_safe(my_completion2);
+    rados_aio_wait_for_complete(my_completion2);
   }
   ASSERT_EQ(0, memcmp(buf, buf2, sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   rados_aio_release(my_completion);
   rados_aio_release(my_completion2);
 }
 
 TEST_F(StriperTestPP, RoundTripPP) {
   AioTestData test_data;
-  AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   bufferlist bl1;
@@ -176,28 +176,26 @@ TEST_F(StriperTestPP, RoundTripPP) {
   ASSERT_EQ(0, striper.aio_write("RoundTripPP", my_completion, bl1, sizeof(buf), 0));
   {
     TestAlarm alarm;
-    sem_wait(test_data.m_sem);
-    sem_wait(test_data.m_sem);
+    test_data.wait();
   }
   bufferlist bl2;
-  AioCompletion *my_completion2 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion2 =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   ASSERT_EQ(0, striper.aio_read("RoundTripPP", my_completion2, &bl2, sizeof(buf), 0));
   {
     TestAlarm alarm;
     my_completion2->wait_for_complete();
   }
   ASSERT_EQ(0, memcmp(buf, bl2.c_str(), sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   my_completion->release();
   my_completion2->release();
 }
 
 TEST_F(StriperTestPP, RoundTripPP2) {
   AioTestData test_data;
-  AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   bufferlist bl1;
@@ -205,20 +203,18 @@ TEST_F(StriperTestPP, RoundTripPP2) {
   ASSERT_EQ(0, striper.aio_write("RoundTripPP2", my_completion, bl1, sizeof(buf), 0));
   {
     TestAlarm alarm;
-    sem_wait(test_data.m_sem);
-    sem_wait(test_data.m_sem);
+    test_data.wait();
   }
   bufferlist bl2;
-  AioCompletion *my_completion2 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion2 =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   ASSERT_EQ(0, striper.aio_read("RoundTripPP2", my_completion2, &bl2, sizeof(buf), 0));
   {
     TestAlarm alarm;
-    my_completion2->wait_for_safe();
+    my_completion2->wait_for_complete();
   }
   ASSERT_EQ(0, memcmp(buf, bl2.c_str(), sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   my_completion->release();
   my_completion2->release();
 }
@@ -226,21 +222,22 @@ TEST_F(StriperTestPP, RoundTripPP2) {
 TEST_F(StriperTest, IsComplete) {
   AioTestData test_data;
   rados_completion_t my_completion;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_write(striper, "IsComplete", my_completion, buf, sizeof(buf), 0));
   {
     TestAlarm alarm;
-    sem_wait(test_data.m_sem);
-    sem_wait(test_data.m_sem);
+    test_data.wait();
   }
   char buf2[128];
   memset(buf2, 0, sizeof(buf2));
   rados_completion_t my_completion2;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion2));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion2));
   ASSERT_EQ(0, rados_striper_aio_read(striper, "IsComplete", my_completion2, buf2, sizeof(buf2), 0));
   {
     TestAlarm alarm;
@@ -253,16 +250,15 @@ TEST_F(StriperTest, IsComplete) {
     }
   }
   ASSERT_EQ(0, memcmp(buf, buf2, sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   rados_aio_release(my_completion);
   rados_aio_release(my_completion2);
 }
 
 TEST_F(StriperTestPP, IsCompletePP) {
   AioTestData test_data;
-  AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   bufferlist bl1;
@@ -270,12 +266,11 @@ TEST_F(StriperTestPP, IsCompletePP) {
   ASSERT_EQ(0, striper.aio_write("IsCompletePP", my_completion, bl1, sizeof(buf), 0));
   {
     TestAlarm alarm;
-    sem_wait(test_data.m_sem);
-    sem_wait(test_data.m_sem);
+    test_data.wait();
   }
   bufferlist bl2;
-  AioCompletion *my_completion2 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion2 =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   ASSERT_EQ(0, striper.aio_read("IsCompletePP", my_completion2, &bl2, sizeof(buf), 0));
   {
     TestAlarm alarm;
@@ -288,8 +283,7 @@ TEST_F(StriperTestPP, IsCompletePP) {
     }
   }
   ASSERT_EQ(0, memcmp(buf, bl2.c_str(), sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   my_completion->release();
   my_completion2->release();
 }
@@ -297,8 +291,9 @@ TEST_F(StriperTestPP, IsCompletePP) {
 TEST_F(StriperTest, IsSafe) {
   AioTestData test_data;
   rados_completion_t my_completion;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_write(striper, "IsSafe", my_completion, buf, sizeof(buf), 0));
@@ -315,59 +310,26 @@ TEST_F(StriperTest, IsSafe) {
   char buf2[128];
   memset(buf2, 0, sizeof(buf2));
   rados_completion_t my_completion2;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion2));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion2));
   ASSERT_EQ(0, rados_striper_aio_read(striper, "IsSafe", my_completion2, buf2, sizeof(buf2), 0));
   {
     TestAlarm alarm;
     rados_aio_wait_for_complete(my_completion2);
   }
   ASSERT_EQ(0, memcmp(buf, buf2, sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   rados_aio_release(my_completion);
   rados_aio_release(my_completion2);
 }
 
-TEST_F(StriperTestPP, IsSafePP) {
-  AioTestData test_data;
-  AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
-  char buf[128];
-  memset(buf, 0xcc, sizeof(buf));
-  bufferlist bl1;
-  bl1.append(buf, sizeof(buf));
-  ASSERT_EQ(0, striper.aio_write("IsSafePP", my_completion, bl1, sizeof(buf), 0));
-  {
-    TestAlarm alarm;
-    // Busy-wait until the AIO completes.
-    // Normally we wouldn't do this, but we want to test rados_aio_is_safe.
-    while (true) {
-      int is_safe = my_completion->is_safe();
-      if (is_safe)
-	break;
-    }
-  }
-  bufferlist bl2;
-  AioCompletion *my_completion2 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
-  ASSERT_EQ(0, striper.aio_read("IsSafePP", my_completion2, &bl2, sizeof(buf), 0));
-  {
-    TestAlarm alarm;
-    my_completion2->wait_for_complete();
-  }
-  ASSERT_EQ(0, memcmp(buf, bl2.c_str(), sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
-  my_completion->release();
-  my_completion2->release();
-}
-
 TEST_F(StriperTest, RoundTripAppend) {
   AioTestData test_data;
   rados_completion_t my_completion, my_completion2, my_completion3;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                           set_completion_complete,
+                                           &my_completion));
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_append(striper, "RoundTripAppend", my_completion, buf, sizeof(buf)));
@@ -377,8 +339,9 @@ TEST_F(StriperTest, RoundTripAppend) {
   }
   char buf2[128];
   memset(buf2, 0xdd, sizeof(buf2));
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion2));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion2));
   ASSERT_EQ(0, rados_striper_aio_append(striper, "RoundTripAppend", my_completion2, buf2, sizeof(buf)));
   {
     TestAlarm alarm;
@@ -386,8 +349,9 @@ TEST_F(StriperTest, RoundTripAppend) {
   }
   char buf3[sizeof(buf) + sizeof(buf2)];
   memset(buf3, 0, sizeof(buf3));
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion3));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion3));
   ASSERT_EQ(0, rados_striper_aio_read(striper, "RoundTripAppend", my_completion3, buf3, sizeof(buf3), 0));
   {
     TestAlarm alarm;
@@ -396,8 +360,7 @@ TEST_F(StriperTest, RoundTripAppend) {
   ASSERT_EQ((int)(sizeof(buf) + sizeof(buf2)), rados_aio_get_return_value(my_completion3));
   ASSERT_EQ(0, memcmp(buf3, buf, sizeof(buf)));
   ASSERT_EQ(0, memcmp(buf3 + sizeof(buf), buf2, sizeof(buf2)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   rados_aio_release(my_completion);
   rados_aio_release(my_completion2);
   rados_aio_release(my_completion3);
@@ -405,8 +368,8 @@ TEST_F(StriperTest, RoundTripAppend) {
 
 TEST_F(StriperTestPP, RoundTripAppendPP) {
   AioTestData test_data;
-  AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   bufferlist bl1;
@@ -420,16 +383,16 @@ TEST_F(StriperTestPP, RoundTripAppendPP) {
   memset(buf2, 0xdd, sizeof(buf2));
   bufferlist bl2;
   bl2.append(buf2, sizeof(buf2));
-  AioCompletion *my_completion2 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion2 =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   ASSERT_EQ(0, striper.aio_append("RoundTripAppendPP", my_completion2, bl2, sizeof(buf2)));
   {
     TestAlarm alarm;
     my_completion2->wait_for_complete();
   }
   bufferlist bl3;
-  AioCompletion *my_completion3 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion3 =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   ASSERT_EQ(0, striper.aio_read("RoundTripAppendPP", my_completion3, &bl3, 2 * sizeof(buf), 0));
   {
     TestAlarm alarm;
@@ -438,8 +401,7 @@ TEST_F(StriperTestPP, RoundTripAppendPP) {
   ASSERT_EQ(sizeof(buf) + sizeof(buf2), (unsigned)my_completion3->get_return_value());
   ASSERT_EQ(0, memcmp(bl3.c_str(), buf, sizeof(buf)));
   ASSERT_EQ(0, memcmp(bl3.c_str() + sizeof(buf), buf2, sizeof(buf2)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   my_completion->release();
   my_completion2->release();
   my_completion3->release();
@@ -448,8 +410,9 @@ TEST_F(StriperTestPP, RoundTripAppendPP) {
 TEST_F(StriperTest, Flush) {
   AioTestData test_data;
   rados_completion_t my_completion;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   char buf[128];
   memset(buf, 0xee, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_write(striper, "Flush", my_completion, buf, sizeof(buf), 0));
@@ -457,24 +420,24 @@ TEST_F(StriperTest, Flush) {
   char buf2[128];
   memset(buf2, 0, sizeof(buf2));
   rados_completion_t my_completion2;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion2));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion2));
   ASSERT_EQ(0, rados_striper_aio_read(striper, "Flush", my_completion2, buf2, sizeof(buf2), 0));
   {
     TestAlarm alarm;
     rados_aio_wait_for_complete(my_completion2);
   }
   ASSERT_EQ(0, memcmp(buf, buf2, sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   rados_aio_release(my_completion);
   rados_aio_release(my_completion2);
 }
 
 TEST_F(StriperTestPP, FlushPP) {
   AioTestData test_data;
-  AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   char buf[128];
   memset(buf, 0xee, sizeof(buf));
   bufferlist bl1;
@@ -482,16 +445,15 @@ TEST_F(StriperTestPP, FlushPP) {
   ASSERT_EQ(0, striper.aio_write("FlushPP", my_completion, bl1, sizeof(buf), 0));
   striper.aio_flush();
   bufferlist bl2;
-  AioCompletion *my_completion2 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion2 =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   ASSERT_EQ(0, striper.aio_read("FlushPP", my_completion2, &bl2, sizeof(buf), 0));
   {
     TestAlarm alarm;
     my_completion2->wait_for_complete();
   }
   ASSERT_EQ(0, memcmp(buf, bl2.c_str(), sizeof(buf)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   my_completion->release();
   my_completion2->release();
 }
@@ -499,8 +461,9 @@ TEST_F(StriperTestPP, FlushPP) {
 TEST_F(StriperTest, RoundTripWriteFull) {
   AioTestData test_data;
   rados_completion_t my_completion, my_completion2, my_completion3;
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   ASSERT_EQ(0, rados_striper_aio_write(striper, "RoundTripWriteFull", my_completion, buf, sizeof(buf), 0));
@@ -510,8 +473,9 @@ TEST_F(StriperTest, RoundTripWriteFull) {
   }
   char buf2[64];
   memset(buf2, 0xdd, sizeof(buf2));
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion2));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion2));
   ASSERT_EQ(0, rados_striper_aio_write_full(striper, "RoundTripWriteFull", my_completion2, buf2, sizeof(buf2)));
   {
     TestAlarm alarm;
@@ -519,8 +483,9 @@ TEST_F(StriperTest, RoundTripWriteFull) {
   }
   char buf3[sizeof(buf) + sizeof(buf2)];
   memset(buf3, 0, sizeof(buf3));
-  ASSERT_EQ(0, rados_aio_create_completion
-            ((void*)&test_data, set_completion_complete, set_completion_safe, &my_completion3));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion3));
   ASSERT_EQ(0, rados_striper_aio_read(striper, "RoundTripWriteFull", my_completion3, buf3, sizeof(buf3), 0));
   {
     TestAlarm alarm;
@@ -528,8 +493,7 @@ TEST_F(StriperTest, RoundTripWriteFull) {
   }
   ASSERT_EQ(sizeof(buf2), (unsigned)rados_aio_get_return_value(my_completion3));
   ASSERT_EQ(0, memcmp(buf3, buf2, sizeof(buf2)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   rados_aio_release(my_completion);
   rados_aio_release(my_completion2);
   rados_aio_release(my_completion3);
@@ -537,8 +501,8 @@ TEST_F(StriperTest, RoundTripWriteFull) {
 
 TEST_F(StriperTestPP, RoundTripWriteFullPP) {
   AioTestData test_data;
-  AioCompletion *my_completion = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   char buf[128];
   memset(buf, 0xcc, sizeof(buf));
   bufferlist bl1;
@@ -552,16 +516,16 @@ TEST_F(StriperTestPP, RoundTripWriteFullPP) {
   memset(buf2, 0xdd, sizeof(buf2));
   bufferlist bl2;
   bl2.append(buf2, sizeof(buf2));
-  AioCompletion *my_completion2 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion2 =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   ASSERT_EQ(0, striper.aio_write_full("RoundTripWriteFullPP", my_completion2, bl2));
   {
     TestAlarm alarm;
     my_completion2->wait_for_complete();
   }
   bufferlist bl3;
-  AioCompletion *my_completion3 = librados::Rados::aio_create_completion
-    ((void*)&test_data, set_completion_complete, set_completion_safe);
+  AioCompletion *my_completion3 =
+    librados::Rados::aio_create_completion(&test_data, set_completion_complete);
   ASSERT_EQ(0, striper.aio_read("RoundTripWriteFullPP", my_completion3, &bl3, sizeof(buf), 0));
   {
     TestAlarm alarm;
@@ -569,8 +533,7 @@ TEST_F(StriperTestPP, RoundTripWriteFullPP) {
   }
   ASSERT_EQ(sizeof(buf2), (unsigned)my_completion3->get_return_value());
   ASSERT_EQ(0, memcmp(bl3.c_str(), buf2, sizeof(buf2)));
-  sem_wait(test_data.m_sem);
-  sem_wait(test_data.m_sem);
+  test_data.wait();
   my_completion->release();
   my_completion2->release();
   my_completion3->release();
@@ -585,8 +548,9 @@ TEST_F(StriperTest, RemoveTest) {
   // async remove it
   AioTestData test_data;
   rados_completion_t my_completion;
-  ASSERT_EQ(0, rados_aio_create_completion((void*)&test_data,
-	      set_completion_complete, set_completion_safe, &my_completion));
+  ASSERT_EQ(0, rados_aio_create_completion2(&test_data,
+                                            set_completion_complete,
+                                            &my_completion));
   ASSERT_EQ(0, rados_striper_aio_remove(striper, "RemoveTest", my_completion));
   {
     TestAlarm alarm;
@@ -604,7 +568,7 @@ TEST_F(StriperTestPP, RemoveTestPP) {
   bufferlist bl;
   bl.append(buf, sizeof(buf));
   ASSERT_EQ(0, striper.write("RemoveTestPP", bl, sizeof(buf), 0));
-  AioCompletion *my_completion = cluster.aio_create_completion(0, 0, 0);
+  AioCompletion *my_completion = cluster.aio_create_completion(nullptr, nullptr);
   ASSERT_EQ(0, striper.aio_remove("RemoveTestPP", my_completion));
   {
     TestAlarm alarm;
