@@ -72,9 +72,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, WriteThrough) {
   MockContext finish_ctx;
   MockContext dispatch_ctx;
   Context* finish_ctx_ptr = &finish_ctx;
-  ASSERT_FALSE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                     nullptr, nullptr, &dispatch_result,
-                                     &finish_ctx_ptr, &dispatch_ctx));
+  ASSERT_FALSE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                     std::nullopt, {}, nullptr, nullptr,
+                                     &dispatch_result, &finish_ctx_ptr,
+                                     &dispatch_ctx));
   ASSERT_EQ(finish_ctx_ptr, &finish_ctx);
 }
 
@@ -95,9 +96,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, WriteThroughUntilFlushed) {
   MockContext finish_ctx;
   MockContext dispatch_ctx;
   Context* finish_ctx_ptr = &finish_ctx;
-  ASSERT_FALSE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                     nullptr, nullptr, &dispatch_result,
-                                     &finish_ctx_ptr, &dispatch_ctx));
+  ASSERT_FALSE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                     std::nullopt, {}, nullptr, nullptr,
+                                     &dispatch_result, &finish_ctx_ptr,
+                                     &dispatch_ctx));
   ASSERT_EQ(finish_ctx_ptr, &finish_ctx);
 
   ASSERT_FALSE(object_dispatch.flush(io::FLUSH_SOURCE_USER, {}, nullptr,
@@ -107,9 +109,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, WriteThroughUntilFlushed) {
   expect_context_complete(dispatch_ctx, 0);
   expect_context_complete(finish_ctx, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                     nullptr, nullptr, &dispatch_result,
-                                     &finish_ctx_ptr, &dispatch_ctx));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr,
+                                    &dispatch_ctx));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr, &finish_ctx);
   ASSERT_EQ(0, dispatch_ctx.wait());
@@ -138,9 +141,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, DispatchIO) {
   expect_context_complete(dispatch_ctx, 0);
   expect_context_complete(finish_ctx, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr, &dispatch_ctx));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr,
+                                    &dispatch_ctx));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr, &finish_ctx);
 
@@ -170,9 +174,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, BlockedIO) {
   expect_context_complete(dispatch_ctx1, 0);
   expect_context_complete(finish_ctx1, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr1, &dispatch_ctx1));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0,  0,
+                                    std::nullopt,{}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr1,
+                                    &dispatch_ctx1));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr1, &finish_ctx1);
 
@@ -183,9 +188,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, BlockedIO) {
   expect_context_complete(dispatch_ctx2, 0);
   expect_context_complete(finish_ctx2, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 4096, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr2, &dispatch_ctx2));
+  ASSERT_TRUE(object_dispatch.write(0, 4096, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr2,
+                                    &dispatch_ctx2));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr2, &finish_ctx2);
 
@@ -193,9 +199,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, BlockedIO) {
   MockContext dispatch_ctx3;
   Context* finish_ctx_ptr3 = &finish_ctx3;
 
-  ASSERT_TRUE(object_dispatch.write(0, 1024, std::move(data), {}, 0,
-                                    {}, nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr3, &dispatch_ctx3));
+  ASSERT_TRUE(object_dispatch.write(0, 1024, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr3,
+                                    &dispatch_ctx3));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr3, &finish_ctx3);
 
@@ -235,9 +242,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, QueuedIO) {
   expect_context_complete(dispatch_ctx1, 0);
   expect_context_complete(finish_ctx1, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr1, &dispatch_ctx1));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr1,
+                                    &dispatch_ctx1));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr1, &finish_ctx1);
 
@@ -245,9 +253,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, QueuedIO) {
   MockContext dispatch_ctx2;
   Context* finish_ctx_ptr2 = &finish_ctx2;
 
-  ASSERT_TRUE(object_dispatch.write(0, 8192, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr2, &dispatch_ctx2));
+  ASSERT_TRUE(object_dispatch.write(0, 8192, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr2,
+                                    &dispatch_ctx2));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr2, &finish_ctx2);
   ASSERT_EQ(0, dispatch_ctx1.wait());
@@ -283,9 +292,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, BlockedAndQueuedIO) {
   expect_context_complete(dispatch_ctx1, 0);
   expect_context_complete(finish_ctx1, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr1, &dispatch_ctx1));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr1,
+                                    &dispatch_ctx1));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr1, &finish_ctx1);
 
@@ -296,9 +306,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, BlockedAndQueuedIO) {
   expect_context_complete(dispatch_ctx2, 0);
   expect_context_complete(finish_ctx2, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 4096, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr2, &dispatch_ctx2));
+  ASSERT_TRUE(object_dispatch.write(0, 4096, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr2,
+                                    &dispatch_ctx2));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr2, &finish_ctx2);
 
@@ -306,9 +317,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, BlockedAndQueuedIO) {
   MockContext dispatch_ctx3;
   Context* finish_ctx_ptr3 = &finish_ctx3;
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0,
-                                    {}, nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr3, &dispatch_ctx3));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr3,
+                                    &dispatch_ctx3));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr3, &finish_ctx3);
 
@@ -368,9 +380,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, FlushQueuedOnInFlightIO) {
   expect_context_complete(dispatch_ctx1, 0);
   expect_context_complete(finish_ctx1, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr1, &dispatch_ctx1));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr1,
+                                    &dispatch_ctx1));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr1, &finish_ctx1);
   ASSERT_EQ(0, dispatch_ctx1.wait());
@@ -413,9 +426,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, FlushQueuedOnQueuedIO) {
   expect_context_complete(dispatch_ctx1, 0);
   expect_context_complete(finish_ctx1, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr1, &dispatch_ctx1));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr1,
+                                    &dispatch_ctx1));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr1, &finish_ctx1);
   ASSERT_EQ(0, dispatch_ctx1.wait());
@@ -424,9 +438,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, FlushQueuedOnQueuedIO) {
   MockContext dispatch_ctx2;
   Context* finish_ctx_ptr2 = &finish_ctx2;
 
-  ASSERT_TRUE(object_dispatch.write(0, 8192, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr2, &dispatch_ctx2));
+  ASSERT_TRUE(object_dispatch.write(0, 8192, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr2,
+                                    &dispatch_ctx2));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr2, &finish_ctx2);
   ASSERT_EQ(0, dispatch_ctx1.wait());
@@ -477,9 +492,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, FlushError) {
   expect_context_complete(dispatch_ctx1, 0);
   expect_context_complete(finish_ctx1, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr1, &dispatch_ctx1));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr1,
+                                    &dispatch_ctx1));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr1, &finish_ctx1);
   ASSERT_EQ(0, dispatch_ctx1.wait());
@@ -547,9 +563,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, UnoptimizedIOInFlightIO) {
   expect_context_complete(dispatch_ctx1, 0);
   expect_context_complete(finish_ctx1, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr1, &dispatch_ctx1));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr1,
+                                    &dispatch_ctx1));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr1, &finish_ctx1);
   ASSERT_EQ(0, dispatch_ctx1.wait());
@@ -593,9 +610,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, UnoptimizedIOBlockedIO) {
   expect_context_complete(dispatch_ctx1, 0);
   expect_context_complete(finish_ctx1, 0);
 
-  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr1, &dispatch_ctx1));
+  ASSERT_TRUE(object_dispatch.write(0, 0, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr1,
+                                    &dispatch_ctx1));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr1, &finish_ctx1);
   ASSERT_EQ(0, dispatch_ctx1.wait());
@@ -604,9 +622,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, UnoptimizedIOBlockedIO) {
   MockContext finish_ctx2;
   MockContext dispatch_ctx2;
   Context* finish_ctx_ptr2 = &finish_ctx2;
-  ASSERT_TRUE(object_dispatch.write(0, 4096, std::move(data), {}, 0, {},
-                                    nullptr, nullptr, &dispatch_result,
-                                    &finish_ctx_ptr2, &dispatch_ctx2));
+  ASSERT_TRUE(object_dispatch.write(0, 4096, std::move(data), {}, 0, 0,
+                                    std::nullopt, {}, nullptr, nullptr,
+                                    &dispatch_result, &finish_ctx_ptr2,
+                                    &dispatch_ctx2));
   ASSERT_EQ(io::DISPATCH_RESULT_CONTINUE, dispatch_result);
   ASSERT_NE(finish_ctx_ptr2, &finish_ctx2);
 
@@ -649,9 +668,10 @@ TEST_F(TestMockCacheWriteAroundObjectDispatch, WriteFUA) {
   MockContext dispatch_ctx;
   Context* finish_ctx_ptr = &finish_ctx;
   ASSERT_FALSE(object_dispatch.write(0, 0, std::move(data), {},
-                                     LIBRADOS_OP_FLAG_FADVISE_FUA, {},
-                                     nullptr, nullptr, &dispatch_result,
-                                     &finish_ctx_ptr, &dispatch_ctx));
+                                     LIBRADOS_OP_FLAG_FADVISE_FUA, 0,
+                                     std::nullopt, {}, nullptr, nullptr,
+                                     &dispatch_result, &finish_ctx_ptr,
+                                     &dispatch_ctx));
   ASSERT_EQ(finish_ctx_ptr, &finish_ctx);
 }
 
