@@ -38,26 +38,28 @@ public:
     std::string oid(ObjectMap<>::object_map_name(ictx->id, snap_id));
     if (snap_id == CEPH_NOSNAP) {
       EXPECT_CALL(get_mock_io_ctx(ictx->md_ctx),
-                  exec(oid, _, StrEq("lock"), StrEq("assert_locked"), _, _, _))
+                  exec(oid, _, StrEq("lock"), StrEq("assert_locked"), _, _, _,
+                       _))
                     .WillOnce(DoDefault());
     }
 
     if (r < 0) {
       EXPECT_CALL(get_mock_io_ctx(ictx->md_ctx),
                   exec(oid, _, StrEq("rbd"), StrEq("object_map_update"),
-                       ContentsEqual(bl), _, _))
+                       ContentsEqual(bl), _, _, _))
                     .WillOnce(Return(r));
     } else {
       EXPECT_CALL(get_mock_io_ctx(ictx->md_ctx),
                   exec(oid, _, StrEq("rbd"), StrEq("object_map_update"),
-                       ContentsEqual(bl), _, _))
+                       ContentsEqual(bl), _, _, _))
                     .WillOnce(DoDefault());
     }
   }
 
   void expect_invalidate(librbd::ImageCtx *ictx) {
     EXPECT_CALL(get_mock_io_ctx(ictx->md_ctx),
-                exec(ictx->header_oid, _, StrEq("rbd"), StrEq("set_flags"), _, _, _))
+                exec(ictx->header_oid, _, StrEq("rbd"), StrEq("set_flags"), _,
+                     _, _, _))
                   .WillOnce(DoDefault());
   }
 };
@@ -232,6 +234,7 @@ TEST_F(TestMockObjectMapUpdateRequest, BatchUpdate) {
                                         no_progress));
   ASSERT_EQ(0, acquire_exclusive_lock(*ictx));
 
+  expect_unlock_exclusive_lock(*ictx);
   InSequence seq;
   expect_update(ictx, CEPH_NOSNAP, 0, 262144, OBJECT_NONEXISTENT, OBJECT_EXISTS,
                 0);
@@ -239,7 +242,6 @@ TEST_F(TestMockObjectMapUpdateRequest, BatchUpdate) {
                 OBJECT_EXISTS, 0);
   expect_update(ictx, CEPH_NOSNAP, 524288, 712312, OBJECT_NONEXISTENT,
                 OBJECT_EXISTS, 0);
-  expect_unlock_exclusive_lock(*ictx);
 
   ceph::shared_mutex object_map_lock = ceph::make_shared_mutex("lock");
   ceph::BitVector<2> object_map;
