@@ -36,6 +36,8 @@ public:
                  const std::string &cookie);
   int unlock_image();
 
+  int flush_writeback_cache(librbd::ImageCtx *image_ctx);
+
   int acquire_exclusive_lock(librbd::ImageCtx &ictx);
 
   static std::string _pool_name;
@@ -44,6 +46,7 @@ public:
   static uint64_t _image_number;
   static std::string _data_pool;
 
+  CephContext* m_cct = nullptr;
   librados::IoCtx m_ioctx;
   librbd::RBD m_rbd;
 
