@@ -25,6 +25,8 @@
 #include "kv/KeyValueDB.h"
 #include "os/ObjectStore.h"
 
+using namespace std;
+
 void usage(const string &name) {
   std::cerr << "Usage: " << name << " [new|continue] store_path store_journal db_path"
 	    << std::endl;
@@ -39,9 +41,7 @@ typename T::iterator rand_choose(T &cont) {
 }
 
 int main(int argc, char **argv) {
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **)argv, args);
-
+  auto args = argv_to_vec(argc, argv);
   auto cct = global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT,
 			 CODE_ENVIRONMENT_UTILITY,
 			 CINIT_FLAG_NO_DEFAULT_CONFIG_FILE);
