@@ -44,7 +44,7 @@ public:
 
 TEST_F(TestInstanceWatcher, InitShutdown)
 {
-  InstanceWatcher<> instance_watcher(m_local_io_ctx, m_threads->work_queue,
+  InstanceWatcher<> instance_watcher(m_local_io_ctx, *m_threads->asio_engine,
                                      nullptr, nullptr, m_instance_id);
   std::vector<std::string> instance_ids;
   get_instances(&instance_ids);
@@ -93,7 +93,7 @@ TEST_F(TestInstanceWatcher, Remove)
   librados::IoCtx io_ctx;
   ASSERT_EQ("", connect_cluster_pp(cluster));
   ASSERT_EQ(0, cluster.ioctx_create(_local_pool_name.c_str(), io_ctx));
-  InstanceWatcher<> instance_watcher(m_local_io_ctx, m_threads->work_queue,
+  InstanceWatcher<> instance_watcher(m_local_io_ctx, *m_threads->asio_engine,
                                      nullptr, nullptr, "instance_id");
   // Init
   ASSERT_EQ(0, instance_watcher.init());
@@ -109,7 +109,7 @@ TEST_F(TestInstanceWatcher, Remove)
 
   // Remove
   C_SaferCond on_remove;
-  InstanceWatcher<>::remove_instance(m_local_io_ctx, m_threads->work_queue,
+  InstanceWatcher<>::remove_instance(m_local_io_ctx, *m_threads->asio_engine,
                                      "instance_id", &on_remove);
   ASSERT_EQ(0, on_remove.wait());
 
@@ -126,7 +126,7 @@ TEST_F(TestInstanceWatcher, Remove)
 
   // Remove NOENT
   C_SaferCond on_remove_noent;
-  InstanceWatcher<>::remove_instance(m_local_io_ctx, m_threads->work_queue,
+  InstanceWatcher<>::remove_instance(m_local_io_ctx, *m_threads->asio_engine,
                                      instance_id, &on_remove_noent);
   ASSERT_EQ(0, on_remove_noent.wait());
 }
