@@ -14,6 +14,8 @@
 #include "common/safe_io.h"
 #include "os/filestore/JournalingObjectStore.h"
 
+using namespace std;
+
 Finisher *finisher;
 ceph::condition_variable sync_cond;
 char path[200];
@@ -62,8 +64,7 @@ unsigned size_mb = 200;
 const char GTEST_PRFIX[] = "--gtest_";
 
 int main(int argc, char **argv) {
-  vector<const char*> args;
-  argv_to_vec(argc, (const char **)argv, args);
+  auto args = argv_to_vec(argc, argv);
 
   auto cct = global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT,
 			 CODE_ENVIRONMENT_UTILITY,
