@@ -23,12 +23,17 @@ using namespace std;
 #undef TYPE_FEATUREFUL_NOCOPY
 #undef MESSAGE
 
-#include "denc_registry.h"
+#include "denc_plugin.h"
 
 // cannot initialize dencoders when initializing static variables, as some of
 // the types are allocated using mempool, and the mempools are initialized as
 // static variables.
-void register_osd_dencoders()
+DENC_API void register_dencoders(DencoderPlugin* plugin)
 {
 #include "osd_types.h"
 }
+
+DENC_API void unregister_dencoders(DencoderPlugin* plugin)
+{
+  plugin->unregister_dencoders();
+}
