@@ -7,12 +7,19 @@
 
 #include "TableTool.h"
 
+using namespace std;
 
 int main(int argc, const char **argv)
 {
-  vector<const char*> args;
-  argv_to_vec(argc, argv, args);
-  env_to_vec(args);
+  auto args = argv_to_vec(argc, argv);
+  if (args.empty()) {
+    cerr << argv[0] << ": -h or --help for usage" << std::endl;
+    exit(1);
+  }
+  if (ceph_argparse_need_usage(args)) {
+    TableTool::usage();
+    exit(0);
+  }
 
   auto cct = global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT,
                          CODE_ENVIRONMENT_UTILITY, 0);
@@ -20,12 +27,6 @@ int main(int argc, const char **argv)
 
   TableTool tt;
 
-  // Handle --help before calling init() so we don't depend on network.
-  if (args.empty() || (args.size() == 1 && (std::string(args[0]) == "--help" || std::string(args[0]) == "-h"))) {
-    tt.usage();
-    return 0;
-  }
-
   // Connect to mon cluster, download MDS map etc
   int rc = tt.init();
   if (rc != 0) {
