@@ -911,7 +911,7 @@ int execute_peer_bootstrap_import(
 
   int fd = STDIN_FILENO;
   if (token_path != "-") {
-    fd = open(token_path.c_str(), O_RDONLY);
+    fd = open(token_path.c_str(), O_RDONLY|O_BINARY);
     if (fd < 0) {
       r = -errno;
       std::cerr << "rbd: error opening " << token_path << std::endl;
@@ -1744,27 +1744,27 @@ Shell::Action action_set(
 
 Shell::Action action_disable(
   {"mirror", "pool", "disable"}, {},
-  "Disable RBD mirroring by default within a pool.", "",
+  "Disable RBD mirroring in a pool or namespace.", "",
   &get_disable_arguments, &execute_disable);
 Shell::Action action_enable(
   {"mirror", "pool", "enable"}, {},
-  "Enable RBD mirroring by default within a pool.", "",
+  "Enable RBD mirroring in a pool or namespace.", "",
   &get_enable_arguments, &execute_enable);
 Shell::Action action_info(
   {"mirror", "pool", "info"}, {},
-  "Show information about the pool mirroring configuration.", {},
+  "Show mirroring configuration for a pool or namespace.", {},
   &get_info_arguments, &execute_info);
 Shell::Action action_status(
   {"mirror", "pool", "status"}, {},
-  "Show status for all mirrored images in the pool.", {},
+  "Show status for all mirrored images in a pool or namespace.", {},
   &get_status_arguments, &execute_status);
 Shell::Action action_promote(
   {"mirror", "pool", "promote"}, {},
-  "Promote all non-primary images in the pool.", {},
+  "Promote all non-primary images in a pool or namespace.", {},
   &get_promote_arguments, &execute_promote);
 Shell::Action action_demote(
   {"mirror", "pool", "demote"}, {},
-  "Demote all primary images in the pool.", {},
+  "Demote all primary images in a pool or namespace.", {},
   &get_demote_arguments, &execute_demote);
 
 } // namespace mirror_pool
